@@ -141,6 +141,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>Up: MoveTop()\n\
  <KeyPress>osfUp: MoveTop()\n\
  <KeyPress>KP_Up: MoveTop()\n\
@@ -180,11 +185,6 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
  <KeyPress>i: Pairs()\n\
- <KeyPress>0x2E: Speed()\n\
- <KeyPress>0x3E: Speed()\n\
- <KeyPress>0x3C: Slow()\n\
- <KeyPress>0x2C: Slow()\n\
- <KeyPress>@: Sound()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
@@ -320,13 +320,16 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(BarrelWidget, barrel.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(BarrelWidget, barrel.started),
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNcheat, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(BarrelWidget, barrel.cheat),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(BarrelWidget, barrel.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(BarrelWidget, barrel.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -539,8 +542,6 @@ CheckSolved(const BarrelWidget w)
 {
 	int color, i, j, inc;
 
-	if (w->barrel.tiles < 2)
-		return True;
 	if (w->barrel.orient) {
 		for (j = 0; j < w->barrel.faces; j++) {
 			color = (w->barrel.tileOfPosition[j * w->barrel.tiles +
@@ -612,7 +613,7 @@ TilePicture(BarrelWidget w, int tile)
 }
 
 #ifdef DEBUG
-static void
+void
 PrintPuzzle(BarrelWidget w)
 {
 	int face, tile, plunger;
@@ -620,8 +621,10 @@ PrintPuzzle(BarrelWidget w)
 	for (face = 0; face < w->barrel.faces; face++) {
 		for (tile = 0; tile < w->barrel.tiles; tile++) {
 			(void) printf("%d %d, ",
-				w->barrel.tileOfPosition[face * w->barrel.tiles + tile],
-				TilePicture(w, w->barrel.tileOfPosition[face * w->barrel.tiles + tile]));
+				w->barrel.tileOfPosition[face *
+					w->barrel.tiles + tile],
+				TilePicture(w, w->barrel.tileOfPosition[face *
+					w->barrel.tiles + tile]));
 			if (!((tile + 1) % w->barrel.tiles))
 				(void) printf("\n");
 		}
@@ -809,51 +812,58 @@ DrawBufferedTile(BarrelWidget w, int pos, int pressedOffset)
 	/* FILLRECTANGLE(w, *dr, tileGC,
 		dx + 1, dy + 1,
 		w->barrel.tileSize.x, w->barrel.tileSize.y - 2); */
-	if (tile > 0 && tile < ((w->barrel.tiles - 2) * w->barrel.faces +
-			NUMPLUNGERS + 1)) {
+	if (tile > 0 && tile <= ((w->barrel.tiles - 2) * w->barrel.faces +
+			NUMPLUNGERS)) {
 		DrawBall(w, *dr, ballGC, pos, pressedOffset, pressedOffset);
-	}
-	buf[0] = '\0';
-	if (w->barrel.orient || w->barrel.mono) {
-		if (w->barrel.mono) {
-			if (w->barrel.orient) {
-				int last;
+		buf[0] = '\0';
+		if (w->barrel.orient || w->barrel.mono) {
+			if (w->barrel.mono) {
+				if (w->barrel.orient) {
+					int last;
 
-				last = int2String(w, buf, tile, w->barrel.base, False);
-				buf[last] =
+					last = int2String(w, buf, tile,
+						w->barrel.base, False);
+					buf[last] =
 #ifdef WINVER
-					w->barrel.faceChar[((tile - 1) / (w->barrel.tiles - 2)) %
-						MAXCOLORS];
+						w->barrel.faceChar[((tile - 1) /
+							(w->barrel.tiles - 2)) %
+							MAXCOLORS];
 #else
-					w->barrel.faceName[((tile - 1) / (w->barrel.tiles -2)) %
-						MAXCOLORS][0];
+						w->barrel.faceName[((tile - 1) /
+							(w->barrel.tiles -2)) %
+							MAXCOLORS][0];
 #endif
-				buf[last + 1] = '\0';
-				t *= w->barrel.base;
+					buf[last + 1] = '\0';
+					t *= w->barrel.base;
+				} else {
+					buf[0] =
+#ifdef WINVER
+						w->barrel.faceChar[((tile - 1) /
+							(w->barrel.tiles - 2)) %
+							MAXCOLORS];
+#else
+						w->barrel.faceName[((tile - 1) /
+							(w->barrel.tiles - 2)) %
+							MAXCOLORS][0];
+#endif
+					buf[1] = '\0';
+					t = 1;
+				}
 			} else {
-				buf[0] =
-#ifdef WINVER
-					w->barrel.faceChar[((tile - 1) /
-						(w->barrel.tiles - 2)) % MAXCOLORS];
-#else
-					w->barrel.faceName[((tile - 1) /
-						(w->barrel.tiles - 2)) % MAXCOLORS][0];
-#endif
-				buf[1] = '\0';
-				t = 1;
+				(void) int2String(w, buf, tile,
+					w->barrel.base, True);
 			}
-		} else {
-			(void) int2String(w, buf, tile, w->barrel.base, True);
+			while (t >= 1) {
+				t /= w->barrel.base;
+				digitOffsetX += w->barrel.digitOffset.x;
+				i++;
+			}
+			DRAWTEXT(w, *dr, tileGC,
+				dx + w->barrel.tileSize.x / 2 - digitOffsetX,
+				dy + w->barrel.tileSize.y / 2 +
+				w->barrel.digitOffset.y,
+				buf, i);
 		}
-		while (t >= 1) {
-			t /= w->barrel.base;
-			digitOffsetX += w->barrel.digitOffset.x;
-			i++;
-		}
-		DRAWTEXT(w, *dr, tileGC,
-			dx + w->barrel.tileSize.x / 2 - digitOffsetX,
-			dy + w->barrel.tileSize.y / 2 + w->barrel.digitOffset.y,
-			buf, i);
 	}
 #if 0
 	DRAWRECTANGLE(w, *dr, borderGC, dx, dy,
@@ -996,11 +1006,11 @@ AnimateRotate(BarrelWidget w, int start, int finish, int direction, int fast)
 	XWindowAttributes xgwa;
 #endif
 	int size = finish - start + 1;
-	int moveI, gapJ, moveJ, inc, i;
+	int moveI, gapJ, moveJ, inc, i, offset = 0;
 
 	moveI = w->barrel.tileSize.x + w->barrel.delta.x;
 	gapJ = w->barrel.tileSize.y * fast / w->barrel.numSlices;
-	moveJ = w->barrel.tileSize.y + w->barrel.delta.y + 2;
+	moveJ = w->barrel.tileSize.y + w->barrel.delta.y + 1;
 	if (gapJ == 0)
 		gapJ++;
 
@@ -1021,7 +1031,7 @@ AnimateRotate(BarrelWidget w, int start, int finish, int direction, int fast)
 	(void) XGetWindowAttributes(XtDisplay(w), XtWindow(w), &xgwa);
 
 	if ((bufferBox = XCreatePixmap(XtDisplay(w), XtWindow(w),
-			size * moveI,
+			size * moveI + 1,
 			w->barrel.puzzleSize.y,
 			xgwa.depth)) == None) {
 		XtError("Not enough memory, exiting.");
@@ -1040,47 +1050,48 @@ AnimateRotate(BarrelWidget w, int start, int finish, int direction, int fast)
 	for (inc = 0; inc < moveJ + gapJ; inc += gapJ) {
 		if (inc > moveJ)
 			inc = moveJ;
-		if (direction == TOP)
+		if (direction == TOP) {
 			i = inc;
-		else
+		} else {
 			i = w->barrel.puzzleSize.y - inc + 1;
+			offset = 3;
+		}
 #ifdef WINVER
 		hOldBitmap = SelectObject(memDC, hBitmap);
 		BitBlt(w->core.hDC,
 			start * moveI + w->barrel.puzzleOffset.x + 1,
-			w->barrel.puzzleOffset.y + 1,
+			w->barrel.puzzleOffset.y + offset,
 			size * moveI,
-			w->barrel.puzzleSize.y - i,
+			w->barrel.puzzleSize.y - i - 1,
 			memDC,
 			0, i,
 			SRCCOPY);
 		BitBlt(w->core.hDC,
 			start * moveI + w->barrel.puzzleOffset.x + 1,
 			w->barrel.puzzleSize.y - i + w->barrel.puzzleOffset.y -
-			w->barrel.delta.y + 1,
+			w->barrel.delta.y - 1 + offset,
 			size * moveI,
-			i - w->barrel.delta.y,
+			i - w->barrel.delta.y - 1,
 			memDC,
 			0, 0,
 			SRCCOPY);
 		(void) SelectObject(memDC, hOldBitmap);
 #else
-
 		XCopyArea(XtDisplay(w), bufferBox, XtWindow(w),
 			w->barrel.frameGC,
 			0, i,
 			size * moveI,
-			w->barrel.puzzleSize.y - i,
+			w->barrel.puzzleSize.y - i - 1,
 			start * moveI + w->barrel.puzzleOffset.x + 1,
-			w->barrel.puzzleOffset.y + 1);
+			w->barrel.puzzleOffset.y + offset);
 		XCopyArea(XtDisplay(w), bufferBox, XtWindow(w),
 			w->barrel.frameGC,
 			0, 0,
 			size * moveI,
-			i - w->barrel.delta.y,
+			i - w->barrel.delta.y - 1,
 			start * moveI + w->barrel.puzzleOffset.x + 1,
 			w->barrel.puzzleSize.y - i + w->barrel.puzzleOffset.y -
-			w->barrel.delta.y + 1);
+			w->barrel.delta.y - 1 + offset);
 #endif
 		FLUSH(w);
 		useTimer(&(w->barrel.oldTime), w->barrel.delay / fast);
@@ -1137,7 +1148,7 @@ DrawFrame(BarrelWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoTiles(BarrelWidget w)
 {
-	setPuzzle(w, PUZZLE_IGNORE);
+	setPuzzle(w, ACTION_IGNORE);
 }
 
 static void
@@ -1219,8 +1230,7 @@ RotateTiles(BarrelWidget w, int start, int finish, int direction, int fast)
 					w->barrel.tiles) % w->barrel.tileFaces;
 		}
 	}
-	/* Would be nice if this was not needed. */
-#ifndef DEBUG
+	/* Needed in case a window is on top. */
 	if (animate) {
 		int moveI = w->barrel.tileSize.x + w->barrel.delta.x;
 		Pixmap dr = 0;
@@ -1231,7 +1241,6 @@ RotateTiles(BarrelWidget w, int start, int finish, int direction, int fast)
 		DrawFrame(w, dr, w->barrel.focus);
 		DrawAllTilesForColumns(w, start, finish);
 	}
-#endif
 #ifdef USE_SOUND
 	if (w->barrel.sound) {
 		playSound((char *) MOVESOUND);
@@ -1288,33 +1297,33 @@ static void
 MoveShift(BarrelWidget w, int direction, int fast)
 {
 	(void) MoveTilesDir(w, direction, 0, True, fast);
-	setPuzzle(w, PUZZLE_CONTROL);
+	setPuzzle(w, ACTION_CONTROL);
 }
 
 #if 0
 static void
 ControlPuzzle(BarrelWidget w, int direction)
 {
-	int reason = PUZZLE_IGNORE;
+	int reason = ACTION_IGNORE;
 
 	switch (direction) {
 	case TOP:
 		if (w->barrel.faces <= MINFACES)
 			return;
-		reason = PUZZLE_DEC_Y;
+		reason = ACTION_DECY;
 		break;
 	case RIGHT:
-		reason = PUZZLE_INC_X;
+		reason = ACTION_INCX;
 		break;
 	case BOTTOM:
 		if (w->barrel.faces >= MAXFACES)
 			return;
-		reason = PUZZLE_INC_Y;
+		reason = ACTION_INCY;
 		break;
 	case LEFT:
 		if (w->barrel.tiles <= MINTILES)
 			return;
-		reason = PUZZLE_DEC_X;
+		reason = ACTION_DECX;
 		break;
 	default:
 		{
@@ -1343,7 +1352,7 @@ MovePuzzle(BarrelWidget w, const int direction, const int tile,
 		return True;
 	} else if (MoveTilesDir(w, direction, tile, False, fast)) {
 		if (!motion) {
-			setPuzzle(w, PUZZLE_MOVED);
+			setPuzzle(w, ACTION_MOVED);
 			setMove(&undo, direction, tile, 0);
 			flushMoves(w, &redo, FALSE);
 		}
@@ -1376,7 +1385,7 @@ MovePosition(BarrelWidget w,
 	if (shift != 0 && (direction == TOP || direction == BOTTOM)) {
 		setMove(&undo, direction, tile, 1);
 	} else {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction, tile, 0);
 	}
 	flushMoves(w, &redo, FALSE);
@@ -1409,7 +1418,7 @@ MovePuzzleDelay(BarrelWidget w, const int direction, const int tile,
 		setMove(&undo, direction,
 			w->barrel.tiles - 1, False);
 	} else {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction, tile, all);
 	}
 	flushMoves(w, &redo, FALSE);
@@ -1425,11 +1434,11 @@ SlidePuzzle(BarrelWidget w, int direction)
 		return;
 	}
 	if (!MovePuzzle(w, direction, 0, FALSE, False, NORMAL)) {
-		setPuzzle(w, PUZZLE_BLOCKED);
+		setPuzzle(w, ACTION_BLOCKED);
 		return;
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return;
 }
@@ -1443,7 +1452,7 @@ RotatePuzzle(BarrelWidget w, int direction, int tile)
 	}
 	(void) MovePuzzle(w, direction, tile, FALSE, False, NORMAL);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return;
 }
@@ -1466,18 +1475,18 @@ MovePuzzleInput(BarrelWidget w, int x, int direction, int shift)
 	} else if (direction == LEFT || direction == RIGHT) {
 		SlidePuzzle(w, direction);
 		if (CheckSolved(w)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 	} else {
 		if (!PositionTile(w, x, &r))
 			return;
 		if (r == 0 || r == w->barrel.tiles - 1) {
-			setPuzzle(w, PUZZLE_BLOCKED);
+			setPuzzle(w, ACTION_BLOCKED);
 			return;
 		}
 		RotatePuzzle(w, direction, r);
 		if (CheckSolved(w)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 	}
 }
@@ -1515,7 +1524,7 @@ SelectSlideTiles(BarrelWidget w, int pos)
 			MoveTiles(w, w->barrel.spacePosition[0] % w->barrel.tiles - w->barrel.tiles + 1,
 				NORMAL);
 		}
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, RIGHT, 0, 0);
 		flushMoves(w, &redo, FALSE);
 	} else if (pos > 0) {
@@ -1530,7 +1539,7 @@ SelectSlideTiles(BarrelWidget w, int pos)
 			MoveTiles(w, w->barrel.spacePosition[0] % w->barrel.tiles + w->barrel.tiles - 1,
 				NORMAL);
 		}
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, LEFT, w->barrel.tiles - 1, 0);
 		flushMoves(w, &redo, FALSE);
 	}
@@ -1590,7 +1599,7 @@ static void
 GetTiles(BarrelWidget w)
 {
 	FILE *fp;
-	int c, pairs, orient, practice, moves;
+	int c, i, tiles, faces, pairs, orient, practice, moves;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -1631,15 +1640,14 @@ GetTiles(BarrelWidget w)
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-#if 0
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &tiles);
 	if (tiles >= MINTILES) {
 		for (i = w->barrel.tiles; i < tiles; i++) {
-			setPuzzle(w, PUZZLE_INC_X);
+			setPuzzle(w, ACTION_INCX);
 		}
 		for (i = w->barrel.tiles; i > tiles; i--) {
-			setPuzzle(w, PUZZLE_DEC_X);
+			setPuzzle(w, ACTION_DECX);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: tiles ");
@@ -1658,10 +1666,10 @@ GetTiles(BarrelWidget w)
 	(void) fscanf(fp, "%d", &faces);
 	if (faces >= MINFACES && faces <= MAXFACES) {
 		for (i = w->barrel.faces; i < faces; i++) {
-			setPuzzle(w, PUZZLE_INC_Y);
+			setPuzzle(w, ACTION_INCY);
 		}
 		for (i = w->barrel.faces; i > faces; i--) {
-			setPuzzle(w, PUZZLE_DEC_Y);
+			setPuzzle(w, ACTION_DECY);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: faces ");
@@ -1678,21 +1686,20 @@ GetTiles(BarrelWidget w)
 		DISPLAY_WARNING(buf2);
 		free(buf2);
 	}
-#endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &pairs);
 	if (w->barrel.pairs != (Boolean) pairs) {
-		setPuzzle(w, PUZZLE_PAIRS);
+		setPuzzle(w, ACTION_PAIRS);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->barrel.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
-	if (w->barrel.orient != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+	if (w->barrel.practice != (Boolean) practice) {
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetTiles(w);
@@ -1700,12 +1707,15 @@ GetTiles(BarrelWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: pairs %d, orient %d, practice %d, moves %d.\n",
-		name, pairs, orient, practice, moves);
+	(void) printf(
+		"%s: tiles %d, faces %d, pairs %s, orient %s, practice %s, moves %d.\n",
+		name, tiles, faces,
+		BOOL_STRING(pairs), BOOL_STRING(orient), BOOL_STRING(practice),
+		moves);
 	free(lname);
 	free(fname);
 	w->barrel.cheat = True; /* Assume the worst. */
@@ -1753,10 +1763,8 @@ WriteTiles(BarrelWidget w)
 		}
 #endif
 	}
-#if 0
 	(void) fprintf(fp, "tiles%c %d\n", SYMBOL, w->barrel.tiles);
 	(void) fprintf(fp, "faces%c %d\n", SYMBOL, w->barrel.faces);
-#endif
 	(void) fprintf(fp, "pairs%c %d\n", SYMBOL,
 		(w->barrel.pairs) ? 1 : 0);
 	(void) fprintf(fp, "orient%c %d\n", SYMBOL,
@@ -1786,9 +1794,9 @@ UndoTiles(BarrelWidget w)
 		if (shift != 0 && (direction == TOP || direction == BOTTOM)) {
 			MoveShift(w, direction, DOUBLE);
 		} else if (MoveTilesDir(w, direction, tile, False, DOUBLE)) {
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else {
 			char *buf1, *buf2;
@@ -1814,9 +1822,9 @@ RedoTiles(BarrelWidget w)
 		if (shift != 0 && (direction == TOP || direction == BOTTOM)) {
 			MoveShift(w, direction, DOUBLE);
 		} else if (MoveTilesDir(w, direction, tile, False, DOUBLE)) {
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else {
 			char *buf1, *buf2;
@@ -1837,13 +1845,13 @@ ClearTiles(BarrelWidget w)
 		return;
 	ResetTiles(w);
 	DrawAllTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
 PracticeTiles(BarrelWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
@@ -1853,7 +1861,7 @@ RandomizeTiles(BarrelWidget w)
 		return;
 	if (w->barrel.practice)
 		PracticeTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 	w->barrel.cheat = False;
 	/* Now move the space around randomly */
 	if (w->barrel.tiles > 1 || w->barrel.faces > 1) {
@@ -1863,7 +1871,7 @@ RandomizeTiles(BarrelWidget w)
 
 		if (big > 100)
 			big = 100;
-#ifdef DEBUG
+#ifdef RDEBUG
 		big = 3;
 #endif
 		while (big-- > 0) {
@@ -1884,7 +1892,7 @@ RandomizeTiles(BarrelWidget w)
 						NRAND(w->barrel.tiles - 2) + 1,
 						FALSE, False, INSTANT);
 				}
-#ifdef DEBUG
+#ifdef RDEBUG
 				sleep(1);
 #endif
 			} else {
@@ -1893,10 +1901,10 @@ RandomizeTiles(BarrelWidget w)
 		}
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1906,23 +1914,27 @@ SolveTiles(BarrelWidget w)
 	if (CheckSolved(w) || w->barrel.currentRef != -w->barrel.tileFaces)
 		return;
 	/* if (w->barrel.tiles == 6 && w->barrel.faces == 5) */
-#ifdef AUTOSOLVE
+#ifdef DEBUG
 	SolveSomeTiles(w);
 #else
-	setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+	if (!w->barrel.orient) {
+		SolveSomeTiles(w);
+	} else {
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
+	}
 #endif
 }
 
 static void
 OrientizeTiles(BarrelWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 PairsTiles(BarrelWidget w)
 {
-	setPuzzle(w, PUZZLE_PAIRS);
+	setPuzzle(w, ACTION_PAIRS);
 }
 
 static void
@@ -1932,7 +1944,7 @@ SpeedTiles(BarrelWidget w)
 	if (w->barrel.delay < 0)
 		w->barrel.delay = 0;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -1941,7 +1953,7 @@ SlowTiles(BarrelWidget w)
 {
 	w->barrel.delay += 5;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2306,48 +2318,48 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		w->barrel.numSlices = ((w->barrel.delay < MAXSLICES) ?
 			w->barrel.delay + 1 : MAXSLICES);
 	}
-	if (w->barrel.menu != -1) {
+	if (w->barrel.menu != ACTION_IGNORE) {
 		int menu = w->barrel.menu;
 
-		w->barrel.menu = -1;
+		w->barrel.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetTiles(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WriteTiles(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoTiles(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoTiles(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearTiles(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizeTiles(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticeTiles(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizeTiles(w);
+			break;
+		case ACTION_SOLVE:
 			SolveTiles(w);
 			break;
-		case MENU_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizeTiles(w);
 			break;
-		case MENU_PAIRS:
+		case ACTION_PAIRS:
 			PairsTiles(w);
 			break;
-		case MENU_SPEED:
+		case ACTION_SPEED:
 			SpeedTiles(w);
 			break;
-		case MENU_SLOW:
+		case ACTION_SLOW:
 			SlowTiles(w);
 			break;
-		case MENU_SOUND:
+		case ACTION_SOUND:
 			SoundTiles(w);
 			break;
 		default:
@@ -2498,7 +2510,7 @@ ResizeTiles(BarrelWidget w)
 	if (!(w->barrel.picture && *(w->barrel.picture))) {
 		DrawAllBufferedTiles(w);
 	} else if (!w->barrel.orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 }
 
@@ -2629,7 +2641,7 @@ HidePuzzle(BarrelWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2693,6 +2705,19 @@ MotionPuzzle(BarrelWidget w
 		return;
 	}
 	pos = SelectTiles(w, x, y, &i, &j);
+	if (shift == 0 && (w->barrel.currentTile == 0 ||
+			w->barrel.currentTile == w->barrel.tiles - 1) &&
+			j != w->barrel.currentFace) {
+		int k = w->barrel.currentTile +
+			w->barrel.currentFace * w->barrel.tiles;
+
+		DrawTile(w, k, True, True, TRUE, 0, 0);
+		DrawTile(w, k, (w->barrel.tileOfPosition[k] <= 0),
+			(w->barrel.tileOfPosition[k] <= 0), FALSE, 0, 0);
+		setPuzzle(w, ACTION_BLOCKED);
+		w->barrel.currentRef = -w->barrel.tileFaces;
+		return;
+	}
 	if (w->barrel.currentTile != i) {
 		int k = w->barrel.currentTile +
 			w->barrel.currentFace * w->barrel.tiles;
@@ -2703,7 +2728,7 @@ MotionPuzzle(BarrelWidget w
 		DiffPosition(w, w->barrel.motionDiff, j, FALSE);
 		DiffPosition(w, w->barrel.shiftDiff, j, TRUE);
 		if (shift != 0 && CheckSolved(w)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 		w->barrel.currentTile = i;
 		w->barrel.currentFace = j;
@@ -2722,7 +2747,7 @@ MotionPuzzle(BarrelWidget w
 				w->barrel.faces) % w->barrel.faces;
 			DiffMove(w, diff, j, shift);
 			if (shift != 0 && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 			w->barrel.motion = True;
 			if (shift == 0)
@@ -2737,8 +2762,10 @@ MotionPuzzle(BarrelWidget w
 		w->barrel.currentFace = j;
 		w->barrel.currentRef = pos;
 		if (shift != 0 || w->barrel.practice || !CheckSolved(w)) {
-			pos = w->barrel.currentTile + w->barrel.currentFace * w->barrel.tiles;
-			DrawTile(w, pos, (w->barrel.tileOfPosition[i + j * w->barrel.tiles] < 0),
+			pos = w->barrel.currentTile +
+				w->barrel.currentFace * w->barrel.tiles;
+			DrawTile(w, pos, (w->barrel.tileOfPosition[i +
+					j * w->barrel.tiles] < 0),
 				False, TRUE, 0, 0);
 		}
 	} else
@@ -2784,7 +2811,7 @@ ReleasePuzzle(BarrelWidget w
 				diff = -w->barrel.tiles + 1;
 			(void) SelectSlideTiles(w, diff);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else {
 			if (shift == 0 && (w->barrel.currentTile == 0 ||
@@ -2805,7 +2832,7 @@ ReleasePuzzle(BarrelWidget w
 			DiffPosition(w, w->barrel.motionDiff, j, FALSE);
 			DiffPosition(w, w->barrel.shiftDiff, j, TRUE);
 			if (shift == 0 && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -2822,7 +2849,7 @@ PracticePuzzleMaybe(BarrelWidget w
 		PracticeTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -2849,7 +2876,7 @@ RandomizePuzzleMaybe(BarrelWidget w
 		RandomizeTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
