@@ -30,7 +30,6 @@
 #include "file.h"
 #ifdef WINVER
 #include "BarrelP.h"
-#include "wbarrel.h"
 #define TITLE "wbarrel"
 
 static BarrelRec widget;
@@ -59,8 +58,8 @@ static BarrelRec widget;
 #include <Xm/Scale.h>
 #include <Xm/ToggleB.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -70,9 +69,19 @@ static BarrelRec widget;
 #include "Barrel.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/barrel.xpm"
+#include "pixmaps/barrel.t.xpm"
+#include "pixmaps/barrel.p.xpm"
+#include "pixmaps/barrel.s.xpm"
+#include "pixmaps/barrel.m.xpm"
+#include "pixmaps/barrel.l.xpm"
+#include "pixmaps/barrel.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?barrel_t_xpm:barrel_p_xpm):\
+(((s)<=24)?barrel_s_xpm:barrel_m_xpm)):\
+(((s)<=48)?barrel_l_xpm:barrel_xpm))
 #endif
-#include "icons/barrel.xbm"
+#include "pixmaps/barrel.xbm"
+#define DEFINE_XBM (char *) barrel_bits, barrel_width, barrel_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -84,7 +93,7 @@ static BarrelRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Barrel Version 7.3\n"
+"Barrel Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -143,8 +152,8 @@ static const char featuresHelp[] = {
 "\n"
 "Press \"C\" or \"c\" keys to clear the puzzle.\n"
 "\n"
-"\"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
+"\"S\" or \"s\" keys to start the auto-solver.  Only works "
+"where oriented is off.\n"
 "\n"
 "Press \"O\" or \"o\" keys to toggle orient mode.  Slightly "
 "more challenging on (default is off).\n"
@@ -188,7 +197,7 @@ static const char referencesHelp[] = {
 #endif
 
 static const char solveHelp[] = {
-"Auto-solver: sorry, not implemented.\n"
+"Auto-solver: sorry, only implemented when oriented is off.\n"
 };
 
 #ifndef SCOREFILE
@@ -206,9 +215,9 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord barrelRecord[2][2];
+static PuzzleRecord puzzleRecord[2][2];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -217,7 +226,7 @@ static char recordDsp[MESSAGELEN] = "NOT RECORDED";
 #define MINSPEED 1
 #define MAXSPEED 50
 static Widget moves, record, message;
-static Widget orientSwitch, pairsSwitch, practiceSwitch;
+static Widget orientizeSwitch, pairsSwitch, practiceSwitch;
 #if 0
 static Widget tile, face;
 #endif
@@ -237,8 +246,8 @@ static char fileName[FILENAMELEN];
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, barrel;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -264,11 +273,11 @@ PrintRecords(void)
 	(void) printf("PAIR ORIENT USER            MOVES\n");
 	for (l = 0; l < 2; l++)
 		for (k = 0; k < 2; k++) {
-			if (barrelRecord[l][k].score > 0)
+			if (puzzleRecord[l][k].score > 0)
 				(void) printf("%4d%7d %-12s%9d\n",
 					l, k,
-					barrelRecord[l][k].name,
-					barrelRecord[l][k].score);
+					puzzleRecord[l][k].name,
+					puzzleRecord[l][k].score);
 		}
 }
 #endif
@@ -280,8 +289,8 @@ InitRecords(void)
 
 	for (l = 0; l < 2; l++)
 		for (k = 0; k < 2; k++) {
-			barrelRecord[l][k].score = NEVER;
-			(void) strncpy(barrelRecord[l][k].name,
+			puzzleRecord[l][k].score = NEVER;
+			(void) strncpy(puzzleRecord[l][k].name,
 				NOACCESS, USERNAMELEN);
 		}
 }
@@ -335,10 +344,10 @@ ReadRecords(void)
 	for (l = 0; l < 2; l++)
 		for (k = 0; k < 2; k++) {
 			(void) fscanf(fp, "%d %s\n", &n, userName);
-			if (n <= barrelRecord[l][k].score ||
-					barrelRecord[l][k].score <= NEVER) {
-				barrelRecord[l][k].score = n;
-				(void) strncpy(barrelRecord[l][k].name,
+			if (n <= puzzleRecord[l][k].score ||
+					puzzleRecord[l][k].score <= NEVER) {
+				puzzleRecord[l][k].score = n;
+				(void) strncpy(puzzleRecord[l][k].name,
 					userName, USERNAMELEN);
 			}
 		}
@@ -382,8 +391,8 @@ WriteRecords(void)
 		for (l = 0; l < 2; l++) {
 			for (k = 0; k < 2; k++) {
 				(void) fprintf(fp, "%d %s\n",
-					barrelRecord[l][k].score,
-					barrelRecord[l][k].name);
+					puzzleRecord[l][k].score,
+					puzzleRecord[l][k].name);
 			}
 			(void) fprintf(fp, "\n");
 		}
@@ -406,12 +415,12 @@ PrintRecord(Boolean orient, Boolean pairs, Boolean practice)
 	} else if (tiles > MAXTILES) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
 #endif
-	} else if (barrelRecord[l][k].score <= NEVER) {
+	} else if (puzzleRecord[l][k].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			barrelRecord[l][k].score,
-			barrelRecord[l][k].name);
+			puzzleRecord[l][k].score,
+			puzzleRecord[l][k].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -439,7 +448,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(barrel), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -449,11 +458,11 @@ HandleSolved(int counter, Boolean orient, Boolean pairs)
 {
 	int k = (orient) ? 1 : 0, l = (pairs) ? 1 : 0;
 
-	if ((counter < barrelRecord[l][k].score ||
-			barrelRecord[l][k].score <= NEVER)) {
+	if ((counter < puzzleRecord[l][k].score ||
+			puzzleRecord[l][k].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		barrelRecord[l][k].score = counter;
-		(void) strncpy(barrelRecord[l][k].name, userNameDsp,
+		puzzleRecord[l][k].score = counter;
+		(void) strncpy(puzzleRecord[l][k].name, userNameDsp,
 			USERNAMELEN);
 		WriteRecords();
 		PrintRecord(orient, pairs, False);
@@ -515,13 +524,13 @@ Widget w
 	XmScaleSetValue(tile, tiles);
 	XmScaleSetValue(face, faces);
 #endif
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(pairsSwitch, pairs, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 	{
 		int delay;
 
-		XtVaGetValues(barrel, XtNdelay, &delay, NULL);
+		XtVaGetValues(w, XtNdelay, &delay, NULL);
 		XmScaleSetValue(speed, MAXSPEED + MINSPEED - delay - 1);
 	}
 #endif
@@ -588,7 +597,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -598,22 +607,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(barrel, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(barrel, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -625,7 +634,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice", MESSAGELEN);
 #ifdef HAVE_MOTIF
@@ -634,30 +643,30 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 		}
 		movesDsp = 0;
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		break;
-	case PUZZLE_BLOCKED:
+	case ACTION_BLOCKED:
 		(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 		break;
-	case PUZZLE_SPACE:
+	case ACTION_SPACE:
 #if 0
 		/* Too annoying */
 		(void) strncpy(messageDsp, "A space can not slide",
 			MESSAGELEN);
 #endif
 		break;
-	case PUZZLE_IGNORE:
+	case ACTION_IGNORE:
 		(void) strncpy(messageDsp, "Randomize to start",
 			MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		return;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -671,7 +680,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 				MESSAGELEN);
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		practice = !practice;
 		if (!practice)
@@ -690,7 +702,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 		XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 #ifdef WINVER
 		w->barrel.practice = False;
@@ -701,7 +713,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 			XtNstart, False, NULL);
 #endif
 		break;
-	case PUZZLE_ORIENTIZE:
+	case ACTION_ORIENTIZE:
 		movesDsp = 0;
 		orient = !orient;
 		PrintRecord(orient, pairs, practice);
@@ -710,11 +722,11 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #else
 		XtVaSetValues(w, XtNorient, orient, NULL);
 #ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
 #endif
 #endif
 		break;
-	case PUZZLE_PAIRS:
+	case ACTION_PAIRS:
 		movesDsp = 0;
 		pairs = !pairs;
 		PrintRecord(orient, pairs, practice);
@@ -728,22 +740,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #endif
 		break;
 #if 0
-	case PUZZLE_DEC_X:
-		movesDsp = 0;
-		tiles--;
-		PrintRecord(tiles, faces, orient, pairs, practice);
-#ifdef WINVER
-		w->barrel.tiles = tiles;
-#else
-		XtVaSetValues(w, XtNtiles, tiles, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(tile, tiles);
-		if (tiles >= MAXTILES)
-			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
-#endif
-#endif
-		break;
-	case PUZZLE_INC_X:
+	case ACTION_INCX:
 		movesDsp = 0;
 		tiles++;
 		PrintRecord(tiles, faces, orient, pairs, practice);
@@ -758,7 +755,35 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_DEC_Y:
+	case ACTION_DECX:
+		movesDsp = 0;
+		tiles--;
+		PrintRecord(tiles, faces, orient, pairs, practice);
+#ifdef WINVER
+		w->barrel.tiles = tiles;
+#else
+		XtVaSetValues(w, XtNtiles, tiles, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(tile, tiles);
+		if (tiles > MAXTILES)
+			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
+#endif
+#endif
+		break;
+	case ACTION_INCY:
+		movesDsp = 0;
+		faces++;
+		PrintRecord(tiles, faces, orient, pairs, practice);
+#ifdef WINVER
+		w->barrel.faces = faces;
+#else
+		XtVaSetValues(w, XtNfaces, faces, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(face, faces);
+#endif
+#endif
+		break;
+	case ACTION_DECY:
 		movesDsp = 0;
 		faces--;
 		PrintRecord(tiles, faces, orient, pairs, practice);
@@ -771,37 +796,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_INC_Y:
-		movesDsp = 0;
-		faces++;
-		PrintRecord(tiles, faces, orient, pairs, practice);
-#ifdef WINVER
-		w->barrel.faces = faces;
-#else
-		XtVaSetValues(w, XtNfaces, faces, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(face, faces);
 #endif
-		break;
-#endif
-#endif
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
-		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
 #ifdef HAVE_MOTIF
-	case PUZZLE_SPEED:
+	case ACTION_SPEED:
 		{
 			int oldDelay, delay;
 
-			XtVaGetValues(barrel, XtNdelay, &delay, NULL);
+			XtVaGetValues(puzzle,
+				XtNdelay, &delay, NULL);
 			oldDelay = delay;
 			if (delay > MAXSPEED - MINSPEED)
 				delay = MAXSPEED - MINSPEED;
@@ -960,94 +970,85 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		break;
 #endif
 	case WM_COMMAND:
-		switch LOWORD((wParam)) {
-		case IDM_GET:
+		switch (LOWORD(wParam)) {
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(&widget, brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_PRACTICE:
+			PracticePuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_PRACTICE:
-			PracticePuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_PAIRS:
+		case ACTION_PAIRS:
 			(void) PairsPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SPEED:
-			SpeedPuzzle(&widget);
-			break;
-		case IDM_SLOW:
-			SlowPuzzle(&widget);
-			break;
-		case IDM_SOUND:
-			SoundPuzzle(&widget);
-			break;
-		case IDM_LEFT:
-		case IDM_RIGHT:
-		case IDM_SHIFT_UP:
-		case IDM_SHIFT_DOWN:
+		case ACTION_LEFT:
+		case ACTION_RIGHT:
+		case ACTION_SHIFT_UP:
+		case ACTION_SHIFT_DOWN:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			(void) MovePuzzleInput(&widget, 0,
-				(int) LOWORD(wParam) - IDM_SHIFT_UP,
+				(int) LOWORD(wParam) - ACTION_SHIFT_UP,
 				!((int) wParam % 2));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_UP:
-		case IDM_DOWN:
+		case ACTION_UP:
+		case ACTION_DOWN:
 			{
 				POINT cursor, origin;
 
@@ -1059,26 +1060,36 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				(void) GetCursorPos(&cursor);
 				(void) MovePuzzleInput(&widget,
 					cursor.x - origin.x,
-					(int) LOWORD(wParam) - IDM_UP, FALSE);
+					(int) LOWORD(wParam) - ACTION_UP,
+					FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1209,7 +1220,7 @@ CallbackPuzzlePractice(Widget w, XtPointer clientData,
 		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(barrel, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
@@ -1218,7 +1229,7 @@ CallbackPuzzleRandomize(Widget w, XtPointer clientData,
 		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(barrel, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
 
@@ -1229,7 +1240,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int tiles = cbs->value, faces, old;
 	Boolean orient, pairs, practice;
 
-	XtVaGetValues(barrel,
+	XtVaGetValues(puzzle,
 		XtNtiles, &old,
 		XtNfaces, &faces,
 		XtNorient, &orient,
@@ -1237,7 +1248,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 		XtNpractice, &practice, NULL);
 	if (old == tiles)
 		return;
-	XtVaSetValues(barrel,
+	XtVaSetValues(puzzle,
 		XtNtiles, tiles, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1253,7 +1264,7 @@ FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int tiles, faces = cbs->value, old;
 	Boolean orient, pairs, practice;
 
-	XtVaGetValues(barrel,
+	XtVaGetValues(puzzle,
 		XtNtiles, &tiles,
 		XtNfaces, &old,
 		XtNorient, &orient,
@@ -1261,7 +1272,7 @@ FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 		XtNpractice, &practice, NULL);
 	if (old == faces)
 		return;
-	XtVaSetValues(barrel,
+	XtVaSetValues(puzzle,
 		XtNfaces, faces, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1278,7 +1289,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	Boolean orient = cbs->set, pairs, practice, oldOrient;
 	char *picture;
 
-	XtVaGetValues(barrel,
+	XtVaGetValues(puzzle,
 		XtNorient, &oldOrient,
 		XtNpairs, &pairs,
 		XtNpicture, &picture,
@@ -1286,10 +1297,10 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	if (oldOrient == orient)
 		return;
 	if (!orient && picture && *picture) {
-		XmToggleButtonSetState(orientSwitch, oldOrient, True);
+		XmToggleButtonSetState(orientizeSwitch, oldOrient, True);
 		return;
 	}
-	XtVaSetValues(barrel,
+	XtVaSetValues(puzzle,
 		XtNorient, orient, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1304,13 +1315,13 @@ PairsToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient, pairs = cbs->set, practice, oldPairs;
 
-	XtVaGetValues(barrel,
+	XtVaGetValues(puzzle,
 		XtNorient, &orient,
 		XtNpairs, &oldPairs,
 		XtNpractice, &practice, NULL);
 	if (oldPairs == pairs)
 		return;
-	XtVaSetValues(barrel,
+	XtVaSetValues(puzzle,
 		XtNpairs, pairs, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1326,13 +1337,13 @@ PracticeToggle(Widget w, XtPointer clientData,
 {
 	Boolean orient, pairs, practice = cbs->set, oldPract;
 
-	XtVaGetValues(barrel,
+	XtVaGetValues(puzzle,
 		XtNorient, &orient,
 		XtNpairs, &pairs,
 		XtNpractice, &oldPract, NULL);
 	if (oldPract == practice)
 		return;
-	XtVaSetValues(barrel,
+	XtVaSetValues(puzzle,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 	movesDsp = 0;
@@ -1351,29 +1362,29 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
-	XtVaGetValues(barrel,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(barrel, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(barrel, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(barrel, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1429,6 +1440,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1465,6 +1477,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1473,8 +1486,8 @@ main(int argc, char **argv)
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
-	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientString, pairsString;
+	XmString clearString, practiceString, randomizeString, solveString;
+	XmString orientizeString, pairsString;
 	XmString speedString, slowString, soundString;
 #endif
 
@@ -1484,34 +1497,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) barrel_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) barrel_bits,
-			barrel_width, barrel_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1528,7 +1514,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1540,22 +1527,23 @@ main(int argc, char **argv)
 	undoString = XmStringCreateSimple((char *) "Undo");
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
-	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "Solve");
-	orientString = XmStringCreateSimple((char *) "Orientize");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	solveString = XmStringCreateSimple((char *) "Auto-Solve");
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
 	pairsString = XmStringCreateSimple((char *) "PAirs");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, pairsString, 'A', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
@@ -1564,10 +1552,10 @@ main(int argc, char **argv)
 	XmStringFree(undoString);
 	XmStringFree(redoString);
 	XmStringFree(clearString);
-	XmStringFree(randomizeString);
 	XmStringFree(practiceString);
+	XmStringFree(randomizeString);
 	XmStringFree(solveString);
-	XmStringFree(orientString);
+	XmStringFree(orientizeString);
 	XmStringFree(pairsString);
 	XmStringFree(speedString);
 	XmStringFree(slowString);
@@ -1682,7 +1670,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 #if 0
 	tilesRowCol = XtVaCreateManagedWidget("tilesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
@@ -1730,10 +1717,10 @@ main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, switchRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	pairsSwitch = XtVaCreateManagedWidget("Pairs rigid",
 		xmToggleButtonWidgetClass, switchRowCol,
@@ -1749,25 +1736,53 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Barrel! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-
-	barrel = XtCreateManagedWidget("barrel",
+	puzzle = XtCreateManagedWidget("barrel",
 		barrelWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	barrel = XtCreateManagedWidget("barrel",
+	puzzle = XtCreateManagedWidget("barrel",
 		barrelWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(barrel, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(barrel);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(barrel), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(barrel), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(barrel),
-		XCreateFontCursor(XtDisplay(barrel), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
