@@ -115,6 +115,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>o: MoveOut()\n\
  <KeyPress>KP_Divide: MoveOut()\n\
  <KeyPress>R5: MoveOut()\n\
@@ -155,11 +160,6 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>c: Clear()\n\
  <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
- <KeyPress>0x2E: Speed()\n\
- <KeyPress>0x3E: Speed()\n\
- <KeyPress>0x3C: Slow()\n\
- <KeyPress>0x2C: Slow()\n\
- <KeyPress>@: Sound()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
@@ -274,7 +274,10 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(CubesWidget, cubes.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(CubesWidget, cubes.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(CubesWidget, cubes.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -820,7 +823,7 @@ AnimateSlide(CubesWidget w, int numBlocks, int dir, int fast, Boolean logMoves)
 	  for (aBlock = 0; aBlock < numBlocks; aBlock++) {
 	    posNext = BlockNFromSpace(w, aBlock + 1, dir);
 	    if (logMoves && inc == 0) {
-	      setPuzzle(w, PUZZLE_MOVED);
+	      setPuzzle(w, ACTION_MOVED);
 	      setMove(&undo, dir);
 	      flushMoves(w, &redo, FALSE);
 	    }
@@ -1089,7 +1092,7 @@ DrawFrame(CubesWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoBlocks(CubesWidget w)
 {
-	setPuzzle(w, PUZZLE_IGNORE);
+	setPuzzle(w, ACTION_IGNORE);
 }
 
 static void
@@ -1213,7 +1216,7 @@ int
 MovePuzzleDir(CubesWidget w, const int direction, const int fast)
 {
 	if (MoveBlocksDir(w, direction, fast)) {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction);
 		flushMoves(w, &redo, FALSE);
 		return TRUE;
@@ -1231,31 +1234,31 @@ MovePuzzle(CubesWidget w, const int direction, const int control)
 	int reason;
 
 	if (control) {
-		reason = PUZZLE_IGNORE;
+		reason = ACTION_IGNORE;
 		switch (direction) {
 		case TOP:
 			if (w->cubes.sizeY <= MINCUBES)
 				return FALSE;
-			reason = PUZZLE_DEC_Y;
+			reason = ACTION_DECY;
 			break;
 		case RIGHT:
-			reason = PUZZLE_INC_X;
+			reason = ACTION_INCX;
 			break;
 		case BOTTOM:
-			reason = PUZZLE_INC_Y;
+			reason = ACTION_INCY;
 			break;
 		case LEFT:
 			if (w->cubes.sizeX <= MINCUBES)
 				return FALSE;
-			reason = PUZZLE_DEC_X;
+			reason = ACTION_DECX;
 			break;
 		case INWARDS:
 			if (w->cubes.sizeZ <= MINCUBES)
 				return FALSE;
-			reason = PUZZLE_DEC_Z;
+			reason = ACTION_DECZ;
 			break;
 		case OUTWARDS:
-			reason = PUZZLE_INC_Z;
+			reason = ACTION_INCZ;
 			break;
 		default:
 			{
@@ -1277,11 +1280,11 @@ MovePuzzle(CubesWidget w, const int direction, const int control)
 		return FALSE;
 	}
 	if (!MovePuzzleDir(w, direction, NORMAL)) {
-		setPuzzle(w, PUZZLE_BLOCKED);
+		setPuzzle(w, ACTION_BLOCKED);
 		return FALSE;
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return TRUE;
 }
@@ -1305,7 +1308,7 @@ static void
 DiscreteMoves(CubesWidget w, int from, int dir, int fast)
 {
 	MoveBlocks(w, from, fast);
-	setPuzzle(w, PUZZLE_MOVED);
+	setPuzzle(w, ACTION_MOVED);
 	setMove(&undo, dir);
 	flushMoves(w, &redo, FALSE);
 }
@@ -1353,13 +1356,13 @@ MovableBlock(CubesWidget w)
 	/* Order important if w->cubes.sizeX = 1 */
 	if (l % w->cubes.sizeX == 0 && k == Stack(w, w->cubes.spacePosition)) {
 		if (l == 0) {
-			return PUZZLE_SPACE;
+			return SPACE;
 		}
 	} else if (!(l / w->cubes.sizeX == 0 &&
 			j == Row(w, w->cubes.spacePosition) &&
 			k == Stack(w, w->cubes.spacePosition)) &&
 			(m % w->cubes.sizeRect != 0)) {
-		return PUZZLE_BLOCKED;
+		return BLOCKED;
 	}
 	return 0;
 }
@@ -1409,7 +1412,7 @@ SelectBlocks(CubesWidget w)
 						TOP, NORMAL);
 			}
 		} else {	/* (l == 0) */
-			intCat(&buf, "SelectBlocks: ", PUZZLE_SPACE);
+			intCat(&buf, "SelectBlocks: ", SPACE);
 			DISPLAY_WARNING(buf);
 			free(buf);
 			return;
@@ -1463,13 +1466,13 @@ SelectBlocks(CubesWidget w)
 			}
 		}
 	} else {
-		intCat(&buf, "SelectBlocks: ", PUZZLE_BLOCKED);
+		intCat(&buf, "SelectBlocks: ", BLOCKED);
 		DISPLAY_WARNING(buf);
 		free(buf);
 		return;
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1601,10 +1604,10 @@ GetBlocks(CubesWidget w)
 	(void) fscanf(fp, "%d", &sizeX);
 	if (sizeX >= MINCUBES) {
 		for (i = w->cubes.sizeX; i < sizeX; i++) {
-			setPuzzle(w, PUZZLE_INC_X);
+			setPuzzle(w, ACTION_INCX);
 		}
 		for (i = w->cubes.sizeX; i > sizeX; i--) {
-			setPuzzle(w, PUZZLE_DEC_X);
+			setPuzzle(w, ACTION_DECX);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeX ");
@@ -1623,10 +1626,10 @@ GetBlocks(CubesWidget w)
 	(void) fscanf(fp, "%d", &sizeY);
 	if (sizeY >= MINCUBES) {
 		for (i = w->cubes.sizeY; i < sizeY; i++) {
-			setPuzzle(w, PUZZLE_INC_Y);
+			setPuzzle(w, ACTION_INCY);
 		}
 		for (i = w->cubes.sizeY; i > sizeY; i--) {
-			setPuzzle(w, PUZZLE_DEC_Y);
+			setPuzzle(w, ACTION_DECY);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeY ");
@@ -1645,10 +1648,10 @@ GetBlocks(CubesWidget w)
 	(void) fscanf(fp, "%d", &sizeZ);
 	if (sizeZ >= MINCUBES) {
 		for (i = w->cubes.sizeZ; i < sizeZ; i++) {
-			setPuzzle(w, PUZZLE_INC_Z);
+			setPuzzle(w, ACTION_INCZ);
 		}
 		for (i = w->cubes.sizeZ; i > sizeZ; i--) {
-			setPuzzle(w, PUZZLE_DEC_Z);
+			setPuzzle(w, ACTION_DECZ);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeZ ");
@@ -1668,7 +1671,7 @@ GetBlocks(CubesWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
@@ -1746,7 +1749,7 @@ UndoBlocks(CubesWidget w)
 		direction = (direction < 4) ? (direction + 2) % 4 :
 			((direction == INWARDS) ? OUTWARDS : INWARDS);
 		if (MoveBlocksDir(w, direction, DOUBLE)) {
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -1769,7 +1772,7 @@ RedoBlocks(CubesWidget w)
 		getMove(&redo, &direction);
 		setMove(&undo, direction);
 		if (MoveBlocksDir(w, direction, DOUBLE)) {
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -1789,7 +1792,7 @@ ClearBlocks(CubesWidget w)
 		return;
 	ResetBlocks(w);
 	DrawAllBlocks(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
@@ -1848,7 +1851,7 @@ RandomizeBlocks(CubesWidget w)
 			(void) MovePuzzleDir(w, INWARDS, INSTANT);
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 #if 0
 	/* Now move the space around randomly */
@@ -1857,7 +1860,7 @@ RandomizeBlocks(CubesWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		setPuzzle(w, PUZZLE_RESET);
+		setPuzzle(w, ACTION_RESET);
 #ifdef DEBUG
 		big = 3;
 #endif
@@ -1881,11 +1884,11 @@ RandomizeBlocks(CubesWidget w)
 		}
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 #endif
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1898,7 +1901,7 @@ SolveBlocks(CubesWidget w)
 			w->cubes.sizeZ == 1) {
 		SolveSomeBlocks(w);
 	} else {
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
@@ -1909,7 +1912,7 @@ SpeedBlocks(CubesWidget w)
 	if (w->cubes.delay < 0)
 		w->cubes.delay = 0;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -1918,7 +1921,7 @@ SlowBlocks(CubesWidget w)
 {
 	w->cubes.delay += 5;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2200,39 +2203,39 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		w->cubes.numSlices = ((w->cubes.delay < MAXSLICES) ?
 			w->cubes.delay + 1 : MAXSLICES);
 	}
-	if (w->cubes.menu != -1) {
+	if (w->cubes.menu != ACTION_IGNORE) {
 		int menu = w->cubes.menu;
 
-		w->cubes.menu = -1;
+		w->cubes.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetBlocks(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WriteBlocks(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoBlocks(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoBlocks(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearBlocks(w);
 			break;
-		case MENU_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			RandomizeBlocks(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_SOLVE:
 			SolveBlocks(w);
 			break;
-		case MENU_SPEED:
+		case ACTION_SPEED:
 			SpeedBlocks(w);
 			break;
-		case MENU_SLOW:
+		case ACTION_SLOW:
 			SlowBlocks(w);
 			break;
-		case MENU_SOUND:
+		case ACTION_SOUND:
 			SoundBlocks(w);
 			break;
 		default:
@@ -2524,7 +2527,7 @@ HidePuzzle(CubesWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2558,12 +2561,12 @@ SelectPuzzle(CubesWidget w
 		rowType = MovableBlock(w);
 		if (rowType < 0) {
 			DrawBlock(w, w->cubes.currentPosition,
-				rowType == PUZZLE_SPACE, False, TRUE, 0, 0);
+				rowType == SPACE, False, TRUE, 0, 0);
 			FLUSH(w);
 			Sleep(100);
 			DrawBlock(w, w->cubes.currentPosition,
 				True, True, TRUE, 0, 0);
-			if (rowType != PUZZLE_SPACE)
+			if (rowType != SPACE)
 				DrawBlock(w, w->cubes.currentPosition,
 					False, False, FALSE, 0, 0);
 			setPuzzle(w, rowType);
@@ -2605,7 +2608,7 @@ ClearPuzzleMaybe(CubesWidget w
 	}
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_CLEAR_QUERY);
+		setPuzzle(w, ACTION_CLEAR_QUERY);
 	}
 #endif
 }
