@@ -7,7 +7,7 @@
 #
 #  Taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 2003 - 2006    David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2007    David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -115,7 +115,6 @@ static Boolean *locked;
 static int *locList;
 
 static Boolean SolvingFlag = False;
-/* TODO Interruptability for Windows */
 #ifdef JMP
 static Boolean AbortSolvingFlag = False;
 static jmp_buf solve_env;
@@ -573,8 +572,9 @@ myInitialize(CubesWidget w)
 	int i;
 	int spX, spY;
 
-sizeX=w->cubes.sizeX;
-sizeY=w->cubes.sizeY;
+	sizeX=w->cubes.sizeX;
+	sizeY=w->cubes.sizeY;
+
 	layers = PUZZLE_SIZE >> 1;
 
 	extraRows    = w->cubes.sizeY - PUZZLE_SIZE;
@@ -819,7 +819,7 @@ SolveSomeBlocks(CubesWidget w)
 		myFree();
 		myInitialize(w);
 	}
-        setPuzzle(w, PUZZLE_RESET);
+        setPuzzle(w, ACTION_RESET);
 	/** determine the position we want to be in when
 	 ** we are done; This position will have the space in
 	 ** the center;  Then, we'll move the space back to
@@ -862,5 +862,5 @@ SolveSomeBlocks(CubesWidget w)
 		locked[i] = False;
 	SolvingFlag = False;
 	w->cubes.cheat = True; /* Assume the worst. */
-        setPuzzle(w, PUZZLE_COMPUTED);
+        setPuzzle(w, ACTION_COMPUTED);
 }
