@@ -23,19 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/11/15 X/Windows
-  Version 5: 95/10/01 Xt/Motif
-  Version 4: 94/06/07 Xt
-  Version 3: 93/04/01 Motif
-  Version 2: 92/12/19 XView
-  Version 1: 91/03/19 SunView
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "CubesP.h"
-#include "wcubes.h"
 #define TITLE "wcubes"
 
 static CubesRec widget;
@@ -63,8 +53,8 @@ static CubesRec widget;
 #include <Xm/CascadeB.h>
 #include <Xm/Scale.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -74,9 +64,19 @@ static CubesRec widget;
 #include "Cubes.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/cubes.xpm"
+#include "pixmaps/cubes.t.xpm"
+#include "pixmaps/cubes.p.xpm"
+#include "pixmaps/cubes.s.xpm"
+#include "pixmaps/cubes.m.xpm"
+#include "pixmaps/cubes.l.xpm"
+#include "pixmaps/cubes.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?cubes_t_xpm:cubes_p_xpm):\
+(((s)<=24)?cubes_s_xpm:cubes_m_xpm)):\
+(((s)<=48)?cubes_l_xpm:cubes_xpm))
 #endif
-#include "icons/cubes.xbm"
+#include "pixmaps/cubes.xbm"
+#define DEFINE_XBM (char *) cubes_bits, cubes_width, cubes_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -88,7 +88,7 @@ static CubesRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Cubes Version 7.3\n"
+"Cubes Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -209,10 +209,10 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord cubesRecord[MAXCUBES - MINCUBES + 1][MAXCUBES - MINCUBES + 1]
-[MAXCUBES - MINCUBES + 1];
+static PuzzleRecord puzzleRecord[MAXCUBES - MINCUBES + 1]
+[MAXCUBES - MINCUBES + 1][MAXCUBES - MINCUBES + 1];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -237,8 +237,8 @@ static Boolean randomized = False;
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, cubes;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -265,11 +265,11 @@ PrintRecords(void)
 	for (i = 0; i < MAXCUBES - MINCUBES + 1; i++)
 		for (j = i; j < MAXCUBES - MINCUBES + 1; j++)
 			for (k = j; k < MAXCUBES - MINCUBES + 1; k++) {
-				if (cubesRecord[k][j][i].score > 0)
+				if (puzzleRecord[k][j][i].score > 0)
 					(void) printf("%4d%4d%4d %-12s%9d\n",
 						i + 1, j + 1, k + 1,
-						cubesRecord[k][j][i].name,
-						cubesRecord[k][j][i].score);
+						puzzleRecord[k][j][i].name,
+						puzzleRecord[k][j][i].score);
 			}
 }
 #endif
@@ -282,20 +282,20 @@ InitRecords(void)
 	for (i = 0; i < MAXCUBES - MINCUBES + 1; i++)
 		for (j = i; j < MAXCUBES - MINCUBES + 1; j++)
 			for (k = j; k < MAXCUBES - MINCUBES + 1; k++) {
-				cubesRecord[k][j][i].score = cubesRecord[k][i][j].score =
-					cubesRecord[j][k][i].score = cubesRecord[j][i][k].score =
-					cubesRecord[i][k][j].score = cubesRecord[i][j][k].score = NEVER;
-				(void) strncpy(cubesRecord[k][j][i].name,
+				puzzleRecord[k][j][i].score = puzzleRecord[k][i][j].score =
+					puzzleRecord[j][k][i].score = puzzleRecord[j][i][k].score =
+					puzzleRecord[i][k][j].score = puzzleRecord[i][j][k].score = NEVER;
+				(void) strncpy(puzzleRecord[k][j][i].name,
 					NOACCESS, USERNAMELEN);
-				(void) strncpy(cubesRecord[k][i][j].name,
+				(void) strncpy(puzzleRecord[k][i][j].name,
 					NOACCESS, USERNAMELEN);
-				(void) strncpy(cubesRecord[j][k][i].name,
+				(void) strncpy(puzzleRecord[j][k][i].name,
 					NOACCESS, USERNAMELEN);
-				(void) strncpy(cubesRecord[j][i][k].name,
+				(void) strncpy(puzzleRecord[j][i][k].name,
 					NOACCESS, USERNAMELEN);
-				(void) strncpy(cubesRecord[i][k][j].name,
+				(void) strncpy(puzzleRecord[i][k][j].name,
 					NOACCESS, USERNAMELEN);
-				(void) strncpy(cubesRecord[i][j][k].name,
+				(void) strncpy(puzzleRecord[i][j][k].name,
 					NOACCESS, USERNAMELEN);
 			}
 }
@@ -350,22 +350,22 @@ ReadRecords(void)
 		for (j = i; j < MAXCUBES - MINCUBES + 1; j++)
 			for (k = j; k < MAXCUBES - MINCUBES + 1; k++) {
 				(void) fscanf(fp, "%d %s", &n, userName);
-				if (n <= cubesRecord[i][j][k].score ||
-						cubesRecord[i][j][k].score <= NEVER) {
-					cubesRecord[k][j][i].score = cubesRecord[k][i][j].score =
-						cubesRecord[j][k][i].score = cubesRecord[j][i][k].score =
-						cubesRecord[i][k][j].score = cubesRecord[i][j][k].score = n;
-					(void) strncpy(cubesRecord[k][j][i].name,
+				if (n <= puzzleRecord[i][j][k].score ||
+						puzzleRecord[i][j][k].score <= NEVER) {
+					puzzleRecord[k][j][i].score = puzzleRecord[k][i][j].score =
+						puzzleRecord[j][k][i].score = puzzleRecord[j][i][k].score =
+						puzzleRecord[i][k][j].score = puzzleRecord[i][j][k].score = n;
+					(void) strncpy(puzzleRecord[k][j][i].name,
 						userName, USERNAMELEN);
-					(void) strncpy(cubesRecord[k][i][j].name,
+					(void) strncpy(puzzleRecord[k][i][j].name,
 						userName, USERNAMELEN);
-					(void) strncpy(cubesRecord[j][k][i].name,
+					(void) strncpy(puzzleRecord[j][k][i].name,
 						userName, USERNAMELEN);
-					(void) strncpy(cubesRecord[j][i][k].name,
+					(void) strncpy(puzzleRecord[j][i][k].name,
 						userName, USERNAMELEN);
-					(void) strncpy(cubesRecord[i][k][j].name,
+					(void) strncpy(puzzleRecord[i][k][j].name,
 						userName, USERNAMELEN);
-					(void) strncpy(cubesRecord[i][j][k].name,
+					(void) strncpy(puzzleRecord[i][j][k].name,
 						userName, USERNAMELEN);
 				}
 			}
@@ -410,8 +410,8 @@ WriteRecords(void)
 			for (j = i; j < MAXCUBES - MINCUBES + 1; j++) {
 				for (k = j; k < MAXCUBES - MINCUBES + 1; k++)
 					(void) fprintf(fp, "%d %s\n",
-						cubesRecord[i][j][k].score,
-						cubesRecord[i][j][k].name);
+						puzzleRecord[i][j][k].score,
+						puzzleRecord[i][j][k].name);
 				(void) fprintf(fp, "\n");
 			}
 			(void) fprintf(fp, "\n");
@@ -431,11 +431,11 @@ PrintRecord(int sizeX, int sizeY, int sizeZ)
 
 	if (sizeX > MAXCUBES || sizeY > MAXCUBES || sizeZ > MAXCUBES) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (cubesRecord[i][j][k].score <= NEVER) {
+	} else if (puzzleRecord[i][j][k].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			cubesRecord[i][j][k].score, cubesRecord[i][j][k].name);
+			puzzleRecord[i][j][k].score, puzzleRecord[i][j][k].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -459,7 +459,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(cubes), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -470,27 +470,27 @@ HandleSolved(int counter, int sizeX, int sizeY, int sizeZ)
 	int i = sizeX - MINCUBES, j = sizeY - MINCUBES, k = sizeZ - MINCUBES;
 
 	if (sizeX <= MAXCUBES && sizeY <= MAXCUBES && sizeZ <= MAXCUBES &&
-			(counter < cubesRecord[i][j][k].score ||
-			cubesRecord[i][j][k].score <= NEVER)) {
+			(counter < puzzleRecord[i][j][k].score ||
+			puzzleRecord[i][j][k].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
 
-		cubesRecord[i][j][k].score = cubesRecord[i][k][j].score =
-			cubesRecord[j][i][k].score =
-			cubesRecord[j][k][i].score =
-			cubesRecord[k][i][j].score =
-			cubesRecord[k][j][i].score =
+		puzzleRecord[i][j][k].score = puzzleRecord[i][k][j].score =
+			puzzleRecord[j][i][k].score =
+			puzzleRecord[j][k][i].score =
+			puzzleRecord[k][i][j].score =
+			puzzleRecord[k][j][i].score =
 			counter;
-		(void) strncpy(cubesRecord[i][j][k].name, userNameDsp,
+		(void) strncpy(puzzleRecord[i][j][k].name, userNameDsp,
 			USERNAMELEN);
-		(void) strncpy(cubesRecord[i][k][j].name, userNameDsp,
+		(void) strncpy(puzzleRecord[i][k][j].name, userNameDsp,
 			USERNAMELEN);
-		(void) strncpy(cubesRecord[j][i][k].name, userNameDsp,
+		(void) strncpy(puzzleRecord[j][i][k].name, userNameDsp,
 			USERNAMELEN);
-		(void) strncpy(cubesRecord[j][k][i].name, userNameDsp,
+		(void) strncpy(puzzleRecord[j][k][i].name, userNameDsp,
 			USERNAMELEN);
-		(void) strncpy(cubesRecord[k][i][j].name, userNameDsp,
+		(void) strncpy(puzzleRecord[k][i][j].name, userNameDsp,
 			USERNAMELEN);
-		(void) strncpy(cubesRecord[k][j][i].name, userNameDsp,
+		(void) strncpy(puzzleRecord[k][j][i].name, userNameDsp,
 			USERNAMELEN);
 		WriteRecords();
 		PrintRecord(sizeX, sizeY, sizeZ);
@@ -625,7 +625,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -635,15 +635,15 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_CLEAR_QUERY:
+	case ACTION_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(clearDialog);
 #else
-		XtVaSetValues(cubes, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -655,30 +655,30 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 #endif
 #endif
 			break;
-	case PUZZLE_RESTORE:
-	case PUZZLE_RESET:
+	case ACTION_RESTORE:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_BLOCKED:
+	case ACTION_BLOCKED:
 		(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 		break;
-	case PUZZLE_SPACE:
+	case ACTION_SPACE:
 #if 0
 		/* Too annoying */
 		(void) strncpy(messageDsp, "A space can not slide",
 			MESSAGELEN);
 #endif
 		break;
-	case PUZZLE_IGNORE:
+	case ACTION_IGNORE:
 		(void) strncpy(messageDsp, "Clear to start",
 			MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (cheat)
 			(void) sprintf(messageDsp,
 				"No cheating %s!!", userNameDsp);
@@ -691,27 +691,15 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 				MESSAGELEN);
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_DEC_X:
-		movesDsp = 0;
-		sizeX--;
-		PrintRecord(sizeX, sizeY, sizeZ);
-#ifdef WINVER
-		w->cubes.sizeX = sizeX;
-#else
-		XtVaSetValues(w, XtNsizeX, sizeX, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(blockX, sizeX);
-		if (sizeX >= MAXCUBES)
-			XtVaSetValues(blockX, XmNmaximum, sizeX, NULL);
-#endif
-#endif
-		break;
-	case PUZZLE_INC_X:
+	case ACTION_INCX:
 		movesDsp = 0;
 		sizeX++;
 		PrintRecord(sizeX, sizeY, sizeZ);
@@ -726,22 +714,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_DEC_Y:
+	case ACTION_DECX:
 		movesDsp = 0;
-		sizeY--;
+		sizeX--;
 		PrintRecord(sizeX, sizeY, sizeZ);
 #ifdef WINVER
-		w->cubes.sizeY = sizeY;
+		w->cubes.sizeX = sizeX;
 #else
-		XtVaSetValues(w, XtNsizeY, sizeY, NULL);
+		XtVaSetValues(w, XtNsizeX, sizeX, NULL);
 #ifdef HAVE_MOTIF
-		XmScaleSetValue(blockY, sizeY);
-		if (sizeY >= MAXCUBES)
-			XtVaSetValues(blockY, XmNmaximum, sizeY, NULL);
+		XmScaleSetValue(blockX, sizeX);
+		if (sizeX >= MAXCUBES)
+			XtVaSetValues(blockX, XmNmaximum, sizeX, NULL);
 #endif
 #endif
 		break;
-	case PUZZLE_INC_Y:
+	case ACTION_INCY:
 		movesDsp = 0;
 		sizeY++;
 		PrintRecord(sizeX, sizeY, sizeZ);
@@ -756,22 +744,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_DEC_Z:
+	case ACTION_DECY:
 		movesDsp = 0;
-		sizeZ--;
+		sizeY--;
 		PrintRecord(sizeX, sizeY, sizeZ);
 #ifdef WINVER
-		w->cubes.sizeZ = sizeZ;
+		w->cubes.sizeY = sizeY;
 #else
-		XtVaSetValues(w, XtNsizeZ, sizeZ, NULL);
+		XtVaSetValues(w, XtNsizeY, sizeY, NULL);
 #ifdef HAVE_MOTIF
-		XmScaleSetValue(blockZ, sizeZ);
-		if (sizeZ >= MAXCUBES)
-			XtVaSetValues(blockZ, XmNmaximum, sizeZ, NULL);
+		XmScaleSetValue(blockY, sizeY);
+		if (sizeY >= MAXCUBES)
+			XtVaSetValues(blockY, XmNmaximum, sizeY, NULL);
 #endif
 #endif
 		break;
-	case PUZZLE_INC_Z:
+	case ACTION_INCZ:
 		movesDsp = 0;
 		sizeZ++;
 		PrintRecord(sizeX, sizeY, sizeZ);
@@ -786,23 +774,36 @@ CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
+	case ACTION_DECZ:
+		movesDsp = 0;
+		sizeZ--;
+		PrintRecord(sizeX, sizeY, sizeZ);
+#ifdef WINVER
+		w->cubes.sizeZ = sizeZ;
+#else
+		XtVaSetValues(w, XtNsizeZ, sizeZ, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(blockZ, sizeZ);
+		if (sizeZ >= MAXCUBES)
+			XtVaSetValues(blockZ, XmNmaximum, sizeZ, NULL);
+#endif
+#endif
 		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
 #ifdef HAVE_MOTIF
-	case PUZZLE_SPEED:
+	case ACTION_SPEED:
 		{
 			int oldDelay, delay;
 
-			XtVaGetValues(cubes, XtNdelay, &delay, NULL);
+			XtVaGetValues(puzzle,
+				XtNdelay, &delay, NULL);
 			oldDelay = delay;
 			if (delay > MAXSPEED - MINSPEED)
 				delay = MAXSPEED - MINSPEED;
@@ -920,11 +921,11 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				if (GetKeyState(VK_CONTROL) >> 1) {
 					SizePuzzle(&widget);
-				(void) InvalidateRect(hWnd, NULL, TRUE);
+					(void) InvalidateRect(hWnd, NULL, TRUE);
 				}
 			} else if (zDelta < -(WHEEL_DELTA >> 1)) {
 				MovePuzzle(&widget, BOTTOM,
-			(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
+					(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				if (GetKeyState(VK_CONTROL) >> 1) {
 					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
@@ -936,106 +937,106 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(&widget, brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_SPEED:
-			SpeedPuzzle(&widget);
-			break;
-		case IDM_SLOW:
-			SlowPuzzle(&widget);
-			break;
-		case IDM_SOUND:
-			SoundPuzzle(&widget);
-			break;
-		case IDM_UP:
-		case IDM_RIGHT:
-		case IDM_DOWN:
-		case IDM_LEFT:
-		case IDM_IN:
-		case IDM_OUT:
+		case ACTION_UP:
+		case ACTION_RIGHT:
+		case ACTION_DOWN:
+		case ACTION_LEFT:
+		case ACTION_IN:
+		case ACTION_OUT:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			(void) MovePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_UP, FALSE);
+				(int) LOWORD(wParam) - ACTION_UP, FALSE);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_DECY:
-		case IDM_INCX:
-		case IDM_INCY:
-		case IDM_DECX:
-		case IDM_DECZ:
-		case IDM_INCZ:
+		case ACTION_INCX:
+		case ACTION_DECX:
+		case ACTION_INCY:
+		case ACTION_DECY:
+		case ACTION_INCZ:
+		case ACTION_DECZ:
 			if (MovePuzzle(&widget,
-					(int) LOWORD(wParam) - IDM_DECY,
+					(int) LOWORD(wParam) - ACTION_DECY,
 					TRUE)) {
 				SizePuzzle(&widget);
 				(void) InvalidateRect(hWnd, NULL, TRUE);
 			}
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1153,7 +1154,7 @@ static void
 CallbackPuzzleClear(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(cubes, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 	}
 }
 
@@ -1162,13 +1163,13 @@ BlockXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX = cbs->value, sizeY, sizeZ, oldX;
 
-	XtVaGetValues(cubes,
+	XtVaGetValues(puzzle,
 		XtNsizeX, &oldX,
 		XtNsizeY, &sizeY,
 		XtNsizeZ, &sizeZ, NULL);
 	if (oldX == sizeX)
 		return;
-	XtVaSetValues(cubes,
+	XtVaSetValues(puzzle,
 		XtNsizeX, sizeX, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1183,13 +1184,13 @@ BlockYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX, sizeY = cbs->value, sizeZ, oldY;
 
-	XtVaGetValues(cubes,
+	XtVaGetValues(puzzle,
 		XtNsizeX, &sizeX,
 		XtNsizeY, &oldY,
 		XtNsizeZ, &sizeZ, NULL);
 	if (oldY == sizeY)
 		return;
-	XtVaSetValues(cubes,
+	XtVaSetValues(puzzle,
 		XtNsizeY, sizeY, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1204,13 +1205,13 @@ BlockZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX, sizeY, sizeZ = cbs->value, oldZ;
 
-	XtVaGetValues(cubes,
+	XtVaGetValues(puzzle,
 		XtNsizeX, &sizeX,
 		XtNsizeY, &sizeY,
 		XtNsizeZ, &oldZ, NULL);
 	if (oldZ == sizeZ)
 		return;
-	XtVaSetValues(cubes,
+	XtVaSetValues(puzzle,
 		XtNsizeZ, sizeZ, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1225,29 +1226,29 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
-	XtVaGetValues(cubes,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(cubes, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(cubes, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(cubes, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1340,6 +1341,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1357,34 +1359,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) cubes_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) cubes_bits,
-			cubes_width, cubes_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1401,7 +1376,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1414,11 +1390,12 @@ main(int argc, char **argv)
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
 	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
-	solveString = XmStringCreateSimple((char *) "Solve");
+	solveString = XmStringCreateSimple((char *) "Auto-Solve");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
@@ -1542,7 +1519,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	blocksRowCol = XtVaCreateManagedWidget("blocksRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1597,24 +1573,53 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Cubes! (use mouse or keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-	cubes = XtCreateManagedWidget("cubes",
+	puzzle = XtCreateManagedWidget("cubes",
 		cubesWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	cubes = XtCreateManagedWidget("cubes",
+	puzzle = XtCreateManagedWidget("cubes",
 		cubesWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(cubes, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(cubes);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(cubes), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(cubes), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(cubes),
-		XCreateFontCursor(XtDisplay(cubes), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
