@@ -27,6 +27,7 @@
 
 #include "file.h"
 #include "rngs.h"
+#include "sound.h"
 #include "DinoP.h"
 #include "Dino2dP.h"
 #include "Dino3dP.h"
@@ -528,7 +529,7 @@ Boolean
 CheckSolved(DinoWidget w)
 {
 	int face, position;
-	DinoCornerLoc test;
+	DinoCornerLoc test = {0, FALSE};
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < MAXORIENT; position++) {
@@ -601,7 +602,7 @@ DrawFrame(const DinoWidget w, const Boolean focus)
 static void
 MoveNoPieces(DinoWidget w)
 {
-	setPuzzle(w, PUZZLE_ILLEGAL);
+	setPuzzle(w, ACTION_ILLEGAL);
 }
 
 static void
@@ -731,7 +732,8 @@ MoveInsideCorners(DinoWidget w, int face, int corner, int direction)
 		dir = direction / 2;
 		newFace = slideCorner[face][corner][dir].face;
 		newCorner = slideCorner[face][corner][dir].side;
-		newDirection = 2 * slideCorner[face][corner][dir].dir + !(newCorner % 2);
+		newDirection = 2 * slideCorner[face][corner][dir].dir +
+			!(newCorner % 2);
 		if (k != MAXROTATE)
 			ReadDiagonal((DinoWidget) w, newFace, newCorner, k);
 		WriteDiagonal(w, newFace, newCorner,
@@ -752,8 +754,10 @@ MoveOutsideCorners(DinoWidget w, int face, int corner, int direction)
 		corner = (corner + 2) % MAXORIENT;
 		dir = direction / 2;
 		newFace = slideCorner[face][corner][dir].face;
-		newCorner = (slideCorner[face][corner][dir].side + 2) % MAXORIENT;
-		newDirection = 2 * slideCorner[face][corner][dir].dir + !(newCorner % 2);
+		newCorner = (slideCorner[face][corner][dir].side + 2) %
+			MAXORIENT;
+		newDirection = 2 * slideCorner[face][corner][dir].dir +
+			!(newCorner % 2);
 		if (k != MAXROTATE)
 			ReadDiagonal((DinoWidget) w, newFace, newCorner, k);
 		WriteDiagonal(w, newFace, newCorner,
@@ -769,7 +773,8 @@ MovePieces(DinoWidget w, int face, int position, int direction, int style)
 {
 	int corner, newCorner;
 
-	corner = (position - !((position + direction) % 2) + MAXORIENT) % MAXORIENT;
+	corner = (position - !((position + direction) % 2) + MAXORIENT) %
+		MAXORIENT;
 	if (style == CORNER) {
 		MoveInsideCorners(w, face, corner, direction);
 	} else if (style == MIDDLE) {
@@ -790,7 +795,8 @@ MovePieces(DinoWidget w, int face, int position, int direction, int style)
 		int k, newFace, rotate, newDirection;
 
 		RotateFace(w, faceToRotate[face][direction % MAXORIENT], CW);
-		RotateFace(w, faceToRotate[face][(direction + 2) % MAXORIENT], CCW);
+		RotateFace(w, faceToRotate[face][(direction + 2) % MAXORIENT],
+			CCW);
 		ReadFace((DinoWidget) w, face, 0);
 		for (k = 1; k <= MAXORIENT; k++) {
 			newFace = slideNextFace[face][direction % MAXORIENT].face;
@@ -811,24 +817,26 @@ MoveControlCb(DinoWidget w, int face, int position, int direction, int style)
 	int newFace, newSide, newDirection, corner, newCorner;
 
 	MovePieces(w, face, position, direction, style);
-	setPuzzleMove(w, PUZZLE_CONTROL, face, position, direction, style);
+	setPuzzleMove(w, ACTION_CONTROL, face, position, direction, style);
 	if (style == CORNER) {
 		newSide = (position + 2) % MAXORIENT;
 		MovePieces(w, face, newSide, direction, MIDDLE);
-		setPuzzleMove(w, PUZZLE_CONTROL, face, newSide, direction, MIDDLE);
-		corner = (position - !((position + direction) % 2) + MAXORIENT) % MAXORIENT;
+		setPuzzleMove(w, ACTION_CONTROL, face, newSide, direction,
+			MIDDLE);
+		corner = (position - !((position + direction) % 2) +
+			MAXORIENT) % MAXORIENT;
 		newFace = oppCorner[face][corner].face;
 		newCorner = oppCorner[face][corner].rotation;
 		newDirection = 2 * ((direction / 2 +
 			(face != 1 && face != 3)) % 2) + !(newCorner % 2);
 		newSide = newCorner;
 		MovePieces(w, newFace, newSide, newDirection, CORNER);
-		setPuzzleMove(w, PUZZLE_CONTROL, newFace, newSide, newDirection,
-			CORNER);
+		setPuzzleMove(w, ACTION_CONTROL, newFace, newSide,
+			newDirection, CORNER);
 	} else if (style == MIDDLE) {
 		newSide = (position + 2) % MAXORIENT;
 		MovePieces(w, face, newSide, direction, CORNER);
-		setPuzzleMove(w, PUZZLE_CONTROL, face, newSide, direction,
+		setPuzzleMove(w, ACTION_CONTROL, face, newSide, direction,
 			CORNER);
 		corner = (position - !((position + direction) % 2) +
 			MAXORIENT) % MAXORIENT;
@@ -841,12 +849,13 @@ MoveControlCb(DinoWidget w, int face, int position, int direction, int style)
 		newDirection = 2 * (newDirection / 2) + !(newDirection % 2);
 		newSide = (newCorner + 2) % MAXORIENT;
 		MovePieces(w, newFace, newSide, newDirection, CORNER);
-		setPuzzleMove(w, PUZZLE_CONTROL, newFace, newSide, newDirection,
-			CORNER);
+		setPuzzleMove(w, ACTION_CONTROL, newFace, newSide,
+			newDirection, CORNER);
 	} else if (style == EDGE) {
 		newSide = (position + 2) % MAXORIENT;
 		MovePieces(w, face, newSide, direction, EDGE);
-		setPuzzleMove(w, PUZZLE_CONTROL, face, newSide, direction, EDGE);
+		setPuzzleMove(w, ACTION_CONTROL, face, newSide, direction,
+			EDGE);
 	}
 }
 
@@ -858,8 +867,14 @@ MovePuzzle(DinoWidget w, int face, int position, int direction, int style,
 		MoveControlCb(w, face, position, direction, style);
 	else {
 		MovePieces(w, face, position, direction, style);
-		setPuzzleMove(w, PUZZLE_MOVED, face, position, direction, style);
+		setPuzzleMove(w, ACTION_MOVED, face, position, direction,
+			style);
 	}
+#ifdef USE_SOUND
+	if (w->dino.sound) {
+		playSound((char *) MOVESOUND);
+	}
+#endif
 	setMove(&undo, face, position, direction, style, control);
 	flushMoves(w, &redo, FALSE);
 }
@@ -942,7 +957,7 @@ MovePuzzleInput(DinoWidget w, int x, int y, int direction, int shift,
 	control = (control) ? 1 : 0;
 	MovePuzzle(w, face, position, direction, style, control);
 	if (!control && CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -960,8 +975,8 @@ ResetPieces(DinoWidget w)
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
 	w->dino.started = False;
-	w->dino.currentFace = PUZZLE_IGNORE;
-	w->dino.currentDirection = PUZZLE_IGNORE;
+	w->dino.currentFace = IGNORE_DIR;
+	w->dino.currentDirection = IGNORE_DIR;
 }
 
 static void
@@ -1013,13 +1028,13 @@ GetPieces(DinoWidget w)
 	(void) fscanf(fp, "%d", &mode);
 	switch (mode) {
 	case PERIOD2:
-		setPuzzle(w, PUZZLE_PERIOD2);
+		setPuzzle(w, ACTION_PERIOD2);
 		break;
 	case PERIOD3:
-		setPuzzle(w, PUZZLE_PERIOD3);
+		setPuzzle(w, ACTION_PERIOD3);
 		break;
 	case BOTH:
-		setPuzzle(w, PUZZLE_BOTH);
+		setPuzzle(w, ACTION_BOTH);
 		break;
 	default:
 		stringCat(&buf1, name, " corrupted: mode ");
@@ -1039,12 +1054,12 @@ GetPieces(DinoWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->dino.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->dino.practice != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
@@ -1052,14 +1067,14 @@ GetPieces(DinoWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 #ifdef WINVER
 	setStartPosition(w);
 #endif
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: mode %d, orient %d, practice %d, moves %d.\n",
-		name, mode, orient, practice, moves);
+	(void) printf("%s: mode %d, orient %s, practice %s, moves %d.\n",
+		name, mode, BOOL_STRING(orient), BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->dino.cheat = True; /* Assume the worst. */
@@ -1138,10 +1153,10 @@ UndoPieces(DinoWidget w)
 			MoveControlCb(w, face, position, direction, style);
 		else {
 			MovePieces(w, face, position, direction, style);
-			setPuzzleMove(w, PUZZLE_UNDO, face, position, direction,
+			setPuzzleMove(w, ACTION_UNDO, face, position, direction,
 				style);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -1161,10 +1176,10 @@ RedoPieces(DinoWidget w)
 			MoveControlCb(w, face, position, direction, style);
 		else {
 			MovePieces(w, face, position, direction, style);
-			setPuzzleMove(w, PUZZLE_REDO, face, position, direction,
+			setPuzzleMove(w, ACTION_REDO, face, position, direction,
 				style);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -1173,13 +1188,13 @@ RedoPieces(DinoWidget w)
 static void
 ClearPieces(DinoWidget w)
 {
-	setPuzzle(w, PUZZLE_CLEAR);
+	setPuzzle(w, ACTION_CLEAR);
 }
 
 static void
 PracticePieces(DinoWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
@@ -1191,7 +1206,7 @@ RandomizePieces(DinoWidget w)
 	w->dino.cheat = False;
 	if (w->dino.practice)
 		PracticePieces(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -1212,9 +1227,9 @@ RandomizePieces(DinoWidget w)
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	setPuzzle(w, PUZZLE_RANDOMIZE);
+	setPuzzle(w, ACTION_RANDOMIZE);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1224,20 +1239,46 @@ SolvePieces(DinoWidget w)
 	if (CheckSolved(w))
 		return;
 	{
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizePieces(DinoWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 ViewPieces(DinoWidget w)
 {
-	setPuzzle(w, PUZZLE_VIEW);
+	setPuzzle(w, ACTION_VIEW);
+}
+
+static void
+SpeedPieces(DinoWidget w)
+{
+	w->dino.delay -= 5;
+	if (w->dino.delay < 0)
+		w->dino.delay = 0;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowPieces(DinoWidget w)
+{
+	w->dino.delay += 5;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundPieces(DinoWidget w)
+{
+	w->dino.sound = !w->dino.sound;
 }
 
 #ifdef WINVER
@@ -1497,56 +1538,65 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		redraw = TRUE;
 	}
 
-	if (w->dino.menu != -1) {
+	if (w->dino.menu != ACTION_IGNORE) {
 		int menu = w->dino.menu;
 
-		w->dino.menu = -1;
+		w->dino.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetPieces(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WritePieces(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoPieces(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoPieces(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearPieces(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizePieces(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePieces(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizePieces(w);
+			break;
+		case ACTION_SOLVE:
 			SolvePieces(w);
 			break;
-		case MENU_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePieces(w);
 			break;
-		case MENU_VIEW:
+		case ACTION_SPEED:
+			SpeedPieces(w);
+			break;
+		case ACTION_SLOW:
+			SlowPieces(w);
+			break;
+		case ACTION_SOUND:
+			SoundPieces(w);
+			break;
+		case ACTION_VIEW:
 			ViewPieces(w);
 			break;
 		default:
 			break;
 		}
 	}
-	if (w->dino.currentDirection == PUZZLE_RESTORE) {
+	if (w->dino.currentDirection == RESTORE_DIR) {
 		setStartPosition(w);
-		w->dino.currentDirection = PUZZLE_IGNORE;
-	} else if (w->dino.currentDirection == PUZZLE_CLEAR) {
+		w->dino.currentDirection = IGNORE_DIR;
+	} else if (w->dino.currentDirection == CLEAR_DIR) {
 		ResetPieces(w);
 		redraw = TRUE;
-		w->dino.currentDirection = PUZZLE_IGNORE;
-	} else if (w->dino.currentDirection > PUZZLE_IGNORE) {
+		w->dino.currentDirection = IGNORE_DIR;
+	} else if (w->dino.currentDirection > IGNORE_DIR) {
 		MovePieces(w, w->dino.currentFace, w->dino.currentPosition,
 			w->dino.currentDirection, w->dino.style);
-		w->dino.currentDirection = PUZZLE_IGNORE;
+		w->dino.currentDirection = IGNORE_DIR;
 	}
 	return redraw;
 }
@@ -1630,7 +1680,7 @@ HidePuzzle(DinoWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 void
@@ -1653,8 +1703,8 @@ SelectPuzzle(DinoWidget w
 			DrawTriangle(w, w->dino.currentFace,
 				w->dino.currentPosition, TRUE);
 	} else {
-		w->dino.currentFace = PUZZLE_IGNORE;
-		w->dino.currentDirection = PUZZLE_IGNORE;
+		w->dino.currentFace = IGNORE_DIR;
+		w->dino.currentDirection = IGNORE_DIR;
 	}
 }
 
@@ -1676,7 +1726,7 @@ ReleasePuzzle(DinoWidget w
 	int style, face, position, count = -1;
 	int direction = -1;
 
-	if (w->dino.currentFace <= PUZZLE_IGNORE)
+	if (w->dino.currentFace <= IGNORE_DIR)
 		return;
 	DrawTriangle(w, w->dino.currentFace, w->dino.currentPosition, FALSE);
 	style = GetStyle(w, shift, control, alt);
@@ -1691,7 +1741,7 @@ ReleasePuzzle(DinoWidget w
 				count = -1;
 		} else {
 			direction = facesToDirection[w->dino.currentFace][face][w->dino.currentPosition];
-			if (direction == PUZZLE_AMBIGUOUS &&
+			if (direction == AMBIGUOUS_DIR &&
 					(style == CORNER || style == MIDDLE)) {
 				/* minimize ambiguity */
 				count = face;
@@ -1702,8 +1752,8 @@ ReleasePuzzle(DinoWidget w
 				if (direction >= 0)
 					direction = (direction + MAXORIENT / 2) % MAXORIENT;
 			}
-			count = (direction == PUZZLE_AMBIGUOUS) ? 2 : 1;
-			if (direction == PUZZLE_IGNORE)
+			count = (direction == AMBIGUOUS_DIR) ? 2 : 1;
+			if (direction == IGNORE_DIR)
 				count = 0;
 		}
 		if (count == 1) {
@@ -1711,15 +1761,15 @@ ReleasePuzzle(DinoWidget w
 				w->dino.currentPosition, direction,
 				style, (control) ? 1 : 0);
 			if (!control && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else if (count == 2) {
-			setPuzzle(w, PUZZLE_AMBIGUOUS);
+			setPuzzle(w, ACTION_AMBIGUOUS);
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->dino.currentFace = PUZZLE_IGNORE;
-	w->dino.currentDirection = PUZZLE_IGNORE;
+	w->dino.currentFace = IGNORE_DIR;
+	w->dino.currentDirection = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -1732,7 +1782,7 @@ PracticePuzzleMaybe(DinoWidget w
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -1757,7 +1807,7 @@ RandomizePuzzleMaybe(DinoWidget w
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
@@ -1870,7 +1920,7 @@ Period2ModePuzzle(DinoWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_PERIOD2);
+	setPuzzle(w, ACTION_PERIOD2);
 }
 
 void
@@ -1878,7 +1928,7 @@ Period3ModePuzzle(DinoWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_PERIOD3);
+	setPuzzle(w, ACTION_PERIOD3);
 }
 
 void
@@ -1886,7 +1936,7 @@ BothModePuzzle(DinoWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_BOTH);
+	setPuzzle(w, ACTION_BOTH);
 }
 #endif
 
@@ -1900,6 +1950,36 @@ ViewPuzzle(DinoWidget w
 	ViewPieces(w);
 }
 
+void
+SpeedPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedPieces(w);
+}
+
+void
+SlowPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowPieces(w);
+}
+
+void
+SoundPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundPieces(w);
+}
+
 void
 EnterPuzzle(DinoWidget w
 #ifndef WINVER
@@ -1926,12 +2006,12 @@ LeavePuzzle(DinoWidget w
 void
 PeriodModePuzzle(DinoWidget w, const int mode)
 {
-	setPuzzle(w, mode + PUZZLE_PERIOD2);
+	setPuzzle(w, mode + ACTION_PERIOD2);
 }
 
 void
 DimPuzzle(DinoWidget w)
 {
-	setPuzzle(w, PUZZLE_DIM);
+	setPuzzle(w, ACTION_DIM);
 }
 #endif
