@@ -45,6 +45,8 @@
 #define XtNfaceColor5 ((char *) "faceColor5")
 #define XtNpieceBorder ((char *) "pieceBorder")
 #define XtNdelay ((char *) "delay")
+#define XtNsound ((char *) "sound")
+#define XtNmoveSound ((char *) "moveSound")
 #define XtNview ((char *) "view")
 #define XtNmode ((char *) "mode")
 #define XtNorient ((char *) "orient")
@@ -60,9 +62,12 @@
 #define XtNpos ((char *) "position")
 #define XtNdirection ((char *) "direction")
 #define XtNstyle ((char *) "style")
+#define XtNpixmapSize ((char *) "pixmapSize")
 #define XtNselectCallback ((char *) "selectCallback")
 #define XtCMono ((char *) "Mono")
 #define XtCDelay ((char *) "Delay")
+#define XtCSound ((char *) "Sound")
+#define XtCMoveSound ((char *) "MoveSound")
 #define XtCView ((char *) "View")
 #define XtCMode ((char *) "Mode")
 #define XtCOrient ((char *) "Orient")
@@ -74,6 +79,7 @@
 #define XtCPos ((char *) "Position")
 #define XtCDirection ((char *) "Direction")
 #define XtCStyle ((char *) "Style")
+#define XtCPixmapSize ((char *) "PixmapSize")
 
 typedef struct _DinoClassRec *DinoWidgetClass;
 
@@ -86,46 +92,107 @@ typedef struct {
 } dinoCallbackStruct;
 #endif
 
-#define PUZZLE_HIDE (-10)
-#define PUZZLE_PRACTICE_QUERY (-9)
-#define PUZZLE_RANDOMIZE_QUERY (-8)
-#define PUZZLE_SOLVE_MESSAGE (-7)
-#define PUZZLE_RESTORE (-6)
-#define PUZZLE_CLEAR (-5)
-#define PUZZLE_RESET (-4)
-#define PUZZLE_AMBIGUOUS (-3)
-#define PUZZLE_ILLEGAL (-2)
-#define PUZZLE_IGNORE (-1)
-#define PUZZLE_MOVED 0
-#define PUZZLE_CONTROL 1
-#define PUZZLE_SOLVED 2
-#define PUZZLE_PRACTICE 3
-#define PUZZLE_RANDOMIZE 4
-#define PUZZLE_DEC 5
-#define PUZZLE_ORIENTIZE 6
-#define PUZZLE_INC 7
-#define PUZZLE_PERIOD2 8
-#define PUZZLE_PERIOD3 9
-#define PUZZLE_BOTH 10
-#define PUZZLE_COMPUTED 11
-#define PUZZLE_UNDO 12
-#define PUZZLE_REDO 13
-#define PUZZLE_DIM 14
-#define PUZZLE_VIEW 15
-
-#define MENU_GET 0
-#define MENU_WRITE 1
-#define MENU_EXIT 2
-#define MENU_UNDO 3
-#define MENU_REDO 4
-#define MENU_CLEAR 5
-#define MENU_RANDOMIZE 6
-#define MENU_PRACTICE 7
-#define MENU_SOLVE 8
-#define MENU_ORIENTIZE 9
-#define MENU_STICKY 10
-#define MENU_VIEW 11
+#define ACTION_GET 100
+#define ACTION_WRITE 101
+#define ACTION_EXIT 102
+#define ACTION_HIDE 103
+#define ACTION_PRACTICE_QUERY 104
+#define ACTION_RANDOMIZE_QUERY 105
+#define ACTION_SOLVE_MESSAGE 106
+#define ACTION_RESTORE 107
+#define ACTION_RESET 108
+#define ACTION_AMBIGUOUS 109
+#define ACTION_ILLEGAL 110
+#define ACTION_MOVED 111
+#define ACTION_CONTROL 112
+#define ACTION_SOLVED 113
+#define ACTION_COMPUTED 114
+#define ACTION_UNDO 200
+#define ACTION_REDO 201
+#define ACTION_CLEAR 202
+#define ACTION_PRACTICE 203
+#define ACTION_RANDOMIZE 204
+#define ACTION_SOLVE 205
+#define ACTION_ORIENTIZE 206
+#define ACTION_SPEED 207
+#define ACTION_SLOW 208
+#define ACTION_SOUND 209
+#define ACTION_VIEW 210
+#define ACTION_DIM 211
+#define ACTION_TR 300
+#define ACTION_BR 301
+#define ACTION_BL 302
+#define ACTION_TL 303
+#define ACTION_CW 305
+#define ACTION_CCW 307
+#define ACTION_TOP 308
+#define ACTION_RIGHT 309
+#define ACTION_BOTTOM 310
+#define ACTION_LEFT 311
+#define ACTION_SHIFT_TR 320
+#define ACTION_SHIFT_BR 321
+#define ACTION_SHIFT_BL 322
+#define ACTION_SHIFT_TL 323
+#define ACTION_SHIFT_CW 325
+#define ACTION_SHIFT_CCW 327
+#define ACTION_SHIFT_TOP 328
+#define ACTION_SHIFT_RIGHT 329
+#define ACTION_SHIFT_BOTTOM 330
+#define ACTION_SHIFT_LEFT 331
+#define ACTION_CONTROL_TR 340
+#define ACTION_CONTROL_BR 341
+#define ACTION_CONTROL_BL 342
+#define ACTION_CONTROL_TL 343
+#define ACTION_CONTROL_CW 345
+#define ACTION_CONTROL_CCW 347
+#define ACTION_CONTROL_TOP 348
+#define ACTION_CONTROL_RIGHT 349
+#define ACTION_CONTROL_BOTTOM 350
+#define ACTION_CONTROL_LEFT 351
+#define ACTION_SHIFTCONTROL_TR 360
+#define ACTION_SHIFTCONTROL_BR 361
+#define ACTION_SHIFTCONTROL_BL 362
+#define ACTION_SHIFTCONTROL_TL 363
+#define ACTION_SHIFTCONTROL_CW 365
+#define ACTION_SHIFTCONTROL_CCW 367
+#define ACTION_SHIFTCONTROL_TOP 368
+#define ACTION_SHIFTCONTROL_RIGHT 369
+#define ACTION_SHIFTCONTROL_BOTTOM 370
+#define ACTION_SHIFTCONTROL_LEFT 371
+#define ACTION_ALT_TR 380
+#define ACTION_ALT_BR 381
+#define ACTION_ALT_BL 382
+#define ACTION_ALT_TL 383
+#define ACTION_ALT_CW 385
+#define ACTION_ALT_CCW 387
+#define ACTION_ALT_TOP 388
+#define ACTION_ALT_RIGHT 389
+#define ACTION_ALT_BOTTOM 390
+#define ACTION_ALT_LEFT 391
+#define ACTION_CONTROLALT_TR 400
+#define ACTION_CONTROLALT_BR 401
+#define ACTION_CONTROLALT_BL 402
+#define ACTION_CONTROLALT_TL 403
+#define ACTION_CONTROLALT_CW 405
+#define ACTION_CONTROLALT_CCW 407
+#define ACTION_CONTROLALT_TOP 408
+#define ACTION_CONTROLALT_RIGHT 409
+#define ACTION_CONTROLALT_BOTTOM 410
+#define ACTION_CONTROLALT_LEFT  411
+#define ACTION_PERIOD2 500
+#define ACTION_PERIOD3 501
+#define ACTION_BOTH 502
+#define ACTION_DESCRIPTION 900
+#define ACTION_FEATURES 901
+#define ACTION_REFERENCES 902
+#define ACTION_ABOUT 903
+#define ACTION_IGNORE 999
 
+#define RESTORE_DIR (-5)
+#define CLEAR_DIR (-4)
+#define AMBIGUOUS_DIR (-3)
+#define ILLEGAL_DIR (-2)
+#define IGNORE_DIR (-1)
 #define PERIOD2 2
 #define PERIOD3 3
 #define BOTH 4
@@ -142,6 +209,8 @@ typedef struct {
 #endif
 #define DEFAULTREVERSE FALSE
 
+#define BOOL_STRING(a) ((a==0)?"false":"true")
+
 typedef struct _DinoRec *DinoWidget;
 
 #endif /* _XtDino_h */
