@@ -62,6 +62,11 @@ static char defaultTranslationsPuzzle3D[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -156,6 +161,9 @@ static XtActionsRec actionsListPuzzle3D[] =
 	{(char *) "Period3", (XtActionProc) Period3ModePuzzle},
 	{(char *) "Both", (XtActionProc) BothModePuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -204,6 +212,15 @@ static XtResource resourcesPuzzle3D[] =
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(DinoWidget, dino.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
+	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
+	 XtOffset(DinoWidget, dino.delay),
+	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(DinoWidget, dino.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(DinoWidget, dino.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(DinoWidget, dino.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -230,7 +247,7 @@ static XtResource resourcesPuzzle3D[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(DinoWidget, dino.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(DinoWidget, dino.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -249,6 +266,9 @@ static XtResource resourcesPuzzle3D[] =
 	{XtNstyle, XtCStyle, XtRInt, sizeof (int),
 	 XtOffset(DinoWidget, dino.style),
 	 XtRString, (caddr_t) "-1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(DinoWidget, dino.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(DinoWidget, dino.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -490,10 +510,11 @@ void
 DrawTriangle3D(Dino3DWidget w, int face, int position, int offset)
 {
 	GC faceGC, borderGC;
-	int corner = 0, dx, dy, letterX, letterY;
+	int corner = 0, dx, dy, rotation, letterX, letterY;
 
 	MapTo3D(w, face, position, &corner);
 	CubeOffset3D(w, face, corner, &dx, &dy);
+	rotation = w->dino.cubeLoc[face][position].rotation;
 	letterX = dx + letter3DList[face][corner].x;
 	letterY = dy + letter3DList[face][corner].y;
 	triangleLoc3D[face][corner][0].x = dx;
@@ -535,13 +556,13 @@ DrawTriangle3D(Dino3DWidget w, int face, int position, int offset)
 	if (w->dino.orient) {
 		DRAWLINE(w, dr, borderGC,
 			letterX +
-			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][0].x,
+			orientTriangle[face][rotation][0].x,
 			letterY +
-			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][0].y,
+			orientTriangle[face][rotation][0].y,
 			letterX +
-			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][1].x,
+			orientTriangle[face][rotation][1].x,
 			letterY +
-			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][1].y);
+			orientTriangle[face][rotation][1].y);
 	}
 }
 
