@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 2006	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1995 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -53,8 +53,8 @@ extern Dino3DClassRec dino3dClassRec;
 #define MAXORIENT 4
 
 #define SQRT_3 1.732050808
-#define MULTIPLY(a) ((int)((double)a*SQRT_3/2.0))
-#define DIVIDE(a) ((int)((double)a*2.0/SQRT_3))
+#define MULTIPLY(a) ((int)(a*SQRT_3/2.0))
+#define DIVIDE(a) ((int)(a*2.0/SQRT_3))
 
 typedef struct _Dino3DPart {
 	Position    cubeDiagonal, cubeDiag, diamondDiagonal, faceDiagonal;
