@@ -151,6 +151,11 @@ static char defaultTranslationsPuzzleGL[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>Home: MoveTl()\n\
  <KeyPress>KP_7: MoveTl()\n\
  <KeyPress>R7: MoveTl()\n\
@@ -236,6 +241,9 @@ static XtActionsRec actionsListPuzzleGL[] =
 	{(char *) "Period3", (XtActionProc) Period3ModePuzzle},
 	{(char *) "Both", (XtActionProc) BothModePuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -287,6 +295,12 @@ static XtResource resourcesPuzzleGL[] =
 	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
 	 XtOffset(DinoWidget, dino.delay),
 	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(DinoWidget, dino.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(DinoWidget, dino.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(DinoWidget, dino.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -316,7 +330,7 @@ static XtResource resourcesPuzzleGL[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(DinoWidget, dino.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(DinoWidget, dino.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -335,6 +349,9 @@ static XtResource resourcesPuzzleGL[] =
 	{XtNstyle, XtCStyle, XtRInt, sizeof (int),
 	 XtOffset(DinoWidget, dino.style),
 	 XtRString, (caddr_t) "-1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+         XtOffset(DinoWidget, dino.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(DinoWidget, dino.select),
 	 XtRCallback, (caddr_t) NULL}
