@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 2006	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1995 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -62,9 +62,12 @@ extern DinoClassRec dinoClassRec;
 extern void QuitPuzzle(DinoWidget w, XEvent *event, char **args, int nArgs);
 #endif
 
+#ifndef MOVESOUND
+#define MOVESOUND "move"
+#endif
+
 #define SYMBOL ':'
 
-#define IGNORE_DIR (-1)
 #define TR 0
 #define BR 1
 #define BL 2
@@ -115,7 +118,7 @@ typedef struct _DinoPart {
 	DinoCornerLoc rowLoc[MAXORIENT][MAXORIENT];
 	DinoCornerLoc spindleLoc[MAXROTATE][2];
 	int         currentFace, currentPosition, currentDirection;
-	Boolean     started, cheat;
+	Boolean     started, cheat, sound;
 	Boolean     practice, orient, vertical, mono, reverse, focus;
 	Boolean     scoreOnly, versionOnly;
 	int         dim; /* This allows us to reuse code between 2d and 3d */
@@ -129,11 +132,11 @@ typedef struct _DinoPart {
 	GC          frameGC, borderGC, inverseGC;
 	GC          faceGC[MAXFACES];
 #ifdef WINVER
-	char        userName[81], scoreFile[81];
+	char        userName[81], scoreFile[81], moveSound[81];
 	char        faceChar[MAXFACES];
 #else
-	int         menu;
-	char       *userName, *scoreFile, *font;
+	int         menu, pixmapSize;
+	char       *userName, *scoreFile, *moveSound, *font;
 	XFontStruct *fontInfo;
 	char       *faceName[MAXFACES];
 	Pixel       foreground, background;
@@ -237,6 +240,21 @@ extern void ViewPuzzle(DinoWidget w
 , XEvent *event, char **args, int nArgs
 #endif
 );
+extern void SpeedPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void SlowPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void SoundPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
 extern void EnterPuzzle(DinoWidget w
 #ifndef WINVER
 , XEvent *event, char **args, int nArgs
