@@ -23,16 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/12/15 Xt/Motif
-  Version 5: 95/10/06 Xt/Motif
-  Version 4: 94/05/30 Xt
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "DinoP.h"
-#include "wdino.h"
 #define TITLE "wdino"
 
 static DinoRec widget;
@@ -61,8 +54,8 @@ static DinoRec widget;
 #include <Xm/ToggleB.h>
 #include <Xm/ToggleBG.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -77,9 +70,19 @@ static DinoRec widget;
 #endif
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/dino.xpm"
+#include "pixmaps/dino.t.xpm"
+#include "pixmaps/dino.p.xpm"
+#include "pixmaps/dino.s.xpm"
+#include "pixmaps/dino.m.xpm"
+#include "pixmaps/dino.l.xpm"
+#include "pixmaps/dino.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?dino_t_xpm:dino_p_xpm):\
+(((s)<=24)?dino_s_xpm:dino_m_xpm)):\
+(((s)<=48)?dino_l_xpm:dino_xpm))
 #endif
-#include "icons/dino.xbm"
+#include "pixmaps/dino.xbm"
+#define DEFINE_XBM (char *) dino_bits, dino_width, dino_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -91,7 +94,7 @@ static DinoRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Dino Version 7.3\n"
+"Dino Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -103,6 +106,7 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3|4|5} {color}] [-{border|bd} {color}]\n"
+"[-delay msecs] [-[no]sound] [-moveSound {filename}]\n"
 "[-{font|fn} {fontname}] [-view {int}]\n"
 "[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n"
 "[-userName {string}] [-scoreFile {filename}] [-scores]\n"
@@ -165,6 +169,14 @@ static const char featuresHelp[] = {
 "\n"
 "Press \"V\" or \"v\" keys to change the view of the cube.\n"
 "\n"
+"Press \">\" or \".\" keys to speed up the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"<\" or \",\" keys to slow down the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"@\" key to toggle the sound.\n"
+"\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
 "Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program.\n"
@@ -223,9 +235,9 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord dinoRecord[MAXMODES][2];
+static PuzzleRecord puzzleRecord[MAXMODES][2];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -242,7 +254,7 @@ static char userNameDsp[USERNAMELEN] = "Guest";
 #else
 #ifdef HAVE_MOTIF
 static Widget moves, record, message, modes[MAXMODES];
-static Widget orientSwitch, practiceSwitch;
+static Widget orientizeSwitch, practiceSwitch;
 static char buff[21];
 static const char *modeString[] =
 {
@@ -258,10 +270,10 @@ static Widget shell;
 static Widget shellGL;
 #endif
 #endif
-static Pixmap icon = None;
-static Widget topLevel, dino2d, dino3d;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle2d, puzzle3d;
 #ifdef HAVE_OPENGL
-static Widget dinoGL;
+static Widget puzzleGL;
 #endif
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -288,11 +300,11 @@ PrintRecords(void)
 	(void) printf("MODE ORIENT USER            MOVES\n");
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++) {
-			if (dinoRecord[mode][orient].score > 0)
+			if (puzzleRecord[mode][orient].score > 0)
 				(void) printf("%4d%7d %-12s%9d\n",
 					mode + 2, orient,
-					dinoRecord[mode][orient].name,
-					dinoRecord[mode][orient].score);
+					puzzleRecord[mode][orient].name,
+					puzzleRecord[mode][orient].score);
 		}
 }
 #endif
@@ -304,8 +316,8 @@ InitRecords(void)
 
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++) {
-			dinoRecord[mode][orient].score = NEVER;
-			(void) strncpy(dinoRecord[mode][orient].name, NOACCESS,
+			puzzleRecord[mode][orient].score = NEVER;
+			(void) strncpy(puzzleRecord[mode][orient].name, NOACCESS,
 				USERNAMELEN);
 		}
 }
@@ -359,10 +371,10 @@ ReadRecords(void)
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++) {
 			(void) fscanf(fp, "%d %s\n", &n, userName);
-			if (n <= dinoRecord[mode][orient].score ||
-					dinoRecord[mode][orient].score <= NEVER) {
-				dinoRecord[mode][orient].score = n;
-				(void) strncpy(dinoRecord[mode][orient].name,
+			if (n <= puzzleRecord[mode][orient].score ||
+					puzzleRecord[mode][orient].score <= NEVER) {
+				puzzleRecord[mode][orient].score = n;
+				(void) strncpy(puzzleRecord[mode][orient].name,
 					userName, USERNAMELEN);
 			}
 		}
@@ -406,8 +418,8 @@ WriteRecords(void)
 		for (mode = 0; mode < MAXMODES; mode++) {
 			for (orient = 0; orient < 2; orient++)
 				(void) fprintf(fp, "%d %s\n",
-					dinoRecord[mode][orient].score,
-					dinoRecord[mode][orient].name);
+					puzzleRecord[mode][orient].score,
+					puzzleRecord[mode][orient].name);
 			(void) fprintf(fp, "\n");
 		}
 #if HAVE_FCNTL_H
@@ -426,11 +438,11 @@ PrintRecord(int mode, Boolean orient, Boolean practice)
 
 	if (practice) {
 		(void) strncpy(recordDsp, "practice", MESSAGELEN);
-	} else if (dinoRecord[i][j].score <= NEVER) {
+	} else if (puzzleRecord[i][j].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			dinoRecord[i][j].score, dinoRecord[i][j].name);
+			puzzleRecord[i][j].score, puzzleRecord[i][j].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -449,7 +461,7 @@ MakeEquivalent(String *userName, String *scoreFile, int *mode,
 	Pixel foreground, background, pieceBorder;
 	String faceColor[MAXFACES];
 
-	XtVaGetValues(dino2d,
+	XtVaGetValues(puzzle2d,
 		XtNuserName, userName,
 		XtNscoreFile, scoreFile,
 		XtNmode, mode,
@@ -480,17 +492,17 @@ MakeEquivalent(String *userName, String *scoreFile, int *mode,
 		PrintRecords();
 		exit(0);
 	}
-	XtVaSetValues(dino2d,
-		XtNdirection, PUZZLE_IGNORE,
+	XtVaSetValues(puzzle2d,
+		XtNdirection, IGNORE_DIR,
 		XtNstart, False, NULL);
-	XtVaSetValues(dino3d,
+	XtVaSetValues(puzzle3d,
 		XtNuserName, *userName,
 		XtNmode, *mode,
 		XtNorient, *orient,
 		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverseVideo, reverse,
-		XtNdirection, PUZZLE_IGNORE,
+		XtNdirection, IGNORE_DIR,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -502,13 +514,13 @@ MakeEquivalent(String *userName, String *scoreFile, int *mode,
 		XtNfaceColor4, faceColor[4],
 		XtNfaceColor5, faceColor[5], NULL);
 #ifdef HAVE_OPENGL
-	XtVaSetValues(dinoGL,
+	XtVaSetValues(puzzleGL,
 		XtNuserName, *userName,
 		XtNorient, *orient,
 		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverseVideo, reverse,
-		XtNdirection, PUZZLE_IGNORE,
+		XtNdirection, IGNORE_DIR,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -554,10 +566,10 @@ PrintStatus(char *msg, int nMoves
 #else
 #ifdef HAVE_OPENGL
 	if (dim == 4)
-		XtVaSetValues(XtParent(dinoGL), XtNtitle, titleDsp, NULL);
+		XtVaSetValues(XtParent(puzzleGL), XtNtitle, titleDsp, NULL);
 	else
 #endif
-	XtVaSetValues(XtParent((dim == 2) ? dino2d : dino3d),
+	XtVaSetValues(XtParent((dim == 2) ? puzzle2d : puzzle3d),
 		XtNtitle, titleDsp, NULL);
 #endif
 #endif
@@ -569,16 +581,16 @@ HandleSolved(int counter, int mode, Boolean orient)
 	int i = mode - PERIOD2;
 	int j = (orient) ? 1 : 0;
 
-	if (counter < dinoRecord[i][j].score ||
-			dinoRecord[i][j].score <= NEVER) {
+	if (counter < puzzleRecord[i][j].score ||
+			puzzleRecord[i][j].score <= NEVER) {
 		ReadRecords();	/* Maybe its been updated by another */
-		dinoRecord[i][j].score = counter;
-		(void) strncpy(dinoRecord[i][j].name, userNameDsp,
+		puzzleRecord[i][j].score = counter;
+		(void) strncpy(puzzleRecord[i][j].name, userNameDsp,
 			USERNAMELEN);
-		if (orient && (counter < dinoRecord[!i][j].score ||
-				dinoRecord[!i][j].score <= NEVER)) {
-			dinoRecord[!i][j].score = counter;
-			(void) strncpy(dinoRecord[!i][j].name, userNameDsp,
+		if (orient && (counter < puzzleRecord[!i][j].score ||
+				puzzleRecord[!i][j].score <= NEVER)) {
+			puzzleRecord[!i][j].score = counter;
+			(void) strncpy(puzzleRecord[!i][j].name, userNameDsp,
 				USERNAMELEN);
 		}
 		WriteRecords();
@@ -618,7 +630,7 @@ void
 	MakeEquivalent(&userName, &scoreFile, &mode, &orient, &practice);
 #ifdef HAVE_MOTIF
 	XmToggleButtonSetState(modes[mode - PERIOD2], True, False);
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 #endif
@@ -689,10 +701,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 #endif
 
-	if (w == dino2d) {
-		otherw1 = dino3d;
+	if (w == puzzle2d) {
+		otherw1 = puzzle3d;
 #ifdef HAVE_OPENGL
-		otherw2 = dinoGL;
+		otherw2 = puzzleGL;
 #endif
 #ifndef HAVE_MOTIF
 		dim = 2;
@@ -701,10 +713,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 		otherdim2 = 4;
 #endif
 #endif
-	} else if (w == dino3d) {
-		otherw1 = dino2d;
+	} else if (w == puzzle3d) {
+		otherw1 = puzzle2d;
 #ifdef HAVE_OPENGL
-		otherw2 = dinoGL;
+		otherw2 = puzzleGL;
 #endif
 #ifndef HAVE_MOTIF
 		dim = 3;
@@ -715,9 +727,9 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 	}
 #ifdef HAVE_OPENGL
-	else if (w == dinoGL) {
-		otherw1 = dino2d;
-		otherw2 = dino3d;
+	else if (w == puzzleGL) {
+		otherw1 = puzzle2d;
+		otherw2 = puzzle3d;
 #ifndef HAVE_MOTIF
 		dim = 4;
 		otherdim1 = 2;
@@ -743,7 +755,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -763,22 +775,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(dino2d, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(dino2d, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -790,7 +802,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice",
 				MESSAGELEN);
@@ -803,37 +815,37 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
-			XtNdirection, PUZZLE_RESTORE, NULL);
+			XtNdirection, RESTORE_DIR, NULL);
 #endif
 		XtVaSetValues(otherw1,
-			XtNdirection, PUZZLE_RESTORE, NULL);
+			XtNdirection, RESTORE_DIR, NULL);
 		XtVaSetValues(w,
-			XtNdirection, PUZZLE_RESTORE, NULL);
+			XtNdirection, RESTORE_DIR, NULL);
 #endif
 		break;
-	case PUZZLE_CLEAR:
+	case ACTION_CLEAR:
 		movesDsp = 0;
 		randomized = False;
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
-			XtNdirection, PUZZLE_CLEAR, NULL);
+			XtNdirection, CLEAR_DIR, NULL);
 #endif
 		XtVaSetValues(otherw1,
-			XtNdirection, PUZZLE_CLEAR, NULL);
+			XtNdirection, CLEAR_DIR, NULL);
 		XtVaSetValues(w,
-			XtNdirection, PUZZLE_CLEAR, NULL);
+			XtNdirection, CLEAR_DIR, NULL);
 #endif
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	 case PUZZLE_AMBIGUOUS:
+	case ACTION_AMBIGUOUS:
 		(void) strncpy(messageDsp, "Ambiguous move",
 			MESSAGELEN);
 		break;
-	case PUZZLE_ILLEGAL:
+	case ACTION_ILLEGAL:
 		if (practice || randomized)
 			(void) strncpy(messageDsp, "Illegal move",
 				MESSAGELEN);
@@ -842,7 +854,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 				"Randomize to start", MESSAGELEN);
 		break;
 #ifndef WINVER
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
@@ -860,7 +872,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 			XtNstyle, callData->style, NULL);
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
 			XtNface, callData->face,
@@ -875,7 +887,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 			XtNstyle, callData->style, NULL);
 		break;
 #endif
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -896,7 +908,16 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 		randomized = False;
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+#ifndef WINVER
+#ifdef HAVE_OPENGL
+		SET_STARTED(otherw2, False);
+#endif
+		SET_STARTED(otherw1, False);
+#endif
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		randomized = False;
 		practice = !practice;
@@ -924,7 +945,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 #ifdef WINVER
@@ -945,27 +966,27 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 		break;
 #ifdef HAVE_OPENGL
-	case PUZZLE_VIEW:
+	case ACTION_VIEW:
 		{
 			int view;
 
 #ifdef WINVER
 			view = w->dino.view;
 #else
-			XtVaGetValues(dinoGL,
+			XtVaGetValues(puzzleGL,
 				XtNview, &view, NULL);
 #endif
 			view = (view + 1) % (2 * MAXVIEWS);
 #ifdef WINVER
 			w->dino.view = view;
 #else
-			XtVaSetValues(dinoGL,
+			XtVaSetValues(puzzleGL,
 				XtNview, view, NULL);
 #endif
 		}
 		break;
 #endif
-	case PUZZLE_ORIENTIZE:
+	case ACTION_ORIENTIZE:
 		movesDsp = 0;
 		orient = !orient;
 		PrintRecord(mode, orient, practice);
@@ -981,15 +1002,15 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 		XtVaSetValues(otherw1,
 			XtNorient, orient, NULL);
 #ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
 #endif
 #endif
 		break;
-	case PUZZLE_PERIOD2:
-	case PUZZLE_PERIOD3:
-	case PUZZLE_BOTH:
+	case ACTION_PERIOD2:
+	case ACTION_PERIOD3:
+	case ACTION_BOTH:
 		movesDsp = 0;
-		mode = reason - PUZZLE_PERIOD2 + PERIOD2;
+		mode = reason - ACTION_PERIOD2 + PERIOD2;
 		PrintRecord(mode, orient, practice);
 #ifdef WINVER
 		w->dino.mode = mode;
@@ -1007,17 +1028,8 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
 #ifndef WINVER
-#ifdef HAVE_OPENGL
-		SET_STARTED(otherw2, False);
-#endif
-		SET_STARTED(otherw1, False);
-#endif
-		break;
-#ifndef WINVER
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
@@ -1037,7 +1049,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 		break;
 #endif
 #ifndef WINVER
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
@@ -1057,7 +1069,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 		break;
 #endif
 #ifdef WINVER
-	case PUZZLE_DIM:
+	case ACTION_DIM:
 		dim++;
 #ifdef HAVE_OPENGL
 		if (dim > 4)
@@ -1091,7 +1103,7 @@ setPuzzleMove(DinoWidget w, int reason, int face, int position, int direction,
 {
 	int mode;
 	/* Boolean orient, practice, cheat; */
-	int dim = 0;
+	int dim;
 
 	(void) strcpy(messageDsp, "");
 	mode = w->dino.mode;
@@ -1102,17 +1114,17 @@ setPuzzleMove(DinoWidget w, int reason, int face, int position, int direction,
 #endif
 	dim = w->dino.dim;
 	switch (reason) {
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
@@ -1240,94 +1252,94 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			ClearPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DIM:
+		case ACTION_DIM:
 			(void) DimPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_VIEW:
+		case ACTION_VIEW:
 			(void) ViewPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SHIFT_TR:
-		case IDM_SHIFT_BR:
-		case IDM_SHIFT_BL:
-		case IDM_SHIFT_TL:
-		case IDM_SHIFT_CW:
-		case IDM_SHIFT_CCW:
-		case IDM_SHIFT_TOP:
-		case IDM_SHIFT_RIGHT:
-		case IDM_SHIFT_BOTTOM:
-		case IDM_SHIFT_LEFT:
+		case ACTION_SHIFT_TR:
+		case ACTION_SHIFT_BR:
+		case ACTION_SHIFT_BL:
+		case ACTION_SHIFT_TL:
+		case ACTION_SHIFT_CW:
+		case ACTION_SHIFT_CCW:
+		case ACTION_SHIFT_TOP:
+		case ACTION_SHIFT_RIGHT:
+		case ACTION_SHIFT_BOTTOM:
+		case ACTION_SHIFT_LEFT:
 			shift = 1;
-		case IDM_TR:
-		case IDM_BR:
-		case IDM_BL:
-		case IDM_TL:
-		case IDM_CW:
-		case IDM_CCW:
-		case IDM_TOP:
-		case IDM_RIGHT:
-		case IDM_BOTTOM:
-		case IDM_LEFT:
+		case ACTION_TR:
+		case ACTION_BR:
+		case ACTION_BL:
+		case ACTION_TL:
+		case ACTION_CW:
+		case ACTION_CCW:
+		case ACTION_TOP:
+		case ACTION_RIGHT:
+		case ACTION_BOTTOM:
+		case ACTION_LEFT:
 			{
 				POINT cursor, origin;
 
@@ -1342,32 +1354,32 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_SHIFT_TR : IDM_TR),
+					((shift) ? ACTION_SHIFT_TR : ACTION_TR),
 					shift, FALSE, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_SHIFTCONTROL_TR:
-		case IDM_SHIFTCONTROL_BR:
-		case IDM_SHIFTCONTROL_BL:
-		case IDM_SHIFTCONTROL_TL:
-		case IDM_SHIFTCONTROL_CW:
-		case IDM_SHIFTCONTROL_CCW:
-		case IDM_SHIFTCONTROL_TOP:
-		case IDM_SHIFTCONTROL_RIGHT:
-		case IDM_SHIFTCONTROL_BOTTOM:
-		case IDM_SHIFTCONTROL_LEFT:
+		case ACTION_SHIFTCONTROL_TR:
+		case ACTION_SHIFTCONTROL_BR:
+		case ACTION_SHIFTCONTROL_BL:
+		case ACTION_SHIFTCONTROL_TL:
+		case ACTION_SHIFTCONTROL_CW:
+		case ACTION_SHIFTCONTROL_CCW:
+		case ACTION_SHIFTCONTROL_TOP:
+		case ACTION_SHIFTCONTROL_RIGHT:
+		case ACTION_SHIFTCONTROL_BOTTOM:
+		case ACTION_SHIFTCONTROL_LEFT:
 			shift = 1;
-		case IDM_CONTROL_TR:
-		case IDM_CONTROL_BR:
-		case IDM_CONTROL_BL:
-		case IDM_CONTROL_TL:
-		case IDM_CONTROL_CW:
-		case IDM_CONTROL_CCW:
-		case IDM_CONTROL_TOP:
-		case IDM_CONTROL_RIGHT:
-		case IDM_CONTROL_BOTTOM:
-		case IDM_CONTROL_LEFT:
+		case ACTION_CONTROL_TR:
+		case ACTION_CONTROL_BR:
+		case ACTION_CONTROL_BL:
+		case ACTION_CONTROL_TL:
+		case ACTION_CONTROL_CW:
+		case ACTION_CONTROL_CCW:
+		case ACTION_CONTROL_TOP:
+		case ACTION_CONTROL_RIGHT:
+		case ACTION_CONTROL_BOTTOM:
+		case ACTION_CONTROL_LEFT:
 			{
 				POINT cursor, origin;
 
@@ -1381,33 +1393,33 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_SHIFTCONTROL_TR :
-					IDM_CONTROL_TR),
+					((shift) ? ACTION_SHIFTCONTROL_TR :
+					ACTION_CONTROL_TR),
 					shift, TRUE, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_CONTROLALT_TR:
-		case IDM_CONTROLALT_BR:
-		case IDM_CONTROLALT_BL:
-		case IDM_CONTROLALT_TL:
-		case IDM_CONTROLALT_CW:
-		case IDM_CONTROLALT_CCW:
-		case IDM_CONTROLALT_TOP:
-		case IDM_CONTROLALT_RIGHT:
-		case IDM_CONTROLALT_BOTTOM:
-		case IDM_CONTROLALT_LEFT:
+		case ACTION_CONTROLALT_TR:
+		case ACTION_CONTROLALT_BR:
+		case ACTION_CONTROLALT_BL:
+		case ACTION_CONTROLALT_TL:
+		case ACTION_CONTROLALT_CW:
+		case ACTION_CONTROLALT_CCW:
+		case ACTION_CONTROLALT_TOP:
+		case ACTION_CONTROLALT_RIGHT:
+		case ACTION_CONTROLALT_BOTTOM:
+		case ACTION_CONTROLALT_LEFT:
 			shift = 1;
-		case IDM_ALT_TR:
-		case IDM_ALT_BR:
-		case IDM_ALT_BL:
-		case IDM_ALT_TL:
-		case IDM_ALT_CW:
-		case IDM_ALT_CCW:
-		case IDM_ALT_TOP:
-		case IDM_ALT_RIGHT:
-		case IDM_ALT_BOTTOM:
-		case IDM_ALT_LEFT:
+		case ACTION_ALT_TR:
+		case ACTION_ALT_BR:
+		case ACTION_ALT_BL:
+		case ACTION_ALT_TL:
+		case ACTION_ALT_CW:
+		case ACTION_ALT_CCW:
+		case ACTION_ALT_TOP:
+		case ACTION_ALT_RIGHT:
+		case ACTION_ALT_BOTTOM:
+		case ACTION_ALT_LEFT:
 			{
 				POINT cursor, origin;
 
@@ -1421,36 +1433,45 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_CONTROLALT_TR :
-					IDM_ALT_TR),
+					((shift) ? ACTION_CONTROLALT_TR :
+					ACTION_ALT_TR),
 					FALSE, shift, TRUE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_PERIOD2:
-		case IDM_PERIOD3:
-		case IDM_BOTH:
+		case ACTION_PERIOD2:
+		case ACTION_PERIOD3:
+		case ACTION_BOTH:
 			PeriodModePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_PERIOD2);
+				(int) LOWORD(wParam) - ACTION_PERIOD2);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1542,6 +1563,9 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face3", (char *) "*dino.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-face4", (char *) "*dino.faceColor4", XrmoptionSepArg, NULL},
 	{(char *) "-face5", (char *) "*dino.faceColor5", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*dino.delay", XrmoptionSepArg, NULL},
+	{(char *) "-sound", (char *) "*dino.sound", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nosound", (char *) "*dino.sound", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-fn", (char *) "*dino.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*dino.font", XrmoptionSepArg, NULL},
 	{(char *) "-view", (char *) "*dino.view", XrmoptionSepArg, NULL},
@@ -1562,7 +1586,7 @@ static void
 CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(dino2d, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
@@ -1570,7 +1594,7 @@ static void
 CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(dino2d, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
 static void
@@ -1579,15 +1603,15 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 	Boolean orient, practice;
 
 	if (cbs->set) {
-		XtVaGetValues(dino2d,
+		XtVaGetValues(puzzle2d,
 			XtNorient, &orient,
 			XtNpractice, &practice, NULL);
-		XtVaSetValues(dino2d,
+		XtVaSetValues(puzzle2d,
 			XtNmode, mode + PERIOD2, NULL);
-		XtVaSetValues(dino3d,
+		XtVaSetValues(puzzle3d,
 			XtNmode, mode + PERIOD2, NULL);
 #ifdef HAVE_OPENGL
-		XtVaSetValues(dinoGL,
+		XtVaSetValues(puzzleGL,
 			XtNmode, mode + PERIOD2, NULL);
 #endif
 		movesDsp = 0;
@@ -1605,15 +1629,15 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int mode;
 	Boolean orient = cbs->set, practice;
 
-	XtVaGetValues(dino2d,
+	XtVaGetValues(puzzle2d,
 		XtNmode, &mode,
 		XtNpractice, &practice, NULL);
-	XtVaSetValues(dino2d,
+	XtVaSetValues(puzzle2d,
 		XtNorient, orient, NULL);
-	XtVaSetValues(dino3d,
+	XtVaSetValues(puzzle3d,
 		XtNorient, orient, NULL);
 #ifdef HAVE_OPENGL
-	XtVaSetValues(dinoGL,
+	XtVaSetValues(puzzleGL,
 		XtNorient, orient, NULL);
 #endif
 	movesDsp = 0;
@@ -1625,23 +1649,24 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
+PracticeToggle(Widget w, XtPointer clientData,
+		XmToggleButtonCallbackStruct *cbs)
 {
 	int mode;
 	Boolean orient, practice = cbs->set;
 
-	XtVaSetValues(dino2d,
+	XtVaSetValues(puzzle2d,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
-	XtVaSetValues(dino3d,
+	XtVaSetValues(puzzle3d,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 #ifdef HAVE_OPENGL
-	XtVaSetValues(dinoGL,
+	XtVaSetValues(puzzleGL,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 #endif
-	XtVaGetValues(dino2d,
+	XtVaGetValues(puzzle2d,
 		XtNmode, &mode,
 		XtNpractice, &orient, NULL);
 	movesDsp = 0;
@@ -1658,19 +1683,19 @@ PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(dino2d, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle2d, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(dino2d, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle2d, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1726,6 +1751,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1762,6 +1788,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1770,8 +1797,9 @@ main(int argc, char **argv)
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
-	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientString, viewString;
+	XmString clearString, practiceString, randomizeString, solveString;
+	XmString orientizeString;
+	XmString speedString, slowString, soundString, viewString;
 	unsigned int i;
 #endif
 
@@ -1781,33 +1809,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) dino_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) dino_bits, dino_width, dino_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1824,7 +1826,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1836,28 +1839,38 @@ main(int argc, char **argv)
 	undoString = XmStringCreateSimple((char *) "Undo");
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
-	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "(Solve)");
-	orientString = XmStringCreateSimple((char *) "Orientize");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	solveString = XmStringCreateSimple((char *) "(Auto-Solve)");
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
+	speedString = XmStringCreateSimple((char *) ">Speed");
+	slowString = XmStringCreateSimple((char *) "<Slow");
+	soundString = XmStringCreateSimple((char *) "@Sound");
 	viewString = XmStringCreateSimple((char *) "View");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
+		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
+		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		XmVaPUSHBUTTON, viewString, 'V', NULL, NULL,
 		NULL);
 	XmStringFree(undoString);
 	XmStringFree(redoString);
 	XmStringFree(clearString);
-	XmStringFree(randomizeString);
 	XmStringFree(practiceString);
+	XmStringFree(randomizeString);
 	XmStringFree(solveString);
-	XmStringFree(orientString);
+	XmStringFree(orientizeString);
+	XmStringFree(speedString);
+	XmStringFree(slowString);
+	XmStringFree(soundString);
 	XmStringFree(viewString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
@@ -1869,7 +1882,7 @@ main(int argc, char **argv)
 	widget = XtVaCreateManagedWidget("Description",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'D', NULL);
-		XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 0);
+	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 0);
 	widget = XtVaCreateManagedWidget("Features",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'F', NULL);
@@ -1993,10 +2006,10 @@ main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, switchRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	practiceSwitch = XtVaCreateManagedWidget("Practice",
 		xmToggleButtonWidgetClass, switchRowCol,
@@ -2010,54 +2023,86 @@ main(int argc, char **argv)
 		XmNpacking, XmPACK_COLUMN, NULL);
 	message = XtVaCreateManagedWidget("Play Dino! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-
-	dino2d = XtCreateManagedWidget("dino",
+	puzzle2d = XtCreateManagedWidget("dino",
 		dino2dWidgetClass, mainPanel, NULL, 0);
-	XtVaSetValues(dino2d,
+	XtVaSetValues(puzzle2d,
 		XtNheight, 200, NULL);
-	XtAddCallback(dino2d, XtNselectCallback,
+	XtAddCallback(puzzle2d, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	dino3d = XtCreateManagedWidget("dino",
+	puzzle3d = XtCreateManagedWidget("dino",
 		dino3dWidgetClass, mainPanel, NULL, 0);
-	XtVaSetValues(dino3d,
+	XtVaSetValues(puzzle3d,
 		XtNheight, 200, NULL);
 #ifdef HAVE_OPENGL
-	dinoGL = XtCreateManagedWidget("dino",
+	puzzleGL = XtCreateManagedWidget("dino",
 		dinoGLWidgetClass, mainPanel, NULL, 0);
-	XtVaSetValues(dinoGL,
+	XtVaSetValues(puzzleGL,
 		XtNheight, 200, NULL);
 #endif
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
 	shell = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
-	XtVaSetValues(shell,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
 #ifdef HAVE_OPENGL
 	shellGL = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
-	XtVaSetValues(shellGL,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
 #endif
-	dino2d = XtCreateManagedWidget("dino",
+	puzzle2d = XtCreateManagedWidget("dino",
 		dino2dWidgetClass, topLevel, NULL, 0);
-	XtAddCallback(dino2d, XtNselectCallback,
-		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	dino3d = XtCreateManagedWidget("dino",
+	puzzle3d = XtCreateManagedWidget("dino",
 		dino3dWidgetClass, shell, NULL, 0);
 #ifdef HAVE_OPENGL
-	dinoGL = XtCreateManagedWidget("dino",
+	puzzleGL = XtCreateManagedWidget("dino",
 		dinoGLWidgetClass, shellGL, NULL, 0);
 #endif
 #endif
-	XtAddCallback(dino3d, XtNselectCallback,
+	XtVaGetValues(puzzle2d,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+#ifdef HAVE_MOTIF
+	XtVaSetValues(topLevel,
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+		XtNiconPixmap, pixmap, NULL);
+#else
+	XtVaSetValues(topLevel,
+		XtNinput, True,
+		XtNiconPixmap, pixmap, NULL);
+	XtVaSetValues(shell,
+		XtNinput, True,
+		XtNiconPixmap, pixmap, NULL);
+#endif
+	XtAddCallback(puzzle2d, XtNselectCallback,
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
+	XtAddCallback(puzzle3d, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #ifdef HAVE_OPENGL
-	XtAddCallback(dinoGL, XtNselectCallback,
+#ifndef HAVE_MOTIF
+	XtVaSetValues(shellGL,
+		XtNinput, True,
+		XtNiconPixmap, pixmap, NULL);
+#endif
+	XtAddCallback(puzzleGL, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #endif
 	Initialize();
@@ -2068,22 +2113,22 @@ main(int argc, char **argv)
 	XtRealizeWidget(shellGL);
 #endif
 #endif
-	XGrabButton(XtDisplay(dino2d), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(dino2d), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle2d), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle2d), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(dino2d),
-		XCreateFontCursor(XtDisplay(dino2d), XC_crosshair));
-	XGrabButton(XtDisplay(dino3d), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(dino3d), TRUE, (unsigned int) (ButtonPressMask |
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle2d),
+		XCreateFontCursor(XtDisplay(puzzle2d), XC_crosshair));
+	XGrabButton(XtDisplay(puzzle3d), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle3d), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(dino3d),
-		XCreateFontCursor(XtDisplay(dino3d), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle3d),
+		XCreateFontCursor(XtDisplay(puzzle3d), XC_crosshair));
 #ifdef HAVE_OPENGL
-	XGrabButton(XtDisplay(dinoGL), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(dinoGL), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzleGL), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzleGL), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(dinoGL),
-		XCreateFontCursor(XtDisplay(dinoGL), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzleGL),
+		XCreateFontCursor(XtDisplay(puzzleGL), XC_crosshair));
 #endif
 	XtMainLoop();
 
