@@ -98,7 +98,7 @@ static void RandomizePuzzle(HexagonsWidget w,
 	XEvent *event, char **args, int nArgs);
 static void SolvePuzzle(HexagonsWidget w,
 	XEvent *event, char **args, int nArgs);
-static void ModePuzzle(HexagonsWidget w,
+static void CornersPuzzle(HexagonsWidget w,
 	XEvent *event, char **args, int nArgs);
 static void SpeedPuzzle(HexagonsWidget w,
 	XEvent *event, char **args, int nArgs);
@@ -136,6 +136,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>Home: MoveTl()\n\
  <KeyPress>KP_7: MoveTl()\n\
  <KeyPress>R7: MoveTl()\n\
@@ -181,12 +186,7 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>c: Clear()\n\
  <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
- <KeyPress>o: Mode()\n\
- <KeyPress>0x2E: Speed()\n\
- <KeyPress>0x3E: Speed()\n\
- <KeyPress>0x3C: Slow()\n\
- <KeyPress>0x2C: Slow()\n\
- <KeyPress>@: Sound()\n\
+ <KeyPress>o: Corners()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
@@ -213,7 +213,7 @@ static XtActionsRec actionsListPuzzle[] =
 	{(char *) "Clear", (XtActionProc) ClearPuzzle},
 	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
 	{(char *) "Solve", (XtActionProc) SolvePuzzle},
-	{(char *) "Mode", (XtActionProc) ModePuzzle},
+	{(char *) "Corners", (XtActionProc) CornersPuzzle},
 	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
 	{(char *) "Slow", (XtActionProc) SlowPuzzle},
 	{(char *) "Sound", (XtActionProc) SoundPuzzle},
@@ -300,7 +300,10 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(HexagonsWidget, hexagons.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(HexagonsWidget, hexagons.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(HexagonsWidget, hexagons.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -1165,16 +1168,16 @@ MovableCornerTiles(HexagonsWidget w, int direction,
 static int
 MovableNoCornTile(HexagonsWidget w)
 {
-	int rowType = PUZZLE_BLOCKED, l;
+	int rowType = BLOCKED, l;
 
 	/* Are the spaces in a "row" with the mouse click?
 	 * (If two, then one clicked on a space). */
 	for (l = 0; l < ROWTYPES; l++) {
 		if (w->hexagons.currentRow[l] == w->hexagons.spaceRow[l]) {
-			if (rowType == PUZZLE_BLOCKED) {
+			if (rowType == BLOCKED) {
 				rowType = l;
 			} else {
-				return PUZZLE_SPACE;
+				return SPACE;
 			}
 		}
 	}
@@ -1299,7 +1302,7 @@ AnimateSlide(HexagonsWidget w, int dir, int fast, Boolean logMoves)
 	}
 	for (aTile = 0; aTile < numTiles; aTile++) {
 		if (logMoves) {
-			setPuzzle(w, PUZZLE_MOVED);
+			setPuzzle(w, ACTION_MOVED);
 			setMove(&undo, direction);
 			flushMoves(w, &redo, FALSE);
 		}
@@ -1532,7 +1535,7 @@ DrawFrame(HexagonsWidget w, Pixmap dr, Boolean all, Boolean focus)
 static void
 MoveNoTiles(HexagonsWidget w)
 {
-	setPuzzle(w, PUZZLE_IGNORE);
+	setPuzzle(w, ACTION_IGNORE);
 }
 
 static void
@@ -1852,7 +1855,7 @@ int
 MovePuzzleDir(HexagonsWidget w, const int direction, const int fast)
 {
 	if (MoveTilesDir(w, direction, fast)) {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction);
 		flushMoves(w, &redo, FALSE);
 		return TRUE;
@@ -1869,7 +1872,7 @@ MovePuzzle(HexagonsWidget w, const int direction, const int control)
 	int reason;
 
 	if (control) {
-		reason = PUZZLE_IGNORE;
+		reason = ACTION_IGNORE;
 		switch (direction) {
 		case TL:
 		case TOP:
@@ -1877,20 +1880,20 @@ MovePuzzle(HexagonsWidget w, const int direction, const int control)
 			if (w->hexagons.sizeY <= MINTILES +
 					w->hexagons.corners)
 				return FALSE;
-			reason = PUZZLE_DEC_Y;
+			reason = ACTION_DECY;
 			break;
 		case RIGHT:
-			reason = PUZZLE_INC_X;
+			reason = ACTION_INCX;
 			break;
 		case BL:
 		case BOTTOM:
 		case BR:
-			reason = PUZZLE_INC_Y;
+			reason = ACTION_INCY;
 			break;
 		case LEFT:
 			if (w->hexagons.sizeX <= MINTILES)
 				return FALSE;
-			reason = PUZZLE_DEC_X;
+			reason = ACTION_DECX;
 			break;
 		default:
 			{
@@ -1911,14 +1914,15 @@ MovePuzzle(HexagonsWidget w, const int direction, const int control)
 		return FALSE;
 	}
 	if (direction == TOP || direction == BOTTOM) {
+		setPuzzle(w, ACTION_AMBIGUOUS);
 		return FALSE;
 	}
 	if (!MovePuzzleDir(w, direction, NORMAL)) {
-		setPuzzle(w, PUZZLE_BLOCKED);
+		setPuzzle(w, ACTION_BLOCKED);
 		return FALSE;
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return TRUE;
 }
@@ -1955,7 +1959,7 @@ DiscreteNoCornMoves(HexagonsWidget w, int dir, int fast)
 			w->hexagons.spaceRow[ROW] - 1;
 		MoveNoCornTiles(w, TileNFromSpace(w, 1, rowType, HIGH),
 			orient, fast);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		switch (rowType) {
 		case TLBR:
 			setMove(&undo, BR);
@@ -1984,7 +1988,7 @@ DiscreteNoCornMoves(HexagonsWidget w, int dir, int fast)
 			w->hexagons.spaceRow[ROW] + 1;
 		MoveNoCornTiles(w, TileNFromSpace(w, 1, rowType, LOW),
 			orient, fast);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		switch (rowType) {
 		case TLBR:
 			setMove(&undo, TL);
@@ -2061,7 +2065,7 @@ MovableCornerTile(HexagonsWidget w)
 	/* (Clicked on a space?) */
 	if (w->hexagons.currentPosition == w->hexagons.spacePosition[LOW] ||
 			w->hexagons.currentPosition == w->hexagons.spacePosition[HIGH]) {
-		return PUZZLE_SPACE;
+		return SPACE;
 	}
 	if (FindTileTriangle(w,
 			w->hexagons.currentPosition, w->hexagons.spacePosition[HIGH],
@@ -2069,7 +2073,7 @@ MovableCornerTile(HexagonsWidget w)
 			w->hexagons.spaceRow[HIGH], w->hexagons.spaceRow[LOW])) {
 		return 0;
 	}
-	return PUZZLE_BLOCKED;
+	return BLOCKED;
 }
 
 static void
@@ -2087,16 +2091,17 @@ SelectCornerTiles(HexagonsWidget w, int space)
 		return;
 	}
 	orient = (w->hexagons.spacePosition[HIGH] <
-		w->hexagons.spacePosition[LOW]) ? !space : space;
+		w->hexagons.spacePosition[LOW]) ? ((space == HIGH) ?
+		LOW : HIGH) : space;
 	setMove(&undo, FindDir(w,
 		w->hexagons.currentPosition, w->hexagons.spacePosition[orient],
 		w->hexagons.currentRow[ROW], w->hexagons.spaceRow[orient]));
 	flushMoves(w, &redo, FALSE);
 	MoveCornerTiles(w, w->hexagons.currentPosition,
 			w->hexagons.currentRow[ROW], orient, NORMAL);
-	setPuzzle(w, PUZZLE_MOVED);
+	setPuzzle(w, ACTION_MOVED);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -2138,7 +2143,7 @@ SelectNoCornTiles(HexagonsWidget w)
 		}
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -2186,7 +2191,7 @@ RandomizeTiles(HexagonsWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		setPuzzle(w, PUZZLE_RESET);
+		setPuzzle(w, ACTION_RESET);
 #ifdef DEBUG
 		big = 3;
 #endif
@@ -2206,10 +2211,10 @@ RandomizeTiles(HexagonsWidget w)
 		}
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -2261,16 +2266,16 @@ GetTiles(HexagonsWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &corners);
 	if (w->hexagons.corners != (Boolean) corners) {
-		setPuzzle(w, PUZZLE_CORNERS);
+		setPuzzle(w, ACTION_CORNERS);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sizeX);
 	if (sizeX >= MINTILES) {
 		for (i = w->hexagons.sizeX; i < sizeX; i++) {
-			setPuzzle(w, PUZZLE_INC_X);
+			setPuzzle(w, ACTION_INCX);
 		}
 		for (i = w->hexagons.sizeX; i > sizeX; i--) {
-			setPuzzle(w, PUZZLE_DEC_X);
+			setPuzzle(w, ACTION_DECX);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeX ");
@@ -2290,10 +2295,10 @@ GetTiles(HexagonsWidget w)
 	(void) fscanf(fp, "%d", &sizeY);
 	if (sizeY >= MINTILES) {
 		for (i = w->hexagons.sizeY; i < sizeY; i++) {
-			setPuzzle(w, PUZZLE_INC_Y);
+			setPuzzle(w, ACTION_INCY);
 		}
 		for (i = w->hexagons.sizeY; i > sizeY; i--) {
-			setPuzzle(w, PUZZLE_DEC_Y);
+			setPuzzle(w, ACTION_DECY);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeY ");
@@ -2314,12 +2319,12 @@ GetTiles(HexagonsWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: corners %d, sizeX %d, sizeY %d, moves %d.\n",
-		name, corners, sizeX, sizeY, moves);
+	(void) printf("%s: corners %s, sizeX %d, sizeY %d, moves %d.\n",
+		name, BOOL_STRING(corners), sizeX, sizeY, moves);
 	free(lname);
 	free(fname);
 	w->hexagons.cheat = True; /* Assume the worst. */
@@ -2392,7 +2397,7 @@ UndoTiles(HexagonsWidget w)
 		setMove(&redo, direction);
 		direction = (direction + (COORD / 2)) % COORD;
 		if (MoveTilesDir(w, direction, DOUBLE)) {
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -2415,7 +2420,7 @@ RedoTiles(HexagonsWidget w)
 		getMove(&redo, &direction);
 		setMove(&undo, direction);
 		if (MoveTilesDir(w, direction, DOUBLE)) {
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -2435,7 +2440,7 @@ ClearTiles(HexagonsWidget w)
 		return;
 	ResetTiles(w);
 	DrawAllTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
@@ -2444,15 +2449,15 @@ SolveTiles(HexagonsWidget w)
 	if (CheckSolved(w) || w->hexagons.currentPosition >= 0)
 		return;
 	{
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
-ModeTiles(HexagonsWidget w)
+CornersTiles(HexagonsWidget w)
 {
 	if (w->hexagons.corners || w->hexagons.sizeY > 1) {
-		setPuzzle(w, PUZZLE_CORNERS);
+		setPuzzle(w, ACTION_CORNERS);
 	}
 }
 
@@ -2463,7 +2468,7 @@ SpeedTiles(HexagonsWidget w)
 	if (w->hexagons.delay < 0)
 		w->hexagons.delay = 0;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2472,7 +2477,7 @@ SlowTiles(HexagonsWidget w)
 {
 	w->hexagons.delay += 5;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2732,42 +2737,42 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		ResizePuzzle(w);
 		redraw = TRUE;
 	}
-	if (w->hexagons.menu != -1) {
+	if (w->hexagons.menu != ACTION_IGNORE) {
 		int menu = w->hexagons.menu;
 
-		w->hexagons.menu = -1;
+		w->hexagons.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetTiles(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WriteTiles(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoTiles(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoTiles(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearTiles(w);
 			break;
-		case MENU_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			RandomizeTiles(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_SOLVE:
 			SolveTiles(w);
 			break;
-		case MENU_MODE:
-			ModeTiles(w);
+		case ACTION_CORNERS:
+			CornersTiles(w);
 			break;
-		case MENU_SPEED:
+		case ACTION_SPEED:
 			SpeedTiles(w);
 			break;
-		case MENU_SLOW:
+		case ACTION_SLOW:
 			SlowTiles(w);
 			break;
-		case MENU_SOUND:
+		case ACTION_SOUND:
 			SoundTiles(w);
 			break;
 		default:
@@ -2956,8 +2961,8 @@ ResizePuzzle(HexagonsWidget w)
 				(3 * w->hexagons.sizeY - 1), 0);
 	w->hexagons.offset.x = w->hexagons.tileSize.x + w->hexagons.delta.x;
 	w->hexagons.offset.y = w->hexagons.tileSize.y + 2 * w->hexagons.delta.y;
-	tempSize.y = (int) ((double) w->hexagons.offset.x / sqrt_3);
-	tempSize.x = (int) ((double) w->hexagons.offset.y * sqrt_3);
+	tempSize.y = (int) (w->hexagons.offset.x / sqrt_3);
+	tempSize.x = (int) (w->hexagons.offset.y * sqrt_3);
 	if (tempSize.y < w->hexagons.offset.y) {
 		w->hexagons.offset.x = w->hexagons.tileSize.x + w->hexagons.delta.x;
 		w->hexagons.offset.y = tempSize.y;
@@ -3075,7 +3080,7 @@ HidePuzzle(HexagonsWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -3111,12 +3116,12 @@ SelectPuzzle(HexagonsWidget w
 			rowType = MovableNoCornTile(w);
 		if (rowType < 0) {
 			DrawTile(w, w->hexagons.currentPosition,
-				rowType == PUZZLE_SPACE, False, TRUE, 0, 0);
+				rowType == SPACE, False, TRUE, 0, 0);
 			FLUSH(w);
 			Sleep(100);
 			DrawTile(w, w->hexagons.currentPosition,
 				True, True, TRUE, 0, 0);
-			if (rowType != PUZZLE_SPACE)
+			if (rowType != SPACE)
 				DrawTile(w, w->hexagons.currentPosition,
 					False, False, FALSE, 0, 0);
 			setPuzzle(w, rowType);
@@ -3182,7 +3187,7 @@ ClearPuzzleMaybe(HexagonsWidget w
 		ClearTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_CLEAR_QUERY);
+		setPuzzle(w, ACTION_CLEAR_QUERY);
 	}
 #endif
 }
@@ -3294,13 +3299,13 @@ SolvePuzzle(HexagonsWidget w
 static
 #endif
 void
-ModePuzzle(HexagonsWidget w
+CornersPuzzle(HexagonsWidget w
 #ifndef WINVER
 , XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	ModeTiles(w);
+	CornersTiles(w);
 }
 
 #ifndef WINVER
