@@ -23,19 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/11/15 X/Windows
-  Version 5: 95/10/01 Xt/Motif
-  Version 4: 94/06/07 Xt
-  Version 3: 93/04/01 Motif
-  Version 2: 92/01/07 XView
-  Version 1: 91/09/05 SunView
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "HexagonsP.h"
-#include "whexagons.h"
 #define TITLE "whexagons"
 
 static HexagonsRec widget;
@@ -64,8 +54,8 @@ static HexagonsRec widget;
 #include <Xm/Scale.h>
 #include <Xm/ToggleB.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -75,9 +65,19 @@ static HexagonsRec widget;
 #include "Hexagons.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/hexagons.xpm"
+#include "pixmaps/hexagons.t.xpm"
+#include "pixmaps/hexagons.p.xpm"
+#include "pixmaps/hexagons.s.xpm"
+#include "pixmaps/hexagons.m.xpm"
+#include "pixmaps/hexagons.l.xpm"
+#include "pixmaps/hexagons.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?hexagons_t_xpm:hexagons_p_xpm):\
+(((s)<=24)?hexagons_s_xpm:hexagons_m_xpm)):\
+(((s)<=48)?hexagons_l_xpm:hexagons_xpm))
 #endif
-#include "icons/hexagons.xbm"
+#include "pixmaps/hexagons.xbm"
+#define DEFINE_XBM (char *) hexagons_bits, hexagons_width, hexagons_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -89,7 +89,7 @@ static HexagonsRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Hexagons Version 7.3\n"
+"Hexagons Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -208,9 +208,10 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord hexagonsRecord[MAXORIENT][MAXTILES - MINTILES + 1][MAXTILES - MINTILES + 1];
+static PuzzleRecord puzzleRecord[MAXORIENT]
+[MAXTILES - MINTILES + 1][MAXTILES - MINTILES + 1];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -238,8 +239,8 @@ static Boolean randomized = False;
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, hexagons;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -266,11 +267,11 @@ PrintRecords(void)
 	for (i = 0; i < MAXORIENT; i++)
 		for (j = 0; j < MAXTILES - MINTILES + 1; j++)
 			for (k = 0; k < MAXTILES - MINTILES + 1; k++) {
-				if (hexagonsRecord[i][j][k].score > 0)
+				if (puzzleRecord[i][j][k].score > 0)
 					(void) printf("%6d%4d%4d %-12s%9d\n",
 						i, j + 1, k + 1,
-						hexagonsRecord[i][j][k].name,
-						hexagonsRecord[i][j][k].score);
+						puzzleRecord[i][j][k].name,
+						puzzleRecord[i][j][k].score);
 			}
 }
 #endif
@@ -283,8 +284,8 @@ InitRecords(void)
 	for (i = 0; i < MAXORIENT; i++)
 		for (j = 0; j < MAXTILES - MINTILES + 1; j++)
 			for (k = 0; k < MAXTILES - MINTILES + 1; k++) {
-				hexagonsRecord[i][j][k].score = NEVER;
-				(void) strncpy(hexagonsRecord[i][j][k].name,
+				puzzleRecord[i][j][k].score = NEVER;
+				(void) strncpy(puzzleRecord[i][j][k].name,
 					NOACCESS, USERNAMELEN);
 			}
 }
@@ -339,10 +340,10 @@ ReadRecords(void)
 		for (j = 0; j < MAXTILES - MINTILES + 1; j++)
 			for (k = 0; k < MAXTILES - MINTILES + 1; k++) {
 				(void) fscanf(fp, "%d %s\n", &n, userName);
-				if (n <= hexagonsRecord[i][j][k].score ||
-						hexagonsRecord[i][j][k].score <= NEVER) {
-					hexagonsRecord[i][j][k].score = n;
-					(void) strncpy(hexagonsRecord[i][j][k].name,
+				if (n <= puzzleRecord[i][j][k].score ||
+						puzzleRecord[i][j][k].score <= NEVER) {
+					puzzleRecord[i][j][k].score = n;
+					(void) strncpy(puzzleRecord[i][j][k].name,
 						userName, USERNAMELEN);
 				}
 			}
@@ -387,8 +388,8 @@ WriteRecords(void)
 			for (j = 0; j < MAXTILES - MINTILES + 1; j++)
 				for (k = 0; k < MAXTILES - MINTILES + 1; k++)
 					(void) fprintf(fp, "%d %s\n",
-						hexagonsRecord[i][j][k].score,
-						hexagonsRecord[i][j][k].name);
+						puzzleRecord[i][j][k].score,
+						puzzleRecord[i][j][k].name);
 			(void) fprintf(fp, "\n");
 		}
 #if HAVE_FCNTL_H
@@ -407,12 +408,12 @@ PrintRecord(int sizeX, int sizeY, Boolean corners)
 
 	if (sizeX > MAXTILES || sizeY > MAXTILES) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (hexagonsRecord[i][j][k].score <= NEVER) {
+	} else if (puzzleRecord[i][j][k].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			hexagonsRecord[i][j][k].score,
-			hexagonsRecord[i][j][k].name);
+			puzzleRecord[i][j][k].score,
+			puzzleRecord[i][j][k].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -436,7 +437,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(hexagons), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -448,11 +449,11 @@ HandleSolved(int counter, int sizeX, int sizeY, int corners)
 	int j = sizeX - MINTILES, k = sizeY - MINTILES;
 
 	if (sizeX <= MAXTILES && sizeY <= MAXTILES &&
-			(counter < hexagonsRecord[i][j][k].score ||
-			hexagonsRecord[i][j][k].score <= NEVER)) {
+			(counter < puzzleRecord[i][j][k].score ||
+			puzzleRecord[i][j][k].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		hexagonsRecord[i][j][k].score = counter;
-		(void) strncpy(hexagonsRecord[i][j][k].name, userNameDsp,
+		puzzleRecord[i][j][k].score = counter;
+		(void) strncpy(puzzleRecord[i][j][k].name, userNameDsp,
 			USERNAMELEN);
 		WriteRecords();
 		PrintRecord(sizeX, sizeY, corners);
@@ -589,7 +590,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -599,15 +600,15 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_CLEAR_QUERY:
+	case ACTION_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(clearDialog);
 #else
-		XtVaSetValues(hexagons, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -619,30 +620,33 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
-	case PUZZLE_RESET:
+	case ACTION_RESTORE:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_BLOCKED:
+	case ACTION_AMBIGUOUS:
+		(void) strncpy(messageDsp, "Ambiguous move", MESSAGELEN);
+		break;
+	case ACTION_BLOCKED:
 		(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 		break;
-	case PUZZLE_SPACE:
+	case ACTION_SPACE:
 #if 0
 		/* Too annoying */
 		(void) sprintf(messageDsp, "%s can not slide",
 			(corners) ? "Spaces" : "A space");
 #endif
 		break;
-	case PUZZLE_IGNORE:
+	case ACTION_IGNORE:
 		(void) strncpy(messageDsp, "Clear to start",
 			MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (cheat)
 			(void) sprintf(messageDsp,
 				"No cheating %s!!", userNameDsp);
@@ -655,27 +659,15 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 				MESSAGELEN);
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_DEC_X:
-		movesDsp = 0;
-		sizeX--;
-		PrintRecord(sizeX, sizeY, corners);
-#ifdef WINVER
-		w->hexagons.sizeX = sizeX;
-#else
-		XtVaSetValues(w, XtNsizeX, sizeX, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(tileX, sizeX);
-		if (sizeX >= MAXTILES)
-			XtVaSetValues(tileX, XmNmaximum, sizeX, NULL);
-#endif
-#endif
-		break;
-	case PUZZLE_INC_X:
+	case ACTION_INCX:
 		movesDsp = 0;
 		sizeX++;
 		PrintRecord(sizeX, sizeY, corners);
@@ -690,22 +682,22 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_DEC_Y:
+	case ACTION_DECX:
 		movesDsp = 0;
-		sizeY--;
+		sizeX--;
 		PrintRecord(sizeX, sizeY, corners);
 #ifdef WINVER
-		w->hexagons.sizeY = sizeY;
+		w->hexagons.sizeX = sizeX;
 #else
-		XtVaSetValues(w, XtNsizeY, sizeY, NULL);
+		XtVaSetValues(w, XtNsizeX, sizeX, NULL);
 #ifdef HAVE_MOTIF
-		XmScaleSetValue(tileY, sizeY);
-		if (sizeY >= MAXTILES)
-			XtVaSetValues(tileY, XmNmaximum, sizeY, NULL);
+		XmScaleSetValue(tileX, sizeX);
+		if (sizeX >= MAXTILES)
+			XtVaSetValues(tileX, XmNmaximum, sizeX, NULL);
 #endif
 #endif
 		break;
-	case PUZZLE_INC_Y:
+	case ACTION_INCY:
 		movesDsp = 0;
 		sizeY++;
 		PrintRecord(sizeX, sizeY, corners);
@@ -720,7 +712,22 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_CORNERS:
+	case ACTION_DECY:
+		movesDsp = 0;
+		sizeY--;
+		PrintRecord(sizeX, sizeY, corners);
+#ifdef WINVER
+		w->hexagons.sizeY = sizeY;
+#else
+		XtVaSetValues(w, XtNsizeY, sizeY, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(tileY, sizeY);
+		if (sizeY >= MAXTILES)
+			XtVaSetValues(tileY, XmNmaximum, sizeY, NULL);
+#endif
+#endif
+		break;
+	case ACTION_CORNERS:
 		movesDsp = 0;
 		corners = !corners;
 		PrintRecord(sizeX, sizeY, corners);
@@ -733,23 +740,20 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
-		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
 #if defined(HAVE_MOTIF) && defined(ANIMATE)
-	case PUZZLE_SPEED:
+	case ACTION_SPEED:
 		{
 			int oldDelay, delay;
 
-			XtVaGetValues(hexagons,
+			XtVaGetValues(puzzle,
 				XtNdelay, &delay, NULL);
 			oldDelay = delay;
 			if (delay > MAXSPEED - MINSPEED)
@@ -884,109 +888,109 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(&widget, brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CORNERS:
-			ModePuzzle(&widget);
+		case ACTION_CORNERS:
+			CornersPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SPEED:
-			SpeedPuzzle(&widget);
-			break;
-		case IDM_SLOW:
-			SlowPuzzle(&widget);
-			break;
-		case IDM_SOUND:
-			SoundPuzzle(&widget);
-			break;
-		case IDM_TR:
-		case IDM_RIGHT:
-		case IDM_BR:
-		case IDM_BL:
-		case IDM_LEFT:
-		case IDM_TL:
+		case ACTION_TR:
+		case ACTION_RIGHT:
+		case ACTION_BR:
+		case ACTION_BL:
+		case ACTION_LEFT:
+		case ACTION_TL:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			(void) MovePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_TR, FALSE);
+				(int) LOWORD(wParam) - ACTION_TR, FALSE);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_DECY:
-		case IDM_INCX:
-		case IDM_INCY:
-		case IDM_DECX:
+		case ACTION_DECY:
+		case ACTION_INCX:
+		case ACTION_INCY:
+		case ACTION_DECX:
 			if (MovePuzzle(&widget,
-					(int) LOWORD(wParam) - IDM_DECY,
+					(int) LOWORD(wParam) - ACTION_DECY,
 					TRUE)) {
 				SizePuzzle(&widget);
 				(void) InvalidateRect(hWnd, NULL, TRUE);
 			}
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1101,7 +1105,7 @@ CallbackPuzzleClear(Widget w, XtPointer clientData,
 		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(hexagons, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 	}
 }
 
@@ -1111,13 +1115,13 @@ TileXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int sizeX = cbs->value, sizeY, oldX;
 	Boolean corners;
 
-	XtVaGetValues(hexagons,
+	XtVaGetValues(puzzle,
 		XtNcorners, &corners,
 		XtNsizeX, &oldX,
 		XtNsizeY, &sizeY, NULL);
 	if (oldX == sizeX)
 		return;
-	XtVaSetValues(hexagons,
+	XtVaSetValues(puzzle,
 		XtNsizeX, sizeX, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1133,7 +1137,7 @@ TileYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int sizeX, sizeY = cbs->value, oldY;
 	Boolean corners;
 
-	XtVaGetValues(hexagons,
+	XtVaGetValues(puzzle,
 		XtNcorners, &corners,
 		XtNsizeX, &sizeX,
 		XtNsizeY, &oldY, NULL);
@@ -1143,7 +1147,7 @@ TileYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 		XmScaleSetValue(tileY, oldY);
 		return;
 	}
-	XtVaSetValues(hexagons,
+	XtVaSetValues(puzzle,
 		XtNsizeY, sizeY, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1160,7 +1164,7 @@ CornersToggle(Widget w, XtPointer clientData,
 	int sizeX, sizeY;
 	Boolean corners = cbs->set, oldCorn;
 
-	XtVaGetValues(hexagons,
+	XtVaGetValues(puzzle,
 		XtNcorners, &oldCorn,
 		XtNsizeX, &sizeX,
 		XtNsizeY, &sizeY, NULL);
@@ -1170,7 +1174,7 @@ CornersToggle(Widget w, XtPointer clientData,
 		XmToggleButtonSetState(cornersSwitch, !corners, True);
 		return;
 	}
-	XtVaSetValues(hexagons,
+	XtVaSetValues(puzzle,
 		XtNcorners, corners, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1186,10 +1190,10 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
-	XtVaGetValues(hexagons,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(hexagons, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 #endif
@@ -1197,18 +1201,19 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(hexagons, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
-	XtVaSetValues(hexagons, XtNmenu, val + MENU_EXIT + 1, NULL);
+	int val = (int) value + ACTION_UNDO;
+
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1264,6 +1269,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1300,6 +1306,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1313,40 +1320,12 @@ main(int argc, char **argv)
 #endif
 
 	progDsp = argv[0];
-
 	topLevel = XtInitialize(argv[0], "Hexagons",
 		options, XtNumber(options), &argc, argv);
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) hexagons_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) hexagons_bits,
-			hexagons_width, hexagons_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1363,7 +1342,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1376,18 +1356,19 @@ main(int argc, char **argv)
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
 	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
-	solveString = XmStringCreateSimple((char *) "(Solve)");
-	modeString = XmStringCreateSimple((char *) "Mode");
+	solveString = XmStringCreateSimple((char *) "(Auto-Solve)");
+	modeString = XmStringCreateSimple((char *) "Corners");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, modeString, 'M', NULL, NULL,
+		XmVaPUSHBUTTON, modeString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
@@ -1507,7 +1488,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	tilesRowCol = XtVaCreateManagedWidget("tilesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1559,24 +1539,53 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Hexagons! (use mouse or keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-	hexagons = XtCreateManagedWidget("hexagons",
+	puzzle = XtCreateManagedWidget("hexagons",
 		hexagonsWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	hexagons = XtCreateManagedWidget("hexagons",
+	puzzle = XtCreateManagedWidget("hexagons",
 		hexagonsWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(hexagons, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(hexagons);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(hexagons), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(hexagons), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(hexagons),
-		XCreateFontCursor(XtDisplay(hexagons), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
