@@ -27,6 +27,7 @@
 
 #include "file.h"
 #include "rngs.h"
+#include "sound.h"
 #include "MballP.h"
 
 #ifdef WINVER
@@ -59,11 +60,6 @@ static char wedgeColorChar[MAXWEDGES] =
 {'Y', 'B', 'R', 'M', 'G', 'O', 'C', 'D', 'P', 'L', 'A', 'T'};
 #else
 
-#include <math.h>
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 #ifndef LOGPATH
 #ifdef VMS
 #define LOGPATH "SYS$SCRATCH:"
@@ -116,6 +112,9 @@ static void Wedge10ModePuzzle(MballWidget w,
 	XEvent *event, char **args, int nArgs);
 static void Wedge12ModePuzzle(MballWidget w,
 	XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(MballWidget w, XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(MballWidget w, XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(MballWidget w, XEvent *event, char **args, int nArgs);
 static void EnterPuzzle(MballWidget w, XEvent *event, char **args, int nArgs);
 static void LeavePuzzle(MballWidget w, XEvent *event, char **args, int nArgs);
 static void MovePuzzleTtl(MballWidget w,
@@ -161,6 +160,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -277,6 +281,9 @@ static XtActionsRec actionsListPuzzle[] =
 	{(char *) "Wedge8", (XtActionProc) Wedge8ModePuzzle},
 	{(char *) "Wedge10", (XtActionProc) Wedge10ModePuzzle},
 	{(char *) "Wedge12", (XtActionProc) Wedge12ModePuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -343,6 +350,15 @@ static XtResource resourcesPuzzle[] =
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(MballWidget, mball.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
+	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
+	 XtOffset(MballWidget, mball.delay),
+	 XtRString, (caddr_t) "10"}, /* DEFAULTDELAY */
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(MballWidget, mball.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(MballWidget, mball.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(MballWidget, mball.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -375,13 +391,16 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(MballWidget, mball.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(MballWidget, mball.started),
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNcheat, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(MballWidget, mball.cheat),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(MballWidget, mball.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(MballWidget, mball.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -596,7 +615,7 @@ Boolean
 CheckSolved(MballWidget w)
 {
 	int wedge, ring;
-	MballLoc test;
+	MballLoc test = {0, FALSE};
 
 	if (w->mball.orient)
 		for (wedge = 0; wedge < w->mball.wedges; wedge++) {
@@ -695,13 +714,13 @@ LetterPosition(MballWidget w, int wedge, int ring, int lengthx, int lengthy,
 {
 	double angle, radius;
 
-	angle = (double) (2 * wedge + 1) * M_PI / w->mball.wedges;
+	angle = (2 * wedge + 1) * M_PI / w->mball.wedges;
 	if (w->mball.rings % 2 && ring == w->mball.rings / 2)
-		radius = ((double) 4.0 * ring + 1.0) / ((double) 4.0 * w->mball.rings);
+		radius = (4.0 * ring + 1.0) / (4.0 * w->mball.rings);
 	else
-		radius = ((double) 2.0 * ring + 1.0) / ((double) 2.0 * w->mball.rings);
-	*dx = lengthx / 2 + (int) ((double) lengthx * radius * cos(angle - M_PI / 2));
-	*dy = lengthy / 2 + (int) ((double) lengthy * radius * sin(angle - M_PI / 2));
+		radius = (2.0 * ring + 1.0) / (2.0 * w->mball.rings);
+	*dx = lengthx / 2 + (int) (lengthx * radius * cos(angle - M_PI / 2));
+	*dy = lengthy / 2 + (int) (lengthy * radius * sin(angle - M_PI / 2));
 }
 
 #if defined(WINVER) && (WINVER <= 0x030a)	/* if WINDOWS 3.1 or less */
@@ -712,11 +731,11 @@ LetterPosition(MballWidget w, int wedge, int ring, int lengthx, int lengthy,
 static void
 OffsetSect(const MballWidget w, const int wedge, int *dx, int *dy)
 {
-	double angle = (double) (2 * wedge + 1) * M_PI / w->mball.wedges
-			- M_PI / 2.0;
+	double angle = (2 * wedge + 1) * M_PI / w->mball.wedges
+		- M_PI / 2.0;
 
-	*dx = (int) ((double) w->mball.dr * cos(angle));
-	*dy = (int) ((double) w->mball.dr * sin(angle));
+	*dx = (int) (w->mball.dr * cos(angle));
+	*dy = (int) (w->mball.dr * sin(angle));
 }
 #endif
 
@@ -757,14 +776,18 @@ SECTOR(MballWidget w, const GC pieceGC, const GC borderGC,
 		r1 = -3;
 	if (r1 <= 3) {
 		d = MAX(2 * (r2 - 1), 3);
-		DRAWPIE(w, pieceGC, xo - d / 2, yo - d / 2, d, d, angle1, angle2);
+		DRAWPIE(w, pieceGC, xo - d / 2, yo - d / 2, d, d,
+			angle1, angle2);
 	} else {
 		d = MAX(r1 + r2 + 2, 2);
-		DRAWARC2(w, dr, pieceGC, t, xo - d / 2, yo - d / 2, d, d, angle1, angle2);
+		DRAWARC2(w, dr, pieceGC, t, xo - d / 2, yo - d / 2, d, d,
+			angle1, angle2);
 		d = MAX(2 * (r1 + 3), 1);
-		DRAWARC(w, dr, borderGC, 1, xo - d / 2, yo - d / 2, d, d, angle1, angle2);
+		DRAWARC(w, dr, borderGC, 1, xo - d / 2, yo - d / 2, d, d,
+			angle1, angle2);
 	}
-	DRAWARC(w, dr, borderGC, 1, xo - d / 2, yo - d / 2, d, d, angle1, angle2);
+	DRAWARC(w, dr, borderGC, 1, xo - d / 2, yo - d / 2, d, d,
+		angle1, angle2);
 #if 0
 	{
 		double ang, x, y;
@@ -774,14 +797,14 @@ SECTOR(MballWidget w, const GC pieceGC, const GC borderGC,
 	x = cos(ang);
 	y = -sin(ang);
 	DRAWLINE(w, dr, pieceGC,
-		(int) ((double) r1 * x) + xo, (int) ((double) r1 * y) + yo,
-		(int) ((double) r2 * x) + xo, (int) ((double) r2 * y) + yo);
+		(int) (r1 * x) + xo, (int) (r1 * y) + yo,
+		(int) (r2 * x) + xo, (int) (r2 * y) + yo);
 	ang = RADIANS((double) angle1+angle2);
 	x = cos(ang);
 	y = -sin(ang);
 	DRAWLINE(w, dr, pieceGC,
-		(int) ((double) r1 * x) + xo, (int) ((double) r1 * y) + yo,
-		(int) ((double) r2 * x) + xo, (int) ((double) r2 * y) + yo);
+		(int) (r1 * x) + xo, (int) (r1 * y) + yo,
+		(int) (r2 * x) + xo, (int) (r2 * y) + yo);
 	}
 #endif
 }
@@ -844,8 +867,10 @@ DrawSector(const MballWidget w,
 
 	startx = 1 + w->mball.puzzleOffset.x;
 	starty = 1 + w->mball.puzzleOffset.y;
-	lengthx = w->mball.viewLength - w->mball.delta + w->mball.puzzleOffset.x;
-	lengthy = w->mball.viewLength - w->mball.delta + w->mball.puzzleOffset.y;
+	lengthx = w->mball.viewLength - w->mball.delta +
+		w->mball.puzzleOffset.x;
+	lengthy = w->mball.viewLength - w->mball.delta +
+		w->mball.puzzleOffset.y;
 	if (offset) {
 		pieceGC = w->mball.borderGC;
 #ifdef FLOODFILL
@@ -1387,7 +1412,7 @@ DrawRadar(const MballWidget w, GC gc, int thick,
 				0, FULLCIRCLE);
 		}
 	}
-	increment = RADIANS(NUM_DEGREES) / (double) w->mball.wedges;
+	increment = RADIANS(NUM_DEGREES) / w->mball.wedges;
 	angle = RADIANS(RT_ANG);
 	stx = startx + lengthx / 2;
 	sty = starty + lengthy / 2;
@@ -1400,8 +1425,8 @@ DrawRadar(const MballWidget w, GC gc, int thick,
 	for (i = 0; i < w->mball.wedges; i++) {
 		DRAWLINE(w, dr, gc,
 			stx, sty,
-			stx + (int) ((double) (lenx) * cos(angle) / 2.0),
-			sty + (int) ((double) (leny) * sin(angle) / 2.0));
+			stx + (int) (lenx * cos(angle) / 2.0),
+			sty + (int) (leny * sin(angle) / 2.0));
 		angle += increment;
 	}
 }
@@ -1426,19 +1451,21 @@ DrawFrame(MballWidget w, Boolean focus)
 	DrawRadar(w, gc, 0, startx, starty, lengthx - startx, lengthy - starty);
 	if (w->mball.vertical) {
 		DRAWLINE(w, dr, w->mball.frameGC, 0, lengthy + 1,
-			(int) w->core.width, lengthy + 1);
-		DrawRadar(w, gc, 0, startx, lengthy + 3, lengthx - startx, lengthy - starty);
+			w->core.width, lengthy + 1);
+		DrawRadar(w, gc, 0, startx, lengthy + 3,
+			lengthx - startx, lengthy - starty);
 	} else {
 		DRAWLINE(w, dr, w->mball.frameGC, lengthx + 1, 0,
-			lengthx + 1, (int) w->core.height);
-		DrawRadar(w, gc, 0, lengthx + 3, starty, lengthx - startx, lengthy - starty);
+			lengthx + 1, w->core.height);
+		DrawRadar(w, gc, 0, lengthx + 3, starty,
+			lengthx - startx, lengthy - starty);
 	}
 }
 
 static void
 MoveNoPieces(MballWidget w)
 {
-	setPuzzle(w, PUZZLE_ILLEGAL);
+	setPuzzle(w, ACTION_ILLEGAL);
 }
 
 static void
@@ -1521,12 +1548,12 @@ MoveControlCb(MballWidget w, int wedge, int direction)
 	if (direction > COORD)
 		for (ring = 0; ring < w->mball.rings; ring++) {
 			MovePieces(w, wedge, ring, direction);
-			setPuzzle(w, PUZZLE_CONTROL);
+			setPuzzle(w, ACTION_CONTROL);
 	} else {
 		MovePieces(w, 0, 0, direction);
-		setPuzzle(w, PUZZLE_CONTROL);
+		setPuzzle(w, ACTION_CONTROL);
 		MovePieces(w, w->mball.wedges / 2, 0, direction);
-		setPuzzle(w, PUZZLE_CONTROL);
+		setPuzzle(w, ACTION_CONTROL);
 	}
 }
 
@@ -1538,8 +1565,13 @@ MovePuzzle(MballWidget w, const int wedge, const int ring, const int direction,
 		MoveControlCb(w, wedge, direction);
 	else {
 		MovePieces(w, wedge, ring, direction);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 	}
+#ifdef USE_SOUND
+	if (w->mball.sound) {
+		playSound((char *) MOVESOUND);
+	}
+#endif
 	setMove(&undo, wedge, ring, direction, control);
 	flushMoves(w, &redo, FALSE);
 }
@@ -1563,14 +1595,13 @@ PositionToSector(MballWidget w, int x, int y, int *wedge, int *ring, int *view)
 	y -= (w->mball.wedgeLength + 1) / 2;
 	radius = sqrt((double) x * x + y * y);
 	if (y >= 0)
-		angle = atan2((double) -x, (double) y) + M_PI;
+		angle = atan2(-x, y) + M_PI;
 	else if (x < 0)
-		angle = 2 * M_PI - atan2((double) -x, (double) -y);
+		angle = 2 * M_PI - atan2(-x, -y);
 	else
-		angle = -atan2((double) -x, (double) -y);
-	*ring = (int) (radius * (double) w->mball.rings /
-		((double) w->mball.wedgeLength));
-	*wedge = (int) (angle * (double) w->mball.wedges / (2.0 * M_PI));
+		angle = -atan2(-x, -y);
+	*ring = (int) (radius * w->mball.rings / w->mball.wedgeLength);
+	*wedge = (int) (angle * w->mball.wedges / (2.0 * M_PI));
 	if (*view == DOWN) {
 		if (w->mball.vertical)
 			*wedge = (3 * w->mball.wedges / 2 - 1 - *wedge) % w->mball.wedges;
@@ -1624,7 +1655,7 @@ MovePuzzleInput(MballWidget w, int x, int y, int direction, int control)
 	control = (control) ? 1 : 0;
 	MovePuzzle(w, wedge, ring, direction, control);
 	if (!control && CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1654,12 +1685,12 @@ ResetPieces(MballWidget w)
 		}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	w->mball.currentWedge = PUZZLE_IGNORE;
+	w->mball.currentWedge = IGNORE_DIR;
 	w->mball.started = False;
 }
 
 static void
-ResizeWedges(MballWidget w)
+ResizePieces(MballWidget w)
 {
 	w->mball.mballLength = w->mball.wedgeLength / (2 * w->mball.wedges) -
 		w->mball.delta - 1;
@@ -1669,7 +1700,7 @@ ResizeWedges(MballWidget w)
 }
 
 static void
-GetWedges(MballWidget w)
+GetPieces(MballWidget w)
 {
 	FILE *fp;
 	int c, i, wedge, ring, orient, practice, moves;
@@ -1717,7 +1748,7 @@ GetWedges(MballWidget w)
 	(void) fscanf(fp, "%d", &wedge);
 	if (wedge >= MINWEDGES && wedge <= MAXWEDGES && !(wedge % 2)) {
 		if (w->mball.wedges != wedge) {
-			setPuzzle(w, (wedge - MINWEDGES) / 2 + PUZZLE_WEDGE2);
+			setPuzzle(w, (wedge - MINWEDGES) / 2 + ACTION_WEDGE2);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: wedge ");
@@ -1738,10 +1769,10 @@ GetWedges(MballWidget w)
 	(void) fscanf(fp, "%d", &ring);
 	if (ring >= MINRINGS) {
 		for (i = w->mball.rings; i < ring; i++) {
-			setPuzzle(w, PUZZLE_INC);
+			setPuzzle(w, ACTION_INCREMENT);
 		}
 		for (i = w->mball.rings; i > ring; i--) {
-			setPuzzle(w, PUZZLE_DEC);
+			setPuzzle(w, ACTION_DECREMENT);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: ring ");
@@ -1759,12 +1790,12 @@ GetWedges(MballWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->mball.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->mball.practice != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
@@ -1772,20 +1803,20 @@ GetWedges(MballWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: wedge %d, ring %d, orient %d, ",
-		name, wedge, ring, orient);
-	(void) printf("practice %d, moves %d\n", practice, moves);
+	(void) printf("%s: wedge %d, ring %d, orient %s, ",
+		name, wedge, ring, BOOL_STRING(orient));
+	(void) printf("practice %s, moves %d\n", BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->mball.cheat = True; /* Assume the worst. */
 }
 
 static void
-WriteWedges(MballWidget w)
+WritePieces(MballWidget w)
 {
 	FILE *fp;
 	char *buf1 = NULL, *buf2 = NULL;
@@ -1842,10 +1873,10 @@ WriteWedges(MballWidget w)
 }
 
 static void
-UndoWedges(MballWidget w)
+UndoPieces(MballWidget w)
 {
 	if (madeMoves(&undo) &&
-			w->mball.currentWedge <= PUZZLE_IGNORE) {
+			w->mball.currentWedge <= IGNORE_DIR) {
 		int wedge, ring, direction, control;
 
 		getMove(&undo, &wedge, &ring, &direction, &control);
@@ -1855,19 +1886,19 @@ UndoWedges(MballWidget w)
 			MoveControlCb(w, wedge, direction);
 		else {
 			MovePieces(w, wedge, ring, direction);
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
 }
 
 static void
-RedoWedges(MballWidget w)
+RedoPieces(MballWidget w)
 {
 	if (madeMoves(&redo) &&
-			w->mball.currentWedge <= PUZZLE_IGNORE) {
+			w->mball.currentWedge <= IGNORE_DIR) {
 		int wedge, ring, direction, control;
 
 		getMove(&redo, &wedge, &ring, &direction, &control);
@@ -1876,41 +1907,41 @@ RedoWedges(MballWidget w)
 			MoveControlCb(w, wedge, direction);
 		else {
 			MovePieces(w, wedge, ring, direction);
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
 }
 
 static void
-ClearWedges(MballWidget w)
+ClearPieces(MballWidget w)
 {
-	if (w->mball.currentWedge > PUZZLE_IGNORE)
+	if (w->mball.currentWedge > IGNORE_DIR)
 		return;
 	ResetPieces(w);
 	DrawAllWedges(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
-PracticeWedges(MballWidget w)
+PracticePieces(MballWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
-RandomizeWedges(MballWidget w)
+RandomizePieces(MballWidget w)
 {
 	int randomDirection, wedge, ring;
 	int big = w->mball.wedges * (w->mball.rings + 1) + NRAND(2);
 
-	if (w->mball.currentWedge > PUZZLE_IGNORE)
+	if (w->mball.currentWedge > IGNORE_DIR)
 		return;
 	if (w->mball.practice)
-		PracticeWedges(w);
-	setPuzzle(w, PUZZLE_RESET);
+		PracticePieces(w);
+	setPuzzle(w, ACTION_RESET);
 	w->mball.cheat = False;
 	if (big > 100)
 		big = 100;
@@ -1935,41 +1966,67 @@ RandomizeWedges(MballWidget w)
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	setPuzzle(w, PUZZLE_RANDOMIZE);
+	setPuzzle(w, ACTION_RANDOMIZE);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
 static void
-SolveWedges(MballWidget w)
+SolvePieces(MballWidget w)
 {
-	if (CheckSolved(w) || w->mball.currentWedge > PUZZLE_IGNORE)
+	if (CheckSolved(w) || w->mball.currentWedge > IGNORE_DIR)
 		return;
 	{
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
-IncrementWedges(MballWidget w)
+IncrementPieces(MballWidget w)
 {
-	setPuzzle(w, PUZZLE_INC);
+	setPuzzle(w, ACTION_INCREMENT);
 }
 
 static Boolean
-DecrementWedges(MballWidget w)
+DecrementPieces(MballWidget w)
 {
 	if (w->mball.rings <= MINRINGS)
 		return False;
-	setPuzzle(w, PUZZLE_DEC);
+	setPuzzle(w, ACTION_DECREMENT);
 	return True;
 }
 
 static void
-OrientizeWedges(MballWidget w)
+OrientizePieces(MballWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
+}
+
+static void
+SpeedPieces(MballWidget w)
+{
+	w->mball.delay -= 5;
+	if (w->mball.delay < 0)
+		w->mball.delay = 0;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowPieces(MballWidget w)
+{
+	w->mball.delay += 5;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundPieces(MballWidget w)
+{
+	w->mball.sound = !w->mball.sound;
 }
 
 #ifdef WINVER
@@ -2201,43 +2258,52 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		ResizePuzzle(w);
 		redraw = True;
 	}
-	if (w->mball.menu != -1) {
+	if (w->mball.menu != ACTION_IGNORE) {
 		int menu = w->mball.menu;
 
-		w->mball.menu = -1;
+		w->mball.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
-			GetWedges(w);
+		case ACTION_GET:
+			GetPieces(w);
 			break;
-		case MENU_WRITE:
-			WriteWedges(w);
+		case ACTION_WRITE:
+			WritePieces(w);
 			break;
-		case MENU_UNDO:
-			UndoWedges(w);
+		case ACTION_UNDO:
+			UndoPieces(w);
 			break;
-		case MENU_REDO:
-			RedoWedges(w);
+		case ACTION_REDO:
+			RedoPieces(w);
 			break;
-		case MENU_CLEAR:
-			ClearWedges(w);
+		case ACTION_CLEAR:
+			ClearPieces(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizeWedges(w);
+		case ACTION_PRACTICE:
+			PracticePieces(w);
 			break;
-		case MENU_PRACTICE:
-			PracticeWedges(w);
+		case ACTION_RANDOMIZE:
+			RandomizePieces(w);
 			break;
-		case MENU_SOLVE:
-			SolveWedges(w);
+		case ACTION_SOLVE:
+			SolvePieces(w);
 			break;
-		case MENU_ORIENTIZE:
-			OrientizeWedges(w);
+		case ACTION_INCREMENT:
+			IncrementPieces(w);
 			break;
-		case MENU_INCREMENT:
-			IncrementWedges(w);
+		case ACTION_DECREMENT:
+			(void) DecrementPieces(w);
 			break;
-		case MENU_DECREMENT:
-			(void) DecrementWedges(w);
+		case ACTION_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case ACTION_SPEED:
+			SpeedPieces(w);
+			break;
+		case ACTION_SLOW:
+			SlowPieces(w);
+			break;
+		case ACTION_SOUND:
+			SoundPieces(w);
 			break;
 		default:
 			break;
@@ -2311,7 +2377,7 @@ ResizePuzzle(MballWidget w)
 	}
 	w->mball.puzzleOffset.x = ((int) w->core.width - w->mball.puzzleSize.x) / 2;
 	w->mball.puzzleOffset.y = ((int) w->core.height - w->mball.puzzleSize.y) / 2;
-	ResizeWedges(w);
+	ResizePieces(w);
 }
 
 #ifndef WINVER
@@ -2403,7 +2469,7 @@ HidePuzzle(MballWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2430,7 +2496,7 @@ SelectPuzzle(MballWidget w
 			DrawSector(w, w->mball.currentWedge, w->mball.currentRing,
 				TRUE);
 	} else
-		w->mball.currentWedge = PUZZLE_IGNORE;
+		w->mball.currentWedge = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -2451,7 +2517,7 @@ ReleasePuzzle(MballWidget w
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
-	if (w->mball.currentWedge <= PUZZLE_IGNORE)
+	if (w->mball.currentWedge <= IGNORE_DIR)
 		return;
 	DrawSector(w, w->mball.currentWedge, w->mball.currentRing, FALSE);
 	if (!control && !w->mball.practice && CheckSolved(w))
@@ -2461,12 +2527,12 @@ ReleasePuzzle(MballWidget w
 			w->mball.wedges;
 		if (ring == w->mball.currentRing) {
 			if (wedge == w->mball.currentWedge) {
-				w->mball.currentWedge = PUZZLE_IGNORE;
+				w->mball.currentWedge = IGNORE_DIR;
 				return;
 			}
 			if (opp == (wedge + 1) % w->mball.wedges ||
 					wedge == (opp + 1) % w->mball.wedges) {
-				setPuzzle(w, PUZZLE_AMBIGUOUS);
+				setPuzzle(w, ACTION_AMBIGUOUS);
 			} else {
 				diff = (w->mball.currentWedge - wedge +
 					w->mball.wedges) % w->mball.wedges;
@@ -2479,11 +2545,11 @@ ReleasePuzzle(MballWidget w
 						MovePuzzle(w, wedge, ring, CCW,
 							control);
 				if (!control && CheckSolved(w)) {
-					setPuzzle(w, PUZZLE_SOLVED);
+					setPuzzle(w, ACTION_SOLVED);
 				}
 			}
 		} else if (wedge == w->mball.currentWedge && w->mball.wedges > 2) {
-			setPuzzle(w, PUZZLE_AMBIGUOUS);
+			setPuzzle(w, ACTION_AMBIGUOUS);
 		} else if (opp == (wedge + 1) % w->mball.wedges)
 			MovePuzzle(w, wedge, ring,
 				mapWedgeToDir[(w->mball.wedges - MINWEDGES) / 2]
@@ -2495,7 +2561,7 @@ ReleasePuzzle(MballWidget w
 		else
 			MoveNoPieces(w);
 	}
-	w->mball.currentWedge = PUZZLE_IGNORE;
+	w->mball.currentWedge = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -2505,10 +2571,10 @@ PracticePuzzleMaybe(MballWidget w
 )
 {
 	if (!w->mball.started)
-		PracticeWedges(w);
+		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -2521,7 +2587,7 @@ PracticePuzzle2(MballWidget w
 #ifdef HAVE_MOTIF
 	if (!w->mball.started)
 #endif
-		PracticeWedges(w);
+		PracticePieces(w);
 }
 #endif
 
@@ -2532,10 +2598,10 @@ RandomizePuzzleMaybe(MballWidget w
 )
 {
 	if (!w->mball.started)
-		RandomizeWedges(w);
+		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
@@ -2548,7 +2614,7 @@ RandomizePuzzle2(MballWidget w
 #ifdef HAVE_MOTIF
 	if (!w->mball.started)
 #endif
-		RandomizeWedges(w);
+		RandomizePieces(w);
 }
 #endif
 
@@ -2562,7 +2628,7 @@ GetPuzzle(MballWidget w
 #endif
 )
 {
-	GetWedges(w);
+	GetPieces(w);
 }
 
 #ifndef WINVER
@@ -2575,7 +2641,7 @@ WritePuzzle(MballWidget w
 #endif
 )
 {
-	WriteWedges(w);
+	WritePieces(w);
 }
 
 #ifndef WINVER
@@ -2588,7 +2654,7 @@ UndoPuzzle(MballWidget w
 #endif
 )
 {
-	UndoWedges(w);
+	UndoPieces(w);
 }
 
 #ifndef WINVER
@@ -2601,7 +2667,7 @@ RedoPuzzle(MballWidget w
 #endif
 )
 {
-	RedoWedges(w);
+	RedoPieces(w);
 }
 
 #ifndef WINVER
@@ -2614,7 +2680,7 @@ ClearPuzzle(MballWidget w
 #endif
 )
 {
-	ClearWedges(w);
+	ClearPieces(w);
 }
 
 #ifndef WINVER
@@ -2627,7 +2693,7 @@ RandomizePuzzle(MballWidget w
 #endif
 )
 {
-	RandomizeWedges(w);
+	RandomizePieces(w);
 }
 
 #ifndef WINVER
@@ -2640,7 +2706,7 @@ SolvePuzzle(MballWidget w
 #endif
 )
 {
-	SolveWedges(w);
+	SolvePieces(w);
 }
 
 #ifndef WINVER
@@ -2653,7 +2719,7 @@ PracticePuzzle(MballWidget w
 #endif
 )
 {
-	PracticeWedges(w);
+	PracticePieces(w);
 }
 
 #ifndef WINVER
@@ -2662,7 +2728,7 @@ Wedge2ModePuzzle(MballWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_WEDGE2);
+	setPuzzle(w, ACTION_WEDGE2);
 }
 
 static void
@@ -2670,7 +2736,7 @@ Wedge4ModePuzzle(MballWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_WEDGE4);
+	setPuzzle(w, ACTION_WEDGE4);
 }
 
 static void
@@ -2678,7 +2744,7 @@ Wedge6ModePuzzle(MballWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_WEDGE6);
+	setPuzzle(w, ACTION_WEDGE6);
 }
 
 static void
@@ -2686,7 +2752,7 @@ Wedge8ModePuzzle(MballWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_WEDGE8);
+	setPuzzle(w, ACTION_WEDGE8);
 }
 
 static void
@@ -2694,7 +2760,7 @@ Wedge10ModePuzzle(MballWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_WEDGE10);
+	setPuzzle(w, ACTION_WEDGE10);
 }
 
 static void
@@ -2702,7 +2768,7 @@ Wedge12ModePuzzle(MballWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_WEDGE12);
+	setPuzzle(w, ACTION_WEDGE12);
 }
 #endif
 
@@ -2716,7 +2782,7 @@ IncrementPuzzle(MballWidget w
 #endif
 )
 {
-	IncrementWedges(w);
+	IncrementPieces(w);
 }
 
 #ifdef WINVER
@@ -2735,7 +2801,7 @@ DecrementPuzzle(MballWidget w
 #else
 	(void)
 #endif
-	DecrementWedges(w);
+	DecrementPieces(w);
 }
 
 #ifndef WINVER
@@ -2748,7 +2814,46 @@ OrientizePuzzle(MballWidget w
 #endif
 )
 {
-	OrientizeWedges(w);
+	OrientizePieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SpeedPuzzle(MballWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SlowPuzzle(MballWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SoundPuzzle(MballWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundPieces(w);
 }
 
 #ifndef WINVER
@@ -2783,7 +2888,7 @@ LeavePuzzle(MballWidget w
 void
 WedgePuzzle(MballWidget w, const int mode)
 {
-	setPuzzle(w, mode + PUZZLE_WEDGE2);
+	setPuzzle(w, mode + ACTION_WEDGE2);
 }
 
 #else
