@@ -23,16 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/11/15 X/Windows
-  Version 5: 95/10/02 Xt/Motif
-  Version 4: 94/09/15 Xt
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "MballP.h"
-#include "wmball.h"
 #define TITLE "wmball"
 
 static MballRec widget;
@@ -62,8 +55,8 @@ static MballRec widget;
 #include <Xm/ToggleB.h>
 #include <Xm/ToggleBG.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -73,9 +66,19 @@ static MballRec widget;
 #include "Mball.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/mball.xpm"
+#include "pixmaps/mball.t.xpm"
+#include "pixmaps/mball.p.xpm"
+#include "pixmaps/mball.s.xpm"
+#include "pixmaps/mball.m.xpm"
+#include "pixmaps/mball.l.xpm"
+#include "pixmaps/mball.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?mball_t_xpm:mball_p_xpm):\
+(((s)<=24)?mball_s_xpm:mball_m_xpm)):\
+(((s)<=48)?mball_l_xpm:mball_xpm))
 #endif
-#include "icons/mball.xbm"
+#include "pixmaps/mball.xbm"
+#define DEFINE_XBM (char *) mball_bits, mball_width, mball_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -87,7 +90,7 @@ static MballRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Mball Version 7.3\n"
+"Mball Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -99,7 +102,8 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-wedge{0|1|2|3|4|5|6|7|8|9|10|11} {color}]\n"
-"[-{border|bd} {color}] [-{font|fn} {fontname}]\n"
+"[-{border|bd} {color}] [-delay msecs] [-[no]sound]\n"
+"[-moveSound {filename}] [-{font|fn} {fontname}]\n"
 "[-wedges {int}] [-ring {int}] [-[no]orient]\n"
 "[-[no]practice] [-base {int}] [-userName {string}]\n"
 "[-scoreFile {filename}] [-scores] [-version]\n"
@@ -171,6 +175,14 @@ static const char featuresHelp[] = {
 "number of wedges, there would be no 180 degree turn and\n"
 "therefore the puzzle would be inoperable.\n"
 "\n"
+"Press \">\" or \".\" keys to speed up the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"<\" or \",\" keys to slow down the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"@\" key to toggle the sound.\n"
+"\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
 "Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program.\n"
@@ -214,16 +226,16 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord mballRecord[2][(MAXWEDGES - MINWEDGES) / 2 + 1]
+static PuzzleRecord puzzleRecord[2][(MAXWEDGES - MINWEDGES) / 2 + 1]
 [MAXRINGS - MINRINGS + 1];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
 
 #ifdef HAVE_MOTIF
-static Widget moves, record, message, orientSwitch, practiceSwitch;
+static Widget moves, record, message, orientizeSwitch, practiceSwitch;
 static Widget wedge[(MAXWEDGES - MINWEDGES) / 2 + 1], ring;
 static char buff[21];
 static const char *wedgeString[] =
@@ -245,8 +257,8 @@ static Boolean randomized = False;
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, mball;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -273,11 +285,11 @@ PrintRecords(void)
 	for (orient = 0; orient < 2; orient++)
 		for (i = 0; i < (MAXWEDGES - MINWEDGES) / 2 + 1; i++)
 			for (j = 0; j < MAXRINGS - MINRINGS + 1; j++) {
-				if (mballRecord[orient][i][j].score > 0)
+				if (puzzleRecord[orient][i][j].score > 0)
 					(void) printf("%6d%6d%5d %-12s%9d\n",
 						orient, 2 * i + 1, j + 1,
-						mballRecord[orient][i][j].name,
-						mballRecord[orient][i][j].score);
+						puzzleRecord[orient][i][j].name,
+						puzzleRecord[orient][i][j].score);
 			}
 }
 #endif
@@ -290,8 +302,8 @@ InitRecords(void)
 	for (orient = 0; orient < 2; orient++)
 		for (i = 0; i < (MAXWEDGES - MINWEDGES) / 2 + 1; i++)
 			for (j = 0; j < MAXRINGS - MINRINGS + 1; j++) {
-				mballRecord[orient][i][j].score = NEVER;
-				(void) strncpy(mballRecord[orient][i][j].name,
+				puzzleRecord[orient][i][j].score = NEVER;
+				(void) strncpy(puzzleRecord[orient][i][j].name,
 					NOACCESS, USERNAMELEN);
 			}
 }
@@ -346,10 +358,10 @@ ReadRecords(void)
 		for (i = 0; i < (MAXWEDGES - MINWEDGES) / 2 + 1; i++)
 			for (j = 0; j < MAXRINGS - MINRINGS + 1; j++) {
 				(void) fscanf(fp, "%d %s\n", &n, userName);
-				if (n <= mballRecord[orient][i][j].score ||
-						mballRecord[orient][i][j].score <= NEVER) {
-					mballRecord[orient][i][j].score = n;
-					(void) strncpy(mballRecord[orient][i][j].name,
+				if (n <= puzzleRecord[orient][i][j].score ||
+						puzzleRecord[orient][i][j].score <= NEVER) {
+					puzzleRecord[orient][i][j].score = n;
+					(void) strncpy(puzzleRecord[orient][i][j].name,
 						userName, USERNAMELEN);
 				}
 			}
@@ -394,8 +406,8 @@ WriteRecords(void)
 			for (i = 0; i < (MAXWEDGES - MINWEDGES) / 2 + 1; i++) {
 				for (j = 0; j < MAXRINGS - MINRINGS + 1; j++)
 					(void) fprintf(fp, "%d %s\n",
-						mballRecord[orient][i][j].score,
-						mballRecord[orient][i][j].name);
+						puzzleRecord[orient][i][j].score,
+						puzzleRecord[orient][i][j].name);
 				(void) fprintf(fp, "\n");
 			}
 			(void) fprintf(fp, "\n");
@@ -419,11 +431,11 @@ PrintRecord(int wedges, int rings, Boolean orient, Boolean practice)
 		(void) strncpy(recordDsp, "practice", MESSAGELEN);
 	} else if (rings > MAXRINGS) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (mballRecord[i][w][r].score <= NEVER) {
+	} else if (puzzleRecord[i][w][r].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			mballRecord[i][w][r].score, mballRecord[i][w][r].name);
+			puzzleRecord[i][w][r].score, puzzleRecord[i][w][r].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -447,7 +459,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(mball), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -459,16 +471,16 @@ HandleSolved(int counter, int wedges, int rings, Boolean orient)
 	int w = (wedges - MINWEDGES) / 2;
 	int r = rings - MINRINGS;
 
-	if (rings <= MAXRINGS && (counter < mballRecord[i][w][r].score ||
-			mballRecord[i][w][r].score <= NEVER)) {
+	if (rings <= MAXRINGS && (counter < puzzleRecord[i][w][r].score ||
+			puzzleRecord[i][w][r].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		mballRecord[i][w][r].score = counter;
-		(void) strncpy(mballRecord[i][w][r].name, userNameDsp,
+		puzzleRecord[i][w][r].score = counter;
+		(void) strncpy(puzzleRecord[i][w][r].name, userNameDsp,
 			USERNAMELEN);
-		if (orient && (counter < mballRecord[!i][w][r].score ||
-				mballRecord[!i][w][r].score <= NEVER)) {
-			mballRecord[!i][w][r].score = counter;
-			(void) strncpy(mballRecord[!i][w][r].name, userNameDsp,
+		if (orient && (counter < puzzleRecord[!i][w][r].score ||
+				puzzleRecord[!i][w][r].score <= NEVER)) {
+			puzzleRecord[!i][w][r].score = counter;
+			(void) strncpy(puzzleRecord[!i][w][r].name, userNameDsp,
 				USERNAMELEN);
 		}
 		WriteRecords();
@@ -531,7 +543,7 @@ Widget w
 	if (rings > MAXRINGS)
 		XtVaSetValues(ring, XmNmaximum, rings, NULL);
 	XmScaleSetValue(ring, rings);
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 #endif
@@ -556,8 +568,8 @@ Widget w
 		}
 		if (!strcmp(userNameDsp, "") ||
 				!strcmp(userNameDsp, "(null)") ||
-				!strcmp(userNameDsp, NOBODY) ||
-				!strcmp(userNameDsp, NOACCESS))
+				!strcmp(userNameDsp, NOACCESS) ||
+				!strcmp(userNameDsp, NOBODY))
 			/* It really IS nobody */
 			(void) sprintf(userNameDsp, "%s", "guest");
 #endif
@@ -602,7 +614,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -612,25 +624,25 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(mball, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(mball, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
-				"Auto-Solve", MB_OK);
+			"Auto-Solve", MB_OK);
 #else
 #ifdef HAVE_MOTIF
 		XtManageChild(solveDialog);
@@ -639,7 +651,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice",
 				MESSAGELEN);
@@ -650,15 +662,15 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_AMBIGUOUS:
+	case ACTION_AMBIGUOUS:
 		(void) strncpy(messageDsp, "Ambiguous move",
 			MESSAGELEN);
 		break;
-	case PUZZLE_ILLEGAL:
+	case ACTION_ILLEGAL:
 		if (practice || randomized)
 			(void) strncpy(messageDsp, "Illegal move",
 				MESSAGELEN);
@@ -666,13 +678,13 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 			(void) strncpy(messageDsp,
 				"Randomize to start", MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		return;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -687,7 +699,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 		SET_STARTED(w, False);
 		randomized = False;
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		randomized = False;
 		practice = !practice;
@@ -707,7 +722,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 #ifdef WINVER
@@ -719,35 +734,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 			XtNstart, False, NULL);
 #endif
 		break;
-	case PUZZLE_DEC:
-		movesDsp = 0;
-		rings--;
-		PrintRecord(wedges, rings, orient, practice);
-#ifdef WINVER
-		w->mball.rings = rings;
-#else
-		XtVaSetValues(w, XtNrings, rings, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(ring, rings);
-		if (rings >= MAXRINGS)
-			XtVaSetValues(ring, XmNmaximum, rings, NULL);
-#endif
-#endif
-		break;
-	case PUZZLE_ORIENTIZE:
-		movesDsp = 0;
-		orient = !orient;
-		PrintRecord(wedges, rings, orient, practice);
-#ifdef WINVER
-		w->mball.orient = orient;
-#else
-		XtVaSetValues(w, XtNorient, orient, NULL);
-#ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
-#endif
-#endif
-		break;
-	case PUZZLE_INC:
+	case ACTION_INCREMENT:
 		movesDsp = 0;
 		rings++;
 		PrintRecord(wedges, rings, orient, practice);
@@ -762,14 +749,42 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_WEDGE2:
-	case PUZZLE_WEDGE4:
-	case PUZZLE_WEDGE6:
-	case PUZZLE_WEDGE8:
-	case PUZZLE_WEDGE10:
-	case PUZZLE_WEDGE12:
+	case ACTION_DECREMENT:
 		movesDsp = 0;
-		wedges = 2 * (reason - PUZZLE_WEDGE2) + MINWEDGES;
+		rings--;
+		PrintRecord(wedges, rings, orient, practice);
+#ifdef WINVER
+		w->mball.rings = rings;
+#else
+		XtVaSetValues(w, XtNrings, rings, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(ring, rings);
+		if (rings >= MAXRINGS)
+			XtVaSetValues(ring, XmNmaximum, rings, NULL);
+#endif
+#endif
+		break;
+	case ACTION_ORIENTIZE:
+		movesDsp = 0;
+		orient = !orient;
+		PrintRecord(wedges, rings, orient, practice);
+#ifdef WINVER
+		w->mball.orient = orient;
+#else
+		XtVaSetValues(w, XtNorient, orient, NULL);
+#ifdef HAVE_MOTIF
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
+#endif
+#endif
+		break;
+	case ACTION_WEDGE2:
+	case ACTION_WEDGE4:
+	case ACTION_WEDGE6:
+	case ACTION_WEDGE8:
+	case ACTION_WEDGE10:
+	case ACTION_WEDGE12:
+		movesDsp = 0;
+		wedges = 2 * (reason - ACTION_WEDGE2) + MINWEDGES;
 		PrintRecord(wedges, rings, orient, practice);
 #ifdef WINVER
 		w->mball.wedges = wedges;
@@ -781,14 +796,11 @@ CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
-		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
@@ -911,79 +923,79 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_PRACTICE:
+			PracticePuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_PRACTICE:
-			PracticePuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 			GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_TOP:
-		case IDM_TTR:
-		case IDM_TR:
-		case IDM_RIGHT:
-		case IDM_BR:
-		case IDM_BBR:
-		case IDM_BOTTOM:
-		case IDM_BBL:
-		case IDM_BL:
-		case IDM_LEFT:
-		case IDM_TL:
-		case IDM_TTL:
-		case IDM_CW:
-		case IDM_CCW:
+		case ACTION_TOP:
+		case ACTION_TTR:
+		case ACTION_TR:
+		case ACTION_RIGHT:
+		case ACTION_BR:
+		case ACTION_BBR:
+		case ACTION_BOTTOM:
+		case ACTION_BBL:
+		case ACTION_BL:
+		case ACTION_LEFT:
+		case ACTION_TL:
+		case ACTION_TTL:
+		case ACTION_CW:
+		case ACTION_CCW:
 			{
 				POINT cursor, origin;
 
@@ -996,24 +1008,24 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				(void) MovePuzzleInput(&widget,
 					cursor.x - origin.x,
 					cursor.y - origin.y,
-					(int) LOWORD(wParam) - IDM_TOP, FALSE);
+					(int) LOWORD(wParam) - ACTION_TOP, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_CONTROL_TOP:
-		case IDM_CONTROL_TTR:
-		case IDM_CONTROL_TR:
-		case IDM_CONTROL_RIGHT:
-		case IDM_CONTROL_BR:
-		case IDM_CONTROL_BBR:
-		case IDM_CONTROL_BOTTOM:
-		case IDM_CONTROL_BBL:
-		case IDM_CONTROL_BL:
-		case IDM_CONTROL_LEFT:
-		case IDM_CONTROL_TL:
-		case IDM_CONTROL_TTL:
-		case IDM_CONTROL_CW:
-		case IDM_CONTROL_CCW:
+		case ACTION_CONTROL_TOP:
+		case ACTION_CONTROL_TTR:
+		case ACTION_CONTROL_TR:
+		case ACTION_CONTROL_RIGHT:
+		case ACTION_CONTROL_BR:
+		case ACTION_CONTROL_BBR:
+		case ACTION_CONTROL_BOTTOM:
+		case ACTION_CONTROL_BBL:
+		case ACTION_CONTROL_BL:
+		case ACTION_CONTROL_LEFT:
+		case ACTION_CONTROL_TL:
+		case ACTION_CONTROL_TTL:
+		case ACTION_CONTROL_CW:
+		case ACTION_CONTROL_CCW:
 			{
 				POINT cursor, origin;
 
@@ -1026,49 +1038,58 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				(void) MovePuzzleInput(&widget,
 					cursor.x - origin.x,
 					cursor.y - origin.y,
-					(int) LOWORD(wParam) - IDM_CONTROL_TOP,
+					(int) LOWORD(wParam) - ACTION_CONTROL_TOP,
 					TRUE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_DEC:
+		case ACTION_INCREMENT:
+			IncrementPuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_DECREMENT:
 			if (DecrementPuzzle(&widget)) {
 				SizePuzzle(&widget);
 				(void) InvalidateRect(hWnd, NULL, TRUE);
 			}
 			break;
-		case IDM_INC:
-			IncrementPuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_WEDGE2:
-		case IDM_WEDGE4:
-		case IDM_WEDGE6:
-		case IDM_WEDGE8:
-		case IDM_WEDGE10:
-		case IDM_WEDGE12:
+		case ACTION_WEDGE2:
+		case ACTION_WEDGE4:
+		case ACTION_WEDGE6:
+		case ACTION_WEDGE8:
+		case ACTION_WEDGE10:
+		case ACTION_WEDGE12:
 			WedgePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_WEDGE2);
+				(int) LOWORD(wParam) - ACTION_WEDGE2);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1167,6 +1188,9 @@ static XrmOptionDescRec options[] =
 	{(char *) "-wedge9", (char *) "*mball.wedgeColor9", XrmoptionSepArg, NULL},
 	{(char *) "-wedge10", (char *) "*mball.wedgeColor10", XrmoptionSepArg, NULL},
 	{(char *) "-wedge11", (char *) "*mball.wedgeColor11", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*mball.delay", XrmoptionSepArg, NULL},
+	{(char *) "-sound", (char *) "*mball.sound", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nosound", (char *) "*mball.sound", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-fn", (char *) "*mball.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*mball.font", XrmoptionSepArg, NULL},
 	{(char *) "-wedges", (char *) "*mball.wedges", XrmoptionSepArg, NULL},
@@ -1184,18 +1208,20 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mball, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mball, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
 
@@ -1206,12 +1232,12 @@ WedgeToggle(Widget w, int bit, XmToggleButtonCallbackStruct *cbs)
 	Boolean orient, practice;
 
 	if (cbs->set) {
-		XtVaGetValues(mball,
+		XtVaGetValues(puzzle,
 			XtNrings, &rings,
 			XtNorient, &orient,
 			XtNpractice, &practice, NULL);
 		wedges = bit * 2 + MINWEDGES;
-		XtVaSetValues(mball,
+		XtVaSetValues(puzzle,
 			XtNwedges, wedges, NULL);
 		movesDsp = 0;
 		(void) sprintf(buff, "%d", movesDsp);
@@ -1228,14 +1254,14 @@ RingSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int wedges, rings = cbs->value, old;
 	Boolean orient, practice;
 
-	XtVaGetValues(mball,
+	XtVaGetValues(puzzle,
 		XtNwedges, &wedges,
 		XtNrings, &old,
 		XtNorient, &orient,
 		XtNpractice, &practice, NULL);
 	if (old == rings)
 		return;
-	XtVaSetValues(mball,
+	XtVaSetValues(puzzle,
 		XtNrings, rings, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1251,14 +1277,14 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int wedges, rings;
 	Boolean orient = cbs->set, practice, oldOrient;
 
-	XtVaGetValues(mball,
+	XtVaGetValues(puzzle,
 		XtNwedges, &wedges,
 		XtNrings, &rings,
 		XtNorient, &oldOrient,
 		XtNpractice, &practice, NULL);
 	if (oldOrient == orient)
 		return;
-	XtVaSetValues(mball,
+	XtVaSetValues(puzzle,
 		XtNorient, orient, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1269,19 +1295,20 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
+PracticeToggle(Widget w, XtPointer clientData,
+		XmToggleButtonCallbackStruct *cbs)
 {
 	int wedges, rings;
 	Boolean orient, practice = cbs->set, oldPract;
 
-	XtVaGetValues(mball,
+	XtVaGetValues(puzzle,
 		XtNwedges, &wedges,
 		XtNrings, &rings,
 		XtNorient, &orient,
 		XtNpractice, &oldPract, NULL);
 	if (oldPract == practice)
 		return;
-	XtVaSetValues(mball,
+	XtVaSetValues(puzzle,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 	movesDsp = 0;
@@ -1298,19 +1325,19 @@ PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(mball, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(mball, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1366,6 +1393,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1402,6 +1430,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1411,7 +1440,8 @@ main(int argc, char **argv)
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
 	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientString,incrementString, decrementString;
+	XmString incrementString, decrementString, orientizeString;
+	XmString speedString, slowString, soundString;
 	unsigned int i;
 #endif
 
@@ -1421,34 +1451,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) mball_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) mball_bits,
-			mball_width, mball_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1465,7 +1468,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1479,20 +1483,27 @@ main(int argc, char **argv)
 	clearString = XmStringCreateSimple((char *) "Clear");
 	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "(Solve)");
-	orientString = XmStringCreateSimple((char *) "Orientize");
-	incrementString = XmStringCreateSimple((char *) "Increment");
-	decrementString = XmStringCreateSimple((char *) "Decrement");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	solveString = XmStringCreateSimple((char *) "(Auto-Solve)");
+	incrementString = XmStringCreateSimple((char *) "Increment Rings");
+	decrementString = XmStringCreateSimple((char *) "Decrement Rings");
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
+	speedString = XmStringCreateSimple((char *) ">Speed");
+	slowString = XmStringCreateSimple((char *) "<Slow");
+	soundString = XmStringCreateSimple((char *) "@Sound");
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
 		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
+		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
+		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
 	XmStringFree(undoString);
 	XmStringFree(redoString);
@@ -1502,7 +1513,10 @@ main(int argc, char **argv)
 	XmStringFree(solveString);
 	XmStringFree(incrementString);
 	XmStringFree(decrementString);
-	XmStringFree(orientString);
+	XmStringFree(orientizeString);
+	XmStringFree(speedString);
+	XmStringFree(slowString);
+	XmStringFree(soundString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1646,40 +1660,67 @@ main(int argc, char **argv)
 		XmNorientation, XmHORIZONTAL, NULL);
 	XtAddCallback(ring, XmNvalueChangedCallback,
 		(XtCallbackProc) RingSlider, (XtPointer) NULL);
-
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, ringsRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	practiceSwitch = XtVaCreateManagedWidget("Practice",
 		xmToggleButtonWidgetClass, ringsRowCol,
 		XmNset, DEFAULTPRACTICE, NULL);
 	XtAddCallback(practiceSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) PracticeToggle, (XtPointer) NULL);
-
 	messageRowCol = XtVaCreateManagedWidget("messageRowCol",
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Masterball! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-	mball = XtCreateManagedWidget("mball",
+	puzzle = XtCreateManagedWidget("mball",
 		mballWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	mball = XtCreateManagedWidget("mball",
+	puzzle = XtCreateManagedWidget("mball",
 		mballWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(mball, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(mball);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(mball), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(mball), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(mball),
-		XCreateFontCursor(XtDisplay(mball), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
