@@ -138,6 +138,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>Up: MoveTop()\n\
  <KeyPress>osfUp: MoveTop()\n\
  <KeyPress>KP_Up: MoveTop()\n\
@@ -177,11 +182,6 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
  <KeyPress>m: Middle()\n\
- <KeyPress>0x2E: Speed()\n\
- <KeyPress>0x3E: Speed()\n\
- <KeyPress>0x3C: Slow()\n\
- <KeyPress>0x2C: Slow()\n\
- <KeyPress>@: Sound()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
@@ -323,13 +323,16 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(MlinkWidget, mlink.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(MlinkWidget, mlink.started),
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNcheat, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(MlinkWidget, mlink.cheat),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(MlinkWidget, mlink.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(MlinkWidget, mlink.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -941,7 +944,7 @@ AnimateSlide(MlinkWidget w, int numTiles, int dir, int fast, Boolean logMoves)
 	  for (aTile = 0; aTile < numTiles; aTile++) {
 	    posNext = TileNFromSpace(w, aTile + 1, dir);
 	    if (logMoves && inc == 0) {
-	      setPuzzle(w, PUZZLE_MOVED);
+	      setPuzzle(w, ACTION_MOVED);
 	      setMove(&undo, dir, posNext, 0);
 	      flushMoves(w, &redo, FALSE);
 	    }
@@ -1027,11 +1030,11 @@ AnimateRotate(MlinkWidget w, int start, int finish, int direction, int fast)
 	XWindowAttributes xgwa;
 #endif
 	int size = finish - start + 1;
-	int moveI, gapJ, moveJ, inc, i;
+	int moveI, gapJ, moveJ, inc, i, offset = 0;
 
 	moveI = w->mlink.tileSize.x + w->mlink.delta.x;
 	gapJ = w->mlink.tileSize.y * fast / w->mlink.numSlices;
-	moveJ = w->mlink.tileSize.y + w->mlink.delta.y + 2;
+	moveJ = w->mlink.tileSize.y + w->mlink.delta.y + 1;
 	if (gapJ == 0)
 		gapJ++;
 
@@ -1070,26 +1073,28 @@ AnimateRotate(MlinkWidget w, int start, int finish, int direction, int fast)
 	for (inc = 0; inc < moveJ + gapJ; inc += gapJ) {
 		if (inc > moveJ)
 			inc = moveJ;
-		if (direction == TOP)
+		if (direction == TOP) {
 			i = inc;
-		else
+		} else {
 			i = w->mlink.puzzleSize.y - inc + 1;
+			offset = 3;
+		}
 #ifdef WINVER
 		hOldBitmap = SelectObject(memDC, hBitmap);
 		BitBlt(w->core.hDC,
 			start * moveI + w->mlink.puzzleOffset.x + 1,
-			w->mlink.puzzleOffset.y + 1,
+			w->mlink.puzzleOffset.y + offset,
 			size * moveI,
-			w->mlink.puzzleSize.y - i,
+			w->mlink.puzzleSize.y - i - 1,
 			memDC,
 			0, i,
 			SRCCOPY);
 		BitBlt(w->core.hDC,
 			start * moveI + w->mlink.puzzleOffset.x + 1,
 			w->mlink.puzzleSize.y - i + w->mlink.puzzleOffset.y -
-			w->mlink.delta.y + 1,
+			w->mlink.delta.y - 1 + offset,
 			size * moveI,
-			i - w->mlink.delta.y,
+			i - w->mlink.delta.y - 1,
 			memDC,
 			0, 0,
 			SRCCOPY);
@@ -1099,17 +1104,17 @@ AnimateRotate(MlinkWidget w, int start, int finish, int direction, int fast)
 			w->mlink.frameGC,
 			0, i,
 			size * moveI,
-			w->mlink.puzzleSize.y - i,
+			w->mlink.puzzleSize.y - i - 1,
 			start * moveI + w->mlink.puzzleOffset.x + 1,
-			w->mlink.puzzleOffset.y + 1);
+			w->mlink.puzzleOffset.y + offset);
 		XCopyArea(XtDisplay(w), bufferBox, XtWindow(w),
 			w->mlink.frameGC,
 			0, 0,
 			size * moveI,
-			i - w->mlink.delta.y,
+			i - w->mlink.delta.y - 1,
 			start * moveI + w->mlink.puzzleOffset.x + 1,
 			w->mlink.puzzleSize.y - i + w->mlink.puzzleOffset.y -
-			w->mlink.delta.y + 1);
+			w->mlink.delta.y - 1 + offset);
 #endif
 		FLUSH(w);
 		useTimer(&(w->mlink.oldTime), w->mlink.delay / fast);
@@ -1167,7 +1172,7 @@ EraseFrame(const MlinkWidget w, Pixmap dr)
 static void
 MoveNoTiles(MlinkWidget w)
 {
-	setPuzzle(w, PUZZLE_IGNORE);
+	setPuzzle(w, ACTION_IGNORE);
 }
 
 static void
@@ -1249,8 +1254,7 @@ RotateTiles(MlinkWidget w, int start, int finish, int direction, int fast)
 		if (space >= 0)
 			w->mlink.spacePosition = space;
 	}
-	/* Would be nice if this was not needed. */
-#ifndef DEBUG
+	/* Needed in case a window is on top. */
 	if (animate) {
 		int moveI = w->mlink.tileSize.x + w->mlink.delta.x;
 		Pixmap dr = 0;
@@ -1261,7 +1265,6 @@ RotateTiles(MlinkWidget w, int start, int finish, int direction, int fast)
 		DrawFrame(w, dr, w->mlink.focus);
 		DrawAllTilesForColumns(w, start, finish);
 	}
-#endif
 	w->mlink.currentRef = -w->mlink.tileFaces;
 #ifdef USE_SOUND
 	if (w->mlink.sound) {
@@ -1334,7 +1337,7 @@ static void
 DiscreteMoves(MlinkWidget w, int from, int dir)
 {
 	MoveTiles(w, from, NORMAL);
-	setPuzzle(w, PUZZLE_MOVED);
+	setPuzzle(w, ACTION_MOVED);
 	setMove(&undo, dir, from, 0);
 	flushMoves(w, &redo, FALSE);
 }
@@ -1343,7 +1346,7 @@ static void
 MoveShift(MlinkWidget w, int direction, int fast)
 {
 	(void) MoveTilesDir(w, direction, 0, True, fast);
-	setPuzzle(w, PUZZLE_CONTROL);
+	setPuzzle(w, ACTION_CONTROL);
 }
 
 static void
@@ -1353,20 +1356,20 @@ ControlPuzzle(MlinkWidget w, int direction)
 	case TOP:
 		if (w->mlink.faces <= MINFACES)
 			return;
-		setPuzzle(w, PUZZLE_DEC_Y);
+		setPuzzle(w, ACTION_DECY);
 		break;
 	case RIGHT:
-		setPuzzle(w, PUZZLE_INC_X);
+		setPuzzle(w, ACTION_INCX);
 		break;
 	case BOTTOM:
 		if (w->mlink.faces >= MAXFACES)
 			return;
-		setPuzzle(w, PUZZLE_INC_Y);
+		setPuzzle(w, ACTION_INCY);
 		break;
 	case LEFT:
 		if (w->mlink.tiles <= MINTILES)
 			return;
-		setPuzzle(w, PUZZLE_DEC_X);
+		setPuzzle(w, ACTION_DECX);
 		break;
 	default:
 		{
@@ -1376,7 +1379,7 @@ ControlPuzzle(MlinkWidget w, int direction)
 				direction);
 			DISPLAY_WARNING(buf);
 			free(buf);
-			setPuzzle(w, PUZZLE_IGNORE);
+			setPuzzle(w, ACTION_IGNORE);
 		}
 	}
 }
@@ -1394,7 +1397,7 @@ MovePuzzle(MlinkWidget w, const int direction, const int tile, const int shift,
 		return True;
 	} else if (MoveTilesDir(w, direction, tile, False, fast)) {
 		if (!motion) {
-			setPuzzle(w, PUZZLE_MOVED);
+			setPuzzle(w, ACTION_MOVED);
 			setMove(&undo, direction, tile, 0);
 			flushMoves(w, &redo, FALSE);
 		}
@@ -1426,7 +1429,7 @@ MovePosition(MlinkWidget w, const int direction, const int tile, const int shift
 	if (shift != 0 && (direction == TOP || direction == BOTTOM)) {
 		setMove(&undo, direction, tile, 1);
 	} else {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction, tile, 0);
 	}
 	flushMoves(w, &redo, FALSE);
@@ -1458,19 +1461,19 @@ MovePuzzleDelay(MlinkWidget w,
 	if (direction == RIGHT) {
 		pos = tile - w->mlink.spacePosition;
 		for (n = (-pos) % w->mlink.tiles - 1; n >= 0; n--) {
-			setPuzzle(w, PUZZLE_MOVED);
+			setPuzzle(w, ACTION_MOVED);
 			setMove(&undo,
 				direction, tile + n, False);
 		}
 	} else if (direction == LEFT) {
 		pos = tile - w->mlink.spacePosition;
 		for (n = pos % w->mlink.tiles - 1; n >= 0; n--) {
-			setPuzzle(w, PUZZLE_MOVED);
+			setPuzzle(w, ACTION_MOVED);
 			setMove(&undo,
 				direction, tile - n, False);
 		}
 	} else {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction, tile, all);
 	}
 	flushMoves(w, &redo, FALSE);
@@ -1486,11 +1489,11 @@ SlidePuzzle(MlinkWidget w, int direction, int pos)
 		return;
 	}
 	if (!MovePuzzle(w, direction, pos, FALSE, False, NORMAL)) {
-		setPuzzle(w, PUZZLE_BLOCKED);
+		setPuzzle(w, ACTION_BLOCKED);
 		return;
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return;
 }
@@ -1504,7 +1507,7 @@ RotatePuzzle(MlinkWidget w, int direction, int tile)
 	}
 	(void) MovePuzzle(w, direction, tile, FALSE, False, NORMAL);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return;
 }
@@ -1527,16 +1530,17 @@ MovePuzzleInput(MlinkWidget w, int x, int direction, int shift, int control)
 		MoveNoTiles(w);
 		return;
 	} else if (direction == LEFT || direction == RIGHT) {
-		SlidePuzzle(w, direction, w->mlink.spacePosition + direction - BOTTOM);
+		SlidePuzzle(w, direction,
+			w->mlink.spacePosition + direction - BOTTOM);
 		if (CheckSolved(w)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 	} else {
 		if (!PositionTile(w, x, &r))
 			return;
 		RotatePuzzle(w, direction, r);
 		if (CheckSolved(w)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 	}
 }
@@ -1672,10 +1676,10 @@ GetTiles(MlinkWidget w)
 	(void) fscanf(fp, "%d", &tiles);
 	if (tiles >= MINTILES) {
 		for (i = w->mlink.tiles; i < tiles; i++) {
-			setPuzzle(w, PUZZLE_INC_X);
+			setPuzzle(w, ACTION_INCX);
 		}
 		for (i = w->mlink.tiles; i > tiles; i--) {
-			setPuzzle(w, PUZZLE_DEC_X);
+			setPuzzle(w, ACTION_DECX);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: tiles ");
@@ -1694,10 +1698,10 @@ GetTiles(MlinkWidget w)
 	(void) fscanf(fp, "%d", &faces);
 	if (faces >= MINFACES && faces <= MAXFACES) {
 		for (i = w->mlink.faces; i < faces; i++) {
-			setPuzzle(w, PUZZLE_INC_Y);
+			setPuzzle(w, ACTION_INCY);
 		}
 		for (i = w->mlink.faces; i > faces; i--) {
-			setPuzzle(w, PUZZLE_DEC_Y);
+			setPuzzle(w, ACTION_DECY);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: faces ");
@@ -1717,17 +1721,17 @@ GetTiles(MlinkWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &middle);
 	if (w->mlink.middle != (Boolean) middle) {
-		setPuzzle(w, PUZZLE_MIDDLE);
+		setPuzzle(w, ACTION_MIDDLE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->mlink.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
-	if (w->mlink.orient != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+	if (w->mlink.practice != (Boolean) practice) {
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetTiles(w);
@@ -1735,13 +1739,14 @@ GetTiles(MlinkWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: tiles %d, faces %d, middle %d, orient %d, ",
-		name, tiles, faces, middle, orient);
-	(void) printf("practice %d, moves %d.\n", practice, moves);
+	(void) printf("%s: tiles %d, faces %d, middle %s, orient %s, ",
+		name, tiles, faces, BOOL_STRING(middle), BOOL_STRING(orient));
+	(void) printf("practice %s, moves %d.\n",
+		BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->mlink.cheat = True; /* Assume the worst. */
@@ -1825,9 +1830,9 @@ UndoTiles(MlinkWidget w)
 			else if (direction == RIGHT)
 				tile = w->mlink.spacePosition - 1;
 			if (MoveTilesDir(w, direction, tile, False, DOUBLE)) {
-				setPuzzle(w, PUZZLE_UNDO);
+				setPuzzle(w, ACTION_UNDO);
 				if (CheckSolved(w)) {
-					setPuzzle(w, PUZZLE_SOLVED);
+					setPuzzle(w, ACTION_SOLVED);
 				}
 			} else {
 				char *buf1, *buf2;
@@ -1859,9 +1864,9 @@ RedoTiles(MlinkWidget w)
 			else if (direction == RIGHT)
 				tile = w->mlink.spacePosition - 1;
 			if (MoveTilesDir(w, direction, tile, False, DOUBLE)) {
-				setPuzzle(w, PUZZLE_REDO);
+				setPuzzle(w, ACTION_REDO);
 				if (CheckSolved(w)) {
-					setPuzzle(w, PUZZLE_SOLVED);
+					setPuzzle(w, ACTION_SOLVED);
 				}
 			} else {
 				char *buf1, *buf2;
@@ -1883,13 +1888,13 @@ ClearTiles(MlinkWidget w)
 		return;
 	ResetTiles(w);
 	DrawAllTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
 PracticeTiles(MlinkWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
@@ -1899,7 +1904,7 @@ RandomizeTiles(MlinkWidget w)
 		return;
 	if (w->mlink.practice)
 		PracticeTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 	w->mlink.cheat = False;
 	/* First interchange tiles an even number of times */
 	if (w->mlink.tiles > 1 && w->mlink.faces > 1 && w->mlink.tileFaces > 4) {
@@ -1944,7 +1949,7 @@ RandomizeTiles(MlinkWidget w)
 				w->mlink.spacePosition - 1, False, INSTANT);
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 #if 0
 	/* Now move the space around randomly */
@@ -1953,7 +1958,7 @@ RandomizeTiles(MlinkWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		setPuzzle(w, PUZZLE_RESET);
+		setPuzzle(w, ACTION_RESET);
 
 #ifdef DEBUG
 		big = 3;
@@ -1978,11 +1983,11 @@ RandomizeTiles(MlinkWidget w)
 		}
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 #endif
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1994,20 +1999,20 @@ SolveTiles(MlinkWidget w)
 	if (w->mlink.tiles == 4 && w->mlink.faces == 4) {
 		SolveSomeTiles(w);
 	} else {
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizeTiles(MlinkWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 MiddleTiles(MlinkWidget w)
 {
-	setPuzzle(w, PUZZLE_MIDDLE);
+	setPuzzle(w, ACTION_MIDDLE);
 }
 
 static void
@@ -2017,7 +2022,7 @@ SpeedTiles(MlinkWidget w)
 	if (w->mlink.delay < 0)
 		w->mlink.delay = 0;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2026,7 +2031,7 @@ SlowTiles(MlinkWidget w)
 {
 	w->mlink.delay += 5;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2394,48 +2399,48 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		w->mlink.numSlices = ((w->mlink.delay < MAXSLICES) ?
 			w->mlink.delay + 1 : MAXSLICES);
 	}
-	if (w->mlink.menu != -1) {
+	if (w->mlink.menu != ACTION_IGNORE) {
 		int menu = w->mlink.menu;
 
-		w->mlink.menu = -1;
+		w->mlink.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetTiles(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WriteTiles(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoTiles(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoTiles(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearTiles(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizeTiles(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticeTiles(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizeTiles(w);
+			break;
+		case ACTION_SOLVE:
 			SolveTiles(w);
 			break;
-		case MENU_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizeTiles(w);
 			break;
-		case MENU_MIDDLE:
+		case ACTION_MIDDLE:
 			MiddleTiles(w);
 			break;
-		case MENU_SPEED:
+		case ACTION_SPEED:
 			SpeedTiles(w);
 			break;
-		case MENU_SLOW:
+		case ACTION_SLOW:
 			SlowTiles(w);
 			break;
-		case MENU_SOUND:
+		case ACTION_SOUND:
 			SoundTiles(w);
 			break;
 		default:
@@ -2585,7 +2590,7 @@ ResizeTiles(MlinkWidget w)
 	if (!(w->mlink.picture && *(w->mlink.picture))) {
 		DrawAllBufferedTiles(w);
 	} else if (!w->mlink.orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 }
 
@@ -2716,7 +2721,7 @@ HidePuzzle(MlinkWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2792,7 +2797,7 @@ MotionPuzzle(MlinkWidget w
 		DiffPosition(w, w->mlink.motionDiff, j, FALSE);
 		DiffPosition(w, w->mlink.shiftDiff, j, TRUE);
 		if (shift == 0 && CheckSolved(w)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 		w->mlink.currentTile = i;
 		w->mlink.currentFace = j;
@@ -2806,7 +2811,7 @@ MotionPuzzle(MlinkWidget w
 				w->mlink.faces;
 			DiffMove(w, diff, j, shift);
 			if (shift == 0 && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 			w->mlink.motion = True;
 			if (shift == 0)
@@ -2873,7 +2878,7 @@ ReleasePuzzle(MlinkWidget w
 					SelectSlideTiles(w, pos);
 					w->mlink.currentTile = w->mlink.tiles;
 					if (CheckSolved(w)) {
-						setPuzzle(w, PUZZLE_SOLVED);
+						setPuzzle(w, ACTION_SOLVED);
 					}
 				}
 			}
@@ -2891,7 +2896,7 @@ ReleasePuzzle(MlinkWidget w
 			DiffPosition(w, w->mlink.motionDiff, j, FALSE);
 			DiffPosition(w, w->mlink.shiftDiff, j, TRUE);
 			if (shift == 0 && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -2908,7 +2913,7 @@ PracticePuzzleMaybe(MlinkWidget w
 		PracticeTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -2935,7 +2940,7 @@ RandomizePuzzleMaybe(MlinkWidget w
 		RandomizeTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
