@@ -13,7 +13,7 @@
 #  reverse better so it does not repeat last move of 3 tiles.
 #  Break ability taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 2003 - 2006    David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2007    David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -66,7 +66,6 @@ else MoveTilePiece(w,LEFT,w->mlink.spacePosition+(n),False)
 else	MoveTilePiece(w,RIGHT,w->mlink.spacePosition-(n),False)
 
 static Boolean SolvingFlag = False;
-/* TODO Interruptability for Windows */
 #ifdef JMP
 static Boolean AbortSolvingFlag = False;
 static jmp_buf solve_env;
@@ -1828,7 +1827,7 @@ SolveRight(MlinkWidget w)
 void
 SolveSomeTiles(MlinkWidget w)
 {
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 	if (SolvingFlag)
 		return;
 
@@ -1853,5 +1852,5 @@ SolveSomeTiles(MlinkWidget w)
 #endif
 	SolvingFlag = False;
 	w->mlink.cheat = True; /* Assume the worst. */
-	setPuzzle(w, PUZZLE_COMPUTED);
+	setPuzzle(w, ACTION_COMPUTED);
 }
