@@ -32,7 +32,6 @@
 #include "file.h"
 #ifdef WINVER
 #include "MlinkP.h"
-#include "wmlink.h"
 #define TITLE "wmlink"
 
 static MlinkRec widget;
@@ -61,8 +60,8 @@ static MlinkRec widget;
 #include <Xm/Scale.h>
 #include <Xm/ToggleB.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -72,9 +71,19 @@ static MlinkRec widget;
 #include "Mlink.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/mlink.xpm"
+#include "pixmaps/mlink.t.xpm"
+#include "pixmaps/mlink.p.xpm"
+#include "pixmaps/mlink.s.xpm"
+#include "pixmaps/mlink.m.xpm"
+#include "pixmaps/mlink.l.xpm"
+#include "pixmaps/mlink.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?mlink_t_xpm:mlink_p_xpm):\
+(((s)<=24)?mlink_s_xpm:mlink_m_xpm)):\
+(((s)<=48)?mlink_l_xpm:mlink_xpm))
 #endif
-#include "icons/mlink.xbm"
+#include "pixmaps/mlink.xbm"
+#define DEFINE_XBM (char *) mlink_bits, mlink_width, mlink_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -86,7 +95,7 @@ static MlinkRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Mlink Version 7.3\n"
+"Mlink Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -207,10 +216,10 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord mlinkRecord[2][2][MAXFACES - MINFACES + 1]
-[MAXTILES - MINTILES + 1];
+static PuzzleRecord puzzleRecord[2][2]
+[MAXFACES - MINFACES + 1][MAXTILES - MINTILES + 1];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -219,7 +228,7 @@ static char recordDsp[MESSAGELEN] = "NOT RECORDED";
 #define MINSPEED 1
 #define MAXSPEED 50
 static Widget moves, record, message;
-static Widget orientSwitch, middleSwitch, practiceSwitch, tile, face, speed;
+static Widget orientizeSwitch, middleSwitch, practiceSwitch, tile, face, speed;
 static char buff[21];
 static Widget descriptionDialog, featuresDialog;
 static Widget optionsDialog, referencesDialog, aboutDialog;
@@ -235,8 +244,8 @@ static char fileName[FILENAMELEN];
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, mlink;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -264,11 +273,11 @@ PrintRecords(void)
 		for (k = 0; k < 2; k++)
 			for (j = 0; j < MAXFACES - MINFACES + 1; j++)
 				for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-					if (mlinkRecord[l][k][j][i].score > 0)
+					if (puzzleRecord[l][k][j][i].score > 0)
 						(void) printf("%6d%7d%5d%5d %-12s%9d\n",
 							l, k, j + 1, i + 1,
-							mlinkRecord[l][k][j][i].name,
-							mlinkRecord[l][k][j][i].score);
+							puzzleRecord[l][k][j][i].name,
+							puzzleRecord[l][k][j][i].score);
 				}
 }
 #endif
@@ -282,8 +291,8 @@ InitRecords(void)
 		for (k = 0; k < 2; k++)
 			for (j = 0; j < MAXFACES - MINFACES + 1; j++)
 				for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-					mlinkRecord[l][k][j][i].score = NEVER;
-					(void) strncpy(mlinkRecord[l][k][j][i].name,
+					puzzleRecord[l][k][j][i].score = NEVER;
+					(void) strncpy(puzzleRecord[l][k][j][i].name,
 						NOACCESS, USERNAMELEN);
 				}
 }
@@ -339,10 +348,10 @@ ReadRecords(void)
 			for (j = 0; j < MAXFACES - MINFACES + 1; j++)
 				for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
 					(void) fscanf(fp, "%d %s\n", &n, userName);
-					if (n <= mlinkRecord[l][k][j][i].score ||
-							mlinkRecord[l][k][j][i].score <= NEVER) {
-						mlinkRecord[l][k][j][i].score = n;
-						(void) strncpy(mlinkRecord[l][k][j][i].name,
+					if (n <= puzzleRecord[l][k][j][i].score ||
+							puzzleRecord[l][k][j][i].score <= NEVER) {
+						puzzleRecord[l][k][j][i].score = n;
+						(void) strncpy(puzzleRecord[l][k][j][i].name,
 							userName, USERNAMELEN);
 					}
 				}
@@ -388,7 +397,7 @@ WriteRecords(void)
 				for (j = 0; j < MAXFACES - MINFACES + 1; j++) {
 					for (i = 0; i < MAXTILES - MINTILES + 1; i++)
 						(void) fprintf(fp, "%d %s\n",
-							mlinkRecord[l][k][j][i].score, mlinkRecord[l][k][j][i].name);
+							puzzleRecord[l][k][j][i].score, puzzleRecord[l][k][j][i].name);
 					(void) fprintf(fp, "\n");
 				}
 				(void) fprintf(fp, "\n");
@@ -414,12 +423,12 @@ PrintRecord(int tiles, int faces, Boolean orient, Boolean middle,
 		(void) strncpy(recordDsp, "practice", MESSAGELEN);
 	} else if (tiles > MAXTILES) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (mlinkRecord[l][k][j][i].score <= NEVER) {
+	} else if (puzzleRecord[l][k][j][i].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			mlinkRecord[l][k][j][i].score,
-			mlinkRecord[l][k][j][i].name);
+			puzzleRecord[l][k][j][i].score,
+			puzzleRecord[l][k][j][i].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -447,7 +456,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(mlink), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -458,15 +467,15 @@ HandleSolved(int counter, int tiles, int faces, Boolean orient, Boolean middle)
 	int i = tiles - MINTILES, j = faces - MINFACES;
 	int k = (orient) ? 1 : 0, l = (middle) ? 1 : 0;
 
-	if (tiles <= MAXTILES && (counter < mlinkRecord[l][k][j][i].score ||
-			mlinkRecord[l][k][j][i].score <= NEVER)) {
+	if (tiles <= MAXTILES && (counter < puzzleRecord[l][k][j][i].score ||
+			puzzleRecord[l][k][j][i].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		mlinkRecord[l][k][j][i].score = counter;
-		(void) strncpy(mlinkRecord[l][k][j][i].name, userNameDsp,
+		puzzleRecord[l][k][j][i].score = counter;
+		(void) strncpy(puzzleRecord[l][k][j][i].name, userNameDsp,
 			USERNAMELEN);
 		if (tiles < 4 || faces < 2) {
-			mlinkRecord[!l][k][j][i].score = counter;
-			(void) strncpy(mlinkRecord[!l][k][j][i].name,
+			puzzleRecord[!l][k][j][i].score = counter;
+			(void) strncpy(puzzleRecord[!l][k][j][i].name,
 				userNameDsp, USERNAMELEN);
 		}
 		WriteRecords();
@@ -531,7 +540,7 @@ Widget w
 		XtVaSetValues(tile, XmNmaximum, tiles, NULL);
 	XmScaleSetValue(tile, tiles);
 	XmScaleSetValue(face, faces);
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(middleSwitch, middle, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 	{
@@ -609,7 +618,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -619,22 +628,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(mlink, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(mlink, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -646,7 +655,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice", MESSAGELEN);
 #ifdef HAVE_MOTIF
@@ -655,30 +664,30 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 		}
 		movesDsp = 0;
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		break;
-	case PUZZLE_BLOCKED:
+	case ACTION_BLOCKED:
 		(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 		break;
-	case PUZZLE_SPACE:
+	case ACTION_SPACE:
 #if 0
 		/* Too annoying */
 		(void) strncpy(messageDsp, "A space can not slide",
 			MESSAGELEN);
 #endif
 		break;
-	case PUZZLE_IGNORE:
+	case ACTION_IGNORE:
 		(void) strncpy(messageDsp, "Randomize to start",
 			MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		return;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -693,7 +702,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 				MESSAGELEN);
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		practice = !practice;
 		if (!practice)
@@ -712,7 +724,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 		XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 #ifdef WINVER
 		w->mlink.practice = False;
@@ -723,7 +735,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 			XtNstart, False, NULL);
 #endif
 		break;
-	case PUZZLE_ORIENTIZE:
+	case ACTION_ORIENTIZE:
 		movesDsp = 0;
 		orient = !orient;
 		PrintRecord(tiles, faces, orient, middle, practice);
@@ -732,11 +744,11 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #else
 		XtVaSetValues(w, XtNorient, orient, NULL);
 #ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
 #endif
 #endif
 		break;
-	case PUZZLE_MIDDLE:
+	case ACTION_MIDDLE:
 		movesDsp = 0;
 		middle = !middle;
 		PrintRecord(tiles, faces, orient, middle, practice);
@@ -749,22 +761,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_DEC_X:
-		movesDsp = 0;
-		tiles--;
-		PrintRecord(tiles, faces, orient, middle, practice);
-#ifdef WINVER
-		w->mlink.tiles = tiles;
-#else
-		XtVaSetValues(w, XtNtiles, tiles, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(tile, tiles);
-		if (tiles >= MAXTILES)
-			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
-#endif
-#endif
-		break;
-	case PUZZLE_INC_X:
+	case ACTION_INCX:
 		movesDsp = 0;
 		tiles++;
 		PrintRecord(tiles, faces, orient, middle, practice);
@@ -779,20 +776,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_DEC_Y:
+	case ACTION_DECX:
 		movesDsp = 0;
-		faces--;
+		tiles--;
 		PrintRecord(tiles, faces, orient, middle, practice);
 #ifdef WINVER
-		w->mlink.faces = faces;
+		w->mlink.tiles = tiles;
 #else
-		XtVaSetValues(w, XtNfaces, faces, NULL);
+		XtVaSetValues(w, XtNtiles, tiles, NULL);
 #ifdef HAVE_MOTIF
-		XmScaleSetValue(face, faces);
+		XmScaleSetValue(tile, tiles);
+		if (tiles >= MAXTILES)
+			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
 #endif
 #endif
 		break;
-	case PUZZLE_INC_Y:
+	case ACTION_INCY:
 		movesDsp = 0;
 		faces++;
 		PrintRecord(tiles, faces, orient, middle, practice);
@@ -805,23 +804,34 @@ CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
+	case ACTION_DECY:
+		movesDsp = 0;
+		faces--;
+		PrintRecord(tiles, faces, orient, middle, practice);
+#ifdef WINVER
+		w->mlink.faces = faces;
+#else
+		XtVaSetValues(w, XtNfaces, faces, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(face, faces);
+#endif
+#endif
 		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
 #ifdef HAVE_MOTIF
-	case PUZZLE_SPEED:
+	case ACTION_SPEED:
 		{
 			int oldDelay, delay;
 
-			XtVaGetValues(mlink, XtNdelay, &delay, NULL);
+			XtVaGetValues(puzzle,
+				XtNdelay, &delay, NULL);
 			oldDelay = delay;
 			if (delay > MAXSPEED - MINSPEED)
 				delay = MAXSPEED - MINSPEED;
@@ -898,7 +908,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	case WM_PAINT:
 		widget.core.hDC = BeginPaint(hWnd, &paint);
 		(void) SelectObject(widget.core.hDC,
-		GetStockObject(NULL_PEN));
+			GetStockObject(NULL_PEN));
 		ExposePuzzle(&widget);
 		(void) EndPaint(hWnd, &paint);
 		break;
@@ -963,10 +973,10 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			(void) GetCursorPos(&cursor);
 			if (zDelta > (WHEEL_DELTA >> 1)) {
 				MovePuzzleInput(&widget, cursor.x - origin.x,
-				TOP,
-				((GetKeyState(VK_SHIFT) >> 1) ||
-				(GetKeyState(VK_CAPITAL) & 1)),
-				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
+					TOP,
+					((GetKeyState(VK_SHIFT) >> 1) ||
+					(GetKeyState(VK_CAPITAL) & 1)),
+					(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				if (GetKeyState(VK_CONTROL) >> 1) {
 					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
@@ -988,93 +998,84 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(&widget, brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_PRACTICE:
+			PracticePuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_PRACTICE:
-			(void) PracticePuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_MIDDLE:
+		case ACTION_MIDDLE:
 			(void) MiddlePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SPEED:
-			SpeedPuzzle(&widget);
-			break;
-		case IDM_SLOW:
-			SlowPuzzle(&widget);
-			break;
-		case IDM_SOUND:
-			SoundPuzzle(&widget);
-			break;
-		case IDM_LEFT:
-		case IDM_RIGHT:
-		case IDM_SHIFT_UP:
-		case IDM_SHIFT_DOWN:
+		case ACTION_LEFT:
+		case ACTION_RIGHT:
+		case ACTION_SHIFT_UP:
+		case ACTION_SHIFT_DOWN:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			(void) MovePuzzleInput(&widget, 0,
-				(int) LOWORD(wParam) - IDM_SHIFT_UP,
+				(int) LOWORD(wParam) - ACTION_SHIFT_UP,
 				!((int) wParam % 2), FALSE);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_UP:
-		case IDM_DOWN:
+		case ACTION_UP:
+		case ACTION_DOWN:
 			{
 				POINT cursor, origin;
 
@@ -1086,36 +1087,46 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				(void) GetCursorPos(&cursor);
 				(void) MovePuzzleInput(&widget,
 					cursor.x - origin.x,
-					(int) LOWORD(wParam) - IDM_UP,
+					(int) LOWORD(wParam) - ACTION_UP,
 					FALSE, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_DECY:
-		case IDM_INCX:
-		case IDM_INCY:
-		case IDM_DECX:
+		case ACTION_DECY:
+		case ACTION_INCX:
+		case ACTION_INCY:
+		case ACTION_DECX:
 			MovePuzzleInput(&widget, 0,
-				(int) LOWORD(wParam) - IDM_DECY, FALSE, TRUE);
+				(int) LOWORD(wParam) - ACTION_DECY,
+				FALSE, TRUE);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1246,7 +1257,7 @@ CallbackPuzzlePractice(Widget w, XtPointer clientData,
 		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mlink, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
@@ -1255,7 +1266,7 @@ CallbackPuzzleRandomize(Widget w, XtPointer clientData,
 		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mlink, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
 
@@ -1265,7 +1276,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int tiles = cbs->value, faces, old;
 	Boolean orient, middle, practice;
 
-	XtVaGetValues(mlink,
+	XtVaGetValues(puzzle,
 		XtNtiles, &old,
 		XtNfaces, &faces,
 		XtNorient, &orient,
@@ -1273,7 +1284,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 		XtNpractice, &practice, NULL);
 	if (old == tiles)
 		return;
-	XtVaSetValues(mlink,
+	XtVaSetValues(puzzle,
 		XtNtiles, tiles, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1289,7 +1300,7 @@ FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	int tiles, faces = cbs->value, old;
 	Boolean orient, middle, practice;
 
-	XtVaGetValues(mlink,
+	XtVaGetValues(puzzle,
 		XtNtiles, &tiles,
 		XtNfaces, &old,
 		XtNorient, &orient,
@@ -1297,7 +1308,7 @@ FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 		XtNpractice, &practice, NULL);
 	if (old == faces)
 		return;
-	XtVaSetValues(mlink,
+	XtVaSetValues(puzzle,
 		XtNfaces, faces, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1314,7 +1325,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	Boolean orient = cbs->set, middle, practice, oldOrient;
 	char *picture;
 
-	XtVaGetValues(mlink,
+	XtVaGetValues(puzzle,
 		XtNtiles, &tiles,
 		XtNfaces, &faces,
 		XtNorient, &oldOrient,
@@ -1324,10 +1335,10 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	if (oldOrient == orient)
 		return;
 	if (!orient && picture && *picture) {
-		XmToggleButtonSetState(orientSwitch, oldOrient, True);
+		XmToggleButtonSetState(orientizeSwitch, oldOrient, True);
 		return;
 	}
-	XtVaSetValues(mlink,
+	XtVaSetValues(puzzle,
 		XtNorient, orient, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1343,7 +1354,7 @@ MiddleToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int tiles, faces;
 	Boolean orient, middle = cbs->set, practice, oldMid;
 
-	XtVaGetValues(mlink,
+	XtVaGetValues(puzzle,
 		XtNtiles, &tiles,
 		XtNfaces, &faces,
 		XtNorient, &orient,
@@ -1351,7 +1362,7 @@ MiddleToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 		XtNpractice, &practice, NULL);
 	if (oldMid == middle)
 		return;
-	XtVaSetValues(mlink,
+	XtVaSetValues(puzzle,
 		XtNmiddle, middle, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1368,7 +1379,7 @@ PracticeToggle(Widget w, XtPointer clientData,
 	int tiles, faces;
 	Boolean orient, middle, practice = cbs->set, oldPract;
 
-	XtVaGetValues(mlink,
+	XtVaGetValues(puzzle,
 		XtNtiles, &tiles,
 		XtNfaces, &faces,
 		XtNorient, &orient,
@@ -1376,7 +1387,7 @@ PracticeToggle(Widget w, XtPointer clientData,
 		XtNpractice, &oldPract, NULL);
 	if (oldPract == practice)
 		return;
-	XtVaSetValues(mlink,
+	XtVaSetValues(puzzle,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 	movesDsp = 0;
@@ -1395,28 +1406,29 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
-	XtVaGetValues(mlink,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(mlink, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(mlink, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
-	XtVaSetValues(mlink, XtNmenu, val + MENU_EXIT + 1, NULL);
+	int val = (int) value + ACTION_UNDO;
+
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1472,6 +1484,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1508,6 +1521,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1516,8 +1530,8 @@ main(int argc, char **argv)
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
-	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientString, middleString;
+	XmString clearString, practiceString, randomizeString, solveString;
+	XmString orientizeString, middleString;
 	XmString speedString, slowString, soundString;
 #endif
 
@@ -1527,34 +1541,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) mlink_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) mlink_bits,
-			mlink_width, mlink_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1571,7 +1558,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1583,22 +1571,23 @@ main(int argc, char **argv)
 	undoString = XmStringCreateSimple((char *) "Undo");
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
-	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "Solve");
-	orientString = XmStringCreateSimple((char *) "Orientize");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	solveString = XmStringCreateSimple((char *) "Auto-Solve");
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
 	middleString = XmStringCreateSimple((char *) "Middle");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, middleString, 'M', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
@@ -1607,10 +1596,10 @@ main(int argc, char **argv)
 	XmStringFree(undoString);
 	XmStringFree(redoString);
 	XmStringFree(clearString);
-	XmStringFree(randomizeString);
 	XmStringFree(practiceString);
+	XmStringFree(randomizeString);
 	XmStringFree(solveString);
-	XmStringFree(orientString);
+	XmStringFree(orientizeString);
 	XmStringFree(middleString);
 	XmStringFree(speedString);
 	XmStringFree(slowString);
@@ -1725,7 +1714,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	tilesRowCol = XtVaCreateManagedWidget("tilesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1771,10 +1759,10 @@ main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, switchRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	middleSwitch = XtVaCreateManagedWidget("Middle rigid",
 		xmToggleButtonWidgetClass, switchRowCol,
@@ -1790,25 +1778,53 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Missing Link! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-
-	mlink = XtCreateManagedWidget("mlink",
+	puzzle = XtCreateManagedWidget("mlink",
 		mlinkWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	mlink = XtCreateManagedWidget("mlink",
+	puzzle = XtCreateManagedWidget("mlink",
 		mlinkWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(mlink, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(mlink);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(mlink), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(mlink), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(mlink),
-		XCreateFontCursor(XtDisplay(mlink), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
