@@ -27,6 +27,7 @@
 
 #include "file.h"
 #include "rngs.h"
+#include "sound.h"
 #include "OctP.h"
 
 #ifdef WINVER
@@ -103,6 +104,9 @@ static void Period4ModePuzzle(OctWidget w,
 	XEvent *event, char **args, int nArgs);
 static void BothModePuzzle(OctWidget w,
 	XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
 static void EnterPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
 static void LeavePuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
 static void MovePuzzleTl(OctWidget w, XEvent *event, char **args, int nArgs);
@@ -131,6 +135,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -229,6 +238,9 @@ static XtActionsRec actionsListPuzzle[] =
 	{(char *) "Period4", (XtActionProc) Period4ModePuzzle},
 	{(char *) "Both", (XtActionProc) BothModePuzzle},
 	{(char *) "Sticky", (XtActionProc) StickyModePuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -284,6 +296,15 @@ static XtResource resourcesPuzzle[] =
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(OctWidget, oct.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
+	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
+	 XtOffset(OctWidget, oct.delay),
+	 XtRString, (caddr_t) "10"}, /* DEFAULTDELAY */
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(OctWidget, oct.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(OctWidget, oct.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(OctWidget, oct.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -316,13 +337,16 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(OctWidget, oct.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(OctWidget, oct.started),
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNcheat, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(OctWidget, oct.cheat),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(OctWidget, oct.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(OctWidget, oct.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -660,7 +684,7 @@ Boolean
 CheckSolved(OctWidget w)
 {
 	int face, position;
-	OctLoc test;
+	OctLoc test = {0, FALSE};
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < w->oct.sizeSize; position++) {
@@ -1056,7 +1080,7 @@ DrawFrame(OctWidget w, Boolean focus)
 static void
 MoveNoPieces(OctWidget w)
 {
-	setPuzzle(w, PUZZLE_ILLEGAL);
+	setPuzzle(w, ACTION_ILLEGAL);
 }
 
 static void
@@ -1377,7 +1401,7 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 				rtt.trbl = i;
 				rtt.tlbr = i;
 				MovePieces(w, face, rtt, direction, style);
-				setPuzzle(w, PUZZLE_CONTROL);
+				setPuzzle(w, ACTION_CONTROL);
 		} else		/* (style == PERIOD4) */
 			for (i = 0; i < 3; i++) {
 				if (i < 2) {
@@ -1411,7 +1435,7 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 							(direction + 6) % MAXFACES, style);
 					}
 				}
-				setPuzzle(w, PUZZLE_CONTROL);
+				setPuzzle(w, ACTION_CONTROL);
 			}
 	} else {
 		if (style == PERIOD3)
@@ -1420,7 +1444,7 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 				rtt.trbl = i;
 				rtt.tlbr = i;
 				MovePieces(w, face, rtt, direction, style);
-				setPuzzle(w, PUZZLE_CONTROL);
+				setPuzzle(w, ACTION_CONTROL);
 		} else		/* (style == PERIOD4) */
 			for (i = 0; i < 2 * w->oct.size - 1; i++) {
 				if (i < w->oct.size) {
@@ -1450,13 +1474,14 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 							(direction + 6) % MAXFACES, style);
 					}
 				}
-				setPuzzle(w, PUZZLE_CONTROL);
+				setPuzzle(w, ACTION_CONTROL);
 			}
 	}
 }
 
 void
-MovePuzzle(OctWidget w, int face, int position, int direction, int style, int control)
+MovePuzzle(OctWidget w, int face, int position, int direction, int style,
+		int control)
 {
 	if (control)
 		MoveControlCb(w, face, direction, style);
@@ -1465,8 +1490,13 @@ MovePuzzle(OctWidget w, int face, int position, int direction, int style, int co
 
 		ToRTT(position, &rtt);
 		MovePieces(w, face, rtt, direction, style);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 	}
+#ifdef USE_SOUND
+	if (w->oct.sound) {
+		playSound((char *) MOVESOUND);
+	}
+#endif
 	setMove(&undo, face, position, direction, style, control);
 	flushMoves(w, &redo, FALSE);
 }
@@ -1940,7 +1970,7 @@ MovePuzzleInput(OctWidget w, int x, int y, int direction, int shift, int control
 	control = (control) ? 1 : 0;
 	MovePuzzle(w, face, ToPosition(rtt), direction, style, control);
 	if (!control && CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1986,7 +2016,7 @@ ResetPieces(OctWidget w)
 		}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	w->oct.currentFace = PUZZLE_IGNORE;
+	w->oct.currentFace = IGNORE_DIR;
 	w->oct.started = False;
 }
 
@@ -2068,10 +2098,10 @@ GetPieces(OctWidget w)
 	(void) fscanf(fp, "%d", &size);
 	if (size >= MINFACETS) {
 		for (i = w->oct.size; i < size; i++) {
-			setPuzzle(w, PUZZLE_INC);
+			setPuzzle(w, ACTION_INCREMENT);
 		}
 		for (i = w->oct.size; i > size; i--) {
-			setPuzzle(w, PUZZLE_DEC);
+			setPuzzle(w, ACTION_DECREMENT);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: size ");
@@ -2090,13 +2120,13 @@ GetPieces(OctWidget w)
 	(void) fscanf(fp, "%d", &mode);
 	switch (mode) {
 	case PERIOD3:
-		setPuzzle(w, PUZZLE_PERIOD3);
+		setPuzzle(w, ACTION_PERIOD3);
 		break;
 	case PERIOD4:
-		setPuzzle(w, PUZZLE_PERIOD4);
+		setPuzzle(w, ACTION_PERIOD4);
 		break;
 	case BOTH:
-		setPuzzle(w, PUZZLE_BOTH);
+		setPuzzle(w, ACTION_BOTH);
 		break;
 	default:
 		stringCat(&buf1, name, " corrupted: mode ");
@@ -2116,17 +2146,17 @@ GetPieces(OctWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sticky);
 	if (w->oct.sticky != (Boolean) sticky) {
-		setPuzzle(w, PUZZLE_STICKY);
+		setPuzzle(w, ACTION_STICKY);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->oct.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->oct.practice != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
@@ -2134,13 +2164,14 @@ GetPieces(OctWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: size %d, mode %d, sticky %d, orient %d",
-		name, size, mode, sticky, orient);
-	(void) printf(", practice %d, moves %d.\n", practice, moves);
+	(void) printf("%s: size %d, mode %d, sticky %s, orient %s",
+		name, size, mode, BOOL_STRING(sticky), BOOL_STRING(orient));
+	(void) printf(", practice %s, moves %d.\n",
+		BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->oct.cheat = True; /* Assume the worst. */
@@ -2208,7 +2239,7 @@ static void
 UndoPieces(OctWidget w)
 {
 	if (madeMoves(&undo) &&
-			w->oct.currentFace <= PUZZLE_IGNORE) {
+			w->oct.currentFace <= IGNORE_DIR) {
 		int face, position, direction, style, control;
 
 		getMove(&undo,
@@ -2224,9 +2255,9 @@ UndoPieces(OctWidget w)
 
 			ToRTT(position, &rtt);
 			MovePieces(w, face, rtt, direction, style);
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -2236,7 +2267,7 @@ static void
 RedoPieces(OctWidget w)
 {
 	if (madeMoves(&redo) &&
-			w->oct.currentFace <= PUZZLE_IGNORE) {
+			w->oct.currentFace <= IGNORE_DIR) {
 		int face, position, direction, style, control;
 
 		getMove(&redo,
@@ -2250,9 +2281,9 @@ RedoPieces(OctWidget w)
 
 			ToRTT(position, &rtt);
 			MovePieces(w, face, rtt, direction, style);
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -2261,17 +2292,17 @@ RedoPieces(OctWidget w)
 static void
 ClearPieces(OctWidget w)
 {
-	if (w->oct.currentFace > PUZZLE_IGNORE)
+	if (w->oct.currentFace > IGNORE_DIR)
 		return;
 	ResetPieces(w);
 	DrawAllPieces(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
 PracticePieces(OctWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
@@ -2280,7 +2311,7 @@ RandomizePieces(OctWidget w)
 	int randomDirection, face, position, style;
 	int big = w->oct.sizeSize * 3 + NRAND(2);
 
-	if (w->oct.currentFace > PUZZLE_IGNORE)
+	if (w->oct.currentFace > IGNORE_DIR)
 		return;
 	if (big > 1000)
 		big = 1000;
@@ -2288,7 +2319,7 @@ RandomizePieces(OctWidget w)
 		PracticePieces(w);
 	if (w->oct.sticky)
 		big /= 3;
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -2355,30 +2386,30 @@ RandomizePieces(OctWidget w)
 			position = NRAND(w->oct.sizeSize);
 		}
 		MovePuzzle(w, face, position, randomDirection, style, FALSE);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	setPuzzle(w, PUZZLE_RANDOMIZE);
+	setPuzzle(w, ACTION_RANDOMIZE);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
 static void
 SolvePieces(OctWidget w)
 {
-	if (CheckSolved(w) && w->oct.currentFace > PUZZLE_IGNORE)
+	if (CheckSolved(w) && w->oct.currentFace > IGNORE_DIR)
 		return;
 	{
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
 IncrementPieces(OctWidget w)
 {
-	setPuzzle(w, PUZZLE_INC);
+	setPuzzle(w, ACTION_INCREMENT);
 }
 
 static Boolean
@@ -2386,20 +2417,46 @@ DecrementPieces(OctWidget w)
 {
 	if (w->oct.size <= MINFACETS)
 		return False;
-	setPuzzle(w, PUZZLE_DEC);
+	setPuzzle(w, ACTION_DECREMENT);
 	return True;
 }
 
 static void
 OrientizePieces(OctWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 StickyPieces(OctWidget w)
 {
-	setPuzzle(w, PUZZLE_STICKY);
+	setPuzzle(w, ACTION_STICKY);
+}
+
+static void
+SpeedPieces(OctWidget w)
+{
+	w->oct.delay -= 5;
+	if (w->oct.delay < 0)
+		w->oct.delay = 0;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowPieces(OctWidget w)
+{
+	w->oct.delay += 5;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundPieces(OctWidget w)
+{
+	w->oct.sound = !w->oct.sound;
 }
 
 #ifdef WINVER
@@ -2627,47 +2684,56 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		ResizePuzzle(w);
 		redraw = True;
 	}
-	if (w->oct.menu != -1) {
+	if (w->oct.menu != ACTION_IGNORE) {
 		int menu = w->oct.menu;
 
-		w->oct.menu = -1;
+		w->oct.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetPieces(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WritePieces(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoPieces(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoPieces(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearPieces(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizePieces(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePieces(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizePieces(w);
+			break;
+		case ACTION_SOLVE:
 			SolvePieces(w);
 			break;
-		case MENU_ORIENTIZE:
-			OrientizePieces(w);
-			break;
-		case MENU_STICKY:
-			StickyPieces(w);
-			break;
-		case MENU_INCREMENT:
+		case ACTION_INCREMENT:
 			IncrementPieces(w);
 			break;
-		case MENU_DECREMENT:
+		case ACTION_DECREMENT:
 			(void) DecrementPieces(w);
 			break;
+		case ACTION_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case ACTION_STICKY:
+			StickyPieces(w);
+			break;
+		case ACTION_SPEED:
+			SpeedPieces(w);
+			break;
+		case ACTION_SLOW:
+			SlowPieces(w);
+			break;
+		case ACTION_SOUND:
+			SoundPieces(w);
+			break;
 		default:
 			break;
 		}
@@ -2834,7 +2900,7 @@ HidePuzzle(OctWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2862,7 +2928,7 @@ SelectPuzzle(OctWidget w
 				w->oct.currentPosition, TRUE);
 		}
 	} else
-		w->oct.currentFace = PUZZLE_IGNORE;
+		w->oct.currentFace = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -2885,7 +2951,7 @@ ReleasePuzzle(OctWidget w
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
-	if (w->oct.currentFace <= PUZZLE_IGNORE)
+	if (w->oct.currentFace <= IGNORE_DIR)
 		return;
 	DrawTriangle(w, w->oct.currentFace, w->oct.currentPosition, FALSE);
 	if (!control && !w->oct.practice && CheckSolved(w))
@@ -2909,15 +2975,15 @@ ReleasePuzzle(OctWidget w
 				w->oct.currentPosition,
 				direction, style, (control) ? 1 : 0);
 			if (!control && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else if (count == 2) {
 			/* count == 3 too annoying */
-			setPuzzle(w, PUZZLE_AMBIGUOUS);
+			setPuzzle(w, ACTION_AMBIGUOUS);
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->oct.currentFace = PUZZLE_IGNORE;
+	w->oct.currentFace = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -2930,7 +2996,7 @@ PracticePuzzleMaybe(OctWidget w
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -2957,7 +3023,7 @@ RandomizePuzzleMaybe(OctWidget w
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
@@ -3142,7 +3208,7 @@ Period3ModePuzzle(OctWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_PERIOD3);
+	setPuzzle(w, ACTION_PERIOD3);
 }
 
 static void
@@ -3150,7 +3216,7 @@ Period4ModePuzzle(OctWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_PERIOD4);
+	setPuzzle(w, ACTION_PERIOD4);
 }
 
 static void
@@ -3158,10 +3224,49 @@ BothModePuzzle(OctWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_BOTH);
+	setPuzzle(w, ACTION_BOTH);
 }
 #endif
 
+#ifndef WINVER
+static
+#endif
+void
+SpeedPuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SlowPuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SoundPuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundPieces(w);
+}
+
 #ifndef WINVER
 static
 #endif
@@ -3194,7 +3299,7 @@ LeavePuzzle(OctWidget w
 void
 PeriodModePuzzle(OctWidget w, const int mode)
 {
-	setPuzzle(w, mode + PUZZLE_PERIOD3);
+	setPuzzle(w, mode + ACTION_PERIOD3);
 }
 
 #else
