@@ -23,19 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/12/15 X/Windows
-  Version 5: 95/10/04 Xt/Motif
-  Version 4: 94/05/31 Xt
-  Version 3: 93/04/01 Motif
-  Version 2: 92/02/01 XView
-  Version 1: 91/06/10 SunView
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "OctP.h"
-#include "woct.h"
 #define TITLE "woct"
 
 static OctRec widget;
@@ -65,8 +55,8 @@ static OctRec widget;
 #include <Xm/ToggleB.h>
 #include <Xm/ToggleBG.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -76,9 +66,19 @@ static OctRec widget;
 #include "Oct.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/oct.xpm"
+#include "pixmaps/oct.t.xpm"
+#include "pixmaps/oct.p.xpm"
+#include "pixmaps/oct.s.xpm"
+#include "pixmaps/oct.m.xpm"
+#include "pixmaps/oct.l.xpm"
+#include "pixmaps/oct.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?oct_t_xpm:oct_p_xpm):\
+(((s)<=24)?oct_s_xpm:oct_m_xpm)):\
+(((s)<=48)?oct_l_xpm:oct_xpm))
 #endif
-#include "icons/oct.xbm"
+#include "pixmaps/oct.xbm"
+#define DEFINE_XBM (char *) oct_bits, oct_width, oct_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -90,7 +90,7 @@ static OctRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Oct Version 7.3\n"
+"Oct Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -102,6 +102,7 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3|4|5|6|7} {color}] [-{border|bd} {color}]\n"
+"[-delay msecs] [-[no]sound] [-moveSound {filename}]\n"
 "[-{font|fn} {fontname}] [-{size {int} | sticky}]\n"
 "[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n"
 "[-userName {string}] [-scoreFile {filename}] [-scores]\n"
@@ -166,7 +167,7 @@ static const char featuresHelp[] = {
 "getting all the faces to be the same color.  To do this "
 "one has to get the lines to be oriented in\n"
 "the same direction, this only matters with center "
-"\"facets\", if at all (i.e. those that are not on a\n"
+"pieces, if at all (i.e. those that are not on a\n"
 "corner or edge).  This does add complexity so there are "
 "2 sets of records.\n"
 "\n"
@@ -184,9 +185,17 @@ static const char featuresHelp[] = {
 "Beware, the \"Sticky\" mode is a hack and much could be done "
 "to improve its look.\n"
 "\n"
-"Press \"I\" or \"i\" keys to increase the number of \"facets\".\n"
+"Press \"I\" or \"i\" keys to increase the number of pieces.\n"
 "\n"
-"Press \"D\" or \"d\" keys to decrease the number of \"facets\".\n"
+"Press \"D\" or \"d\" keys to decrease the number of pieces.\n"
+"\n"
+"Press \">\" or \".\" keys to speed up the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"<\" or \",\" keys to slow down the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"@\" key to toggle the sound.\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -237,16 +246,16 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord octRecord[MAXMODES][2][MAXFACETS - MINFACETS + 2];
+static PuzzleRecord puzzleRecord[MAXMODES][2][MAXFACETS - MINFACETS + 2];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
 static int oldSize;
 
 #ifdef HAVE_MOTIF
-static Widget moves, record, message, modes[MAXMODES], orientSwitch;
+static Widget moves, record, message, modes[MAXMODES], orientizeSwitch;
 static Widget stickySwitch, practiceSwitch, facets;
 static char buff[21];
 static const char *modeString[] =
@@ -268,8 +277,8 @@ static Boolean randomized = False;
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, oct;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -296,11 +305,11 @@ PrintRecords(void)
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++) {
-				if (octRecord[mode][orient][i].score > 0)
+				if (puzzleRecord[mode][orient][i].score > 0)
 					(void) printf("%4d%7d%6d %-12s%9d\n",
 						mode + 3, orient, i + 1,
-						octRecord[mode][orient][i].name,
-						octRecord[mode][orient][i].score);
+						puzzleRecord[mode][orient][i].name,
+						puzzleRecord[mode][orient][i].score);
 			}
 }
 #endif
@@ -313,8 +322,8 @@ InitRecords(void)
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++) {
-				octRecord[mode][orient][i].score = NEVER;
-				(void) strncpy(octRecord[mode][orient][i].name,
+				puzzleRecord[mode][orient][i].score = NEVER;
+				(void) strncpy(puzzleRecord[mode][orient][i].name,
 					NOACCESS, USERNAMELEN);
 			}
 }
@@ -369,10 +378,10 @@ ReadRecords(void)
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++) {
 				(void) fscanf(fp, "%d %s\n", &n, userName);
-				if (n <= octRecord[mode][orient][i].score ||
-						octRecord[mode][orient][i].score <= NEVER) {
-					octRecord[mode][orient][i].score = n;
-					(void) strncpy(octRecord[mode][orient][i].name,
+				if (n <= puzzleRecord[mode][orient][i].score ||
+						puzzleRecord[mode][orient][i].score <= NEVER) {
+					puzzleRecord[mode][orient][i].score = n;
+					(void) strncpy(puzzleRecord[mode][orient][i].name,
 						userName, USERNAMELEN);
 				}
 			}
@@ -417,7 +426,7 @@ WriteRecords(void)
 			for (orient = 0; orient < 2; orient++) {
 				for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++)
 					(void) fprintf(fp, "%d %s\n",
-						octRecord[mode][orient][i].score, octRecord[mode][orient][i].name);
+						puzzleRecord[mode][orient][i].score, puzzleRecord[mode][orient][i].name);
 				(void) fprintf(fp, "\n");
 			}
 			(void) fprintf(fp, "\n");
@@ -442,11 +451,11 @@ PrintRecord(int size, int mode,
 		(void) strncpy(recordDsp, "practice", MESSAGELEN);
 	} else if (!sticky && size > MAXFACETS) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (octRecord[i][j][k].score <= NEVER) {
+	} else if (puzzleRecord[i][j][k].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			octRecord[i][j][k].score, octRecord[i][j][k].name);
+			puzzleRecord[i][j][k].score, puzzleRecord[i][j][k].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -480,7 +489,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(oct), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -493,17 +502,17 @@ HandleSolved(int counter, int size, int mode, Boolean orient, Boolean sticky)
 	int k = (sticky) ? MAXFACETS - MINFACETS + 1 : size - MINFACETS;
 
 	if ((sticky || size <= MAXFACETS) &&
-			(counter < octRecord[i][j][k].score ||
-			octRecord[i][j][k].score <= NEVER)) {
+			(counter < puzzleRecord[i][j][k].score ||
+			puzzleRecord[i][j][k].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		octRecord[i][j][k].score = counter;
-		(void) strncpy(octRecord[i][j][k].name, userNameDsp, USERNAMELEN);
+		puzzleRecord[i][j][k].score = counter;
+		(void) strncpy(puzzleRecord[i][j][k].name, userNameDsp, USERNAMELEN);
 		if ((size < 2 && mode != PERIOD4) ||
 				(size < 4 && mode == PERIOD4) || (orient &&
-				(counter < octRecord[i][!j][k].score ||
-				octRecord[i][!j][k].score <= NEVER))) {
-			octRecord[i][!j][k].score = counter;
-			(void) strncpy(octRecord[i][!j][k].name, userNameDsp,
+				(counter < puzzleRecord[i][!j][k].score ||
+				puzzleRecord[i][!j][k].score <= NEVER))) {
+			puzzleRecord[i][!j][k].score = counter;
+			(void) strncpy(puzzleRecord[i][!j][k].name, userNameDsp,
 				USERNAMELEN);
 		}
 		WriteRecords();
@@ -568,7 +577,7 @@ Widget w
 		XtVaSetValues(facets, XmNmaximum, size, NULL);
 	XmScaleSetValue(facets, size);
 	XmToggleButtonSetState(modes[mode - PERIOD3], True, False);
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(stickySwitch, sticky, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
@@ -643,7 +652,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -653,22 +662,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(oct, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(oct, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -680,7 +689,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice",
 				MESSAGELEN);
@@ -691,15 +700,15 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_AMBIGUOUS:
+	case ACTION_AMBIGUOUS:
 		(void) strncpy(messageDsp, "Ambiguous move",
 			MESSAGELEN);
 		break;
-	case PUZZLE_ILLEGAL:
+	case ACTION_ILLEGAL:
 		if (practice || randomized)
 			(void) strncpy(messageDsp, "Illegal move",
 				MESSAGELEN);
@@ -707,13 +716,13 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 			(void) strncpy(messageDsp,
 				"Randomize to start", MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		return;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -728,7 +737,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 		SET_STARTED(w, False);
 		randomized = False;
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		randomized = False;
 		practice = !practice;
@@ -748,7 +760,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 #ifdef WINVER
@@ -760,38 +772,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 			XtNstart, False, NULL);
 #endif
 		break;
-	case PUZZLE_DEC:
-		if (!sticky) {
-			movesDsp = 0;
-			size--;
-			oldSize = size;
-			PrintRecord(size, mode, orient, sticky, practice);
-#ifdef WINVER
-			w->oct.size = size;
-#else
-			XtVaSetValues(w, XtNsize, size, NULL);
-#ifdef HAVE_MOTIF
-			XmScaleSetValue(facets, size);
-			if (size >= MAXFACETS)
-				XtVaSetValues(facets, XmNmaximum, size, NULL);
-#endif
-#endif
-		}
-		break;
-	case PUZZLE_ORIENTIZE:
-		movesDsp = 0;
-		orient = !orient;
-		PrintRecord(size, mode, orient, sticky, practice);
-#ifdef WINVER
-		w->oct.orient = orient;
-#else
-		XtVaSetValues(w, XtNorient, orient, NULL);
-#ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
-#endif
-#endif
-		break;
-	case PUZZLE_INC:
+	case ACTION_INCREMENT:
 		if (!sticky) {
 			movesDsp = 0;
 			size++;
@@ -809,11 +790,42 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 		}
 		break;
-	case PUZZLE_PERIOD3:
-	case PUZZLE_PERIOD4:
-	case PUZZLE_BOTH:
+	case ACTION_DECREMENT:
+		if (!sticky) {
+			movesDsp = 0;
+			size--;
+			oldSize = size;
+			PrintRecord(size, mode, orient, sticky, practice);
+#ifdef WINVER
+			w->oct.size = size;
+#else
+			XtVaSetValues(w, XtNsize, size, NULL);
+#ifdef HAVE_MOTIF
+			XmScaleSetValue(facets, size);
+			if (size >= MAXFACETS)
+				XtVaSetValues(facets, XmNmaximum, size, NULL);
+#endif
+#endif
+		}
+		break;
+	case ACTION_ORIENTIZE:
 		movesDsp = 0;
-		mode = reason - PUZZLE_PERIOD3 + PERIOD3;
+		orient = !orient;
+		PrintRecord(size, mode, orient, sticky, practice);
+#ifdef WINVER
+		w->oct.orient = orient;
+#else
+		XtVaSetValues(w, XtNorient, orient, NULL);
+#ifdef HAVE_MOTIF
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
+#endif
+#endif
+		break;
+	case ACTION_PERIOD3:
+	case ACTION_PERIOD4:
+	case ACTION_BOTH:
+		movesDsp = 0;
+		mode = reason - ACTION_PERIOD3 + PERIOD3;
 		PrintRecord(size, mode, orient, sticky, practice);
 #ifdef WINVER
 		w->oct.mode = mode;
@@ -824,7 +836,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_STICKY:
+	case ACTION_STICKY:
 		movesDsp = 0;
 		sticky = !sticky;
 		if (sticky)
@@ -845,14 +857,11 @@ CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
-		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
@@ -982,91 +991,91 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_STICKY:
+		case ACTION_STICKY:
 			(void) StickyModePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SHIFT_TOP:
-		case IDM_SHIFT_TR:
-		case IDM_SHIFT_RIGHT:
-		case IDM_SHIFT_BR:
-		case IDM_SHIFT_BOTTOM:
-		case IDM_SHIFT_BL:
-		case IDM_SHIFT_LEFT:
-		case IDM_SHIFT_TL:
-		case IDM_SHIFT_CW:
-		case IDM_SHIFT_CCW:
+		case ACTION_SHIFT_TOP:
+		case ACTION_SHIFT_TR:
+		case ACTION_SHIFT_RIGHT:
+		case ACTION_SHIFT_BR:
+		case ACTION_SHIFT_BOTTOM:
+		case ACTION_SHIFT_BL:
+		case ACTION_SHIFT_LEFT:
+		case ACTION_SHIFT_TL:
+		case ACTION_SHIFT_CW:
+		case ACTION_SHIFT_CCW:
 			shift = 1;
-		case IDM_TOP:
-		case IDM_TR:
-		case IDM_RIGHT:
-		case IDM_BR:
-		case IDM_BOTTOM:
-		case IDM_BL:
-		case IDM_LEFT:
-		case IDM_TL:
-		case IDM_CW:
-		case IDM_CCW:
+		case ACTION_TOP:
+		case ACTION_TR:
+		case ACTION_RIGHT:
+		case ACTION_BR:
+		case ACTION_BOTTOM:
+		case ACTION_BL:
+		case ACTION_LEFT:
+		case ACTION_TL:
+		case ACTION_CW:
+		case ACTION_CCW:
 			{
 				POINT cursor, origin;
 
@@ -1080,32 +1089,32 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_SHIFT_TOP : IDM_TOP),
+					((shift) ? ACTION_SHIFT_TOP : ACTION_TOP),
 					shift, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_SHIFTCONTROL_TOP:
-		case IDM_SHIFTCONTROL_TR:
-		case IDM_SHIFTCONTROL_RIGHT:
-		case IDM_SHIFTCONTROL_BR:
-		case IDM_SHIFTCONTROL_BOTTOM:
-		case IDM_SHIFTCONTROL_BL:
-		case IDM_SHIFTCONTROL_LEFT:
-		case IDM_SHIFTCONTROL_TL:
-		case IDM_SHIFTCONTROL_CW:
-		case IDM_SHIFTCONTROL_CCW:
+		case ACTION_SHIFTCONTROL_TOP:
+		case ACTION_SHIFTCONTROL_TR:
+		case ACTION_SHIFTCONTROL_RIGHT:
+		case ACTION_SHIFTCONTROL_BR:
+		case ACTION_SHIFTCONTROL_BOTTOM:
+		case ACTION_SHIFTCONTROL_BL:
+		case ACTION_SHIFTCONTROL_LEFT:
+		case ACTION_SHIFTCONTROL_TL:
+		case ACTION_SHIFTCONTROL_CW:
+		case ACTION_SHIFTCONTROL_CCW:
 			shift = 1;
-		case IDM_CONTROL_TOP:
-		case IDM_CONTROL_TR:
-		case IDM_CONTROL_RIGHT:
-		case IDM_CONTROL_BR:
-		case IDM_CONTROL_BOTTOM:
-		case IDM_CONTROL_BL:
-		case IDM_CONTROL_LEFT:
-		case IDM_CONTROL_TL:
-		case IDM_CONTROL_CW:
-		case IDM_CONTROL_CCW:
+		case ACTION_CONTROL_TOP:
+		case ACTION_CONTROL_TR:
+		case ACTION_CONTROL_RIGHT:
+		case ACTION_CONTROL_BR:
+		case ACTION_CONTROL_BOTTOM:
+		case ACTION_CONTROL_BL:
+		case ACTION_CONTROL_LEFT:
+		case ACTION_CONTROL_TL:
+		case ACTION_CONTROL_CW:
+		case ACTION_CONTROL_CCW:
 			{
 				POINT cursor, origin;
 
@@ -1119,47 +1128,56 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_SHIFTCONTROL_TOP :
-					IDM_CONTROL_TOP),
+					((shift) ? ACTION_SHIFTCONTROL_TOP :
+					ACTION_CONTROL_TOP),
 					shift, TRUE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_DEC:
+		case ACTION_INCREMENT:
+			IncrementPuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_DECREMENT:
 			if (DecrementPuzzle(&widget)) {
 				SizePuzzle(&widget);
 				(void) InvalidateRect(hWnd, NULL, TRUE);
 			}
 			break;
-		case IDM_INC:
-			IncrementPuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_PERIOD3:
-		case IDM_PERIOD4:
-		case IDM_BOTH:
+		case ACTION_PERIOD3:
+		case ACTION_PERIOD4:
+		case ACTION_BOTH:
 			PeriodModePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_PERIOD3);
+				(int) LOWORD(wParam) - ACTION_PERIOD3);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1254,6 +1272,9 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face5", (char *) "*oct.faceColor5", XrmoptionSepArg, NULL},
 	{(char *) "-face6", (char *) "*oct.faceColor6", XrmoptionSepArg, NULL},
 	{(char *) "-face7", (char *) "*oct.faceColor7", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*oct.delay", XrmoptionSepArg, NULL},
+	{(char *) "-sound", (char *) "*oct.sound", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nosound", (char *) "*oct.sound", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-fn", (char *) "*oct.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*oct.font", XrmoptionSepArg, NULL},
 	{(char *) "-size", (char *) "*oct.size", XrmoptionSepArg, NULL},
@@ -1272,27 +1293,29 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(oct, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(oct, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
 static void
-FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
+PieceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int size = cbs->value, mode;
 	Boolean orient, sticky, practice;
 
-	XtVaGetValues(oct,
+	XtVaGetValues(puzzle,
 		XtNsize, &oldSize,
 		XtNmode, &mode,
 		XtNorient, &orient,
@@ -1301,7 +1324,7 @@ FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	if (sticky)
 		XmScaleSetValue(w, oldSize);
 	else if (oldSize != size) {
-		XtVaSetValues(oct,
+		XtVaSetValues(puzzle,
 			XtNsize, size, NULL);
 		oldSize = size;
 		movesDsp = 0;
@@ -1320,12 +1343,12 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 	Boolean orient, sticky, practice;
 
 	if (cbs->set) {
-		XtVaGetValues(oct,
+		XtVaGetValues(puzzle,
 			XtNsize, &size,
 			XtNorient, &orient,
 			XtNsticky, &sticky,
 			XtNpractice, &practice, NULL);
-		XtVaSetValues(oct,
+		XtVaSetValues(puzzle,
 			XtNmode, mode + PERIOD3, NULL);
 		movesDsp = 0;
 		(void) sprintf(buff, "%d", movesDsp);
@@ -1342,12 +1365,12 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int size, mode;
 	Boolean orient = cbs->set, sticky, practice;
 
-	XtVaGetValues(oct,
+	XtVaGetValues(puzzle,
 		XtNsize, &size,
 		XtNmode, &mode,
 		XtNsticky, &sticky,
 		XtNpractice, &practice, NULL);
-	XtVaSetValues(oct,
+	XtVaSetValues(puzzle,
 		XtNorient, orient, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1363,12 +1386,12 @@ StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int size, mode;
 	Boolean orient, sticky = cbs->set, practice;
 
-	XtVaGetValues(oct,
+	XtVaGetValues(puzzle,
 		XtNsize, &size,
 		XtNmode, &mode,
 		XtNorient, &orient,
 		XtNpractice, &practice, NULL);
-	XtVaSetValues(oct,
+	XtVaSetValues(puzzle,
 		XtNsticky, sticky, NULL);
 	if ((sticky && size != 4) || (!sticky && oldSize != 4)) {
 		if (sticky)
@@ -1377,7 +1400,7 @@ StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 			size = oldSize;
 		if (size <= MAXFACETS)
 			XmScaleSetValue(facets, size);
-		XtVaSetValues(oct,
+		XtVaSetValues(puzzle,
 			XtNsize, size, NULL);
 	}
 	movesDsp = 0;
@@ -1395,10 +1418,10 @@ PracticeToggle(Widget w, XtPointer clientData,
 	int size, mode;
 	Boolean orient, sticky, practice = cbs->set;
 
-	XtVaSetValues(oct,
+	XtVaSetValues(puzzle,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
-	XtVaGetValues(oct,
+	XtVaGetValues(puzzle,
 		XtNsize, &size,
 		XtNmode, &mode,
 		XtNorient, &orient,
@@ -1417,19 +1440,19 @@ PracticeToggle(Widget w, XtPointer clientData,
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(oct, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(oct, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1521,6 +1544,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1530,8 +1554,9 @@ main(int argc, char **argv)
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
 	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientizeString, stickyString;
 	XmString incrementString, decrementString;
+	XmString orientizeString, stickyString;
+	XmString speedString, slowString, soundString;
 	unsigned int i;
 #endif
 
@@ -1541,34 +1566,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) oct_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) oct_bits,
-			oct_width, oct_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1585,7 +1583,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1599,33 +1598,43 @@ main(int argc, char **argv)
 	clearString = XmStringCreateSimple((char *) "Clear");
 	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "(Solve)");
-	orientizeString = XmStringCreateSimple((char *) "Orientize");
-	stickyString = XmStringCreateSimple((char *) "StickY");
+	solveString = XmStringCreateSimple((char *) "(Auto-Solve)");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
+	stickyString = XmStringCreateSimple((char *) "StickY");
+	speedString = XmStringCreateSimple((char *) ">Speed");
+	slowString = XmStringCreateSimple((char *) "<Slow");
+	soundString = XmStringCreateSimple((char *) "@Sound");
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
-		XmVaPUSHBUTTON, stickyString, 'Y', NULL, NULL,
 		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
 		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, stickyString, 'Y', NULL, NULL,
+		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
+		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
+		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
 	XmStringFree(undoString);
 	XmStringFree(redoString);
 	XmStringFree(clearString);
 	XmStringFree(randomizeString);
-	XmStringFree(solveString);
 	XmStringFree(practiceString);
-	XmStringFree(orientizeString);
-	XmStringFree(stickyString);
+	XmStringFree(solveString);
 	XmStringFree(incrementString);
 	XmStringFree(decrementString);
+	XmStringFree(orientizeString);
+	XmStringFree(stickyString);
+	XmStringFree(speedString);
+	XmStringFree(slowString);
+	XmStringFree(soundString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1682,7 +1691,7 @@ main(int argc, char **argv)
 		XmNseparatorOn, False,
 		XmNsashWidth, 1,
 		XmNsashHeight, 1, NULL);
-	movesRowCol = XtVaCreateManagedWidget("Rowcol",
+	movesRowCol = XtVaCreateManagedWidget("movesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 2,
 		XmNorientation, XmHORIZONTAL,
@@ -1736,7 +1745,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	facetsRowCol = XtVaCreateManagedWidget("facetsRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1744,14 +1752,14 @@ main(int argc, char **argv)
 		XmNpacking, XmPACK_COLUMN, NULL);
 	facets = XtVaCreateManagedWidget("facets",
 		xmScaleWidgetClass, facetsRowCol,
-		XtVaTypedArg, XmNtitleString, XmRString, "Size", 6,
+		XtVaTypedArg, XmNtitleString, XmRString, "Size", 5,
 		XmNminimum, MINFACETS,
 		XmNmaximum, MAXFACETS,
 		XmNvalue, MINFACETS,
 		XmNshowValue, True,
 		XmNorientation, XmHORIZONTAL, NULL);
 	XtAddCallback(facets, XmNvalueChangedCallback,
-		(XtCallbackProc) FacetSlider, (XtPointer) NULL);
+		(XtCallbackProc) PieceSlider, (XtPointer) NULL);
 	labelRowCol = XtVaCreateManagedWidget("labelRowCol",
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	(void) XtVaCreateManagedWidget("PeriodText",
@@ -1775,10 +1783,10 @@ main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, switchRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	stickySwitch = XtVaCreateManagedWidget("Sticky",
 		xmToggleButtonWidgetClass, switchRowCol,
@@ -1794,24 +1802,53 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Oct! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-	oct = XtCreateManagedWidget("oct",
+	puzzle = XtCreateManagedWidget("oct",
 		octWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	oct = XtCreateManagedWidget("oct",
+	puzzle = XtCreateManagedWidget("oct",
 		octWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(oct, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(oct);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(oct), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(oct), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(oct),
-		XCreateFontCursor(XtDisplay(oct), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
