@@ -78,14 +78,14 @@ static void UndoPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 static void RedoPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 static void ClearPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 static void SolvePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
-static void SpeedPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
-static void SlowPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
-static void SoundPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 static void IncrementPuzzle(PanexWidget w,
 	XEvent *event, char **args, int nArgs);
 static void DecrementPuzzle(PanexWidget w,
 	XEvent *event, char **args, int nArgs);
 static void ModePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 static void EnterPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 static void LeavePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 
@@ -97,6 +97,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <Btn1Down>: Select()\n\
  <Btn1Motion>: Motion()\n\
  <Btn1Up>: Release()\n\
@@ -108,11 +113,6 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>r: Redo()\n\
  <KeyPress>c: Clear()\n\
  <KeyPress>s: Solve()\n\
- <KeyPress>0x2E: Speed()\n\
- <KeyPress>0x3E: Speed()\n\
- <KeyPress>0x3C: Slow()\n\
- <KeyPress>0x2C: Slow()\n\
- <KeyPress>@: Sound()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>d: Decrement()\n\
  <KeyPress>m: Mode()\n\
@@ -211,13 +211,16 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(PanexWidget, panex.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(PanexWidget, panex.started),
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNcheat, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(PanexWidget, panex.cheat),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(PanexWidget, panex.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(PanexWidget, panex.select), XtRCallback, (caddr_t) NULL}
 };
@@ -302,10 +305,10 @@ loadFont(PanexWidget w)
 		}
 	}
 	if (w->panex.fontInfo) {
-		w->panex.letterOffset.x = XTextWidth(w->panex.fontInfo, "8", 1)
-			/ 2;
-		w->panex.letterOffset.y = w->panex.fontInfo->max_bounds.ascent
-			/ 2;
+		w->panex.letterOffset.x =
+			XTextWidth(w->panex.fontInfo, "8", 1) / 2;
+		w->panex.letterOffset.y =
+			w->panex.fontInfo->max_bounds.ascent / 2;
 	} else
 #endif
 	{
@@ -910,7 +913,7 @@ DrawFrame(PanexWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoTiles(PanexWidget w)
 {
-	setPuzzle(w, PUZZLE_IGNORE);
+	setPuzzle(w, ACTION_IGNORE);
 }
 
 int
@@ -1221,10 +1224,10 @@ GetTiles(PanexWidget w)
 	(void) fscanf(fp, "%d", &tiles);
 	if (tiles >= MINTILES) {
 		for (i = w->panex.tiles; i < tiles; i++) {
-			setPuzzle(w, PUZZLE_INC);
+			setPuzzle(w, ACTION_INCREMENT);
 		}
 		for (i = w->panex.tiles; i > tiles; i--) {
-			setPuzzle(w, PUZZLE_DEC);
+			setPuzzle(w, ACTION_DECREMENT);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: tiles ");
@@ -1243,7 +1246,7 @@ GetTiles(PanexWidget w)
 	(void) fscanf(fp, "%d", &mode);
 	if (mode >= HANOI && mode <= PANEX) {
 		if (w->panex.mode != mode) {
-			setPuzzle(w, PUZZLE_MODE);
+			setPuzzle(w, ACTION_MODE);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: mode ");
@@ -1266,7 +1269,7 @@ GetTiles(PanexWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
@@ -1359,7 +1362,7 @@ UndoTiles(PanexWidget w)
 			DISPLAY_WARNING(buf2);
 			free(buf2);
 		} else {
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 		}
 	}
 }
@@ -1388,7 +1391,7 @@ RedoTiles(PanexWidget w)
 			DISPLAY_WARNING(buf2);
 			free(buf2);
 		} else {
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 		}
 	}
 }
@@ -1402,7 +1405,7 @@ ClearTiles(PanexWidget w)
 	ResetTiles(w);
 	DrawSlots(w, 0);
 	DrawAllTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
@@ -1413,7 +1416,7 @@ SolveTiles(PanexWidget w)
 	ResetTiles(w);
 	DrawSlots(w, 0);
 	DrawAllTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 	SolveTilesFromStart(w);
 }
 
@@ -1424,7 +1427,7 @@ SpeedTiles(PanexWidget w)
 	if (w->panex.delay < 0)
 		w->panex.delay = 0;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -1433,7 +1436,7 @@ SlowTiles(PanexWidget w)
 {
 	w->panex.delay += 5;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -1446,7 +1449,7 @@ SoundTiles(PanexWidget w)
 static void
 IncrementTiles(PanexWidget w)
 {
-	setPuzzle(w, PUZZLE_INC);
+	setPuzzle(w, ACTION_INCREMENT);
 }
 
 static void
@@ -1454,13 +1457,13 @@ DecrementTiles(PanexWidget w)
 {
 	if (w->panex.tiles <= MINTILES)
 		return;
-	setPuzzle(w, PUZZLE_DEC);
+	setPuzzle(w, ACTION_DECREMENT);
 }
 
 static void
 ModeTiles(PanexWidget w)
 {
-	setPuzzle(w, PUZZLE_MODE);
+	setPuzzle(w, ACTION_MODE);
 }
 
 int
@@ -1471,7 +1474,7 @@ MovePuzzle(PanexWidget w, int fromStack, int fromPosition, int toStack,
 
 	if ((toPosition = MoveTile(w, fromStack, fromPosition, toStack,
 			fast)) >= 0) {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, fromStack, toStack);
 		flushMoves(w, &redo, FALSE);
 	}
@@ -1826,48 +1829,47 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		w->panex.numSlices = ((w->panex.delay < MAXSLICES) ?
 			w->panex.delay + 1 : MAXSLICES);
 	}
-	if (w->panex.menu != -1) {
+	if (w->panex.menu != ACTION_IGNORE) {
 		int menu = w->panex.menu;
 
-		w->panex.menu = -1;
+		w->panex.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetTiles(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WriteTiles(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoTiles(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoTiles(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearTiles(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_SOLVE:
 			SolveTiles(w);
 			break;
-		case MENU_MODE:
+		case ACTION_MODE:
 			ModeTiles(w);
 			break;
-		case MENU_INCREMENT:
+		case ACTION_INCREMENT:
 			IncrementTiles(w);
 			break;
-		case MENU_DECREMENT:
+		case ACTION_DECREMENT:
 			DecrementTiles(w);
 			break;
-		case MENU_SPEED:
+		case ACTION_SPEED:
 			SpeedTiles(w);
 			break;
-		case MENU_SLOW:
+		case ACTION_SLOW:
 			SlowTiles(w);
 			break;
-		case MENU_SOUND:
+		case ACTION_SOUND:
 			SoundTiles(w);
 			break;
-
 		default:
 			break;
 		}
@@ -2085,7 +2087,7 @@ HidePuzzle(PanexWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2205,7 +2207,7 @@ ReleasePuzzle(PanexWidget w
 				w->panex.currentPosition, toStack,
 				NORMAL)) >= 0) {
 			if (w->panex.previousStack != toStack) {
-				setPuzzle(w, PUZZLE_MOVED);
+				setPuzzle(w, ACTION_MOVED);
 				if (w->panex.previousStack >= 0) {
 					setMove(&undo, w->panex.previousStack,
 						toStack);
@@ -2216,11 +2218,11 @@ ReleasePuzzle(PanexWidget w
 					flushMoves(w, &redo, FALSE);
 				}
 				if (CheckSolved(w)) {
-					setPuzzle(w, PUZZLE_SOLVED);
+					setPuzzle(w, ACTION_SOLVED);
 				}
 			}
 		} else if (toPosition == -1) {
-			setPuzzle(w, PUZZLE_BLOCKED);
+			setPuzzle(w, ACTION_BLOCKED);
 			if (w->panex.previousStack != -1) {
 				if ((toPosition = MoveTile(w,
 						w->panex.currentStack,
@@ -2229,7 +2231,7 @@ ReleasePuzzle(PanexWidget w
 						NORMAL)) >= 0) {
 			if (w->panex.previousStack != toStack &&
 					w->panex.previousStack != w->panex.currentStack) {
-					setPuzzle(w, PUZZLE_MOVED);
+					setPuzzle(w, ACTION_MOVED);
 			if (w->panex.previousStack >= 0) {
 				setMove(&undo,
 					w->panex.previousStack,
@@ -2242,13 +2244,13 @@ ReleasePuzzle(PanexWidget w
 				flushMoves(w, &redo, FALSE);
 			}
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 					}
 				}
 			}
 		} else {
-			setPuzzle(w, PUZZLE_ILLEGAL);
+			setPuzzle(w, ACTION_ILLEGAL);
 			if (w->panex.previousStack != -1) {
 				toPosition = MoveTile(w, w->panex.currentStack,
 				w->panex.currentPosition,
@@ -2272,7 +2274,7 @@ ClearPuzzleMaybe(PanexWidget w
 #ifdef HAVE_MOTIF
 	else {
 		/* Check if one really wants to destroy current state. */
-		setPuzzle(w, PUZZLE_CLEAR_QUERY);
+		setPuzzle(w, ACTION_CLEAR_QUERY);
 	}
 #endif
 }
