@@ -53,6 +53,7 @@
 #define XtNmenu ((char *) "menu")
 #define XtNstart ((char *) "start")
 #define XtNcheat ((char *) "cheat")
+#define XtNpixmapSize ((char *) "pixmapSize")
 #define XtNselectCallback ((char *) "selectCallback")
 #define XtCMono ((char *) "Mono")
 #define XtCDelay ((char *) "Delay")
@@ -63,6 +64,7 @@
 #define XtCUserName ((char *) "UserName")
 #define XtCScoreFile ((char *) "ScoreFile")
 #define XtCMenu ((char *) "Menu")
+#define XtCPixmapSize ((char *) "PixmapSize")
 
 typedef struct _PanexClassRec *PanexWidgetClass;
 
@@ -74,37 +76,34 @@ typedef struct {
 } panexCallbackStruct;
 #endif
 
-#define PUZZLE_HIDE (-8)
-#define PUZZLE_CLEAR_QUERY (-7)
-#define PUZZLE_RESTORE (-6)
-#define PUZZLE_RESET (-5)
-#define PUZZLE_ILLEGAL (-4)
-#define PUZZLE_BLOCKED (-3)
-#define PUZZLE_SPACE (-2)
-#define PUZZLE_IGNORE (-1)
-#define PUZZLE_MOVED 0
-#define PUZZLE_SOLVED 1
-#define PUZZLE_DEC 2
-#define PUZZLE_INC 3
-#define PUZZLE_MODE 4
-#define PUZZLE_COMPUTED 5
-#define PUZZLE_UNDO 6
-#define PUZZLE_REDO 7
-#define PUZZLE_SPEED 8
-
-#define MENU_GET 0
-#define MENU_WRITE 1
-#define MENU_EXIT 2
-#define MENU_UNDO 3
-#define MENU_REDO 4
-#define MENU_CLEAR 5
-#define MENU_SOLVE 6
-#define MENU_MODE 7
-#define MENU_INCREMENT 8
-#define MENU_DECREMENT 9
-#define MENU_SPEED 10
-#define MENU_SLOW 11
-#define MENU_SOUND 12
+#define ACTION_GET 100
+#define ACTION_WRITE 101
+#define ACTION_EXIT 102
+#define ACTION_HIDE 103
+#define ACTION_CLEAR_QUERY 104
+#define ACTION_RESTORE 105
+#define ACTION_RESET 106
+#define ACTION_ILLEGAL 107
+#define ACTION_BLOCKED 108
+#define ACTION_SPACE 109
+#define ACTION_MOVED 110
+#define ACTION_SOLVED 111
+#define ACTION_COMPUTED 112
+#define ACTION_UNDO 200
+#define ACTION_REDO 201
+#define ACTION_CLEAR 202
+#define ACTION_SOLVE 203
+#define ACTION_MODE 204
+#define ACTION_INCREMENT 205
+#define ACTION_DECREMENT 206
+#define ACTION_SPEED 207
+#define ACTION_SLOW 208
+#define ACTION_SOUND 209
+#define ACTION_DESCRIPTION 900
+#define ACTION_FEATURES 901
+#define ACTION_REFERENCES 902
+#define ACTION_ABOUT 903
+#define ACTION_IGNORE 999
 
 #define MINTILES 1
 #define DEFAULTTILES 10
@@ -120,6 +119,8 @@ typedef struct {
 #define DEFAULTREVERSE FALSE
 #define DEFAULTDELAY 10
 
+#define BOOL_STRING(a) ((a==0)?"false":"true")
+
 typedef struct _PanexRec *PanexWidget;
 
 #endif /* _XtPanex_h */
