@@ -16,7 +16,7 @@
 #  Dubrovsky.
 #  Break ability taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -44,7 +44,6 @@
 #include "PanexP.h"
 
 static Boolean SolvingFlag = False;
-/* TODO Interruptability for Windows */
 #ifdef JMP
 static Boolean AbortSolvingFlag = False;
 static jmp_buf solve_env;
@@ -227,7 +226,7 @@ SolveHanoiMode(PanexWidget w)
 #endif
 	SolvingFlag = False;
 	w->panex.cheat = True; /* Assume the worst. */
-	setPuzzle(w, PUZZLE_COMPUTED);
+	setPuzzle(w, ACTION_COMPUTED);
 }
 
 #ifdef DEBUG
@@ -1338,7 +1337,7 @@ SolvePanexMode(PanexWidget w)
 #endif
 	SolvingFlag = False;
 	w->panex.cheat = True; /* Assume the worst. */
-	setPuzzle(w, PUZZLE_COMPUTED);
+	setPuzzle(w, ACTION_COMPUTED);
 #ifdef DEBUG
 	counter = 0;
 #endif
