@@ -23,15 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/11/15 X/Windows
-  Version 5.3: 96/04/30 Xt/Motif
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "PanexP.h"
-#include "wpanex.h"
 #define TITLE "wpanex"
 
 static PanexRec widget;
@@ -61,8 +55,8 @@ static PanexRec widget;
 #include <Xm/ToggleB.h>
 #include <Xm/ToggleBG.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -72,9 +66,19 @@ static PanexRec widget;
 #include "Panex.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/panex.xpm"
+#include "pixmaps/panex.t.xpm"
+#include "pixmaps/panex.p.xpm"
+#include "pixmaps/panex.s.xpm"
+#include "pixmaps/panex.m.xpm"
+#include "pixmaps/panex.l.xpm"
+#include "pixmaps/panex.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?panex_t_xpm:panex_p_xpm):\
+(((s)<=24)?panex_s_xpm:panex_m_xpm)):\
+(((s)<=48)?panex_l_xpm:panex_xpm))
 #endif
-#include "icons/panex.xbm"
+#include "pixmaps/panex.xbm"
+#define DEFINE_XBM (char *) panex_bits, panex_width, panex_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -86,7 +90,7 @@ static PanexRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Panex Version 7.3\n"
+"Panex Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -203,9 +207,9 @@ static const char referencesHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord panexRecord[MAXMODES][MAXTILES - MINTILES + 1];
+static PuzzleRecord puzzleRecord[MAXMODES][MAXTILES - MINTILES + 1];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -233,8 +237,8 @@ static char fileName[FILENAMELEN];
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, panex;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -260,11 +264,11 @@ PrintRecords(void)
 	(void) printf("PANEX TILES USER            MOVES\n");
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-			if (panexRecord[mode][i].score > 0)
+			if (puzzleRecord[mode][i].score > 0)
 				(void) printf("%5d%6d %-12s%9d\n",
 					mode, i + 1,
-					panexRecord[mode][i].name,
-					panexRecord[mode][i].score);
+					puzzleRecord[mode][i].name,
+					puzzleRecord[mode][i].score);
 		}
 }
 #endif
@@ -276,8 +280,8 @@ InitRecords(void)
 
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-			panexRecord[mode][i].score = NEVER;
-			(void) strncpy(panexRecord[mode][i].name, NOACCESS,
+			puzzleRecord[mode][i].score = NEVER;
+			(void) strncpy(puzzleRecord[mode][i].name, NOACCESS,
 				USERNAMELEN);
 		}
 }
@@ -331,10 +335,10 @@ ReadRecords(void)
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
 			(void) fscanf(fp, "%d %s\n", &n, userName);
-			if (n <= panexRecord[mode][i].score ||
-					panexRecord[mode][i].score <= NEVER) {
-				panexRecord[mode][i].score = n;
-				(void) strncpy(panexRecord[mode][i].name,
+			if (n <= puzzleRecord[mode][i].score ||
+					puzzleRecord[mode][i].score <= NEVER) {
+				puzzleRecord[mode][i].score = n;
+				(void) strncpy(puzzleRecord[mode][i].name,
 					userName, USERNAMELEN);
 			}
 		}
@@ -355,7 +359,7 @@ WriteRecords(void)
 		return;
 	}
 	{
-#ifdef HAVE_FCNTL_H
+#if HAVE_FCNTL_H
 		int lfd;
 		char lockFile[FILENAMELEN];
 
@@ -378,8 +382,8 @@ WriteRecords(void)
 		for (mode = 0; mode < MAXMODES; mode++) {
 			for (i = 0; i < MAXTILES - MINTILES + 1; i++)
 				(void) fprintf(fp, "%d %s\n",
-					panexRecord[mode][i].score,
-					panexRecord[mode][i].name);
+					puzzleRecord[mode][i].score,
+					puzzleRecord[mode][i].name);
 			(void) fprintf(fp, "\n");
 		}
 #if HAVE_FCNTL_H
@@ -397,11 +401,11 @@ PrintRecord(int tiles, int mode)
 
 	if (tiles > MAXTILES)
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	else if (panexRecord[j][i].score <= NEVER) {
+	else if (puzzleRecord[j][i].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			panexRecord[j][i].score, panexRecord[j][i].name);
+			puzzleRecord[j][i].score, puzzleRecord[j][i].name);
 
 	}
 #ifdef HAVE_MOTIF
@@ -426,7 +430,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(panex), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -436,11 +440,12 @@ HandleSolved(int counter, int tiles, int mode)
 {
 	int i = tiles - MINTILES, j = mode - HANOI;
 
-	if (tiles <= MAXTILES && (counter < panexRecord[j][i].score ||
-			panexRecord[j][i].score <= NEVER)) {
+	if (tiles <= MAXTILES && (counter < puzzleRecord[j][i].score ||
+			puzzleRecord[j][i].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		panexRecord[j][i].score = counter;
-		(void) strncpy(panexRecord[j][i].name, userNameDsp, USERNAMELEN);
+		puzzleRecord[j][i].score = counter;
+		(void) strncpy(puzzleRecord[j][i].name, userNameDsp,
+			USERNAMELEN);
 		WriteRecords();
 		PrintRecord(tiles, mode);
 		return True;
@@ -499,7 +504,7 @@ Widget w
 			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
 		XmScaleSetValue(tile, tiles);
 		XmToggleButtonSetState(modes[mode - HANOI], True, False);
-		XtVaGetValues(panex, XtNdelay, &delay, NULL);
+		XtVaGetValues(puzzle, XtNdelay, &delay, NULL);
 		XmScaleSetValue(speed, MAXSPEED + MINSPEED - delay - 1);
 	}
 #endif
@@ -565,7 +570,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, panexCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -575,40 +580,40 @@ CallbackPuzzle(Widget w, caddr_t clientData, panexCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_CLEAR_QUERY:
+	case ACTION_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(clearDialog);
 #else
-		XtVaSetValues(panex, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_RESTORE:
-	case PUZZLE_RESET:
+	case ACTION_RESTORE:
+	case ACTION_RESET:
 		movesDsp = 0;
 		break;
-	case PUZZLE_ILLEGAL:
+	case ACTION_ILLEGAL:
 		(void) strncpy(messageDsp, "Illegal move", MESSAGELEN);
 		break;
-	case PUZZLE_BLOCKED:
+	case ACTION_BLOCKED:
 		(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 		break;
-	case PUZZLE_SPACE:
+	case ACTION_SPACE:
 #if 0
 		/* Too annoying */
 		(void) strncpy(messageDsp, "Spaces can not slide",
-			MESSAGELEN));
+			MESSAGELEN);
 #endif
 		break;
-	case PUZZLE_IGNORE:
+	case ACTION_IGNORE:
 		(void) strncpy(messageDsp, "Clear to start",
 			MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (cheat)
 			(void) sprintf(messageDsp,
 				"No cheating %s!!", userNameDsp);
@@ -620,22 +625,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, panexCallbackStruct *callData)
 				MESSAGELEN);
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_DEC:
-		movesDsp = 0;
-		tiles--;
-		PrintRecord(tiles, mode);
-#ifdef WINVER
-		w->panex.tiles = tiles;
-#else
-		XtVaSetValues(w, XtNtiles, tiles, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(tile, tiles);
-		if (tiles >= MAXTILES)
-			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
-#endif
-#endif
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
 		break;
-	case PUZZLE_INC:
+	case ACTION_INCREMENT:
 		movesDsp = 0;
 		tiles++;
 		PrintRecord(tiles, mode);
@@ -650,7 +643,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, panexCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_MODE:
+	case ACTION_DECREMENT:
+		movesDsp = 0;
+		tiles--;
+		PrintRecord(tiles, mode);
+#ifdef WINVER
+		w->panex.tiles = tiles;
+#else
+		XtVaSetValues(w, XtNtiles, tiles, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(tile, tiles);
+		if (tiles >= MAXTILES)
+			XtVaSetValues(tile, XmNmaximum, tiles, NULL);
+#endif
+#endif
+		break;
+	case ACTION_MODE:
 		movesDsp = 0;
 		mode = (mode == PANEX) ? HANOI : PANEX;
 		PrintRecord(tiles, mode);
@@ -664,23 +672,20 @@ CallbackPuzzle(Widget w, caddr_t clientData, panexCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
-		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
 #ifdef HAVE_MOTIF
-	case PUZZLE_SPEED:
+	case ACTION_SPEED:
 		{
 			int oldDelay, delay;
 
-			XtVaGetValues(panex, XtNdelay, &delay, NULL);
+			XtVaGetValues(puzzle, XtNdelay, &delay, NULL);
 			oldDelay = delay;
 			if (delay > MAXSPEED - MINSPEED)
 				delay = MAXSPEED - MINSPEED;
@@ -795,86 +800,86 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		break;
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_MODE:
+		case ACTION_MODE:
 			(void) ModePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DEC:
-		case IDM_INC:
-			((LOWORD(wParam) == IDM_DEC) ?
+		case ACTION_INCREMENT:
+		case ACTION_DECREMENT:
+			((LOWORD(wParam) == ACTION_DECREMENT) ?
 				DecrementPuzzle(&widget) :
 				IncrementPuzzle(&widget));
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SPEED:
+		case ACTION_SPEED:
 			SpeedPuzzle(&widget);
 			break;
-		case IDM_SLOW:
+		case ACTION_SLOW:
 			SlowPuzzle(&widget);
 			break;
-		case IDM_SOUND:
+		case ACTION_SOUND:
 			SoundPuzzle(&widget);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
-			break;
-		case IDM_DESCRIPTION:
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -985,7 +990,7 @@ static void
 CallbackPuzzleClear(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(panex, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 	}
 }
 
@@ -994,12 +999,12 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int tiles = cbs->value, mode, old;
 
-	XtVaGetValues(panex,
+	XtVaGetValues(puzzle,
 		XtNtiles, &old,
 		XtNmode, &mode, NULL);
 	if (old == tiles)
 		return;
-	XtVaSetValues(panex,
+	XtVaSetValues(puzzle,
 		XtNtiles, tiles, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1014,12 +1019,12 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 {
 	int tiles, old;
 
-	XtVaGetValues(panex,
+	XtVaGetValues(puzzle,
 		XtNtiles, &tiles,
 		XtNmode, &old, NULL);
 	if (old == mode)
 		return;
-	XtVaSetValues(panex,
+	XtVaSetValues(puzzle,
 		XtNmode, mode + HANOI, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1034,29 +1039,29 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
-	XtVaGetValues(panex,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(panex, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(panex, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(panex, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1112,6 +1117,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1148,10 +1154,11 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
-	Widget movesRowCol, radioRowCol, tileRowCol, messageRowCol;
+	Widget movesRowCol, radioRowCol, tilesRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
@@ -1162,40 +1169,12 @@ main(int argc, char **argv)
 #endif
 
 	progDsp = argv[0];
-
 	topLevel = XtInitialize(argv[0], "Panex",
 		options, XtNumber(options), &argc, argv);
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) panex_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) panex_bits,
-			panex_width, panex_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1212,7 +1191,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1224,14 +1204,15 @@ main(int argc, char **argv)
 	undoString = XmStringCreateSimple((char *) "Undo");
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
-	solveString = XmStringCreateSimple((char *) "Solve");
-	mode = XmStringCreateSimple((char *) "Mode");
+	solveString = XmStringCreateSimple((char *) "Auto-Solve");
+	mode = XmStringCreateSimple((char *) "Mode (Hanoi/Panex)");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
@@ -1357,7 +1338,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	radioRowCol = XtVaCreateManagedWidget("movesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1371,13 +1351,13 @@ main(int argc, char **argv)
 		XtAddCallback(modes[i], XmNvalueChangedCallback,
 			(XtCallbackProc) ModeToggle, (XtPointer) i);
 	}
-	tileRowCol = XtVaCreateManagedWidget("tileRowCol",
+	tilesRowCol = XtVaCreateManagedWidget("tilesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
 	tile = XtVaCreateManagedWidget("tile",
-		xmScaleWidgetClass, tileRowCol,
+		xmScaleWidgetClass, tilesRowCol,
 		XtVaTypedArg, XmNtitleString, XmRString, "Size", 5,
 		XmNminimum, MINTILES,
 		XmNmaximum, MAXTILES,
@@ -1386,9 +1366,9 @@ main(int argc, char **argv)
 		XmNorientation, XmHORIZONTAL, NULL);
 	XtAddCallback(tile, XmNvalueChangedCallback,
 		(XtCallbackProc) TileSlider, (XtPointer) NULL);
-	speed = XtVaCreateManagedWidget("speed",
-		xmScaleWidgetClass, tileRowCol,
-		XtVaTypedArg, XmNtitleString, XmRString, "Animation Speed", 8,
+	speed = XtVaCreateManagedWidget("Animation Speed",
+		xmScaleWidgetClass, tilesRowCol,
+		XtVaTypedArg, XmNtitleString, XmRString, "Animation Speed", 16,
 		XmNminimum, MINSPEED,
 		XmNmaximum, MAXSPEED,
 		XmNvalue, MAXSPEED - DEFAULTDELAY,
@@ -1396,30 +1376,57 @@ main(int argc, char **argv)
 		XmNorientation, XmHORIZONTAL, NULL);
 	XtAddCallback(speed, XmNvalueChangedCallback,
 		(XtCallbackProc) SpeedSlider, (XtPointer) NULL);
-
 	messageRowCol = XtVaCreateManagedWidget("messageRowCol",
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Panex!",
 		xmLabelWidgetClass, messageRowCol, NULL);
-
-	panex = XtCreateManagedWidget("panex",
+	puzzle = XtCreateManagedWidget("panex",
 		panexWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	panex = XtCreateManagedWidget("panex",
+	puzzle = XtCreateManagedWidget("panex",
 		panexWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(panex, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(panex);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(panex), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(panex), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(panex),
-		XCreateFontCursor(XtDisplay(panex), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
