@@ -27,6 +27,7 @@
 
 #include "file.h"
 #include "rngs.h"
+#include "sound.h"
 #include "PyraminxP.h"
 
 #ifdef WINVER
@@ -107,6 +108,12 @@ static void Period3ModePuzzle(PyraminxWidget w,
 	XEvent *event, char **args, int nArgs);
 static void BothModePuzzle(PyraminxWidget w,
 	XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(PyraminxWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(PyraminxWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(PyraminxWidget w,
+	XEvent *event, char **args, int nArgs);
 static void EnterPuzzle(PyraminxWidget w,
 	XEvent *event, char **args, int nArgs);
 static void LeavePuzzle(PyraminxWidget w,
@@ -142,6 +149,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Up: MoveTop()\n\
@@ -232,6 +244,9 @@ static XtActionsRec actionsListPuzzle[] =
 	{(char *) "Period3", (XtActionProc) Period3ModePuzzle},
 	{(char *) "Both", (XtActionProc) BothModePuzzle},
 	{(char *) "Sticky", (XtActionProc) StickyModePuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -277,7 +292,13 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
 	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
 	 XtOffset(PyraminxWidget, pyraminx.delay),
-	 XtRString, (caddr_t) "300"},
+	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(PyraminxWidget, pyraminx.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(PyraminxWidget, pyraminx.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(PyraminxWidget, pyraminx.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -310,13 +331,16 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(PyraminxWidget, pyraminx.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(PyraminxWidget, pyraminx.started),
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNcheat, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(PyraminxWidget, pyraminx.cheat),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(PyraminxWidget, pyraminx.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(PyraminxWidget, pyraminx.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -530,7 +554,7 @@ Boolean
 CheckSolved(PyraminxWidget w)
 {
 	int face, position;
-	PyraminxLoc test;
+	PyraminxLoc test = {0, FALSE};
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < w->pyraminx.sizeSize; position++) {
@@ -899,7 +923,7 @@ DrawFrame(PyraminxWidget w, Boolean focus)
 static void
 MoveNoPieces(PyraminxWidget w)
 {
-	setPuzzle(w, PUZZLE_ILLEGAL);
+	setPuzzle(w, ACTION_ILLEGAL);
 }
 
 static void
@@ -1469,7 +1493,7 @@ MoveControlCb(PyraminxWidget w, int face, int direction, int style)
 					crd.diagonal = crd.row + crd.column;
 					MovePieces(w, face, crd, direction, style);
 				}
-				setPuzzle(w, PUZZLE_CONTROL);
+				setPuzzle(w, ACTION_CONTROL);
 		} else		/* (style == PERIOD3) */
 			for (i = 0; i < 2; i++) {
 				if (direction == TR || direction == BL) {
@@ -1483,7 +1507,7 @@ MoveControlCb(PyraminxWidget w, int face, int direction, int style)
 					crd.diagonal = crd.row + crd.column + faceOnView;
 					MovePieces(w, face, crd, direction, style);
 				}
-				setPuzzle(w, PUZZLE_CONTROL);
+				setPuzzle(w, ACTION_CONTROL);
 			}
 	} else {
 		for (i = 0; i < w->pyraminx.size; i++) {
@@ -1505,7 +1529,7 @@ MoveControlCb(PyraminxWidget w, int face, int direction, int style)
 				crd.diagonal = crd.column + crd.row + faceOnView;
 				MovePieces(w, face, crd, direction, style);
 			}
-			setPuzzle(w, PUZZLE_CONTROL);
+			setPuzzle(w, ACTION_CONTROL);
 		}
 	}
 }
@@ -1521,8 +1545,13 @@ MovePuzzle(PyraminxWidget w, int face, int position, int direction,
 
 		ToCRD(w, face, position, &crd);
 		MovePieces(w, face, crd, direction, style);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 	}
+#ifdef USE_SOUND
+	if (w->pyraminx.sound) {
+		playSound((char *) MOVESOUND);
+	}
+#endif
 	setMove(&undo, face, position, direction, style, control);
 	flushMoves(w, &redo, FALSE);
 }
@@ -1877,7 +1906,7 @@ MovePuzzleInput(PyraminxWidget w, int x, int y, int direction, int shift,
 	control = (control) ? 1 : 0;
 	MovePuzzle(w, face, ToPosition(w, crd), direction, style, control);
 	if (!control && CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1925,7 +1954,7 @@ ResetPieces(PyraminxWidget w)
 		}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	w->pyraminx.currentFace = PUZZLE_IGNORE;
+	w->pyraminx.currentFace = IGNORE_DIR;
 	w->pyraminx.started = False;
 }
 
@@ -2009,10 +2038,10 @@ GetPieces(PyraminxWidget w)
 	(void) fscanf(fp, "%d", &size);
 	if (size >= MINFACETS) {
 		for (i = w->pyraminx.size; i < size; i++) {
-			setPuzzle(w, PUZZLE_INC);
+			setPuzzle(w, ACTION_INCREMENT);
 		}
 		for (i = w->pyraminx.size; i > size; i--) {
-			setPuzzle(w, PUZZLE_DEC);
+			setPuzzle(w, ACTION_DECREMENT);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: size ");
@@ -2031,13 +2060,13 @@ GetPieces(PyraminxWidget w)
 	(void) fscanf(fp, "%d", &mode);
 	switch (mode) {
 	case PERIOD2:
-		setPuzzle(w, PUZZLE_PERIOD2);
+		setPuzzle(w, ACTION_PERIOD2);
 		break;
 	case PERIOD3:
-		setPuzzle(w, PUZZLE_PERIOD3);
+		setPuzzle(w, ACTION_PERIOD3);
 		break;
 	case BOTH:
-		setPuzzle(w, PUZZLE_BOTH);
+		setPuzzle(w, ACTION_BOTH);
 		break;
 	default:
 		stringCat(&buf1, name, " corrupted: mode ");
@@ -2057,17 +2086,17 @@ GetPieces(PyraminxWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sticky);
 	if (w->pyraminx.sticky != (Boolean) sticky) {
-		setPuzzle(w, PUZZLE_STICKY);
+		setPuzzle(w, ACTION_STICKY);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->pyraminx.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->pyraminx.practice != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
@@ -2075,13 +2104,14 @@ GetPieces(PyraminxWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: size %d, mode %d, sticky %d, orient %d",
-		name, size, mode, sticky, orient);
-	(void) printf(", practice %d, moves %d.\n", practice, moves);
+	(void) printf("%s: size %d, mode %d, sticky %s, orient %s",
+		name, size, mode, BOOL_STRING(sticky), BOOL_STRING(orient));
+	(void) printf(", practice %s, moves %d.\n",
+		BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->pyraminx.cheat = True; /* Assume the worst. */
@@ -2151,7 +2181,7 @@ static void
 UndoPieces(PyraminxWidget w)
 {
 	if (madeMoves(&undo) &&
-			w->pyraminx.currentFace <= PUZZLE_IGNORE) {
+			w->pyraminx.currentFace <= IGNORE_DIR) {
 		int face, position, direction, style, control;
 
 		getMove(&undo,
@@ -2167,9 +2197,9 @@ UndoPieces(PyraminxWidget w)
 
 			ToCRD(w, face, position, &crd);
 			MovePieces(w, face, crd, direction, style);
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -2179,7 +2209,7 @@ static void
 RedoPieces(PyraminxWidget w)
 {
 	if (madeMoves(&redo) &&
-			w->pyraminx.currentFace <= PUZZLE_IGNORE) {
+			w->pyraminx.currentFace <= IGNORE_DIR) {
 		int face, position, direction, style, control;
 
 		getMove(&redo,
@@ -2193,9 +2223,9 @@ RedoPieces(PyraminxWidget w)
 
 			ToCRD(w, face, position, &crd);
 			MovePieces(w, face, crd, direction, style);
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -2204,17 +2234,17 @@ RedoPieces(PyraminxWidget w)
 static void
 ClearPieces(PyraminxWidget w)
 {
-	if (w->pyraminx.currentFace > PUZZLE_IGNORE)
+	if (w->pyraminx.currentFace > IGNORE_DIR)
 		return;
 	ResetPieces(w);
 	DrawAllPieces(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
 PracticePieces(PyraminxWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
@@ -2223,7 +2253,7 @@ RandomizePieces(PyraminxWidget w)
 	int randomDirection, face, position, style;
 	int big = w->pyraminx.sizeSize * 3 + NRAND(2);
 
-	if (w->pyraminx.currentFace > PUZZLE_IGNORE)
+	if (w->pyraminx.currentFace > IGNORE_DIR)
 		return;
 	w->pyraminx.cheat = False;
 	if (big > 1000)
@@ -2232,7 +2262,7 @@ RandomizePieces(PyraminxWidget w)
 		PracticePieces(w);
 	if (w->pyraminx.sticky)
 		big /= 3;
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -2269,33 +2299,33 @@ RandomizePieces(PyraminxWidget w)
 				style = w->pyraminx.mode;
 		}
 		MovePuzzle(w, face, position, randomDirection, style, FALSE);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	setPuzzle(w, PUZZLE_RANDOMIZE);
+	setPuzzle(w, ACTION_RANDOMIZE);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
 static void
 SolvePieces(PyraminxWidget w)
 {
-	if (CheckSolved(w) && w->pyraminx.currentFace > PUZZLE_IGNORE)
+	if (CheckSolved(w) && w->pyraminx.currentFace > IGNORE_DIR)
 		return;
 	if ((w->pyraminx.size == 2 || w->pyraminx.size == 3) &&
 			w->pyraminx.mode == PERIOD3)
 		SolveSomePieces(w);
 	else {
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
 IncrementPieces(PyraminxWidget w)
 {
-	setPuzzle(w, PUZZLE_INC);
+	setPuzzle(w, ACTION_INCREMENT);
 }
 
 static Boolean
@@ -2303,20 +2333,46 @@ DecrementPieces(PyraminxWidget w)
 {
 	if (w->pyraminx.size <= MINFACETS)
 		return False;
-	setPuzzle(w, PUZZLE_DEC);
+	setPuzzle(w, ACTION_DECREMENT);
 	return True;
 }
 
 static void
 OrientizePieces(PyraminxWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 StickyPieces(PyraminxWidget w)
 {
-	setPuzzle(w, PUZZLE_STICKY);
+	setPuzzle(w, ACTION_STICKY);
+}
+
+static void
+SpeedPieces(PyraminxWidget w)
+{
+	w->pyraminx.delay -= 5;
+	if (w->pyraminx.delay < 0)
+		w->pyraminx.delay = 0;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowPieces(PyraminxWidget w)
+{
+	w->pyraminx.delay += 5;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundPieces(PyraminxWidget w)
+{
+	w->pyraminx.sound = !w->pyraminx.sound;
 }
 
 #ifdef WINVER
@@ -2545,47 +2601,56 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		ResizePuzzle(w);
 		redraw = True;
 	}
-	if (w->pyraminx.menu != -1) {
+	if (w->pyraminx.menu != ACTION_IGNORE) {
 		int menu = w->pyraminx.menu;
 
-		w->pyraminx.menu = -1;
+		w->pyraminx.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetPieces(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WritePieces(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoPieces(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoPieces(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearPieces(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizePieces(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePieces(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizePieces(w);
+			break;
+		case ACTION_SOLVE:
 			SolvePieces(w);
 			break;
-		case MENU_ORIENTIZE:
-			OrientizePieces(w);
-			break;
-		case MENU_STICKY:
-			StickyPieces(w);
-			break;
-		case MENU_INCREMENT:
+		case ACTION_INCREMENT:
 			IncrementPieces(w);
 			break;
-		case MENU_DECREMENT:
+		case ACTION_DECREMENT:
 			(void) DecrementPieces(w);
 			break;
+		case ACTION_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case ACTION_STICKY:
+			StickyPieces(w);
+			break;
+		case ACTION_SPEED:
+			SpeedPieces(w);
+			break;
+		case ACTION_SLOW:
+			SlowPieces(w);
+			break;
+		case ACTION_SOUND:
+			SoundPieces(w);
+			break;
 		default:
 			break;
 		}
@@ -2758,7 +2823,7 @@ HidePuzzle(PyraminxWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2785,7 +2850,7 @@ SelectPuzzle(PyraminxWidget w
 			DrawTriangle(w, w->pyraminx.currentFace,
 				w->pyraminx.currentPosition, TRUE);
 	} else
-		w->pyraminx.currentFace = PUZZLE_IGNORE;
+		w->pyraminx.currentFace = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -2809,7 +2874,7 @@ ReleasePuzzle(PyraminxWidget w
 #endif
 
 
-	if (w->pyraminx.currentFace <= PUZZLE_IGNORE)
+	if (w->pyraminx.currentFace <= IGNORE_DIR)
 		return;
 	DrawTriangle(w, w->pyraminx.currentFace, w->pyraminx.currentPosition,
 		FALSE);
@@ -2836,15 +2901,15 @@ ReleasePuzzle(PyraminxWidget w
 				w->pyraminx.currentPosition,
 				direction, style, (control) ? 1 : 0);
 			if (!control && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else if (count == 2) {
 			/* count == 3 too annoying */
-			setPuzzle(w, PUZZLE_AMBIGUOUS);
+			setPuzzle(w, ACTION_AMBIGUOUS);
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->pyraminx.currentFace = PUZZLE_IGNORE;
+	w->pyraminx.currentFace = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -2857,7 +2922,7 @@ PracticePuzzleMaybe(PyraminxWidget w
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -2884,7 +2949,7 @@ RandomizePuzzleMaybe(PyraminxWidget w
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
@@ -3069,7 +3134,7 @@ Period2ModePuzzle(PyraminxWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_PERIOD2);
+	setPuzzle(w, ACTION_PERIOD2);
 }
 
 static void
@@ -3077,7 +3142,7 @@ Period3ModePuzzle(PyraminxWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_PERIOD3);
+	setPuzzle(w, ACTION_PERIOD3);
 }
 
 static
@@ -3086,10 +3151,49 @@ BothModePuzzle(PyraminxWidget w
 , XEvent *event, char **args, int nArgs
 )
 {
-	setPuzzle(w, PUZZLE_BOTH);
+	setPuzzle(w, ACTION_BOTH);
 }
 #endif
 
+#ifndef WINVER
+static
+#endif
+void
+SpeedPuzzle(PyraminxWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SlowPuzzle(PyraminxWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SoundPuzzle(PyraminxWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundPieces(w);
+}
+
 #ifndef WINVER
 static
 #endif
@@ -3122,7 +3226,7 @@ LeavePuzzle(PyraminxWidget w
 void
 PeriodModePuzzle(PyraminxWidget w, const int mode)
 {
-	setPuzzle(w, mode + PUZZLE_PERIOD2);
+	setPuzzle(w, mode + ACTION_PERIOD2);
 }
 
 #else
