@@ -98,7 +98,6 @@ static PyraminxSLoc EdgeMate[MAXFACES][3] =
 };
 
 static Boolean SolvingFlag = False;
-/* TODO Interruptability for Windows */
 #ifdef JMP
 static Boolean AbortSolvingFlag = False;
 static jmp_buf solve_env;
@@ -610,7 +609,7 @@ From Puzzle It Out: Cubes, Groups and Puzzles
 void
 SolveSomePieces(PyraminxWidget w)
 {
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 	if (SolvingFlag)
 		return;
 #ifdef JMP
@@ -631,5 +630,5 @@ SolveSomePieces(PyraminxWidget w)
 #endif
 	SolvingFlag = False;
 	w->pyraminx.cheat = True; /* Assume the worst. */
-	setPuzzle(w, PUZZLE_COMPUTED);
+	setPuzzle(w, ACTION_COMPUTED);
 }
