@@ -23,19 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/12/15 X/Windows
-  Version 5: 95/10/04 Xt/Motif
-  Version 4: 94/05/31 Xt
-  Version 3: 93/04/01 Motif
-  Version 2: 92/01/29 XView
-  Version 1: 91/03/19 SunView
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "PyraminxP.h"
-#include "wpyraminx.h"
 #define TITLE "wpyraminx"
 
 static PyraminxRec widget;
@@ -65,8 +55,8 @@ static PyraminxRec widget;
 #include <Xm/ToggleB.h>
 #include <Xm/ToggleBG.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -76,9 +66,19 @@ static PyraminxRec widget;
 #include "Pyraminx.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/pyraminx.xpm"
+#include "pixmaps/pyraminx.t.xpm"
+#include "pixmaps/pyraminx.p.xpm"
+#include "pixmaps/pyraminx.s.xpm"
+#include "pixmaps/pyraminx.m.xpm"
+#include "pixmaps/pyraminx.l.xpm"
+#include "pixmaps/pyraminx.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?pyraminx_t_xpm:pyraminx_p_xpm):\
+(((s)<=24)?pyraminx_s_xpm:pyraminx_m_xpm)):\
+(((s)<=48)?pyraminx_l_xpm:pyraminx_xpm))
 #endif
-#include "icons/pyraminx.xbm"
+#include "pixmaps/pyraminx.xbm"
+#define DEFINE_XBM (char *) pyraminx_bits, pyraminx_width, pyraminx_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -90,7 +90,7 @@ static PyraminxRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Pyraminx Version 7.3\n"
+"Pyraminx Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -102,10 +102,11 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3} {color}] [-{border|bd} {color}]\n"
-"[-delay msecs] [-{font|fn} {fontname}]\n"
-"[-{size {int} | sticky}] [-{mode {int} | both}]\n"
-"[-[no]orient] [-[no]practice] [-userName {string}]\n"
-"[-scoreFile {filename}] [-scores] [-version]\n"
+"[-delay msecs] [-[no]sound] [-moveSound {filename}]\n"
+"[-{font|fn} {fontname}] [-{size {int} | sticky}]\n"
+"[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n"
+"[-userName {string}] [-scoreFile {filename}] [-scores]\n"
+"[-version]\n"
 };
 #endif
 
@@ -164,7 +165,7 @@ static const char featuresHelp[] = {
 "\n"
 "Press \"U\" or \"u\" keys to undo a move.\n"
 "\n"
-"Press \"R\" or \"r\" keys to undo a move.\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
 "\n"
 "Press \"C\" or \"c\" keys to clear the puzzle.\n"
 "\n"
@@ -176,7 +177,7 @@ static const char featuresHelp[] = {
 "getting all the faces to be the same color.  To do this "
 "one has to get the lines to be oriented in\n"
 "the same direction, this only matters with center "
-"\"facets\", if at all (i.e. those that are not on a\n"
+"pieces, if at all (i.e. those that are not on a\n"
 "corner or edge).  This does add complexity so there are "
 "2 sets of records.\n"
 "\n"
@@ -194,9 +195,15 @@ static const char featuresHelp[] = {
 "Beware, the \"Sticky\" mode is a hack and much could be done "
 "to improve its look.\n"
 "\n"
-"Press \"I\" or \"i\" keys to increase the number of \"facets\".\n"
+"Press \"I\" or \"i\" keys to increase the number of pieces.\n"
 "\n"
-"Press \"D\" or \"d\" keys to decrease the number of \"facets\".\n"
+"Press \"D\" or \"d\" keys to decrease the number of pieces.\n"
+"\n"
+"Press \">\" or \".\" keys to speed up the movement of pieces.\n"
+"\n"
+"Press \"<\" or \",\" keys to slow down the movement of pieces.\n"
+"\n"
+"Press \"@\" key to toggle the sound.\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -255,9 +262,9 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord pyraminxRecord[MAXMODES][2][MAXFACETS - MINFACETS + 2];
+static PuzzleRecord puzzleRecord[MAXMODES][2][MAXFACETS - MINFACETS + 2];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -267,7 +274,7 @@ static int oldSize;
 #define MINSPEED 1
 #define MAXSPEED 50
 #define MULTSPEED 20
-static Widget moves, record, message, modes[MAXMODES], orientSwitch;
+static Widget moves, record, message, modes[MAXMODES], orientizeSwitch;
 static Widget stickySwitch, practiceSwitch, facets, speed;
 static char buff[21];
 static const char *modeString[] =
@@ -289,8 +296,8 @@ static Boolean randomized = False;
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, pyraminx;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -317,11 +324,11 @@ PrintRecords(void)
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++) {
-				if (pyraminxRecord[mode][orient][i].score > 0)
+				if (puzzleRecord[mode][orient][i].score > 0)
 					(void) printf("%4d%7d%6d %-12s%9d\n",
 						mode + 2, orient, i + 1,
-						pyraminxRecord[mode][orient][i].name,
-						pyraminxRecord[mode][orient][i].score);
+						puzzleRecord[mode][orient][i].name,
+						puzzleRecord[mode][orient][i].score);
 			}
 }
 #endif
@@ -334,8 +341,8 @@ InitRecords(void)
 	for (mode = 0; mode < MAXMODES; mode++)
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++) {
-				pyraminxRecord[mode][orient][i].score = NEVER;
-				(void) strncpy(pyraminxRecord[mode][orient][i].name,
+				puzzleRecord[mode][orient][i].score = NEVER;
+				(void) strncpy(puzzleRecord[mode][orient][i].name,
 					NOACCESS, USERNAMELEN);
 			}
 }
@@ -390,10 +397,10 @@ ReadRecords(void)
 		for (orient = 0; orient < 2; orient++)
 			for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++) {
 				(void) fscanf(fp, "%d %s\n", &n, userName);
-				if (n <= pyraminxRecord[mode][orient][i].score ||
-						pyraminxRecord[mode][orient][i].score <= NEVER) {
-					pyraminxRecord[mode][orient][i].score = n;
-					(void) strncpy(pyraminxRecord[mode][orient][i].name,
+				if (n <= puzzleRecord[mode][orient][i].score ||
+						puzzleRecord[mode][orient][i].score <= NEVER) {
+					puzzleRecord[mode][orient][i].score = n;
+					(void) strncpy(puzzleRecord[mode][orient][i].name,
 						userName, USERNAMELEN);
 				}
 			}
@@ -438,8 +445,8 @@ WriteRecords(void)
 			for (orient = 0; orient < 2; orient++) {
 				for (i = 0; i <= MAXFACETS - MINFACETS + 1; i++)
 					(void) fprintf(fp, "%d %s\n",
-					pyraminxRecord[mode][orient][i].score,
-					pyraminxRecord[mode][orient][i].name);
+					puzzleRecord[mode][orient][i].score,
+					puzzleRecord[mode][orient][i].name);
 				(void) fprintf(fp, "\n");
 			}
 			(void) fprintf(fp, "\n");
@@ -464,12 +471,12 @@ PrintRecord(int size, int mode,
 		(void) strncpy(recordDsp, "practice", MESSAGELEN);
 	} else if (!sticky && size > MAXFACETS) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (pyraminxRecord[i][j][k].score <= NEVER) {
+	} else if (puzzleRecord[i][j][k].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			pyraminxRecord[i][j][k].score,
-			pyraminxRecord[i][j][k].name);
+			puzzleRecord[i][j][k].score,
+			puzzleRecord[i][j][k].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -503,7 +510,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(pyraminx), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -516,17 +523,17 @@ HandleSolved(int counter, int size, int mode, Boolean orient, Boolean sticky)
 	int k = (sticky) ? MAXFACETS - MINFACETS + 1 : size - MINFACETS;
 
 	if ((sticky || size <= MAXFACETS) &&
-			(counter < pyraminxRecord[i][j][k].score ||
-			pyraminxRecord[i][j][k].score <= NEVER)) {
+			(counter < puzzleRecord[i][j][k].score ||
+			puzzleRecord[i][j][k].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		pyraminxRecord[i][j][k].score = counter;
-		(void) strncpy(pyraminxRecord[i][j][k].name, userNameDsp,
+		puzzleRecord[i][j][k].score = counter;
+		(void) strncpy(puzzleRecord[i][j][k].name, userNameDsp,
 			USERNAMELEN);
 		if (size < 4 || mode == PERIOD2 || (orient &&
-				(counter < pyraminxRecord[i][!j][k].score ||
-				pyraminxRecord[i][!j][k].score <= NEVER))) {
-			pyraminxRecord[i][!j][k].score = counter;
-			(void) strncpy(pyraminxRecord[i][!j][k].name, userNameDsp,
+				(counter < puzzleRecord[i][!j][k].score ||
+				puzzleRecord[i][!j][k].score <= NEVER))) {
+			puzzleRecord[i][!j][k].score = counter;
+			(void) strncpy(puzzleRecord[i][!j][k].name, userNameDsp,
 				USERNAMELEN);
 		}
 		WriteRecords();
@@ -591,7 +598,7 @@ Widget w
 		XtVaSetValues(facets, XmNmaximum, size, NULL);
 	XmScaleSetValue(facets, size);
 	XmToggleButtonSetState(modes[mode - PERIOD2], True, False);
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(stickySwitch, sticky, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 	{
@@ -674,7 +681,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -684,22 +691,22 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(pyraminx, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(pyraminx, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -711,7 +718,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice",
 				MESSAGELEN);
@@ -722,15 +729,15 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_AMBIGUOUS:
+	case ACTION_AMBIGUOUS:
 		(void) strncpy(messageDsp, "Ambiguous move",
 			MESSAGELEN);
 		break;
-	case PUZZLE_ILLEGAL:
+	case ACTION_ILLEGAL:
 		if (practice || randomized)
 			(void) strncpy(messageDsp, "Illegal move",
 				MESSAGELEN);
@@ -738,13 +745,13 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 			(void) strncpy(messageDsp,
 				"Randomize to start", MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		return;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -759,7 +766,10 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 		SET_STARTED(w, False);
 		randomized = False;
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		randomized = False;
 		practice = !practice;
@@ -779,7 +789,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 #ifdef WINVER
@@ -791,38 +801,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 			XtNstart, False, NULL);
 #endif
 		break;
-	case PUZZLE_DEC:
-		if (!sticky) {
-			movesDsp = 0;
-			size--;
-			oldSize = size;
-			PrintRecord(size, mode, orient, sticky, practice);
-#ifdef WINVER
-			w->pyraminx.size = size;
-#else
-			XtVaSetValues(w, XtNsize, size, NULL);
-#ifdef HAVE_MOTIF
-			XmScaleSetValue(facets, size);
-			if (size >= MAXFACETS)
-				XtVaSetValues(facets, XmNmaximum, size, NULL);
-#endif
-#endif
-		}
-		break;
-	case PUZZLE_ORIENTIZE:
-		movesDsp = 0;
-		orient = !orient;
-		PrintRecord(size, mode, orient, sticky, practice);
-#ifdef WINVER
-		w->pyraminx.orient = orient;
-#else
-		XtVaSetValues(w, XtNorient, orient, NULL);
-#ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
-#endif
-#endif
-		break;
-	case PUZZLE_INC:
+	case ACTION_INCREMENT:
 		if (!sticky) {
 			movesDsp = 0;
 			size++;
@@ -840,11 +819,42 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 		}
 		break;
-	case PUZZLE_PERIOD2:
-	case PUZZLE_PERIOD3:
-	case PUZZLE_BOTH:
+	case ACTION_DECREMENT:
+		if (!sticky) {
+			movesDsp = 0;
+			size--;
+			oldSize = size;
+			PrintRecord(size, mode, orient, sticky, practice);
+#ifdef WINVER
+			w->pyraminx.size = size;
+#else
+			XtVaSetValues(w, XtNsize, size, NULL);
+#ifdef HAVE_MOTIF
+			XmScaleSetValue(facets, size);
+			if (size >= MAXFACETS)
+				XtVaSetValues(facets, XmNmaximum, size, NULL);
+#endif
+#endif
+		}
+		break;
+	case ACTION_ORIENTIZE:
 		movesDsp = 0;
-		mode = reason - PUZZLE_PERIOD2 + PERIOD2;
+		orient = !orient;
+		PrintRecord(size, mode, orient, sticky, practice);
+#ifdef WINVER
+		w->pyraminx.orient = orient;
+#else
+		XtVaSetValues(w, XtNorient, orient, NULL);
+#ifdef HAVE_MOTIF
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
+#endif
+#endif
+		break;
+	case ACTION_PERIOD2:
+	case ACTION_PERIOD3:
+	case ACTION_BOTH:
+		movesDsp = 0;
+		mode = reason - ACTION_PERIOD2 + PERIOD2;
 		PrintRecord(size, mode, orient, sticky, practice);
 #ifdef WINVER
 		w->pyraminx.mode = mode;
@@ -855,7 +865,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_STICKY:
+	case ACTION_STICKY:
 		movesDsp = 0;
 		sticky = !sticky;
 		if (sticky)
@@ -876,13 +886,35 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
-		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
+	case ACTION_REDO:
+		movesDsp++;
+		SET_STARTED(w, True);
+		break;
+#ifdef HAVE_MOTIF
+	case ACTION_SPEED:
+		{
+			int oldDelay, delay;
+
+			XtVaGetValues(puzzle, XtNdelay, &delay, NULL);
+			oldDelay = delay;
+			if (delay > MAXSPEED - MINSPEED)
+				delay = MAXSPEED - MINSPEED;
+			if (delay < 0)
+				delay = 0;
+			if (delay != oldDelay) {
+				XtVaSetValues(w,
+					XtNdelay, delay, NULL);
+			}
+			XmScaleSetValue(speed,
+				MAXSPEED + MINSPEED - delay - 1);
+		}
+		return;
+#endif
+
 	}
 	PrintStatus(messageDsp, movesDsp
 #ifndef HAVE_MOTIF
@@ -1009,87 +1041,87 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
-				break;
-		case IDM_PRACTICE:
+			break;
+		case ACTION_PRACTICE:
 			PracticePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_STICKY:
+		case ACTION_STICKY:
 			(void) StickyModePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SHIFT_TOP:
-		case IDM_SHIFT_TR:
-		case IDM_SHIFT_RIGHT:
-		case IDM_SHIFT_BOTTOM:
-		case IDM_SHIFT_BL:
-		case IDM_SHIFT_LEFT:
-		case IDM_SHIFT_CW:
-		case IDM_SHIFT_CCW:
+		case ACTION_SHIFT_TOP:
+		case ACTION_SHIFT_TR:
+		case ACTION_SHIFT_RIGHT:
+		case ACTION_SHIFT_BOTTOM:
+		case ACTION_SHIFT_BL:
+		case ACTION_SHIFT_LEFT:
+		case ACTION_SHIFT_CW:
+		case ACTION_SHIFT_CCW:
 			shift = 1;
-		case IDM_TOP:
-		case IDM_TR:
-		case IDM_RIGHT:
-		case IDM_BOTTOM:
-		case IDM_BL:
-		case IDM_LEFT:
-		case IDM_CW:
-		case IDM_CCW:
+		case ACTION_TOP:
+		case ACTION_TR:
+		case ACTION_RIGHT:
+		case ACTION_BOTTOM:
+		case ACTION_BL:
+		case ACTION_LEFT:
+		case ACTION_CW:
+		case ACTION_CCW:
 			{
 				POINT cursor, origin;
 
@@ -1104,28 +1136,28 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_SHIFT_TOP : IDM_TOP),
+					((shift) ? ACTION_SHIFT_TOP : ACTION_TOP),
 					shift, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_SHIFTCONTROL_TOP:
-		case IDM_SHIFTCONTROL_TR:
-		case IDM_SHIFTCONTROL_RIGHT:
-		case IDM_SHIFTCONTROL_BOTTOM:
-		case IDM_SHIFTCONTROL_BL:
-		case IDM_SHIFTCONTROL_LEFT:
-		case IDM_SHIFTCONTROL_CW:
-		case IDM_SHIFTCONTROL_CCW:
+		case ACTION_SHIFTCONTROL_TOP:
+		case ACTION_SHIFTCONTROL_TR:
+		case ACTION_SHIFTCONTROL_RIGHT:
+		case ACTION_SHIFTCONTROL_BOTTOM:
+		case ACTION_SHIFTCONTROL_BL:
+		case ACTION_SHIFTCONTROL_LEFT:
+		case ACTION_SHIFTCONTROL_CW:
+		case ACTION_SHIFTCONTROL_CCW:
 			shift = 1;
-		case IDM_CONTROL_TOP:
-		case IDM_CONTROL_TR:
-		case IDM_CONTROL_RIGHT:
-		case IDM_CONTROL_BOTTOM:
-		case IDM_CONTROL_BL:
-		case IDM_CONTROL_LEFT:
-		case IDM_CONTROL_CW:
-		case IDM_CONTROL_CCW:
+		case ACTION_CONTROL_TOP:
+		case ACTION_CONTROL_TR:
+		case ACTION_CONTROL_RIGHT:
+		case ACTION_CONTROL_BOTTOM:
+		case ACTION_CONTROL_BL:
+		case ACTION_CONTROL_LEFT:
+		case ACTION_CONTROL_CW:
+		case ACTION_CONTROL_CCW:
 			{
 				POINT cursor, origin;
 
@@ -1139,47 +1171,56 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					cursor.x - origin.x,
 					cursor.y - origin.y,
 					(int) LOWORD(wParam) -
-					((shift) ? IDM_SHIFTCONTROL_TOP :
-					IDM_CONTROL_TOP),
+					((shift) ? ACTION_SHIFTCONTROL_TOP :
+					ACTION_CONTROL_TOP),
 					shift, TRUE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_DEC:
+		case ACTION_INCREMENT:
+			IncrementPuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_DECREMENT:
 			if (DecrementPuzzle(&widget)) {
 				SizePuzzle(&widget);
 				(void) InvalidateRect(hWnd, NULL, TRUE);
 			}
 			break;
-		case IDM_INC:
-			IncrementPuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_PERIOD2:
-		case IDM_PERIOD3:
-		case IDM_BOTH:
+		case ACTION_PERIOD2:
+		case ACTION_PERIOD3:
+		case ACTION_BOTH:
 			PeriodModePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_PERIOD2);
+				(int) LOWORD(wParam) - ACTION_PERIOD2);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1271,6 +1312,8 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face2", (char *) "*pyraminx.faceColor2", XrmoptionSepArg, NULL},
 	{(char *) "-face3", (char *) "*pyraminx.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-delay", (char *) "*pyraminx.delay", XrmoptionSepArg, NULL},
+	{(char *) "-sound", (char *) "*pyraminx.sound", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nosound", (char *) "*pyraminx.sound", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-fn", (char *) "*pyraminx.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*pyraminx.font", XrmoptionSepArg, NULL},
 	{(char *) "-size", (char *) "*pyraminx.size", XrmoptionSepArg, NULL},
@@ -1289,27 +1332,30 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(pyraminx, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(pyraminx, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
+
 static void
-FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
+PieceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int size = cbs->value, mode;
 	Boolean orient, sticky, practice;
 
-	XtVaGetValues(pyraminx,
+	XtVaGetValues(puzzle,
 		XtNsize, &oldSize,
 		XtNmode, &mode,
 		XtNorient, &orient,
@@ -1318,7 +1364,7 @@ FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 	if (sticky)
 		XmScaleSetValue(w, oldSize);
 	else if (oldSize != size) {
-		XtVaSetValues(pyraminx,
+		XtVaSetValues(puzzle,
 			XtNsize, size, NULL);
 		oldSize = size;
 		movesDsp = 0;
@@ -1337,12 +1383,12 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 	Boolean orient, sticky, practice;
 
 	if (cbs->set) {
-		XtVaGetValues(pyraminx,
+		XtVaGetValues(puzzle,
 			XtNsize, &size,
 			XtNorient, &orient,
 			XtNsticky, &sticky,
 			XtNpractice, &practice, NULL);
-		XtVaSetValues(pyraminx,
+		XtVaSetValues(puzzle,
 			XtNmode, mode + PERIOD2, NULL);
 		movesDsp = 0;
 		(void) sprintf(buff, "%d", movesDsp);
@@ -1359,12 +1405,12 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int size, mode;
 	Boolean orient = cbs->set, sticky, practice;
 
-	XtVaGetValues(pyraminx,
+	XtVaGetValues(puzzle,
 		XtNsize, &size,
 		XtNmode, &mode,
 		XtNsticky, &sticky,
 		XtNpractice, &practice, NULL);
-	XtVaSetValues(pyraminx,
+	XtVaSetValues(puzzle,
 		XtNorient, orient, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1380,12 +1426,12 @@ StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 	int size, mode;
 	Boolean orient, sticky = cbs->set, practice;
 
-	XtVaGetValues(pyraminx,
+	XtVaGetValues(puzzle,
 		XtNsize, &size,
 		XtNmode, &mode,
 		XtNorient, &orient,
 		XtNpractice, &practice, NULL);
-	XtVaSetValues(pyraminx,
+	XtVaSetValues(puzzle,
 		XtNsticky, sticky, NULL);
 	if ((sticky && size != 4) || (!sticky && oldSize != 4)) {
 		if (sticky)
@@ -1394,7 +1440,7 @@ StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 			size = oldSize;
 		if (size <= MAXFACETS)
 			XmScaleSetValue(facets, size);
-		XtVaSetValues(pyraminx,
+		XtVaSetValues(puzzle,
 			XtNsize, size, NULL);
 	}
 	movesDsp = 0;
@@ -1412,10 +1458,10 @@ PracticeToggle(Widget w, XtPointer clientData,
 	int size, mode;
 	Boolean orient, sticky, practice = cbs->set;
 
-	XtVaSetValues(pyraminx,
+	XtVaSetValues(puzzle,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
-	XtVaGetValues(pyraminx,
+	XtVaGetValues(puzzle,
 		XtNsize, &size,
 		XtNmode, &mode,
 		XtNorient, &orient,
@@ -1436,29 +1482,29 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MULTSPEED * (MAXSPEED + MINSPEED - cbs->value - 1), oldDelay;
 
-	XtVaGetValues(pyraminx,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(pyraminx, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(pyraminx, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(pyraminx, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1514,6 +1560,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1550,6 +1597,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1559,8 +1607,9 @@ main(int argc, char **argv)
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
 	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientizeString, stickyString;
 	XmString incrementString, decrementString;
+	XmString orientizeString, stickyString;
+	XmString speedString, slowString, soundString;
 	unsigned int i;
 #endif
 
@@ -1570,34 +1619,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) pyraminx_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) pyraminx_bits,
-			pyraminx_width, pyraminx_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1614,7 +1636,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1628,22 +1651,29 @@ main(int argc, char **argv)
 	clearString = XmStringCreateSimple((char *) "Clear");
 	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "Solve");
-	orientizeString = XmStringCreateSimple((char *) "Orientize");
-	stickyString = XmStringCreateSimple((char *) "StickY");
+	solveString = XmStringCreateSimple((char *) "Auto-Solve");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
+	stickyString = XmStringCreateSimple((char *) "StickY");
+	speedString = XmStringCreateSimple((char *) ">Speed");
+	slowString = XmStringCreateSimple((char *) "<Slow");
+	soundString = XmStringCreateSimple((char *) "@Sound");
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
-		XmVaPUSHBUTTON, stickyString, 'Y', NULL, NULL,
 		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
 		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, stickyString, 'Y', NULL, NULL,
+		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
+		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
+		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
 	XmStringFree(undoString);
 	XmStringFree(redoString);
@@ -1651,10 +1681,13 @@ main(int argc, char **argv)
 	XmStringFree(randomizeString);
 	XmStringFree(practiceString);
 	XmStringFree(solveString);
-	XmStringFree(orientizeString);
-	XmStringFree(stickyString);
 	XmStringFree(incrementString);
 	XmStringFree(decrementString);
+	XmStringFree(orientizeString);
+	XmStringFree(stickyString);
+	XmStringFree(speedString);
+	XmStringFree(slowString);
+	XmStringFree(soundString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1765,7 +1798,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	facetsRowCol = XtVaCreateManagedWidget("facetsRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1780,7 +1812,7 @@ main(int argc, char **argv)
 		XmNshowValue, True,
 		XmNorientation, XmHORIZONTAL, NULL);
 	XtAddCallback(facets, XmNvalueChangedCallback,
-		(XtCallbackProc) FacetSlider, (XtPointer) NULL);
+		(XtCallbackProc) PieceSlider, (XtPointer) NULL);
 	speed = XtVaCreateManagedWidget("speed",
 		xmScaleWidgetClass, facetsRowCol,
 		XtVaTypedArg, XmNtitleString, XmRString, "Animation Speed", 16,
@@ -1814,10 +1846,10 @@ main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, switchRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	stickySwitch = XtVaCreateManagedWidget("Sticky",
 		xmToggleButtonWidgetClass, switchRowCol,
@@ -1831,28 +1863,55 @@ main(int argc, char **argv)
 		(XtCallbackProc) PracticeToggle, (XtPointer) NULL);
 	messageRowCol = XtVaCreateManagedWidget("messageRowCol",
 		xmRowColumnWidgetClass, controlPanel, NULL);
-
 	message = XtVaCreateManagedWidget("Play Pyraminx! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-
-	pyraminx = XtCreateManagedWidget("pyraminx",
+	puzzle = XtCreateManagedWidget("pyraminx",
 		pyraminxWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	pyraminx = XtCreateManagedWidget("pyraminx",
+	puzzle = XtCreateManagedWidget("pyraminx",
 		pyraminxWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(pyraminx, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(pyraminx);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(pyraminx), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(pyraminx), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(pyraminx),
-		XCreateFontCursor(XtDisplay(pyraminx), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
