@@ -27,6 +27,7 @@
 
 #include "file.h"
 #include "rngs.h"
+#include "sound.h"
 #include "RubikP.h"
 #include "Rubik2dP.h"
 #include "Rubik3dP.h"
@@ -46,12 +47,12 @@
 
 static const char *faceColorString[MAXFACES] =
 {
-        "255 0 0",
-        "255 255 0",
-        "255 255 255",
-        "0 255 0",
-        "255 165 0",
-        "0 0 255"
+	"255 0 0",
+	"255 255 0",
+	"255 255 255",
+	"0 255 0",
+	"255 165 0",
+	"0 0 255"
 };
 
 static const char faceColorChar[MAXFACES] =
@@ -328,7 +329,7 @@ faceSizes(const RubikWidget w, const int face,
 int
 sizeFace(const RubikWidget w, const int face)
 {
-	int         sizeOfRow, sizeOfColumn;
+	int sizeOfRow, sizeOfColumn;
 
 	faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
 	return (sizeOfRow * sizeOfColumn);
@@ -337,7 +338,7 @@ sizeFace(const RubikWidget w, const int face)
 int
 sizeRow(const RubikWidget w, const int face)
 {
-	int         sizeOfRow, sizeOfColumn;
+	int sizeOfRow, sizeOfColumn;
 
 	faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
 	return sizeOfRow;
@@ -346,7 +347,7 @@ sizeRow(const RubikWidget w, const int face)
 static Boolean
 checkFaceSquare(RubikWidget w, int face)
 {
-	int         sizeOfRow, sizeOfColumn;
+	int sizeOfRow, sizeOfColumn;
 
 	faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
 	return (sizeOfRow == sizeOfColumn);
@@ -366,8 +367,8 @@ checkFaceSquare(RubikWidget w, int face)
 Boolean
 CheckSolved(const RubikWidget w, Boolean rotation)
 {
-	int         face, position;
-	RubikLoc    test = {0, FALSE};
+	int face, position;
+	RubikLoc test = {0, FALSE};
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < sizeFace(w, face); position++) {
@@ -387,13 +388,13 @@ CheckSolved(const RubikWidget w, Boolean rotation)
 void
 PrintCube(RubikWidget w)
 {
-	int         face, position, sizeOfRow, sizeOfColumn;
+	int face, position, sizeOfRow, sizeOfColumn;
 
 	for (face = 0; face < MAXFACES; face++) {
 		faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
 		for (position = 0; position < sizeOfRow * sizeOfColumn; position++) {
 			(void) printf("%d %d  ", w->rubik.cubeLoc[face][position].face,
-				  w->rubik.cubeLoc[face][position].rotation);
+				w->rubik.cubeLoc[face][position].rotation);
 			if (!((position + 1) % sizeOfRow))
 				(void) printf("\n");
 		}
@@ -405,12 +406,12 @@ PrintCube(RubikWidget w)
 void
 PrintRow(RubikWidget w, int orient, int size)
 {
-	int         i;
+	int i;
 
 	(void) printf("Row %d:\n", orient);
 	for (i = 0; i < size; i++)
 		(void) printf("%d %d  ", w->rubik.rowLoc[orient][i].face,
-			      w->rubik.rowLoc[orient][i].rotation);
+			w->rubik.rowLoc[orient][i].rotation);
 	(void) printf("\n");
 }
 
@@ -428,7 +429,7 @@ DrawSquare(const RubikWidget w, const int face, const int position, const int of
 void
 DrawAllPieces(const RubikWidget w)
 {
-	int         face, position;
+	int face, position;
 
 #ifdef HAVE_OPENGL
 	if (w->rubik.dim == 4) {
@@ -457,20 +458,20 @@ DrawFrame(const RubikWidget w, const Boolean focus)
 static void
 MoveNoPieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_ILLEGAL);
+	setPuzzle(w, ACTION_ILLEGAL);
 }
 
 static void
 RotateFace(RubikWidget w, int face, int direction)
 {
-	int         position, i, j, sizeOfRow, sizeOfColumn, sizeOnPlane;
-	RubikLoc   *faceLoc = NULL;
+	int position, i, j, sizeOfRow, sizeOfColumn, sizeOnPlane;
+	RubikLoc *faceLoc = NULL;
 
 	faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
 	sizeOnPlane = sizeOfRow * sizeOfColumn;
 	if ((faceLoc = (RubikLoc *) malloc(sizeOnPlane * sizeof (RubikLoc))) ==
-	    NULL) {
-        	DISPLAY_ERROR("Not enough memory for rubik face position info, exiting.");
+				NULL) {
+			DISPLAY_ERROR("Not enough memory for rubik face position info, exiting.");
 	}
 	/* Read Face */
 	for (position = 0; position < sizeOnPlane; position++)
@@ -517,7 +518,7 @@ SlideRC(RubikWidget w, int face, int direction, int h, int sizeOnOppAxis,
 			*newH = sizeOnOppAxis - 1 - h;
 			*reverse = False;
 		} else {	/* *newDirection == RIGHT ||
-			           *newDirection == LEFT */
+				   *newDirection == LEFT */
 			*newH = h;
 			*reverse = True;
 		}
@@ -531,7 +532,7 @@ SlideRC(RubikWidget w, int face, int direction, int h, int sizeOnOppAxis,
 			*newH = h;
 			*reverse = True;
 		} else {	/* *newDirection == RIGHT ||
-		                   *newDirection == LEFT */
+				   *newDirection == LEFT */
 			*newH = sizeOnOppAxis - 1 - h;
 			*reverse = False;
 		}
@@ -550,7 +551,7 @@ SlideRC(RubikWidget w, int face, int direction, int h, int sizeOnOppAxis,
 static void
 ReadRC(RubikWidget w, int face, int dir, int h, int orient, int size)
 {
-	int         g, sizeOfRow;
+	int g, sizeOfRow;
 
 	sizeOfRow = sizeRow(w, face);
 	if (dir == TOP || dir == BOTTOM)
@@ -566,7 +567,7 @@ ReadRC(RubikWidget w, int face, int dir, int h, int orient, int size)
 static void
 RotateRC(RubikWidget w, int rotate, int orient, int size)
 {
-	int         g;
+	int g;
 
 	for (g = 0; g < size; g++)
 		w->rubik.rowLoc[orient][g].rotation =
@@ -576,8 +577,8 @@ RotateRC(RubikWidget w, int rotate, int orient, int size)
 static void
 ReverseRC(RubikWidget w, int orient, int size)
 {
-	int         g;
-	RubikLoc    temp;
+	int g;
+	RubikLoc temp;
 
 	for (g = 0; g < size / 2; g++) {
 		temp = w->rubik.rowLoc[orient][size - 1 - g];
@@ -589,7 +590,7 @@ ReverseRC(RubikWidget w, int orient, int size)
 static void
 WriteRC(RubikWidget w, int face, int dir, int h, int orient, int size)
 {
-	int         g, position, sizeOfRow;
+	int g, position, sizeOfRow;
 
 	sizeOfRow = sizeRow(w, face);
 	if (dir == TOP || dir == BOTTOM) {
@@ -611,10 +612,10 @@ static void
 MovePieces(RubikWidget w, int face, int position, int direction,
 		Boolean control, int fast)
 {
-	int         newFace, newDirection, rotate;
-	int         h, k, newH = 0;
-	int         i, j, sizeOfRow, sizeOfColumn, sizeOnAxis, sizeOnOppAxis;
-	Boolean     reverse = FALSE;
+	int newFace, newDirection, rotate;
+	int h, k, newH = 0;
+	int i, j, sizeOfRow, sizeOfColumn, sizeOnAxis, sizeOnOppAxis;
+	Boolean reverse = FALSE;
 
 #ifdef HAVE_OPENGL
 	if (!control && fast != INSTANT && w->rubik.dim == 4) {
@@ -652,13 +653,13 @@ MovePieces(RubikWidget w, int face, int position, int direction,
 			RotateFace(w, rowToRotate[face][newDirection], HALF);
 		else if (direction == TOP || direction == RIGHT)
 			RotateFace(w, rowToRotate[face][newDirection], CCW);
-		else		/* direction == BOTTOM  || direction == LEFT */
+		else		/* direction == BOTTOM || direction == LEFT */
 			RotateFace(w, rowToRotate[face][newDirection], CW);
 	}
 	/* Slide rows */
 	ReadRC(w, face, direction, h, 0, sizeOnAxis);
 	if (w->rubik.degreeTurn == 180) {
-		int         sizeOnDepthAxis;
+		int sizeOnDepthAxis;
 
 		SlideRC(w, face, direction, h, sizeOnOppAxis,
 			&newFace, &newDirection, &newH, &rotate, &reverse);
@@ -686,7 +687,7 @@ MovePieces(RubikWidget w, int face, int position, int direction,
 				     (k % 2) ? sizeOnDepthAxis : sizeOnAxis);
 				if (k != MAXORIENT + 1)
 					ReverseRC(w, k - 1,
-						  (k % 2) ? sizeOnAxis : sizeOnDepthAxis);
+						(k % 2) ? sizeOnAxis : sizeOnDepthAxis);
 			}
 			WriteRC(w, newFace, newDirection, newH, k - 2,
 				(k % 2) ? sizeOnDepthAxis : sizeOnAxis);
@@ -723,7 +724,7 @@ static Boolean
 MoveAllPieces(RubikWidget w, const int face, const int position,
 		const int direction, const Boolean control, int fast)
 {
-	int  sizeOfRow, sizeOfColumn, sizeOnOppAxis, factor;
+	int sizeOfRow, sizeOfColumn, sizeOnOppAxis, factor;
 	Boolean newControl;
 
 	faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
@@ -735,7 +736,7 @@ MoveAllPieces(RubikWidget w, const int face, const int position,
 		factor = sizeOfRow;
 	}
 	w->rubik.degreeTurn = (checkFaceSquare(w,
-				   rowToRotate[face][direction])) ? 90 : 180;
+				rowToRotate[face][direction])) ? 90 : 180;
 	if (control || sizeOnOppAxis == 1) {
 		int k, newPosition, newFast;
 
@@ -765,9 +766,14 @@ void
 MovePuzzle(RubikWidget w, const int face, const int position,
 		const int direction, const Boolean control, int fast)
 {
-	setPuzzleMove(w, PUZZLE_MOVED, face, position, direction,
+	setPuzzleMove(w, ACTION_MOVED, face, position, direction,
 		MoveAllPieces(w, face, position, direction, control, fast),
 		fast);
+#ifdef USE_SOUND
+	if (w->rubik.sound) {
+		playSound((char *) MOVESOUND);
+	}
+#endif
 	setMove(&undo, face, position, direction, control);
 	flushMoves(w, &redo, FALSE);
 }
@@ -785,10 +791,10 @@ SelectPieces(RubikWidget w, const int x, const int y, int *face, int *position)
 {
 	if (w->rubik.dim == 2)
 		return SelectPieces2D((Rubik2DWidget) w, x, y,
-					   face, position);
+					face, position);
 	else if (w->rubik.dim == 3)
 		return SelectPieces3D((Rubik3DWidget) w, x, y,
-					   face, position);
+					face, position);
 	return False;
 }
 
@@ -796,8 +802,8 @@ static int
 CheckMoveDir(RubikWidget w, int face, int position1, int position2,
 		int *direction)
 {
-	int         count = 0;
-	int         column1, column2, row1, row2, sizeOfRow;
+	int count = 0;
+	int column1, column2, row1, row2, sizeOfRow;
 
 	sizeOfRow = sizeRow(w, face);
 	column1 = position1 % sizeOfRow;
@@ -837,7 +843,7 @@ PositionPieces(RubikWidget w, int x, int y, int *face, int *position, int *direc
 void
 MovePuzzleInput(RubikWidget w, int x, int y, int direction, int control)
 {
-	int         face, position;
+	int face, position;
 
 	if (!w->rubik.practice && !control && CheckSolved(w, w->rubik.orient)) {
 		MoveNoPieces(w);
@@ -848,27 +854,27 @@ MovePuzzleInput(RubikWidget w, int x, int y, int direction, int control)
 	control = (control) ? 1 : 0;
 	MovePuzzle(w, face, position, direction, control, NORMAL);
 	if (!control && CheckSolved(w, w->rubik.orient)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
 void
 ResetPieces(RubikWidget w)
 {
-	int         face, position, orient, sizeOfFace;
+	int face, position, orient, sizeOfFace;
 
 	for (face = 0; face < MAXFACES; face++) {
 		sizeOfFace = sizeFace(w, face);
 		if (w->rubik.cubeLoc[face])
 			free(w->rubik.cubeLoc[face]);
 		if (!(w->rubik.cubeLoc[face] = (RubikLoc *)
-		      malloc(sizeof (RubikLoc) * sizeOfFace))) {
+				malloc(sizeof (RubikLoc) * sizeOfFace))) {
 			DISPLAY_ERROR("Not enough memory, exiting.");
 		}
 		if (startLoc[face])
 			free(startLoc[face]);
 		if (!(startLoc[face] = (RubikLoc *)
-		      malloc(sizeof (RubikLoc) * sizeOfFace))) {
+				malloc(sizeof (RubikLoc) * sizeOfFace))) {
 			DISPLAY_ERROR("Not enough memory, exiting.");
 		}
 		for (position = 0; position < sizeOfFace; position++) {
@@ -880,22 +886,22 @@ ResetPieces(RubikWidget w)
 		if (w->rubik.rowLoc[orient])
 			free(w->rubik.rowLoc[orient]);
 		if (!(w->rubik.rowLoc[orient] = (RubikLoc *)
-		      malloc(sizeof (RubikLoc) * MAXMAXSIZE))) {
+				malloc(sizeof (RubikLoc) * MAXMAXSIZE))) {
 			DISPLAY_ERROR("Not enough memory, exiting.");
 		}
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
 	w->rubik.started = False;
-	w->rubik.currentFace = PUZZLE_IGNORE;
-	w->rubik.currentDirection = PUZZLE_IGNORE;
+	w->rubik.currentFace = IGNORE_DIR;
+	w->rubik.currentDirection = IGNORE_DIR;
 }
 
 static void
 GetPieces(RubikWidget w)
 {
-	FILE       *fp;
-	int         c, i, sizex, sizey, sizez, orient, practice, moves;
+	FILE *fp;
+	int c, i, sizex, sizey, sizez, orient, practice, moves;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -940,10 +946,10 @@ GetPieces(RubikWidget w)
 	(void) fscanf(fp, "%d", &sizex);
 	if (sizex >= MINFACETS) {
 		for (i = w->rubik.sizex; i < sizex; i++) {
-			setPuzzle(w, PUZZLE_INCX);
+			setPuzzle(w, ACTION_INCX);
 		}
 		for (i = w->rubik.sizex; i > sizex; i--) {
-			setPuzzle(w, PUZZLE_DECX);
+			setPuzzle(w, ACTION_DECX);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizex ");
@@ -962,10 +968,10 @@ GetPieces(RubikWidget w)
 	(void) fscanf(fp, "%d", &sizey);
 	if (sizey >= MINFACETS) {
 		for (i = w->rubik.sizey; i < sizey; i++) {
-			setPuzzle(w, PUZZLE_INCY);
+			setPuzzle(w, ACTION_INCY);
 		}
 		for (i = w->rubik.sizey; i > sizey; i--) {
-			setPuzzle(w, PUZZLE_DECY);
+			setPuzzle(w, ACTION_DECY);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizey ");
@@ -984,10 +990,10 @@ GetPieces(RubikWidget w)
 	(void) fscanf(fp, "%d", &sizez);
 	if (sizez >= MINFACETS) {
 		for (i = w->rubik.sizez; i < sizez; i++) {
-			setPuzzle(w, PUZZLE_INCZ);
+			setPuzzle(w, ACTION_INCZ);
 		}
 		for (i = w->rubik.sizez; i > sizez; i--) {
-			setPuzzle(w, PUZZLE_DECZ);
+			setPuzzle(w, ACTION_DECZ);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizez ");
@@ -1005,12 +1011,12 @@ GetPieces(RubikWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->rubik.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->rubik.practice != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
@@ -1018,14 +1024,15 @@ GetPieces(RubikWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 #ifdef WINVER
 	setStartPosition(w);
 #endif
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: sizex %d, sizey %d, sizez %d, orient %d, practice %d, moves %d.\n",
-		name, sizex, sizey, sizez, orient, practice, moves);
+	(void) printf("%s: sizex %d, sizey %d, sizez %d, orient %s, practice %s, moves %d.\n",
+		name, sizex, sizey, sizez,
+		BOOL_STRING(orient), BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->rubik.cheat = True; /* Assume the worst. */
@@ -1034,7 +1041,7 @@ GetPieces(RubikWidget w)
 static void
 WritePieces(RubikWidget w)
 {
-	FILE       *fp;
+	FILE *fp;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -1094,20 +1101,20 @@ static void
 UndoPieces(RubikWidget w)
 {
 	if (madeMoves(&undo)) {
-		int         face, position, direction, control;
+		int face, position, direction, control;
 
 		getMove(&undo,
 			&face, &position, &direction, &control);
-                setMove(&redo,
-                        face, position, direction, control);
+		setMove(&redo,
+			face, position, direction, control);
 		direction = (direction < MAXORIENT) ?
 			(direction + MAXORIENT / 2) % MAXORIENT :
 			3 * MAXORIENT - direction;
-		setPuzzleMove(w, PUZZLE_UNDO, face, position, direction,
+		setPuzzleMove(w, ACTION_UNDO, face, position, direction,
 			MoveAllPieces(w, face, position, direction, control,
 			DOUBLE), DOUBLE);
 		if (!control && CheckSolved(w, w->rubik.orient)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 	}
 }
@@ -1116,17 +1123,17 @@ static void
 RedoPieces(RubikWidget w)
 {
 	if (madeMoves(&redo)) {
-		int         face, position, direction, control;
+		int face, position, direction, control;
 
 		getMove(&redo,
 			&face, &position, &direction, &control);
-                setMove(&undo,
-                        face, position, direction, control);
-		setPuzzleMove(w, PUZZLE_REDO, face, position, direction,
+		setMove(&undo,
+			face, position, direction, control);
+		setPuzzleMove(w, ACTION_REDO, face, position, direction,
 			MoveAllPieces(w, face, position, direction, control,
 			DOUBLE), DOUBLE);
 		if (!control && CheckSolved(w, w->rubik.orient)) {
-			setPuzzle(w, PUZZLE_SOLVED);
+			setPuzzle(w, ACTION_SOLVED);
 		}
 	}
 }
@@ -1134,20 +1141,20 @@ RedoPieces(RubikWidget w)
 static void
 ClearPieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_CLEAR);
+	setPuzzle(w, ACTION_CLEAR);
 }
 
 static void
 PracticePieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
 RandomizePieces(RubikWidget w)
 {
-	int         face, position, direction;
-	int         big = w->rubik.sizex * w->rubik.sizey * w->rubik.sizez *
+	int face, position, direction;
+	int big = w->rubik.sizex * w->rubik.sizey * w->rubik.sizez *
 			3 + NRAND(2);
 
 	w->rubik.cheat = False;
@@ -1155,7 +1162,7 @@ RandomizePieces(RubikWidget w)
 		big = 1000;
 	if (w->rubik.practice)
 		PracticePieces(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 #ifdef DEBUG
 	big = 3;
 #endif
@@ -1167,9 +1174,9 @@ RandomizePieces(RubikWidget w)
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	setPuzzle(w, PUZZLE_RANDOMIZE);
+	setPuzzle(w, ACTION_RANDOMIZE);
 	if (CheckSolved(w, w->rubik.orient)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1182,42 +1189,68 @@ SolvePieces(RubikWidget w)
 	    (w->rubik.sizex == 3 && w->rubik.sizey == 3 && w->rubik.sizez == 3))
 		SolveSomePieces(w);
 	else {
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizePieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 IncrementPieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_INCX);
-	setPuzzle(w, PUZZLE_INCY);
-	setPuzzle(w, PUZZLE_INCZ);
+	setPuzzle(w, ACTION_INCX);
+	setPuzzle(w, ACTION_INCY);
+	setPuzzle(w, ACTION_INCZ);
 }
 
 static void
 DecrementPieces(RubikWidget w)
 {
 	if (w->rubik.sizex > MINFACETS) {
-		setPuzzle(w, PUZZLE_DECX);
+		setPuzzle(w, ACTION_DECX);
 	}
 	if (w->rubik.sizey > MINFACETS) {
-		setPuzzle(w, PUZZLE_DECY);
+		setPuzzle(w, ACTION_DECY);
 	}
 	if (w->rubik.sizez > MINFACETS) {
-		setPuzzle(w, PUZZLE_DECZ);
+		setPuzzle(w, ACTION_DECZ);
 	}
 }
 
 static void
 ViewPieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_VIEW);
+	setPuzzle(w, ACTION_VIEW);
+}
+
+static void
+SpeedPieces(RubikWidget w)
+{
+	w->rubik.delay -= 5;
+	if (w->rubik.delay < 0)
+		w->rubik.delay = 0;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowPieces(RubikWidget w)
+{
+	w->rubik.delay += 5;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundPieces(RubikWidget w)
+{
+	w->rubik.sound = !w->rubik.sound;
 }
 
 #ifdef WINVER
@@ -1227,8 +1260,8 @@ SetValuesPuzzle(RubikWidget w)
 	struct tagColor {
 		int red, green, blue;
 	} color;
-	char        szBuf[80], buf[20], charbuf[2];
-	int         face;
+	char szBuf[80], buf[20], charbuf[2];
+	int face;
 
 	w->rubik.sizex = GetPrivateProfileInt(SECTION, "sizeX",
 		DEFAULTFACETS, INIFILE);
@@ -1335,9 +1368,9 @@ ExposePuzzle(RubikWidget w)
 static void
 GetColor(RubikWidget w, int face)
 {
-	XGCValues   values;
-	XtGCMask    valueMask;
-	XColor      colorCell, rgb;
+	XGCValues values;
+	XtGCMask valueMask;
+	XColor colorCell, rgb;
 
 	valueMask = GCForeground | GCBackground;
 	if (w->rubik.reverse) {
@@ -1390,9 +1423,9 @@ GetColor(RubikWidget w, int face)
 void
 SetAllColors(RubikWidget w)
 {
-	XGCValues   values;
-	XtGCMask    valueMask;
-	int         face;
+	XGCValues values;
+	XtGCMask valueMask;
+	int face;
 
 	valueMask = GCForeground | GCBackground;
 
@@ -1447,8 +1480,8 @@ static Boolean
 SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	RubikWidget c = (RubikWidget) current, w = (RubikWidget) renew;
-	Boolean     redraw = False, setColors = False;
-	int         face;
+	Boolean redraw = False, setColors = False;
+	int face;
 
 	CheckPieces(w);
 	for (face = 0; face < MAXFACES; face++) {
@@ -1477,63 +1510,72 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		ResetPieces(w);
 		redraw = True;
 	}
-	if (w->rubik.menu != -1) {
+	if (w->rubik.menu != ACTION_IGNORE) {
 		int menu = w->rubik.menu;
 
-		w->rubik.menu = -1;
+		w->rubik.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetPieces(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WritePieces(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoPieces(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoPieces(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearPieces(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizePieces(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePieces(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizePieces(w);
+			break;
+		case ACTION_SOLVE:
 			SolvePieces(w);
 			break;
-		case MENU_ORIENTIZE:
-			OrientizePieces(w);
-			break;
-		case MENU_INCREMENT:
+		case ACTION_INCREMENT:
 			IncrementPieces(w);
 			break;
-		case MENU_DECREMENT:
+		case ACTION_DECREMENT:
 			(void) DecrementPieces(w);
 			break;
-		case MENU_VIEW:
+		case ACTION_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case ACTION_SPEED:
+			SpeedPieces(w);
+			break;
+		case ACTION_SLOW:
+			SlowPieces(w);
+			break;
+		case ACTION_SOUND:
+			SoundPieces(w);
+			break;
+		case ACTION_VIEW:
 			ViewPieces(w);
 			break;
 		default:
 			break;
 		}
 	}
-	if (w->rubik.currentDirection == PUZZLE_RESTORE) {
+	if (w->rubik.currentDirection == RESTORE_DIR) {
 		setStartPosition(w);
-		w->rubik.currentDirection = PUZZLE_IGNORE;
-	} else if (w->rubik.currentDirection == PUZZLE_CLEAR) {
+		w->rubik.currentDirection = IGNORE_DIR;
+	} else if (w->rubik.currentDirection == CLEAR_DIR) {
 		ResetPieces(w);
 		redraw = True;
-		w->rubik.currentDirection = PUZZLE_IGNORE;
-	} else if (w->rubik.currentDirection > PUZZLE_IGNORE) {
+		w->rubik.currentDirection = IGNORE_DIR;
+	} else if (w->rubik.currentDirection > IGNORE_DIR) {
 		(void) MoveAllPieces(w, w->rubik.currentFace,
 			w->rubik.currentPosition, w->rubik.currentDirection,
 			w->rubik.currentControl, w->rubik.currentFast);
-		w->rubik.currentDirection = PUZZLE_IGNORE;
+		w->rubik.currentDirection = IGNORE_DIR;
 	}
 	return redraw;
 }
@@ -1579,7 +1621,7 @@ Widget request, Widget renew
 #endif
 )
 {
-	int         face, orient;
+	int face, orient;
 #ifdef WINVER
 	SetValuesPuzzle(w);
 #else
@@ -1621,7 +1663,7 @@ HidePuzzle(RubikWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 void
@@ -1634,7 +1676,7 @@ SelectPuzzle(RubikWidget w
 )
 {
 #ifndef WINVER
-	int  x = event->xbutton.x, y = event->xbutton.y;
+	int x = event->xbutton.x, y = event->xbutton.y;
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
@@ -1642,10 +1684,10 @@ SelectPuzzle(RubikWidget w
 		     &(w->rubik.currentFace), &(w->rubik.currentPosition))) {
 		if (control || w->rubik.practice || !CheckSolved(w, w->rubik.orient))
 			DrawSquare(w, w->rubik.currentFace, w->rubik.currentPosition,
-				   TRUE);
+				TRUE);
 	} else {
-		w->rubik.currentFace = PUZZLE_IGNORE;
-		w->rubik.currentDirection = PUZZLE_IGNORE;
+		w->rubik.currentFace = IGNORE_DIR;
+		w->rubik.currentDirection = IGNORE_DIR;
 	}
 }
 
@@ -1658,13 +1700,13 @@ ReleasePuzzle(RubikWidget w
 #endif
 )
 {
-	int         face, position, count = -1, direction = -1;
+	int face, position, count = -1, direction = -1;
 #ifndef WINVER
-	int  x = event->xbutton.x, y = event->xbutton.y;
+	int x = event->xbutton.x, y = event->xbutton.y;
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
-	if (w->rubik.currentFace <= PUZZLE_IGNORE)
+	if (w->rubik.currentFace <= IGNORE_DIR)
 		return;
 	DrawSquare(w, w->rubik.currentFace, w->rubik.currentPosition, FALSE);
 	if (!control && !w->rubik.practice && CheckSolved(w, w->rubik.orient))
@@ -1677,7 +1719,7 @@ ReleasePuzzle(RubikWidget w
 					&direction);
 		} else {
 			direction = facesToDirection[w->rubik.currentFace][face];
-			count = (direction == PUZZLE_AMBIGUOUS) ? 2 : 1;
+			count = (direction == AMBIGUOUS_DIR) ? 2 : 1;
 			/* can look closer to minimize this, but then
 			must also be able to handle double moves */
 		}
@@ -1685,15 +1727,15 @@ ReleasePuzzle(RubikWidget w
 			MovePuzzle(w, face, w->rubik.currentPosition,
 				direction, (control) ? 1 : 0, NORMAL);
 			if (!control && CheckSolved(w, w->rubik.orient)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else if (count == 2) {
-			setPuzzle(w, PUZZLE_AMBIGUOUS);
+			setPuzzle(w, ACTION_AMBIGUOUS);
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->rubik.currentFace = PUZZLE_IGNORE;
-	w->rubik.currentDirection = PUZZLE_IGNORE;
+	w->rubik.currentFace = IGNORE_DIR;
+	w->rubik.currentDirection = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -1706,7 +1748,7 @@ PracticePuzzleMaybe(RubikWidget w
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -1731,7 +1773,7 @@ RandomizePuzzleMaybe(RubikWidget w
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
@@ -1868,6 +1910,36 @@ ViewPuzzle(RubikWidget w
 	ViewPieces(w);
 }
 
+void
+SpeedPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedPieces(w);
+}
+
+void
+SlowPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowPieces(w);
+}
+
+void
+SoundPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundPieces(w);
+}
+
 void
 EnterPuzzle(RubikWidget w
 #ifndef WINVER
@@ -1897,24 +1969,24 @@ IncrementAxisPuzzle(RubikWidget w, char axis)
 	switch (axis) {
 	case 'X':
 		if (w->rubik.sizex > MINFACETS)
-			setPuzzle(w, PUZZLE_DECX);
+			setPuzzle(w, ACTION_DECX);
 		return;
 	case 'x':
-		setPuzzle(w, PUZZLE_INCX);
+		setPuzzle(w, ACTION_INCX);
 		return;
 	case 'Y':
 		if (w->rubik.sizey > MINFACETS)
-			setPuzzle(w, PUZZLE_DECY);
+			setPuzzle(w, ACTION_DECY);
 		return;
 	case 'y':
-		setPuzzle(w, PUZZLE_INCY);
+		setPuzzle(w, ACTION_INCY);
 		return;
 	case 'Z':
 		if (w->rubik.sizez > MINFACETS)
-			setPuzzle(w, PUZZLE_DECZ);
+			setPuzzle(w, ACTION_DECZ);
 		return;
 	case 'z':
-		setPuzzle(w, PUZZLE_INCZ);
+		setPuzzle(w, ACTION_INCZ);
 		return;
 	}
 	return;
@@ -1923,7 +1995,7 @@ IncrementAxisPuzzle(RubikWidget w, char axis)
 void
 DimPuzzle(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_DIM);
+	setPuzzle(w, ACTION_DIM);
 }
 
 #else
@@ -1933,9 +2005,9 @@ IncrementXPuzzle(RubikWidget w, XEvent *event, char **args, int nArgs)
 	if (event->xbutton.state & (ShiftMask | LockMask)) {
 		if (w->rubik.sizex <= MINFACETS)
 			return;
-		setPuzzle(w, PUZZLE_DECX);
+		setPuzzle(w, ACTION_DECX);
 	} else
-		setPuzzle(w, PUZZLE_INCX);
+		setPuzzle(w, ACTION_INCX);
 }
 
 void
@@ -1944,9 +2016,9 @@ IncrementYPuzzle(RubikWidget w , XEvent *event, char **args, int nArgs)
 	if (event->xbutton.state & (ShiftMask | LockMask)) {
 		if (w->rubik.sizey <= MINFACETS)
 			return;
-		setPuzzle(w, PUZZLE_DECY);
+		setPuzzle(w, ACTION_DECY);
 	} else
-		setPuzzle(w, PUZZLE_INCY);
+		setPuzzle(w, ACTION_INCY);
 }
 
 void
@@ -1955,22 +2027,22 @@ IncrementZPuzzle(RubikWidget w , XEvent *event, char **args, int nArgs)
 	if (event->xbutton.state & (ShiftMask | LockMask)) {
 		if (w->rubik.sizez <= MINFACETS)
 			return;
-		setPuzzle(w, PUZZLE_DECZ);
+		setPuzzle(w, ACTION_DECZ);
 	} else
-		setPuzzle(w, PUZZLE_INCZ);
+		setPuzzle(w, ACTION_INCZ);
 }
 
 void
 MovePuzzleCcw(RubikWidget w, XEvent *event, char **args, int nArgs)
 {
 	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CCW,
-		       (int) (event->xbutton.state & ControlMask));
+		(int) (event->xbutton.state & ControlMask));
 }
 
 void
 MovePuzzleCw(RubikWidget w, XEvent *event, char **args, int nArgs)
 {
 	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CW,
-		       (int) (event->xkey.state & ControlMask));
+		(int) (event->xkey.state & ControlMask));
 }
 #endif
