@@ -58,6 +58,11 @@ static char defaultTranslationsPuzzle3D[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -111,12 +116,12 @@ static char defaultTranslationsPuzzle3D[] =
  <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
  <KeyPress>p: Practice()\n\
- <KeyPress>o: Orientize()\n\
- <KeyPress>d: Decrement()\n\
  <KeyPress>i: Increment()\n\
+ <KeyPress>d: Decrement()\n\
  <KeyPress>x: IncrementX()\n\
  <KeyPress>y: IncrementY()\n\
  <KeyPress>f: IncrementZ()\n\
+ <KeyPress>o: Orientize()\n\
  <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
@@ -149,13 +154,16 @@ static XtActionsRec actionsListPuzzle3D[] =
 	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
 	{(char *) "Solve", (XtActionProc) SolvePuzzle},
 	{(char *) "Practice", (XtActionProc) PracticePuzzle},
-	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
-	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
 	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
 	{(char *) "IncrementX", (XtActionProc) IncrementXPuzzle},
 	{(char *) "IncrementY", (XtActionProc) IncrementYPuzzle},
 	{(char *) "IncrementZ", (XtActionProc) IncrementZPuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -207,6 +215,12 @@ static XtResource resourcesPuzzle3D[] =
 	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.delay),
 	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(RubikWidget, rubik.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(RubikWidget, rubik.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -239,7 +253,7 @@ static XtResource resourcesPuzzle3D[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(RubikWidget, rubik.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -261,6 +275,9 @@ static XtResource resourcesPuzzle3D[] =
 	{XtNfast, XtCFast, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.currentFast),
 	 XtRString, (caddr_t) "1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(RubikWidget, rubik.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(RubikWidget, rubik.select),
 	 XtRCallback, (caddr_t) NULL}
