@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 2003 - 2006	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -200,6 +200,11 @@ static char defaultTranslationsPuzzleGL[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -251,12 +256,12 @@ static char defaultTranslationsPuzzleGL[] =
  <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
  <KeyPress>p: Practice()\n\
- <KeyPress>o: Orientize()\n\
- <KeyPress>d: Decrement()\n\
  <KeyPress>i: Increment()\n\
+ <KeyPress>d: Decrement()\n\
  <KeyPress>x: IncrementX()\n\
  <KeyPress>y: IncrementY()\n\
  <KeyPress>f: IncrementZ()\n\
+ <KeyPress>o: Orientize()\n\
  <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
@@ -296,6 +301,9 @@ static XtActionsRec actionsListPuzzleGL[] =
 	{(char *) "IncrementY", (XtActionProc) IncrementYPuzzle},
 	{(char *) "IncrementZ", (XtActionProc) IncrementZPuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -347,6 +355,12 @@ static XtResource resourcesPuzzleGL[] =
 	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.delay),
 	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(RubikWidget, rubik.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(RubikWidget, rubik.moveSound),
+         XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -382,7 +396,7 @@ static XtResource resourcesPuzzleGL[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(RubikWidget, rubik.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -404,6 +418,9 @@ static XtResource resourcesPuzzleGL[] =
 	{XtNfast, XtCFast, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.currentFast),
 	 XtRString, (caddr_t) "1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(RubikWidget, rubik.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(RubikWidget, rubik.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -1077,7 +1094,7 @@ DrawCube(RubikGLWidget w)
 #define MIDY(a) (((GLfloat)(2*a-MAXSIZEY+1))/2.0)
 #define MIDZ(a) (((GLfloat)(2*a-MAXSIZEZ+1))/2.0)
 #define DRAW_CUBIT(w,b,f,l,r,bm,t) if (!DrawCubit(w,b,f,l,r,bm,t)) return False
-	RubikSlice slice;
+	RubikSlice slice = {NO_FACE, NO_ROTATION, NO_DEPTH};
 	GLfloat rotateStep;
 	int i, j, k;
 
@@ -1101,8 +1118,8 @@ DrawCube(RubikGLWidget w)
  * There is a sequence for drawing cubies for each axis being moved...
  */
 #if 0
-slice.face = 0; /* DAB */
-slice.face = FRONT_FACE; /* DAB */
+slice.face = 0;
+slice.face = FRONT_FACE;
 (void) printf("slice.face %d\n", slice.face);
 #endif
 	switch (slice.face) {
