@@ -11,7 +11,7 @@
 #  Taken from the algorithm in the Ideal Solution book.
 #  Break ability taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -86,7 +86,7 @@ static int mapIdealToGeneral[MAXFACES] =
 {2, 3, 5, 1, 0, 4};		/* Remember to subtract 1 */
 
 static Boolean SolvingFlag = False;
-/* TODO Interruptability for Windows */
+
 #ifdef JMP
 static Boolean AbortSolvingFlag = False;
 static jmp_buf solve_env;
@@ -1358,7 +1358,7 @@ SolveCenters(RubikWidget w)
 void
 SolveSomePieces(RubikWidget w)
 {
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 	if (SolvingFlag)
 		return;
 #ifdef JMP
@@ -1388,5 +1388,5 @@ SolveSomePieces(RubikWidget w)
 #endif
 	SolvingFlag = False;
 	w->rubik.cheat = True; /* Assume the worst. */
-	setPuzzle(w, PUZZLE_COMPUTED);
+	setPuzzle(w, ACTION_COMPUTED);
 }
