@@ -27,6 +27,7 @@
 
 #include "file.h"
 #include "rngs.h"
+#include "sound.h"
 #include "SkewbP.h"
 #include "Skewb2dP.h"
 #include "Skewb3dP.h"
@@ -605,7 +606,7 @@ Boolean
 CheckSolved(SkewbWidget w)
 {
 	int face, position;
-	SkewbLoc test;
+	SkewbLoc test = {0, FALSE};
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < MAXCUBES; position++) {
@@ -690,7 +691,7 @@ DrawFrame(const SkewbWidget w, const Boolean focus)
 static void
 MoveNoPieces(SkewbWidget w)
 {
-	setPuzzle(w, PUZZLE_ILLEGAL);
+	setPuzzle(w, ACTION_ILLEGAL);
 }
 
 static void
@@ -870,20 +871,20 @@ MoveControlCb(SkewbWidget w, int face, int position, int direction)
 		direction = newpos.direction;
 	}
 	MovePieces(w, face, position, direction);
-	setPuzzleMove(w, PUZZLE_CONTROL, face, position, direction);
+	setPuzzleMove(w, ACTION_CONTROL, face, position, direction);
 	newFace = minToMaj[face][position].face;
 	rotate = minToMaj[face][position].rotation % MAXORIENT;
 	direction = (rotate + direction) % MAXORIENT;
 	position = (position + rotate + 2) % MAXORIENT;
 	MovePieces(w, newFace, position, direction);
-	setPuzzleMove(w, PUZZLE_CONTROL, newFace, position, direction);
+	setPuzzleMove(w, ACTION_CONTROL, newFace, position, direction);
 }
 
 static void
 MoveAltCb(SkewbWidget w, int face, int position, int direction)
 {
 	MovePieces(w, face, position, direction);
-	setPuzzleMove(w, PUZZLE_CONTROL, face, position, direction);
+	setPuzzleMove(w, ACTION_CONTROL, face, position, direction);
 }
 
 void
@@ -902,9 +903,14 @@ MovePuzzle(SkewbWidget w, int face, int position, int direction, int control)
 		if (direction >= 2 * MAXORIENT)
 			newpos = orthToDiag[face][position][direction % MAXORIENT];
 		MovePieces(w, newpos.face, newpos.position, newpos.direction);
-		setPuzzleMove(w, PUZZLE_MOVED, newpos.face, newpos.position,
+		setPuzzleMove(w, ACTION_MOVED, newpos.face, newpos.position,
 			newpos.direction);
 	}
+#ifdef USE_SOUND
+	if (w->skewb.sound) {
+		playSound((char *) MOVESOUND);
+	}
+#endif
 	setMove(&undo, face, position, direction, control);
 	flushMoves(w, &redo, FALSE);
 }
@@ -986,7 +992,7 @@ MovePuzzleInput(SkewbWidget w, int x, int y, int direction, int control,
 		return;
 	MovePuzzle(w, face, position, direction, control);
 	if (!control && CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1003,8 +1009,8 @@ ResetPieces(SkewbWidget w)
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
 	w->skewb.started = False;
-	w->skewb.currentFace = PUZZLE_IGNORE;
-	w->skewb.currentDirection = PUZZLE_IGNORE;
+	w->skewb.currentFace = IGNORE_DIR;
+	w->skewb.currentDirection = IGNORE_DIR;
 }
 
 static void
@@ -1055,12 +1061,12 @@ GetPieces(SkewbWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->skewb.orient != (Boolean) orient) {
-		setPuzzle(w, PUZZLE_ORIENTIZE);
+		setPuzzle(w, ACTION_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->skewb.practice != (Boolean) practice) {
-		setPuzzle(w, PUZZLE_PRACTICE);
+		setPuzzle(w, ACTION_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
@@ -1068,14 +1074,14 @@ GetPieces(SkewbWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 #ifdef WINVER
 	setStartPosition(w);
 #endif
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
-	(void) printf("%s: orient %d, practice %d, moves %d.\n",
-		name, orient, practice, moves);
+	(void) printf("%s: orient %s, practice %s, moves %d.\n",
+		name, BOOL_STRING(orient), BOOL_STRING(practice), moves);
 	free(lname);
 	free(fname);
 	w->skewb.cheat = True; /* Assume the worst. */
@@ -1169,9 +1175,9 @@ UndoPieces(SkewbWidget w)
 				direction = newpos.direction;
 			}
 			MovePieces(w, face, position, direction);
-			setPuzzleMove(w, PUZZLE_UNDO, face, position, direction);
+			setPuzzleMove(w, ACTION_UNDO, face, position, direction);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -1201,9 +1207,9 @@ RedoPieces(SkewbWidget w)
 				direction = newpos.direction;
 			}
 			MovePieces(w, face, position, direction);
-			setPuzzleMove(w, PUZZLE_REDO, face, position, direction);
+			setPuzzleMove(w, ACTION_REDO, face, position, direction);
 			if (CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		}
 	}
@@ -1212,13 +1218,13 @@ RedoPieces(SkewbWidget w)
 static void
 ClearPieces(SkewbWidget w)
 {
-	setPuzzle(w, PUZZLE_CLEAR);
+	setPuzzle(w, ACTION_CLEAR);
 }
 
 static void
 PracticePieces(SkewbWidget w)
 {
-	setPuzzle(w, PUZZLE_PRACTICE);
+	setPuzzle(w, ACTION_PRACTICE);
 }
 
 static void
@@ -1230,7 +1236,7 @@ RandomizePieces(SkewbWidget w)
 	w->skewb.cheat = False;
 	if (w->skewb.practice)
 		PracticePieces(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -1244,9 +1250,9 @@ RandomizePieces(SkewbWidget w)
 	}
 	flushMoves(w, &undo, TRUE);
 	flushMoves(w, &redo, FALSE);
-	setPuzzle(w, PUZZLE_RANDOMIZE);
+	setPuzzle(w, ACTION_RANDOMIZE);
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1256,20 +1262,46 @@ SolvePieces(SkewbWidget w)
 	if (CheckSolved(w))
 		return;
 	{
-		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+		setPuzzle(w, ACTION_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizePieces(SkewbWidget w)
 {
-	setPuzzle(w, PUZZLE_ORIENTIZE);
+	setPuzzle(w, ACTION_ORIENTIZE);
 }
 
 static void
 ViewPieces(SkewbWidget w)
 {
-	setPuzzle(w, PUZZLE_VIEW);
+	setPuzzle(w, ACTION_VIEW);
+}
+
+static void
+SpeedPieces(SkewbWidget w)
+{
+	w->skewb.delay -= 5;
+	if (w->skewb.delay < 0)
+		w->skewb.delay = 0;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowPieces(SkewbWidget w)
+{
+	w->skewb.delay += 5;
+#ifdef HAVE_MOTIF
+	setPuzzle(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundPieces(SkewbWidget w)
+{
+	w->skewb.sound = !w->skewb.sound;
 }
 
 #ifdef WINVER
@@ -1520,56 +1552,65 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		ResetPieces(w);
 		redraw = True;
 	}
-	if (w->skewb.menu != -1) {
+	if (w->skewb.menu != ACTION_IGNORE) {
 		int menu = w->skewb.menu;
 
-		w->skewb.menu = -1;
+		w->skewb.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetPieces(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WritePieces(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoPieces(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoPieces(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearPieces(w);
 			break;
-		case MENU_RANDOMIZE:
-			RandomizePieces(w);
-			break;
-		case MENU_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePieces(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_RANDOMIZE:
+			RandomizePieces(w);
+			break;
+		case ACTION_SOLVE:
 			SolvePieces(w);
 			break;
-		case MENU_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePieces(w);
 			break;
-		case MENU_VIEW:
+		case ACTION_SPEED:
+			SpeedPieces(w);
+			break;
+		case ACTION_SLOW:
+			SlowPieces(w);
+			break;
+		case ACTION_SOUND:
+			SoundPieces(w);
+			break;
+		case ACTION_VIEW:
 			ViewPieces(w);
 			break;
 		default:
 			break;
 		}
 	}
-	if (w->skewb.currentDirection == PUZZLE_RESTORE) {
+	if (w->skewb.currentDirection == RESTORE_DIR) {
 		setStartPosition(w);
-		w->skewb.currentDirection = PUZZLE_IGNORE;
-	} else if (w->skewb.currentDirection == PUZZLE_CLEAR) {
+		w->skewb.currentDirection = IGNORE_DIR;
+	} else if (w->skewb.currentDirection == CLEAR_DIR) {
 		ResetPieces(w);
 		redraw = True;
-		w->skewb.currentDirection = PUZZLE_IGNORE;
-	} else if (w->skewb.currentDirection > PUZZLE_IGNORE) {
+		w->skewb.currentDirection = IGNORE_DIR;
+	} else if (w->skewb.currentDirection > IGNORE_DIR) {
 		MovePieces(w, w->skewb.currentFace, w->skewb.currentPosition,
 			w->skewb.currentDirection);
-		w->skewb.currentDirection = PUZZLE_IGNORE;
+		w->skewb.currentDirection = IGNORE_DIR;
 	}
 	return redraw;
 }
@@ -1652,7 +1693,7 @@ HidePuzzle(SkewbWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 void
@@ -1678,16 +1719,16 @@ SelectPuzzle(SkewbWidget w
 				FLUSH(w);
 				Sleep(100);
 				DrawDiamond(w, w->skewb.currentFace, FALSE);
-				w->skewb.currentFace = PUZZLE_IGNORE;
-				w->skewb.currentDirection = PUZZLE_IGNORE;
+				w->skewb.currentFace = IGNORE_DIR;
+				w->skewb.currentDirection = IGNORE_DIR;
 			}
 		} else if (control || alt || w->skewb.practice || !CheckSolved(w)) {
 			DrawTriangle(w, w->skewb.currentFace, w->skewb.currentPosition,
 				TRUE);
 		}
 	} else {
-		w->skewb.currentFace = PUZZLE_IGNORE;
-		w->skewb.currentDirection = PUZZLE_IGNORE;
+		w->skewb.currentFace = IGNORE_DIR;
+		w->skewb.currentDirection = IGNORE_DIR;
 	}
 }
 
@@ -1707,7 +1748,7 @@ ReleasePuzzle(SkewbWidget w
 #endif
 	int face, position, count = -1, direction = 0, ctrl = control;
 
-	if (w->skewb.currentFace <= PUZZLE_IGNORE)
+	if (w->skewb.currentFace <= IGNORE_DIR)
 		return;
 	if (w->skewb.currentPosition != MAXORIENT)
 		DrawTriangle(w, w->skewb.currentFace, w->skewb.currentPosition,
@@ -1745,13 +1786,13 @@ ReleasePuzzle(SkewbWidget w
 				w->skewb.currentPosition, direction,
 				ctrl);
 			if (!ctrl && CheckSolved(w)) {
-				setPuzzle(w, PUZZLE_SOLVED);
+				setPuzzle(w, ACTION_SOLVED);
 			}
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->skewb.currentFace = PUZZLE_IGNORE;
-	w->skewb.currentDirection = PUZZLE_IGNORE;
+	w->skewb.currentFace = IGNORE_DIR;
+	w->skewb.currentDirection = IGNORE_DIR;
 }
 
 #ifndef WINVER
@@ -1764,7 +1805,7 @@ PracticePuzzleMaybe(SkewbWidget w
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
+		setPuzzle(w, ACTION_PRACTICE_QUERY);
 	}
 #endif
 }
@@ -1789,7 +1830,7 @@ RandomizePuzzleMaybe(SkewbWidget w
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
+		setPuzzle(w, ACTION_RANDOMIZE_QUERY);
 	}
 #endif
 }
@@ -1906,6 +1947,36 @@ ViewPuzzle(SkewbWidget w
 	ViewPieces(w);
 }
 
+void
+SpeedPuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedPieces(w);
+}
+
+void
+SlowPuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowPieces(w);
+}
+
+void
+SoundPuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundPieces(w);
+}
+
 void
 EnterPuzzle(SkewbWidget w
 #ifndef WINVER
@@ -1932,7 +2003,7 @@ LeavePuzzle(SkewbWidget w
 void
 DimPuzzle(SkewbWidget w)
 {
-	setPuzzle(w, PUZZLE_DIM);
+	setPuzzle(w, ACTION_DIM);
 }
 
 #else
