@@ -45,6 +45,8 @@
 #define XtNfaceColor5 ((char *) "faceColor5")
 #define XtNpieceBorder ((char *) "pieceBorder")
 #define XtNdelay ((char *) "delay")
+#define XtNsound ((char *) "sound")
+#define XtNmoveSound ((char *) "moveSound")
 #define XtNview ((char *) "view")
 #define XtNorient ((char *) "orient")
 #define XtNpractice ((char *) "practice")
@@ -58,9 +60,12 @@
 #define XtNface ((char *) "face")
 #define XtNpos ((char *) "pos")
 #define XtNdirection ((char *) "direction")
+#define XtNpixmapSize ((char *) "pixmapSize")
 #define XtNselectCallback ((char *) "selectCallback")
 #define XtCMono ((char *) "Mono")
 #define XtCDelay ((char *) "Delay")
+#define XtCSound ((char *) "Sound")
+#define XtCMoveSound ((char *) "MoveSound")
 #define XtCView ((char *) "View")
 #define XtCOrient ((char *) "Orient")
 #define XtCPractice ((char *) "Practice")
@@ -70,6 +75,7 @@
 #define XtCFace ((char *) "Face")
 #define XtCPos ((char *) "Pos")
 #define XtCDirection ((char *) "Direction")
+#define XtCPixmapSize ((char *) "PixmapSize")
 
 typedef struct _SkewbClassRec *SkewbWidgetClass;
 
@@ -82,41 +88,64 @@ typedef struct {
 } skewbCallbackStruct;
 #endif
 
-#define PUZZLE_HIDE (-9)
-#define PUZZLE_PRACTICE_QUERY (-8)
-#define PUZZLE_RANDOMIZE_QUERY (-7)
-#define PUZZLE_SOLVE_MESSAGE (-6)
-#define PUZZLE_RESTORE (-5)
-#define PUZZLE_CLEAR (-4)
-#define PUZZLE_RESET (-3)
-#define PUZZLE_ILLEGAL (-2)
-#define PUZZLE_IGNORE (-1)
-#define PUZZLE_MOVED 0
-#define PUZZLE_CONTROL 1
-#define PUZZLE_SOLVED 2
-#define PUZZLE_PRACTICE 3
-#define PUZZLE_RANDOMIZE 4
-#define PUZZLE_DEC 5
-#define PUZZLE_ORIENTIZE 6
-#define PUZZLE_INC 7
-#define PUZZLE_COMPUTED 8
-#define PUZZLE_UNDO 9
-#define PUZZLE_REDO 10
-#define PUZZLE_DIM 11
-#define PUZZLE_VIEW 12
-
-#define MENU_GET 0
-#define MENU_WRITE 1
-#define MENU_EXIT 2
-#define MENU_UNDO 3
-#define MENU_REDO 4
-#define MENU_CLEAR 5
-#define MENU_RANDOMIZE 6
-#define MENU_PRACTICE 7
-#define MENU_SOLVE 8
-#define MENU_ORIENTIZE 9
-#define MENU_VIEW 10
+#define ACTION_GET 100
+#define ACTION_WRITE 101
+#define ACTION_EXIT 102
+#define ACTION_HIDE 103
+#define ACTION_PRACTICE_QUERY 104
+#define ACTION_RANDOMIZE_QUERY 105
+#define ACTION_SOLVE_MESSAGE 106
+#define ACTION_RESTORE 107
+#define ACTION_RESET 108
+#define ACTION_AMBIGUOUS 109
+#define ACTION_ILLEGAL 110
+#define ACTION_MOVED 111
+#define ACTION_CONTROL 112
+#define ACTION_SOLVED 113
+#define ACTION_COMPUTED 114
+#define ACTION_UNDO 200
+#define ACTION_REDO 201
+#define ACTION_CLEAR 202
+#define ACTION_PRACTICE 203
+#define ACTION_RANDOMIZE 204
+#define ACTION_SOLVE 205
+#define ACTION_ORIENTIZE 206
+#define ACTION_SPEED 207
+#define ACTION_SLOW 208
+#define ACTION_SOUND 209
+#define ACTION_VIEW 210
+#define ACTION_DIM 211
+#define ACTION_TR 300
+#define ACTION_BR 301
+#define ACTION_BL 302
+#define ACTION_TL 303
+#define ACTION_CW 305
+#define ACTION_CCW 307
+#define ACTION_TOP 308
+#define ACTION_RIGHT 309
+#define ACTION_BOTTOM 310
+#define ACTION_LEFT 311
+#define ACTION_CONTROL_TR 340
+#define ACTION_CONTROL_BR 341
+#define ACTION_CONTROL_BL 342
+#define ACTION_CONTROL_TL 343
+#define ACTION_CONTROL_CW 345
+#define ACTION_CONTROL_CCW 347
+#define ACTION_CONTROL_TOP 348
+#define ACTION_CONTROL_RIGHT 349
+#define ACTION_CONTROL_BOTTOM 350
+#define ACTION_CONTROL_LEFT 351
+#define ACTION_DESCRIPTION 900
+#define ACTION_FEATURES 901
+#define ACTION_REFERENCES 902
+#define ACTION_ABOUT 903
+#define ACTION_IGNORE 999
 
+#define RESTORE_DIR (-5)
+#define CLEAR_DIR (-4)
+#define AMBIGUOUS_DIR (-3)
+#define ILLEGAL_DIR (-2)
+#define IGNORE_DIR (-1)
 #define MAXFACES 6
 #define MAXVIEWS 2
 #define DEFAULTORIENT FALSE
@@ -128,6 +157,8 @@ typedef struct {
 #endif
 #define DEFAULTREVERSE FALSE
 
+#define BOOL_STRING(a) ((a==0)?"false":"true")
+
 typedef struct _SkewbRec *SkewbWidget;
 
 #endif /* _XtSkewb_h */
