@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -58,6 +58,11 @@ static char defaultTranslationsPuzzle2D[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -146,6 +151,9 @@ static XtActionsRec actionsListPuzzle2D[] =
 	{(char *) "Practice", (XtActionProc) PracticePuzzle},
 	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -194,6 +202,15 @@ static XtResource resourcesPuzzle2D[] =
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(SkewbWidget, skewb.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
+	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
+	 XtOffset(SkewbWidget, skewb.delay),
+	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(SkewbWidget, skewb.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(SkewbWidget, skewb.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -217,7 +234,7 @@ static XtResource resourcesPuzzle2D[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(SkewbWidget, skewb.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -233,6 +250,9 @@ static XtResource resourcesPuzzle2D[] =
 	{XtNdirection, XtCDirection, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.currentDirection),
 	 XtRString, (caddr_t) "-1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(SkewbWidget, skewb.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(SkewbWidget, skewb.select),
 	 XtRCallback, (caddr_t) NULL}
