@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2007	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -50,6 +50,11 @@ static char defaultTranslationsPuzzle3D[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -138,6 +143,9 @@ static XtActionsRec actionsListPuzzle3D[] =
 	{(char *) "Practice", (XtActionProc) PracticePuzzle},
 	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -186,6 +194,15 @@ static XtResource resourcesPuzzle3D[] =
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(SkewbWidget, skewb.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
+	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
+	 XtOffset(SkewbWidget, skewb.delay),
+	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(SkewbWidget, skewb.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(SkewbWidget, skewb.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -209,7 +226,7 @@ static XtResource resourcesPuzzle3D[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(SkewbWidget, skewb.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -225,6 +242,9 @@ static XtResource resourcesPuzzle3D[] =
 	{XtNdirection, XtCDirection, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.currentDirection),
 	 XtRString, (caddr_t) "-1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(SkewbWidget, skewb.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(SkewbWidget, skewb.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -437,10 +457,11 @@ void
 DrawTriangle3D(Skewb3DWidget w, int face, int position, int offset)
 {
 	GC faceGC, borderGC;
-	int side = 0, dx, dy, letterX, letterY;
+	int side = 0, dx, dy, rotation, letterX, letterY;
 
 	MapTo3D(w, face, position, &side);
 	CubeOffset3D(w, face, side, &dx, &dy);
+	rotation = w->skewb.cubeLoc[face][position].rotation;
 	letterX = dx + letter3DList[face][side].x;
 	letterY = dy + letter3DList[face][side].y;
 	triangleLoc3D[face][side][0].x = dx;
@@ -483,13 +504,13 @@ DrawTriangle3D(Skewb3DWidget w, int face, int position, int offset)
 	if (w->skewb.orient) {
 		DRAWLINE(w, dr, borderGC,
 			letterX +
-			orientTriangle[face][w->skewb.cubeLoc[face][position].rotation][0].x,
+			orientTriangle[face][rotation][0].x,
 			letterY +
-			orientTriangle[face][w->skewb.cubeLoc[face][position].rotation][0].y,
+			orientTriangle[face][rotation][0].y,
 			letterX +
-			orientTriangle[face][w->skewb.cubeLoc[face][position].rotation][1].x,
+			orientTriangle[face][rotation][1].x,
 			letterY +
-			orientTriangle[face][w->skewb.cubeLoc[face][position].rotation][1].y);
+			orientTriangle[face][rotation][1].y);
 	}
 }
 
@@ -497,9 +518,10 @@ void
 DrawDiamond3D(Skewb3DWidget w, int face, int offset)
 {
 	GC faceGC, borderGC;
-	int dx, dy;
+	int dx, dy, rotation;
 
 	DiamondOffset3D(w, face, &dx, &dy);
+	rotation = w->skewb.cubeLoc[face][MAXORIENT].rotation;
 	diamondLoc3D[face][0].x = dx + cubeLoc3D[face][1].x / 2;
 	diamondLoc3D[face][0].y = dy + cubeLoc3D[face][1].y / 2;
 	if (offset) {
@@ -541,13 +563,13 @@ DrawDiamond3D(Skewb3DWidget w, int face, int offset)
 	if (w->skewb.orient) {
 		DRAWLINE(w, dr, borderGC,
 			dx +
-			orientDiamond[face][w->skewb.cubeLoc[face][MAXORIENT].rotation][0].x,
+			orientDiamond[face][rotation][0].x,
 			dy +
-			orientDiamond[face][w->skewb.cubeLoc[face][MAXORIENT].rotation][0].y,
+			orientDiamond[face][rotation][0].y,
 			dx +
-			orientDiamond[face][w->skewb.cubeLoc[face][MAXORIENT].rotation][1].x,
+			orientDiamond[face][rotation][1].x,
 			dy +
-			orientDiamond[face][w->skewb.cubeLoc[face][MAXORIENT].rotation][1].y);
+			orientDiamond[face][rotation][1].y);
 	}
 }
 
