@@ -313,6 +313,11 @@ static char defaultTranslationsPuzzleGL[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveCcw()\n\
  <KeyPress>R5: MoveCcw()\n\
  <KeyPress>Home: MoveTl()\n\
@@ -399,6 +404,9 @@ static XtActionsRec actionsListPuzzleGL[] =
 	{(char *) "Practice", (XtActionProc) PracticePuzzle},
 	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
 	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
 	{(char *) "Enter", (XtActionProc) EnterPuzzle},
 	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
@@ -450,6 +458,12 @@ static XtResource resourcesPuzzleGL[] =
 	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.delay),
 	 XtRString, (caddr_t) "10"},
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(SkewbWidget, skewb.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(SkewbWidget, skewb.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.font),
 	 XtRString, (caddr_t) "9x15bold"},
@@ -476,7 +490,7 @@ static XtResource resourcesPuzzleGL[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
 	{XtNstart, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(SkewbWidget, skewb.started),
 	 XtRString, (caddr_t) "FALSE"},
@@ -492,6 +506,9 @@ static XtResource resourcesPuzzleGL[] =
 	{XtNdirection, XtCDirection, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.currentDirection),
 	 XtRString, (caddr_t) "-1"},
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(SkewbWidget, skewb.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(SkewbWidget, skewb.select),
 	 XtRCallback, (caddr_t) NULL}
