@@ -23,19 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/12/15 Xt/Motif
-  Version 5: 95/10/06 Xt/Motif
-  Version 4: 94/05/30 Xt
-  Version 3: 93/10/03 Motif
-  Version 2: 92/01/22 XView
-  Version 1: 91/03/19 SunView
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "SkewbP.h"
-#include "wskewb.h"
 #define TITLE "wskewb"
 
 static SkewbRec widget;
@@ -63,8 +53,8 @@ static SkewbRec widget;
 #include <Xm/CascadeB.h>
 #include <Xm/ToggleB.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -79,9 +69,19 @@ static SkewbRec widget;
 #endif
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/skewb.xpm"
+#include "pixmaps/skewb.t.xpm"
+#include "pixmaps/skewb.p.xpm"
+#include "pixmaps/skewb.s.xpm"
+#include "pixmaps/skewb.m.xpm"
+#include "pixmaps/skewb.l.xpm"
+#include "pixmaps/skewb.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?skewb_t_xpm:skewb_p_xpm):\
+(((s)<=24)?skewb_s_xpm:skewb_m_xpm)):\
+(((s)<=48)?skewb_l_xpm:skewb_xpm))
 #endif
-#include "icons/skewb.xbm"
+#include "pixmaps/skewb.xbm"
+#define DEFINE_XBM (char *) skewb_bits, skewb_width, skewb_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -93,7 +93,7 @@ static SkewbRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Skewb Version 7.3\n"
+"Skewb Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -105,6 +105,7 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3|4|5} {color}] [-{border|bd} {color}]\n"
+"[-delay msecs] [-[no]sound] [-moveSound {filename}]\n"
 "[-{font|fn} {fontname}] [-view {int}] [-[no]orient]\n"
 "[-[no]practice] [-userName {string}]\n"
 "[-scoreFile {filename}] [-scores] [-version]\n"
@@ -175,6 +176,14 @@ static const char featuresHelp[] = {
 "\n"
 "Press \"V\" or \"v\" keys to change the view of the cube.\n"
 "\n"
+"Press \">\" or \".\" keys to speed up the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"<\" or \",\" keys to slow down the movement of pieces "
+"(not implemented).\n"
+"\n"
+"Press \"@\" key to toggle the sound.\n"
+"\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
 "Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program.\n"
@@ -208,7 +217,7 @@ static const char featuresHelp[] = {
 "same is \"5\" is Clockwise and \"/\" is Counterclockwise.\n"
 "\n"
 "Use the control key and the left mouse button, keypad, or "
-"arrow keys  to move the whole cube.  This is\n"
+"arrow keys to move the whole cube.  This is\n"
 "not recorded as a turn.\n"
 };
 
@@ -240,9 +249,9 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord skewbRecord[2];
+static PuzzleRecord puzzleRecord[2];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -258,7 +267,7 @@ static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
 #ifdef HAVE_MOTIF
-static Widget moves, record, message, orientSwitch, practiceSwitch;
+static Widget moves, record, message, orientizeSwitch, practiceSwitch;
 static char buff[21];
 static Widget descriptionDialog, featuresDialog;
 static Widget optionsDialog, referencesDialog, aboutDialog;
@@ -270,10 +279,10 @@ static Widget shell;
 static Widget shellGL;
 #endif
 #endif
-static Pixmap icon = None;
-static Widget topLevel, skewb2d, skewb3d;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle2d, puzzle3d;
 #ifdef HAVE_OPENGL
-static Widget skewbGL;
+static Widget puzzleGL;
 #endif
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -299,11 +308,11 @@ PrintRecords(void)
 	(void) printf("    SKEWB  HALL OF FAME\n\n");
 	(void) printf("ORIENT USER            MOVES\n");
 	for (orient = 0; orient < 2; orient++) {
-		if (skewbRecord[orient].score > 0)
+		if (puzzleRecord[orient].score > 0)
 			(void) printf("%6d %-12s%9d\n",
 				orient,
-				skewbRecord[orient].name,
-				skewbRecord[orient].score);
+				puzzleRecord[orient].name,
+				puzzleRecord[orient].score);
 	}
 }
 #endif
@@ -314,8 +323,8 @@ InitRecords(void)
 	int orient;
 
 	for (orient = 0; orient < 2; orient++) {
-		skewbRecord[orient].score = NEVER;
-		(void) strncpy(skewbRecord[orient].name, NOACCESS, USERNAMELEN);
+		puzzleRecord[orient].score = NEVER;
+		(void) strncpy(puzzleRecord[orient].name, NOACCESS, USERNAMELEN);
 	}
 }
 
@@ -367,10 +376,10 @@ ReadRecords(void)
 	free(fname);
 	for (orient = 0; orient < 2; orient++) {
 		(void) fscanf(fp, "%d %s\n", &n, userName);
-		if (n <= skewbRecord[orient].score ||
-				skewbRecord[orient].score <= NEVER) {
-			skewbRecord[orient].score = n;
-			(void) strncpy(skewbRecord[orient].name, userName,
+		if (n <= puzzleRecord[orient].score ||
+				puzzleRecord[orient].score <= NEVER) {
+			puzzleRecord[orient].score = n;
+			(void) strncpy(puzzleRecord[orient].name, userName,
 				USERNAMELEN);
 		}
 	}
@@ -413,7 +422,7 @@ WriteRecords(void)
 #endif
 		for (orient = 0; orient < 2; orient++)
 			(void) fprintf(fp, "%d %s\n",
-			skewbRecord[orient].score, skewbRecord[orient].name);
+			puzzleRecord[orient].score, puzzleRecord[orient].name);
 #if HAVE_FCNTL_H
 		(void) close(lfd);
 		(void) unlink(lockFile);
@@ -429,11 +438,11 @@ PrintRecord(Boolean orient, Boolean practice)
 
 	if (practice) {
 		(void) strncpy(recordDsp, "practice", MESSAGELEN);
-	} else if (skewbRecord[i].score <= NEVER) {
+	} else if (puzzleRecord[i].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			skewbRecord[i].score, skewbRecord[i].name);
+			puzzleRecord[i].score, puzzleRecord[i].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -452,7 +461,7 @@ MakeEquivalent(String *userName, String *scoreFile,
 	Pixel foreground, background, pieceBorder;
 	String faceColor[MAXFACES];
 
-	XtVaGetValues(skewb2d,
+	XtVaGetValues(puzzle2d,
 		XtNuserName, userName,
 		XtNscoreFile, scoreFile,
 		XtNorient, orient,
@@ -482,16 +491,16 @@ MakeEquivalent(String *userName, String *scoreFile,
 		PrintRecords();
 		exit(0);
 	}
-	XtVaSetValues(skewb2d,
-		XtNdirection, PUZZLE_IGNORE,
+	XtVaSetValues(puzzle2d,
+		XtNdirection, IGNORE_DIR,
 		XtNstart, False, NULL);
-	XtVaSetValues(skewb3d,
+	XtVaSetValues(puzzle3d,
 		XtNuserName, *userName,
 		XtNorient, *orient,
 		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverseVideo, reverse,
-		XtNdirection, PUZZLE_IGNORE,
+		XtNdirection, IGNORE_DIR,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -503,13 +512,13 @@ MakeEquivalent(String *userName, String *scoreFile,
 		XtNfaceColor4, faceColor[4],
 		XtNfaceColor5, faceColor[5], NULL);
 #ifdef HAVE_OPENGL
-	XtVaSetValues(skewbGL,
+	XtVaSetValues(puzzleGL,
 		XtNuserName, *userName,
 		XtNorient, *orient,
 		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverseVideo, reverse,
-		XtNdirection, PUZZLE_IGNORE,
+		XtNdirection, IGNORE_DIR,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -549,10 +558,10 @@ PrintStatus(char *msg, int nMoves
 #else
 #ifdef HAVE_OPENGL
 	if (dim == 4)
-		XtVaSetValues(XtParent(skewbGL), XtNtitle, titleDsp, NULL);
+		XtVaSetValues(XtParent(puzzleGL), XtNtitle, titleDsp, NULL);
 	else
 #endif
-	XtVaSetValues(XtParent((dim == 2) ? skewb2d : skewb3d),
+	XtVaSetValues(XtParent((dim == 2) ? puzzle2d : puzzle3d),
 		XtNtitle, titleDsp, NULL);
 #endif
 #endif
@@ -563,14 +572,14 @@ HandleSolved(int counter, Boolean orient)
 {
 	int i = (orient) ? 1 : 0;
 
-	if (counter < skewbRecord[i].score || skewbRecord[i].score <= NEVER) {
+	if (counter < puzzleRecord[i].score || puzzleRecord[i].score <= NEVER) {
 		ReadRecords();	/* Maybe its been updated by another */
-		skewbRecord[i].score = counter;
-		(void) strncpy(skewbRecord[i].name, userNameDsp, USERNAMELEN);
-		if (orient && (counter < skewbRecord[!i].score ||
-				skewbRecord[!i].score <= NEVER)) {
-			skewbRecord[!i].score = counter;
-			(void) strncpy(skewbRecord[!i].name, userNameDsp,
+		puzzleRecord[i].score = counter;
+		(void) strncpy(puzzleRecord[i].name, userNameDsp, USERNAMELEN);
+		if (orient && (counter < puzzleRecord[!i].score ||
+				puzzleRecord[!i].score <= NEVER)) {
+			puzzleRecord[!i].score = counter;
+			(void) strncpy(puzzleRecord[!i].name, userNameDsp,
 				USERNAMELEN);
 		}
 		WriteRecords();
@@ -607,7 +616,7 @@ void
 #else
 	MakeEquivalent(&userName, &scoreFile, &orient, &practice);
 #ifdef HAVE_MOTIF
-	XmToggleButtonSetState(orientSwitch, orient, True);
+	XmToggleButtonSetState(orientizeSwitch, orient, True);
 	XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 #endif
@@ -677,10 +686,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 #endif
 
-	if (w == skewb2d) {
-		otherw1 = skewb3d;
+	if (w == puzzle2d) {
+		otherw1 = puzzle3d;
 #ifdef HAVE_OPENGL
-		otherw2 = skewbGL;
+		otherw2 = puzzleGL;
 #endif
 #ifndef HAVE_MOTIF
 		dim = 2;
@@ -689,10 +698,10 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 		otherdim2 = 4;
 #endif
 #endif
-	} else if (w == skewb3d) {
-		otherw1 = skewb2d;
+	} else if (w == puzzle3d) {
+		otherw1 = puzzle2d;
 #ifdef HAVE_OPENGL
-		otherw2 = skewbGL;
+		otherw2 = puzzleGL;
 #endif
 #ifndef HAVE_MOTIF
 		dim = 3;
@@ -703,9 +712,9 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 	}
 #ifdef HAVE_OPENGL
-	else if (w == skewbGL) {
-		otherw1 = skewb2d;
-		otherw2 = skewb3d;
+	else if (w == puzzleGL) {
+		otherw1 = puzzle2d;
+		otherw2 = puzzle3d;
 #ifndef HAVE_MOTIF
 		dim = 4;
 		otherdim1 = 2;
@@ -729,7 +738,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -749,22 +758,22 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_PRACTICE_QUERY:
+	case ACTION_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(practiceDialog);
 #else
-		XtVaSetValues(skewb2d, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_PRACTICE, NULL);
 #endif
 		break;
-	case PUZZLE_RANDOMIZE_QUERY:
+	case ACTION_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(randomizeDialog);
 #else
-		XtVaSetValues(skewb2d, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_RANDOMIZE, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -776,7 +785,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RESTORE:
+	case ACTION_RESTORE:
 		if (practice) {
 			(void) strncpy(recordDsp, "practice",
 				MESSAGELEN);
@@ -789,33 +798,33 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
-			XtNdirection, PUZZLE_RESTORE, NULL);
+			XtNdirection, RESTORE_DIR, NULL);
 #endif
 		XtVaSetValues(otherw1,
-			XtNdirection, PUZZLE_RESTORE, NULL);
+			XtNdirection, RESTORE_DIR, NULL);
 		XtVaSetValues(w,
-			XtNdirection, PUZZLE_RESTORE, NULL);
+			XtNdirection, RESTORE_DIR, NULL);
 #endif
 		break;
-	case PUZZLE_CLEAR:
+	case ACTION_CLEAR:
 		movesDsp = 0;
 		randomized = False;
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
-			XtNdirection, PUZZLE_CLEAR, NULL);
+			XtNdirection, CLEAR_DIR, NULL);
 #endif
 		XtVaSetValues(otherw1,
-			XtNdirection, PUZZLE_CLEAR, NULL);
+			XtNdirection, CLEAR_DIR, NULL);
 		XtVaSetValues(w,
-			XtNdirection, PUZZLE_CLEAR, NULL);
+			XtNdirection, CLEAR_DIR, NULL);
 #endif
 		break;
-	case PUZZLE_RESET:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_ILLEGAL:
+	case ACTION_ILLEGAL:
 		if (practice || randomized)
 			(void) strncpy(messageDsp, "Illegal move",
 				MESSAGELEN);
@@ -824,7 +833,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 				"Randomize to start", MESSAGELEN);
 		break;
 #ifndef WINVER
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
@@ -840,7 +849,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 			XtNdirection, callData->direction, NULL);
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
 			XtNface, callData->face,
@@ -853,7 +862,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 			XtNdirection, callData->direction, NULL);
 		break;
 #endif
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (practice)
 			movesDsp = 0;
 		else if (cheat)
@@ -874,7 +883,18 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 		randomized = False;
 		break;
-	case PUZZLE_PRACTICE:
+	case ACTION_COMPUTED:
+#ifdef WINVER
+		SET_STARTED(w, False);
+#else
+		SET_STARTED(w, False);
+#ifdef HAVE_OPENGL
+		SET_STARTED(otherw2, False);
+#endif
+		SET_STARTED(otherw1, False);
+#endif
+		break;
+	case ACTION_PRACTICE:
 		movesDsp = 0;
 		randomized = False;
 		practice = !practice;
@@ -902,7 +922,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 #endif
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 #ifdef WINVER
@@ -923,27 +943,27 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 		break;
 #ifdef HAVE_OPENGL
-	case PUZZLE_VIEW:
+	case ACTION_VIEW:
 		{
 			int view;
 
 #ifdef WINVER
 			view = w->skewb.view;
 #else
-			XtVaGetValues(skewbGL,
+			XtVaGetValues(puzzleGL,
 				XtNview, &view, NULL);
 #endif
 			view = (view + 1) % (2 * MAXVIEWS);
 #ifdef WINVER
 			w->skewb.view = view;
 #else
-			XtVaSetValues(skewbGL,
+			XtVaSetValues(puzzleGL,
 				XtNview, view, NULL);
 #endif
 		}
 		break;
 #endif
-	case PUZZLE_ORIENTIZE:
+	case ACTION_ORIENTIZE:
 		movesDsp = 0;
 		orient = !orient;
 		PrintRecord(orient, practice);
@@ -959,23 +979,12 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 		XtVaSetValues(otherw1,
 			XtNorient, orient, NULL);
 #ifdef HAVE_MOTIF
-		XmToggleButtonSetState(orientSwitch, orient, True);
+		XmToggleButtonSetState(orientizeSwitch, orient, True);
 #endif
-#endif
-		break;
-	case PUZZLE_COMPUTED:
-#ifdef WINVER
-		SET_STARTED(w, False);
-#else
-		SET_STARTED(w, False);
-#ifdef HAVE_OPENGL
-		SET_STARTED(otherw2, False);
-#endif
-		SET_STARTED(otherw1, False);
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
@@ -993,7 +1002,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 		break;
 #endif
 #ifndef WINVER
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 #ifdef HAVE_OPENGL
 		XtVaSetValues(otherw2,
@@ -1011,7 +1020,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 		break;
 #endif
 #ifdef WINVER
-	case PUZZLE_DIM:
+	case ACTION_DIM:
 		dim++;
 #ifdef HAVE_OPENGL
 		if (dim > 4)
@@ -1043,7 +1052,7 @@ CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 void
 setPuzzleMove(SkewbWidget w, int reason, int face, int position, int direction)
 {
-	/*Boolean orient, practice, cheat; */
+	/* Boolean orient, practice, cheat; */
 	int dim;
 
 	(void) strcpy(messageDsp, "");
@@ -1054,17 +1063,17 @@ setPuzzleMove(SkewbWidget w, int reason, int face, int position, int direction)
 #endif
 	dim = w->skewb.dim;
 	switch (reason) {
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_CONTROL:
+	case ACTION_CONTROL:
 		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
@@ -1186,83 +1195,83 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			ClearPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_PRACTICE:
+		case ACTION_PRACTICE:
 			PracticePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_ORIENTIZE:
+		case ACTION_ORIENTIZE:
 			OrientizePuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DIM:
+		case ACTION_DIM:
 			(void) DimPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_VIEW:
+		case ACTION_VIEW:
 			(void) ViewPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_TR:
-		case IDM_BR:
-		case IDM_BL:
-		case IDM_TL:
-		case IDM_CW:
-		case IDM_CCW:
-		case IDM_TOP:
-		case IDM_RIGHT:
-		case IDM_BOTTOM:
-		case IDM_LEFT:
+		case ACTION_TR:
+		case ACTION_BR:
+		case ACTION_BL:
+		case ACTION_TL:
+		case ACTION_CW:
+		case ACTION_CCW:
+		case ACTION_TOP:
+		case ACTION_RIGHT:
+		case ACTION_BOTTOM:
+		case ACTION_LEFT:
 			{
 				POINT cursor, origin;
 
@@ -1275,21 +1284,21 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				(void) MovePuzzleInput(&widget,
 					cursor.x - origin.x,
 					cursor.y - origin.y,
-					(int) LOWORD(wParam) - IDM_TR,
+					(int) LOWORD(wParam) - ACTION_TR,
 					FALSE, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_CONTROL_TR:
-		case IDM_CONTROL_BR:
-		case IDM_CONTROL_BL:
-		case IDM_CONTROL_TL:
-		case IDM_CONTROL_CW:
-		case IDM_CONTROL_CCW:
-		case IDM_CONTROL_TOP:
-		case IDM_CONTROL_RIGHT:
-		case IDM_CONTROL_BOTTOM:
-		case IDM_CONTROL_LEFT:
+		case ACTION_CONTROL_TR:
+		case ACTION_CONTROL_BR:
+		case ACTION_CONTROL_BL:
+		case ACTION_CONTROL_TL:
+		case ACTION_CONTROL_CW:
+		case ACTION_CONTROL_CCW:
+		case ACTION_CONTROL_TOP:
+		case ACTION_CONTROL_RIGHT:
+		case ACTION_CONTROL_BOTTOM:
+		case ACTION_CONTROL_LEFT:
 			{
 				POINT cursor, origin;
 
@@ -1302,27 +1311,36 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				(void) MovePuzzleInput(&widget,
 					cursor.x - origin.x,
 					cursor.y - origin.y,
-					(int) LOWORD(wParam) - IDM_CONTROL_TR,
+					(int) LOWORD(wParam) - ACTION_CONTROL_TR,
 					TRUE, FALSE);
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 			}
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -1414,6 +1432,9 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face3", (char *) "*skewb.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-face4", (char *) "*skewb.faceColor4", XrmoptionSepArg, NULL},
 	{(char *) "-face5", (char *) "*skewb.faceColor5", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*skewb.delay", XrmoptionSepArg, NULL},
+	{(char *) "-sound", (char *) "*skewb.sound", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nosound", (char *) "*skewb.sound", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-fn", (char *) "*skewb.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*skewb.font", XrmoptionSepArg, NULL},
 	{(char *) "-view", (char *) "*skewb.view", XrmoptionSepArg, NULL},
@@ -1432,7 +1453,7 @@ static void
 CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(skewb2d, XtNmenu, MENU_PRACTICE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_PRACTICE, NULL);
 	}
 }
 
@@ -1440,7 +1461,7 @@ static void
 CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(skewb2d, XtNmenu, MENU_RANDOMIZE, NULL);
+		XtVaSetValues(puzzle2d, XtNmenu, ACTION_RANDOMIZE, NULL);
 	}
 }
 
@@ -1449,14 +1470,14 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient = cbs->set, practice;
 
-	XtVaGetValues(skewb2d,
+	XtVaGetValues(puzzle2d,
 		XtNpractice, &practice, NULL);
-	XtVaSetValues(skewb2d,
+	XtVaSetValues(puzzle2d,
 		XtNorient, orient, NULL);
-	XtVaSetValues(skewb3d,
+	XtVaSetValues(puzzle3d,
 		XtNorient, orient, NULL);
 #ifdef HAVE_OPENGL
-	XtVaSetValues(skewbGL,
+	XtVaSetValues(puzzleGL,
 		XtNorient, orient, NULL);
 #endif
 	movesDsp = 0;
@@ -1468,22 +1489,23 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
+PracticeToggle(Widget w, XtPointer clientData,
+		XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient, practice = cbs->set;
 
-	XtVaSetValues(skewb2d,
+	XtVaSetValues(puzzle2d,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
-	XtVaSetValues(skewb3d,
+	XtVaSetValues(puzzle3d,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 #ifdef HAVE_OPENGL
-	XtVaSetValues(skewbGL,
+	XtVaSetValues(puzzleGL,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
 #endif
-	XtVaGetValues(skewb2d,
+	XtVaGetValues(puzzle2d,
 		XtNpractice, &orient, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1499,19 +1521,19 @@ PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(skewb2d, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle2d, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(skewb2d, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle2d, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1567,6 +1589,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1603,6 +1626,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1610,8 +1634,9 @@ main(int argc, char **argv)
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
 	XmString undoString, redoString;
-	XmString clearString, randomizeString, practiceString, solveString;
-	XmString orientString, viewString;
+	XmString clearString, practiceString, randomizeString, solveString;
+	XmString orientizeString;
+	XmString speedString, slowString, soundString, viewString;
 #endif
 
 	progDsp = argv[0];
@@ -1620,34 +1645,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) skewb_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) skewb_bits,
-			skewb_width, skewb_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1664,7 +1662,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1676,28 +1675,38 @@ main(int argc, char **argv)
 	undoString = XmStringCreateSimple((char *) "Undo");
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
-	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	practiceString = XmStringCreateSimple((char *) "Practice");
-	solveString = XmStringCreateSimple((char *) "(Solve)");
-	orientString = XmStringCreateSimple((char *) "Orientize");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	solveString = XmStringCreateSimple((char *) "(Auto-Solve)");
+	orientizeString = XmStringCreateSimple((char *) "Orientize");
+	speedString = XmStringCreateSimple((char *) ">Speed");
+	slowString = XmStringCreateSimple((char *) "<Slow");
+	soundString = XmStringCreateSimple((char *) "@Sound");
 	viewString = XmStringCreateSimple((char *) "View");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
+		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
+		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
+		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		XmVaPUSHBUTTON, viewString, 'V', NULL, NULL,
 		NULL);
 	XmStringFree(undoString);
 	XmStringFree(redoString);
 	XmStringFree(clearString);
-	XmStringFree(randomizeString);
 	XmStringFree(practiceString);
+	XmStringFree(randomizeString);
 	XmStringFree(solveString);
-	XmStringFree(orientString);
+	XmStringFree(orientizeString);
+	XmStringFree(speedString);
+	XmStringFree(slowString);
+	XmStringFree(soundString);
 	XmStringFree(viewString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
@@ -1815,10 +1824,10 @@ main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-	orientSwitch = XtVaCreateManagedWidget("Oriented",
+	orientizeSwitch = XtVaCreateManagedWidget("Oriented",
 		xmToggleButtonWidgetClass, switchRowCol,
 		XmNset, DEFAULTORIENT, NULL);
-	XtAddCallback(orientSwitch, XmNvalueChangedCallback,
+	XtAddCallback(orientizeSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) OrientToggle, (XtPointer) NULL);
 	practiceSwitch = XtVaCreateManagedWidget("Practice",
 		xmToggleButtonWidgetClass, switchRowCol,
@@ -1829,54 +1838,86 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Skewb! (use mouse and keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-
-	skewb2d = XtCreateManagedWidget("skewb",
+	puzzle2d = XtCreateManagedWidget("skewb",
 		skewb2dWidgetClass, mainPanel, NULL, 0);
-	XtVaSetValues(skewb2d,
+	XtVaSetValues(puzzle2d,
 		XtNheight, 200, NULL);
-	XtAddCallback(skewb2d, XtNselectCallback,
+	XtAddCallback(puzzle2d, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	skewb3d = XtCreateManagedWidget("skewb",
+	puzzle3d = XtCreateManagedWidget("skewb",
 		skewb3dWidgetClass, mainPanel, NULL, 0);
-	XtVaSetValues(skewb3d,
+	XtVaSetValues(puzzle3d,
 		XtNheight, 200, NULL);
 #ifdef HAVE_OPENGL
-	skewbGL = XtCreateManagedWidget("skewb",
+	puzzleGL = XtCreateManagedWidget("skewb",
 		skewbGLWidgetClass, mainPanel, NULL, 0);
-	XtVaSetValues(skewbGL,
+	XtVaSetValues(puzzleGL,
 		XtNheight, 200, NULL);
 #endif
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
 	shell = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
-	XtVaSetValues(shell,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
 #ifdef HAVE_OPENGL
 	shellGL = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
-	XtVaSetValues(shellGL,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
 #endif
-	skewb2d = XtCreateManagedWidget("skewb",
+	puzzle2d = XtCreateManagedWidget("skewb",
 		skewb2dWidgetClass, topLevel, NULL, 0);
-	XtAddCallback(skewb2d, XtNselectCallback,
-		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	skewb3d = XtCreateManagedWidget("skewb",
+	puzzle3d = XtCreateManagedWidget("skewb",
 		skewb3dWidgetClass, shell, NULL, 0);
 #ifdef HAVE_OPENGL
-	skewbGL = XtCreateManagedWidget("skewb",
+	puzzleGL = XtCreateManagedWidget("skewb",
 		skewbGLWidgetClass, shellGL, NULL, 0);
 #endif
 #endif
-	XtAddCallback(skewb3d, XtNselectCallback,
+	XtVaGetValues(puzzle2d,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+#ifdef HAVE_MOTIF
+	XtVaSetValues(topLevel,
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+		XtNiconPixmap, pixmap, NULL);
+#else
+	XtVaSetValues(topLevel,
+		XtNinput, True,
+		XtNiconPixmap, pixmap, NULL);
+	XtVaSetValues(shell,
+		XtNinput, True,
+		XtNiconPixmap, pixmap, NULL);
+#endif
+	XtAddCallback(puzzle2d, XtNselectCallback,
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
+	XtAddCallback(puzzle3d, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #ifdef HAVE_OPENGL
-	XtAddCallback(skewbGL, XtNselectCallback,
+#ifndef HAVE_MOTIF
+	XtVaSetValues(shellGL,
+		XtNinput, True,
+		XtNiconPixmap, pixmap, NULL);
+#endif
+	XtAddCallback(puzzleGL, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #endif
 	Initialize();
@@ -1887,22 +1928,22 @@ main(int argc, char **argv)
 	XtRealizeWidget(shellGL);
 #endif
 #endif
-	XGrabButton(XtDisplay(skewb2d), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(skewb2d), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle2d), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle2d), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(skewb2d),
-		XCreateFontCursor(XtDisplay(skewb2d), XC_crosshair));
-	XGrabButton(XtDisplay(skewb3d), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(skewb3d), TRUE, (unsigned int) (ButtonPressMask |
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle2d),
+		XCreateFontCursor(XtDisplay(puzzle2d), XC_crosshair));
+	XGrabButton(XtDisplay(puzzle3d), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle3d), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(skewb3d),
-		XCreateFontCursor(XtDisplay(skewb3d), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle3d),
+		XCreateFontCursor(XtDisplay(puzzle3d), XC_crosshair));
 #ifdef HAVE_OPENGL
-	XGrabButton(XtDisplay(skewbGL), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(skewbGL), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzleGL), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzleGL), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(skewbGL),
-		XCreateFontCursor(XtDisplay(skewbGL), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzleGL),
+		XCreateFontCursor(XtDisplay(puzzleGL), XC_crosshair));
 #endif
 	XtMainLoop();
 
