@@ -39,9 +39,6 @@
 
 #define SECTION "setup"
 #else
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
 
 #ifndef DATAFILE
 #define DATAFILE "threed.dat"
@@ -60,6 +57,9 @@ static void SurfaceThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
 static void ObjectThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
+static void SpeedThreeD(ThreeDWidget w, XEvent *event, char **args, int nArgs);
+static void SlowThreeD(ThreeDWidget w, XEvent *event, char **args, int nArgs);
+static void SoundThreeD(ThreeDWidget w, XEvent *event, char **args, int nArgs);
 static void EnterThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int nArgs);
 static void LeaveThreeD(ThreeDWidget w, XEvent *event, char **args,
@@ -113,6 +113,11 @@ static char defaultTranslationsThreeD[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>KP_Divide: MoveRaisePsi()\n\
  <KeyPress>R5: MoveRaisePsi()\n\
  <KeyPress>Home: MoveRaiseThetaRaisePhi()\n\
@@ -194,6 +199,9 @@ static XtActionsRec actionsListThreeD[] = {
 	{(char *) "Release", (XtActionProc) ReleaseThreeD},
 	{(char *) "Surface", (XtActionProc) SurfaceThreeD},
 	{(char *) "Object", (XtActionProc) ObjectThreeD},
+	{(char *) "Speed", (XtActionProc) SpeedThreeD},
+	{(char *) "Slow", (XtActionProc) SlowThreeD},
+	{(char *) "Sound", (XtActionProc) SoundThreeD},
 	{(char *) "Enter", (XtActionProc) EnterThreeD},
 	{(char *) "Leave", (XtActionProc) LeaveThreeD}
 };
@@ -241,6 +249,18 @@ static XtResource resourcesThreeD[] = {
 	{XtNborderColor, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(ThreeDWidget, threed.borderColor),
 	 XtRString, (caddr_t) "gray75" /* XtDefaultForeground*/},
+	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
+	 XtOffset(ThreeDWidget, threed.delay),
+	 XtRString, (caddr_t) "50"}, /* DEFAULTDELAY */
+	{XtNsound, XtCSound, XtRBoolean, sizeof (Boolean),
+	 XtOffset(ThreeDWidget, threed.sound),
+	 XtRString, (caddr_t) "FALSE"},
+	{XtNbumpSound, XtCBumpSound, XtRString, sizeof (String),
+	 XtOffset(ThreeDWidget, threed.bumpSound),
+	 XtRString, (caddr_t) BUMPSOUND},
+	{XtNmoveSound, XtCMoveSound, XtRString, sizeof (String),
+	 XtOffset(ThreeDWidget, threed.moveSound),
+	 XtRString, (caddr_t) MOVESOUND},
 	{XtNxDistance, XtCXDistance, XtRInt, sizeof (int),
 	 XtOffset(ThreeDWidget, threed.distance.x),
 	 XtRString, (caddr_t) "0"},
@@ -261,7 +281,7 @@ static XtResource resourcesThreeD[] = {
 	 XtRString, (caddr_t) "0"},
 	{XtNsurface, XtCSurface, XtRBoolean, sizeof (Boolean),
 	 XtOffset(ThreeDWidget, threed.surface),
-	 XtRString, (caddr_t) "TRUE"}, /* DEFAULTSURFACE */
+	 XtRString, (caddr_t) "TRUE"}, /* DEFAULT_SURFACE */
 	{XtNobject, XtCObject, XtRInt, sizeof (int),
 	 XtOffset(ThreeDWidget, threed.object),
 	 XtRString, (caddr_t) "0"},
@@ -277,6 +297,12 @@ static XtResource resourcesThreeD[] = {
 	{XtNversionOnly, XtCBoolean, XtRBoolean, sizeof (Boolean),
 	 XtOffset(ThreeDWidget, threed.versionOnly),
 	 XtRString, (caddr_t) "FALSE"},
+	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
+	 XtOffset(ThreeDWidget, threed.menu),
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(ThreeDWidget, threed.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(ThreeDWidget, threed.select), XtRCallback, NULL}
 };
@@ -335,8 +361,8 @@ setThreed(ThreeDWidget w, int reason)
 
 #endif
 
-#define RADIANS(x) (M_PI*(x)/(double) ST_ANGLE)
-#define DEGREES(x) ((x)/M_PI*(double) ST_ANGLE)
+#define RADIANS(x) (M_PI*(x)/ST_ANGLE)
+#define DEGREES(x) ((x)/M_PI*ST_ANGLE)
 
 #ifdef DEPTHINDEX
 static Object3D *OBJ;
@@ -612,6 +638,19 @@ DrawObject(ThreeDWidget w, Object3D *obj)
 				&points[0], vert, obj->convex, True);
 		}
 	}
+#ifdef USE_SOUND
+	if (w->threed.sound) {
+		static int oldSurface = -1;
+
+		if (clickSurface != oldSurface && clickSurface >= 0) {
+			playSound((char *) BUMPSOUND);
+			oldSurface = clickSurface;
+		} else if (clickSurface < 0) {
+			oldSurface = -1;
+		}
+	}
+#endif
+
 }
 
 static void
@@ -655,6 +694,32 @@ DrawFrame(ThreeDWidget w, Pixmap dr, Boolean focus)
 			0, 0, w->core.width - 1, w->core.height - 1);
 }
 
+static void
+SpeedObjects(ThreeDWidget w)
+{
+	w->threed.delay -= 5;
+	if (w->threed.delay < 0)
+		w->threed.delay = 0;
+#ifdef HAVE_MOTIF
+	setThreed(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SlowObjects(ThreeDWidget w)
+{
+	w->threed.delay += 5;
+#ifdef HAVE_MOTIF
+	setThreed(w, ACTION_SPEED);
+#endif
+}
+
+static void
+SoundObjects(ThreeDWidget w)
+{
+	w->threed.sound = !w->threed.sound;
+}
+
 static char *
 readLine(FILE *fp, char *string, char special)
 {
@@ -977,26 +1042,35 @@ ReadThreeD(ThreeDWidget w)
 	} color;
 
 	fp = OpenIni(w->core.hWnd, DATAFILE);
-	string = readLine(fp, buffer, SYMBOL);
-	(void) sscanf(string, "%d", &i);
-	w->threed.surface = (i != 0);
-	string = readLine(fp, buffer, SYMBOL);
-	(void) sscanf(string, "%d", &(w->threed.object));
 	for (i = 0; i <= COLORS; i++) {
 		string = readLine(fp, buffer, SYMBOL);
 		(void) sscanf(string, "%d %d %d",
 			&(color.red), &(color.green), &(color.blue));
 		w->threed.shadeGC[i] = RGB(color.red, color.green, color.blue);
 	}
+	string = readLine(fp, buffer, SYMBOL);
+	(void) sscanf(string, "%d", &(w->threed.delay));
+	string = readLine(fp, buffer, SYMBOL);
+	(void) sscanf(string, "%d", &i);
+	w->threed.sound = (i != 0);
+	string = readLine(fp, buffer, SYMBOL);
+	(void) strcpy(w->threed.bumpSound, string);
+	string = readLine(fp, buffer, SYMBOL);
+	(void) strcpy(w->threed.moveSound, string);
+	string = readLine(fp, buffer, SYMBOL);
+	(void) sscanf(string, "%d", &i);
+	w->threed.surface = (i != 0);
+	string = readLine(fp, buffer, SYMBOL);
+	(void) sscanf(string, "%d", &(w->threed.object));
 #ifdef DEBUG
-	(void) fprintf(stderr, "Surface%c%d\n", SYMBOL,
-		(w->threed.surface) ? 1 : 0 );
-	(void) fprintf(stderr, "Object%c%d\n", SYMBOL, w->threed.object);
 	for (i = 0; i <= COLORS; i++)
 		(void) fprintf(stderr, "brush%d%c%d %d %d\n", i, SYMBOL,
 			GetRValue(w->threed.shadeGC[i]),
 			GetGValue(w->threed.shadeGC[i]),
 			GetBValue(w->threed.shadeGC[i]));
+	(void) fprintf(stderr, "Surface%c%d\n", SYMBOL,
+		(w->threed.surface) ? 1 : 0 );
+	(void) fprintf(stderr, "Object%c%d\n", SYMBOL, w->threed.object);
 #endif
 	readCommon(w, fp, buffer);
 	(void) fclose(fp);
@@ -1086,6 +1160,24 @@ SetValuesThreeD(Widget current, Widget request, Widget renew)
 		ResizeThreeD(w);
 		redraw = TRUE;
 	}
+	if (w->threed.menu != ACTION_IGNORE) {
+		int menu = w->threed.menu;
+
+		w->threed.menu = ACTION_IGNORE;
+		switch (menu) {
+		case ACTION_SPEED:
+			SpeedObjects(w);
+			break;
+		case ACTION_SLOW:
+			SlowObjects(w);
+			break;
+		case ACTION_SOUND:
+			SoundObjects(w);
+			break;
+		default:
+			break;
+		}
+	}
 	return (redraw);
 }
 
@@ -1362,7 +1454,7 @@ HideThreeD(ThreeDWidget w
 #endif
 )
 {
-	setThreed(w, THREED_HIDE);
+	setThreed(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -1414,7 +1506,7 @@ MotionThreeD(ThreeDWidget w
 		NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 	w->threed.currentPosition.x = x;
 	w->threed.currentPosition.y = y;
@@ -1435,6 +1527,36 @@ ReleaseThreeD(ThreeDWidget w
 	RepaintGraphicsThreeD(w);
 }
 
+void
+SpeedThreeD(ThreeDWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SpeedObjects(w);
+}
+
+void
+SlowThreeD(ThreeDWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SlowObjects(w);
+}
+
+void
+SoundThreeD(ThreeDWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	SoundObjects(w);
+}
+
 #ifdef WINVER
 void
 setSurfaceThreeD(ThreeDWidget w , Boolean surface)
@@ -1456,7 +1578,7 @@ SurfaceThreeD(ThreeDWidget w
 , XEvent *event, char **args, int n_args
 )
 {
-	setThreed(w, THREED_SURFACE);
+	setThreed(w, ACTION_SURFACE);
 }
 
 static
@@ -1465,7 +1587,7 @@ ObjectThreeD(ThreeDWidget w
 , XEvent *event, char **args, int n_args
 )
 {
-	setThreed(w, THREED_OBJECT);
+	setThreed(w, ACTION_OBJECT);
 }
 
 void
@@ -1499,7 +1621,7 @@ MoveLowerPhiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 	if (w->threed.angle.phi < MINDEGREES)
 		w->threed.angle.phi += NUM_DEGREES;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAngle;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1512,7 +1634,7 @@ MoveLowerThetaThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 	if (w->threed.angle.theta < MINDEGREES)
 		w->threed.angle.theta += NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngle;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1525,7 +1647,7 @@ MoveRaiseThetaThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 	if (w->threed.angle.theta > MAXDEGREES)
 		w->threed.angle.theta -= NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngle;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1538,7 +1660,7 @@ MoveRaisePhiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 	if (w->threed.angle.phi > MAXDEGREES)
 		w->threed.angle.phi -= NUM_DEGREES;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAngle;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1551,7 +1673,7 @@ MoveLowerPsiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 	if (w->threed.angle.psi < MINDEGREES)
 		w->threed.angle.psi += NUM_DEGREES;
 	w->threed.deltaAngle.psi = w->threed.angle.psi - oldAngle;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1564,7 +1686,7 @@ MoveRaisePsiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 	if (w->threed.angle.psi > MAXDEGREES)
 		w->threed.angle.psi -= NUM_DEGREES;
 	w->threed.deltaAngle.psi = w->threed.angle.psi - oldAngle;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1583,7 +1705,7 @@ MoveLowerThetaLowerPhiThreeD(ThreeDWidget w, XEvent *event,
 		w->threed.angle.phi += NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1602,7 +1724,7 @@ MoveLowerThetaRaisePhiThreeD(ThreeDWidget w, XEvent *event,
 		w->threed.angle.phi -= NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1621,7 +1743,7 @@ MoveRaiseThetaLowerPhiThreeD(ThreeDWidget w, XEvent *event,
 		w->threed.angle.phi += NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1640,7 +1762,7 @@ MoveRaiseThetaRaisePhiThreeD(ThreeDWidget w, XEvent *event,
 		w->threed.angle.phi -= NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1649,7 +1771,7 @@ MoveRightThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.x < MAXDISTANCE)
 		w->threed.distance.x += DELTADISTANCE;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1658,7 +1780,7 @@ MoveLeftThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.x > MINDISTANCE)
 		w->threed.distance.x -= DELTADISTANCE;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1667,7 +1789,7 @@ MoveUpThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.y < MAXDISTANCE)
 		w->threed.distance.y += DELTADISTANCE;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1676,7 +1798,7 @@ MoveDownThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.y > MINDISTANCE)
 		w->threed.distance.y -= DELTADISTANCE;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1685,7 +1807,7 @@ MoveOutThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.z < MAXDEPTH)
 		w->threed.distance.z += DELTADEPTH;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 
@@ -1694,7 +1816,7 @@ MoveInThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.z > MINDEPTH)
 		w->threed.distance.z -= DELTADEPTH;
-	setThreed(w, THREED_SET);
+	setThreed(w, ACTION_SET);
 	RepaintGraphicsThreeD(w);
 }
 #endif
