@@ -23,18 +23,9 @@
 #
 */
 
-/*
-  Version 7: 03/11/15 X/Windows
-  Version 5: 95/10/02 Xt/Motif
-  Version 4: 94/11/17 Xt
-  Version 3: 93/05/17 Motif
-  Version 2: 93/05/14 XView
-  Version 1: 93/03/01 MS Windows by Thomas W. Olsen "The C Users Journal"
-*/
-
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Threed Version 7.3\n"
+"Threed Version 7.3.1\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -48,7 +39,9 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-{foreground|fg} {color}]\n"
 "[-{background|bg} {color}] [-frame {color}]\n"
 "[-white {color}] [-[lt|dk]gray {color}] [-black {color}]\n"
-"[-another {color}] [-{border|bd} {color}] [-[no]surface]\n"
+"[-another {color}] [-{border|bd} {color}] [-delay msecs]\n"
+"[-[no]sound] [-bumpSound {filename}]\n"
+"[-moveSound {filename}] [-[no]surface]\n"
 "[-object {int}] [-x {int}] [-y {int}] [-z {int}]\n"
 "[-theta {int}] [-phi {int}] [psi {int}] [-version]\n"
 };
@@ -61,15 +54,33 @@ static const char descriptionHelp[] = {
 
 static const char featuresHelp[] = {
 "Press \"L\" or \"l\" keys to move object left.\n"
+"\n"
 "Press \"R\" or \"r\" keys to move object right.\n"
+"\n"
 "Press \"U\" or \"u\" keys to move object up.\n"
+"\n"
 "Press \"D\" or \"d\" keys to move object down.\n"
+"\n"
 "Press \"I\" or \"i\" keys to move object in.\n"
+"\n"
 "Press \"O\" or \"o\" keys to move object out.\n"
+"\n"
 "Press \"S\" or \"s\" keys to change surface.\n"
+"\n"
 "Press \"B\" or \"b\" keys to change object.\n"
+"\n"
+"Press \">\" or \".\" keys to speed up the movement "
+"(not implemented).\n"
+"\n"
+"Press \"<\" or \",\" keys to slow down the movement "
+"(not implemented).\n"
+"\n"
+"Press \"@\" key to toggle the sound.\n"
+"\n"
 "Press \"Esc\" key to hide program.\n"
+"\n"
 "Press \"Q\", \"fBq\", or \"CTRL-C\" keys kill program.\n"
+"\n"
 "Use the key pad or arrow keys to rotate object without the mouse.\n"
 "Key pad is defined for Threed as:\n"
 "  /     Counterclockwise\n"
@@ -99,7 +110,6 @@ static const char referencesHelp[] = {
 
 #ifdef WINVER
 #include "ThreedP.h"
-#include "wthreed.h"
 #define TITLE "wthreed"
 
 static ThreeDRec widget;
@@ -115,23 +125,23 @@ Initialize(ThreeDWidget w, HBRUSH brush)
 
 	hMenu = GetMenu(w->core.hWnd);
 	(void) CheckMenuItem(hMenu, (unsigned int)
-		w->threed.object + IDM_TETRA, MF_CHECKED);
+		w->threed.object + ACTION_TETRA, MF_CHECKED);
 	(void) CheckMenuItem(hMenu, (unsigned int)
-		w->threed.surface + IDM_WIRE, MF_CHECKED);
+		w->threed.surface + ACTION_WIRE, MF_CHECKED);
 }
 
 void
 setThreed(ThreeDWidget w, int reason)
 {
 	switch (reason) {
-	case THREED_HIDE:
+	case ACTION_HIDE:
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 		break;
 #if 0
-	case THREED_SURFACE:
+	case ACTION_SURFACE:
 		w->threed.surface = !w->threed.surface;
 		break;
-	case THREED_OBJECT:
+	case ACTION_OBJECT:
 		w->threed.object = (w->threed.object + 1) %
 			w->threed.numObjects;
 		break;
@@ -258,50 +268,42 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #endif
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyThreeD(brush);
 			break;
-		case IDM_TETRA:
-		case IDM_HEXA:
-		case IDM_OCTA:
-		case IDM_DODECA:
-		case IDM_ICOSA:
-		case IDM_RHOMBIC:
-		case IDM_CONTA:
-		case IDM_CUBES:
-		case IDM_F16:
+		case ACTION_TETRA:
+		case ACTION_HEXA:
+		case ACTION_OCTA:
+		case ACTION_DODECA:
+		case ACTION_ICOSA:
+		case ACTION_RHOMBIC:
+		case ACTION_CONTA:
+		case ACTION_CUBES:
+		case ACTION_F16:
 			hMenu = GetMenu(hWnd);
 			(void) CheckMenuItem(hMenu,
-				(unsigned int) widget.threed.object + IDM_TETRA,
+				(unsigned int) widget.threed.object + ACTION_TETRA,
 				MF_UNCHECKED);
 			(void) CheckMenuItem(hMenu,
 				LOWORD(wParam), MF_CHECKED);
-			setObjectThreeD(&widget, LOWORD(wParam) - IDM_TETRA);
+			setObjectThreeD(&widget, LOWORD(wParam) - ACTION_TETRA);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HideThreeD(&widget);
 			break;
-		case IDM_WIRE:
-		case IDM_SOLID:
+		case ACTION_WIRE:
+		case ACTION_SOLID:
 			hMenu = GetMenu(hWnd);
 			(void) CheckMenuItem(hMenu,
-				(unsigned int) widget.threed.surface + IDM_WIRE,
+				(unsigned int) widget.threed.surface + ACTION_WIRE,
 				MF_UNCHECKED);
 			(void) CheckMenuItem(hMenu, LOWORD(wParam), MF_CHECKED);
 			setSurfaceThreeD(&widget, (BOOL) (LOWORD(wParam) -
-				IDM_WIRE));
+				ACTION_WIRE));
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DISTXD:
-			if (widget.threed.distance.x >= MINDISTANCE +
-					DELTADISTANCE)
-				widget.threed.distance.x -= DELTADISTANCE;
-			(void) SetScrollPos(hWnd, SB_HORZ,
-				widget.threed.distance.x, TRUE);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_DISTXI:
+		case ACTION_DISTXI:
 			if (widget.threed.distance.x <= MAXDISTANCE -
 					DELTADISTANCE)
 				widget.threed.distance.x += DELTADISTANCE;
@@ -309,15 +311,15 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.distance.x, TRUE);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DISTYD:
-			if (widget.threed.distance.y >= MINDISTANCE +
+		case ACTION_DISTXD:
+			if (widget.threed.distance.x >= MINDISTANCE +
 					DELTADISTANCE)
-				widget.threed.distance.y -= DELTADISTANCE;
-			(void) SetScrollPos(hWnd, SB_VERT,
-				widget.threed.distance.y, TRUE);
+				widget.threed.distance.x -= DELTADISTANCE;
+			(void) SetScrollPos(hWnd, SB_HORZ,
+				widget.threed.distance.x, TRUE);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DISTYI:
+		case ACTION_DISTYI:
 			if (widget.threed.distance.y <= MAXDISTANCE -
 					DELTADISTANCE)
 				widget.threed.distance.y += DELTADISTANCE;
@@ -325,17 +327,34 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.distance.y, TRUE);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DEPTHZD:
-			if (widget.threed.distance.z >= MINDEPTH + DELTADEPTH)
-				widget.threed.distance.z -= DELTADEPTH;
+		case ACTION_DISTYD:
+			if (widget.threed.distance.y >= MINDISTANCE +
+					DELTADISTANCE)
+				widget.threed.distance.y -= DELTADISTANCE;
+			(void) SetScrollPos(hWnd, SB_VERT,
+				widget.threed.distance.y, TRUE);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DEPTHZI:
+		case ACTION_DEPTHZI:
 			if (widget.threed.distance.z <= MAXDEPTH - DELTADEPTH)
 				widget.threed.distance.z += DELTADEPTH;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_THETAD:
+		case ACTION_DEPTHZD:
+			if (widget.threed.distance.z >= MINDEPTH + DELTADEPTH)
+				widget.threed.distance.z -= DELTADEPTH;
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_THETAI:
+			widget.threed.deltaAngle.theta =
+				-widget.threed.angle.theta;
+			widget.threed.angle.theta = (widget.threed.angle.theta +
+				DELTADEGREES) % NUM_DEGREES;
+			widget.threed.deltaAngle.theta +=
+				widget.threed.angle.theta;
+			(void) InvalidateRect(hWnd, NULL, TRUE);
+			break;
+		case ACTION_THETAD:
 			widget.threed.deltaAngle.theta =
 				-widget.threed.angle.theta;
 			widget.threed.angle.theta = (NUM_DEGREES +
@@ -345,16 +364,16 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.theta;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_THETAI:
-			widget.threed.deltaAngle.theta =
-				-widget.threed.angle.theta;
-			widget.threed.angle.theta = (widget.threed.angle.theta +
+		case ACTION_PHII:
+			widget.threed.deltaAngle.phi =
+				-widget.threed.angle.phi;
+			widget.threed.angle.phi = (widget.threed.angle.phi +
 				DELTADEGREES) % NUM_DEGREES;
-			widget.threed.deltaAngle.theta +=
-				widget.threed.angle.theta;
+			widget.threed.deltaAngle.phi +=
+				widget.threed.angle.phi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_PHID:
+		case ACTION_PHID:
 			widget.threed.deltaAngle.phi =
 				-widget.threed.angle.phi;
 			widget.threed.angle.phi = (NUM_DEGREES +
@@ -364,16 +383,16 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.phi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_PHII:
-			widget.threed.deltaAngle.phi =
-				-widget.threed.angle.phi;
-			widget.threed.angle.phi = (widget.threed.angle.phi +
+		case ACTION_PSII:
+			widget.threed.deltaAngle.psi =
+				-widget.threed.angle.psi;
+			widget.threed.angle.psi = (widget.threed.angle.psi +
 				DELTADEGREES) % NUM_DEGREES;
-			widget.threed.deltaAngle.phi +=
-				widget.threed.angle.phi;
+			widget.threed.deltaAngle.psi +=
+				widget.threed.angle.psi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_PSID:
+		case ACTION_PSID:
 			widget.threed.deltaAngle.psi =
 				-widget.threed.angle.psi;
 			widget.threed.angle.psi = (NUM_DEGREES +
@@ -383,16 +402,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.psi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_PSII:
-			widget.threed.deltaAngle.psi =
-				-widget.threed.angle.psi;
-			widget.threed.angle.psi = (widget.threed.angle.psi +
-				DELTADEGREES) % NUM_DEGREES;
-			widget.threed.deltaAngle.psi +=
-				widget.threed.angle.psi;
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_DIAG0:
+		case ACTION_DIAG0:
 			widget.threed.deltaAngle.theta =
 				-widget.threed.angle.theta;
 			widget.threed.deltaAngle.phi =
@@ -408,7 +418,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.phi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DIAG1:
+		case ACTION_DIAG1:
 			widget.threed.deltaAngle.theta =
 				-widget.threed.angle.theta;
 			widget.threed.deltaAngle.phi =
@@ -425,7 +435,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.phi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DIAG2:
+		case ACTION_DIAG2:
 			widget.threed.deltaAngle.theta =
 				-widget.threed.angle.theta;
 			widget.threed.deltaAngle.phi =
@@ -441,7 +451,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.phi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DIAG3:
+		case ACTION_DIAG3:
 			widget.threed.deltaAngle.theta =
 				-widget.threed.angle.theta;
 			widget.threed.deltaAngle.phi =
@@ -456,22 +466,31 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				widget.threed.angle.phi;
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_SPEED:
+			SpeedThreeD(&widget);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SLOW:
+			SlowThreeD(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundThreeD(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	case WM_HSCROLL:
@@ -619,9 +638,19 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #include "Threed.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/threed.xpm"
+#include "pixmaps/threed.t.xpm"
+#include "pixmaps/threed.p.xpm"
+#include "pixmaps/threed.s.xpm"
+#include "pixmaps/threed.m.xpm"
+#include "pixmaps/threed.l.xpm"
+#include "pixmaps/threed.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?threed_t_xpm:threed_p_xpm):\
+(((s)<=24)?threed_s_xpm:threed_m_xpm)):\
+(((s)<=48)?threed_l_xpm:threed_xpm))
 #endif
-#include "icons/threed.xbm"
+#include "pixmaps/threed.xbm"
+#define DEFINE_XBM (char *) threed_bits, threed_width, threed_height
 
 #define FILENAMELEN 1024
 #define TITLELEN 2048
@@ -636,7 +665,7 @@ static char *progDsp;
 #else
 static char titleDsp[TITLELEN] = "";
 #endif
-static Pixmap threedIcon = None;
+static Pixmap pixmap = None;
 static Widget topLevel, threed;
 
 static void
@@ -667,6 +696,11 @@ static XrmOptionDescRec options[] = {
 	{(char *) "-another", (char *) "*threed.anotherBrush", XrmoptionSepArg, NULL},
 	{(char *) "-border", (char *) "*threed.surfaceBorder", XrmoptionSepArg, NULL},
 	{(char *) "-bd", (char *) "*threed.surfaceBorder", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*threed.delay", XrmoptionSepArg, NULL},
+	{(char *) "-sound", (char *) "*threed.sound", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nosound", (char *) "*threed.sound", XrmoptionNoArg, (char *) "FALSE"},
+	{(char *) "-bumpSound", (char *) "*threed.bumpSound", XrmoptionSepArg, NULL},
+	{(char *) "-moveSound", (char *) "*threed.moveSound", XrmoptionSepArg, NULL},
 	{(char *) "-surface", (char *) "*threed.surface", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nosurface", (char *) "*threed.surface", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-object", (char *) "*threed.object", XrmoptionSepArg, (char *) "0"},
@@ -778,18 +812,18 @@ static void CallbackThreeD(Widget w, caddr_t clientData,
 		XtNobjectName, &name,
 		NULL);
 	switch (callData->reason) {
-	case THREED_HIDE:
+	case ACTION_HIDE:
 		(void) XIconifyWindow(XtDisplay(topLevel), XtWindow(topLevel),
 			XScreenNumberOfScreen(XtScreen(topLevel)));
 		break;
-	case THREED_SURFACE:
+	case ACTION_SURFACE:
 		surface = !surface;
 		XtVaSetValues(w, XtNsurface, surface, NULL);
 #ifdef HAVE_MOTIF
 		XmToggleButtonSetState(surfaceSwitch, surface, True);
 #endif
 		break;
-	case THREED_OBJECT:
+	case ACTION_OBJECT:
 		object = (object + 1) % numObjects;
 		XtVaSetValues(w, XtNobject, object, NULL);
 		XtVaGetValues(w,
@@ -1030,6 +1064,14 @@ fileCB(Widget w, void *value, void *clientData)
 	exit(0);
 }
 
+static void
+controlsCB(Widget w, void *value, void *clientData)
+{
+	int val = (int) value;
+
+	XtVaSetValues(threed, XtNmenu, val + ACTION_SPEED, NULL);
+}
+
 
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
@@ -1066,11 +1108,14 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 
 int main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget panel, mainPanel, controlPanel;
 	Widget distRowCol, angleRowCol, shapeRowCol;
-	XmString fileString, quitString;
+	XmString fileString, controlsString;
+	XmString quitString;
+	XmString speedString, slowString, soundString;
 	char **list;
 	int object, number;
 
@@ -1081,49 +1126,38 @@ int main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) threed_xpm, &threedIcon, NULL,
-			&xpmAttributes);
-	}
-	if (threedIcon == (Pixmap) NULL)
-#endif
-		threedIcon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) threed_bits,
-			threed_width, threed_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, threedIcon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	panel = XtVaCreateManagedWidget("panel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
 		XmNsashWidth, 1,
 		XmNsashHeight, 1, NULL);
 	fileString = XmStringCreateSimple((char *) "File");
+	controlsString = XmStringCreateSimple((char *) "Controls");
 	menuBar = XmVaCreateSimpleMenuBar(panel, (char *) "menuBar",
 		XmVaCASCADEBUTTON, fileString, 'F',
+		XmVaCASCADEBUTTON, controlsString, 'C',
 		NULL);
 	XmStringFree(fileString);
+	XmStringFree(controlsString);
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, quitString, 'Q', NULL, NULL,
 		NULL);
 	XmStringFree(quitString);
+	speedString = XmStringCreateSimple((char *) ">Speed");
+	slowString = XmStringCreateSimple((char *) "<Slow");
+	soundString = XmStringCreateSimple((char *) "@Sound");
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "controlsMenu",
+		1, controlsCB,
+		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
+		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
+		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
+		NULL);
+	XmStringFree(speedString);
+	XmStringFree(slowString);
+	XmStringFree(soundString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1245,15 +1279,53 @@ int main(int argc, char **argv)
 		XmNnumColumns, 1,
 		XmNorientation, XmHORIZONTAL,
 		XmNpacking, XmPACK_COLUMN, NULL);
-
-	threed = XtCreateManagedWidget("threed", threedWidgetClass, panel,
+	threed = XtCreateManagedWidget("threed",
+		threedWidgetClass, panel, NULL, 0);
+#else
+	threed = XtCreateManagedWidget("threed",
+		threedWidgetClass, topLevel,
 		NULL, 0);
-	XtAddCallback(threed, XtNselectCallback,
-		(XtCallbackProc) CallbackThreeD, (XtPointer) NULL);
+#endif
 	XtVaGetValues(threed,
+#ifdef HAVE_MOTIF
 		XtNobject, &object,
 		XtNobjectNumber, &number,
-		XtNobjectList, &list, NULL);
+		XtNobjectList, &list,
+#endif
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(threed, XtNselectCallback,
+		(XtCallbackProc) CallbackThreeD, (XtPointer) NULL);
+#ifdef HAVE_MOTIF
 	/*createMenu(&objectMenu, shapeRowCol, object);*/
 	createList(&shapeRowCol, list, object, number);
 	surfaceSwitch = XtVaCreateManagedWidget("Surface",
@@ -1261,17 +1333,7 @@ int main(int argc, char **argv)
 		XmNset, DEFAULTSURFACE, NULL);
 	XtAddCallback(surfaceSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) SurfaceToggle, (XtPointer) NULL);
-#else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, threedIcon,
-		XtNinput, True, NULL);
-	threed = XtCreateManagedWidget("threed",
-		threedWidgetClass, topLevel,
-		NULL, 0);
 #endif
-	XtAddCallback(threed,
-		XtNselectCallback, (XtCallbackProc) CallbackThreeD,
-		(XtPointer) NULL);
 	Initialize(threed);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(threed), (unsigned int) AnyButton, AnyModifier,
