@@ -133,6 +133,11 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>osfEscape: Hide()\n\
  Ctrl<KeyPress>[: Hide()\n\
  <KeyPress>0x1B: Hide()\n\
+ <KeyPress>0x2E: Speed()\n\
+ <KeyPress>0x3E: Speed()\n\
+ <KeyPress>0x3C: Slow()\n\
+ <KeyPress>0x2C: Slow()\n\
+ Shift<KeyPress>2: Sound()\n\
  <KeyPress>Home: MoveTl()\n\
  <KeyPress>KP_7: MoveTl()\n\
  <KeyPress>R7: MoveTl()\n\
@@ -168,11 +173,6 @@ static char defaultTranslationsPuzzle[] =
  <KeyPress>s: Solve()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>d: Decrement()\n\
- <KeyPress>0x2E: Speed()\n\
- <KeyPress>0x3E: Speed()\n\
- <KeyPress>0x3C: Slow()\n\
- <KeyPress>0x2C: Slow()\n\
- <KeyPress>@: Sound()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
@@ -279,7 +279,10 @@ static XtResource resourcesPuzzle[] =
 	 XtRString, (caddr_t) "FALSE"},
 	{XtNmenu, XtCMenu, XtRInt, sizeof (int),
 	 XtOffset(TrianglesWidget, triangles.menu),
-	 XtRString, (caddr_t) "-1"},
+	 XtRString, (caddr_t) "999"}, /* ACTION_IGNORE */
+	{XtNpixmapSize, XtCPixmapSize, XtRInt, sizeof (int),
+	 XtOffset(TrianglesWidget, triangles.pixmapSize),
+	 XtRString, (caddr_t) "64"},
 	{XtNselectCallback, XtCCallback, XtRCallback, sizeof (caddr_t),
 	 XtOffset(TrianglesWidget, triangles.select),
 	 XtRCallback, (caddr_t) NULL}
@@ -835,17 +838,17 @@ DrawAllTiles(const TrianglesWidget w)
 static int
 MovableTile(TrianglesWidget w)
 {
-	int rowType = PUZZLE_BLOCKED, l;
+	int rowType = BLOCKED, l;
 
 	/* Are the spaces in a "row" with the mouse click?
 	 * (If two, then one clicked on a space). */
 	for (l = 0; l < ROWTYPES; l++) {
 		if (w->triangles.currentRow[l] == w->triangles.spaceRow[DOWN][l] &&
 		w->triangles.currentRow[l] == w->triangles.spaceRow[UP][l]) {
-			if (rowType == PUZZLE_BLOCKED) {
+			if (rowType == BLOCKED) {
 				rowType = l;
 			} else {
-				return PUZZLE_SPACE;
+				return SPACE;
 			}
 		}
 	}
@@ -1071,7 +1074,7 @@ AnimateSlide(TrianglesWidget w, int dir, int fast, Boolean logMoves)
 #endif
 	for (aTile = 0; aTile < numTiles; aTile++) {
 		if (logMoves) {
-			setPuzzle(w, PUZZLE_MOVED);
+			setPuzzle(w, ACTION_MOVED);
 			setMove(&undo, direction);
 			flushMoves(w, &redo, FALSE);
 		}
@@ -1274,7 +1277,7 @@ DrawFrame(TrianglesWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoTiles(TrianglesWidget w)
 {
-	setPuzzle(w, PUZZLE_IGNORE);
+	setPuzzle(w, ACTION_IGNORE);
 }
 
 static void
@@ -1418,7 +1421,7 @@ int
 MovePuzzleDir(TrianglesWidget w, const int direction, const int fast)
 {
 	if (MoveTilesDir(w, direction, fast)) {
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		setMove(&undo, direction);
 		flushMoves(w, &redo, FALSE);
 		return TRUE;
@@ -1437,11 +1440,11 @@ MovePuzzle(TrianglesWidget w, const int direction)
 		return FALSE;
 	}
 	if (!MovePuzzleDir(w, direction, NORMAL)) {
-		setPuzzle(w, PUZZLE_BLOCKED);
+		setPuzzle(w, ACTION_BLOCKED);
 		return FALSE;
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 	return TRUE;
 }
@@ -1481,7 +1484,7 @@ DiscreteMoves(TrianglesWidget w, int dir, int fast)
 		next = TileNFromSpace(w, 1, rowType, orient, UP);
 		orient = (orient == UP) ? DOWN : UP;
 		MoveTiles(w, next, orient, fast);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		switch (rowType) {
 		case TLBR:
 			setMove(&undo, BR);
@@ -1512,7 +1515,7 @@ DiscreteMoves(TrianglesWidget w, int dir, int fast)
 		next = TileNFromSpace(w, 1, rowType, orient, DOWN);
 		orient = (orient == UP) ? DOWN : UP;
 		MoveTiles(w, next, orient, fast);
-		setPuzzle(w, PUZZLE_MOVED);
+		setPuzzle(w, ACTION_MOVED);
 		switch (rowType) {
 		case TLBR:
 			setMove(&undo, TL);
@@ -1617,7 +1620,7 @@ SelectTiles(TrianglesWidget w)
 		}
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1658,7 +1661,7 @@ RandomizeTiles(TrianglesWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		setPuzzle(w, PUZZLE_RESET);
+		setPuzzle(w, ACTION_RESET);
 
 		if (w->triangles.size == 2)
 			big *= big;
@@ -1685,10 +1688,10 @@ RandomizeTiles(TrianglesWidget w)
 		}
 		flushMoves(w, &undo, TRUE);
 		flushMoves(w, &redo, FALSE);
-		setPuzzle(w, PUZZLE_RANDOMIZE);
+		setPuzzle(w, ACTION_RANDOMIZE);
 	}
 	if (CheckSolved(w)) {
-		setPuzzle(w, PUZZLE_SOLVED);
+		setPuzzle(w, ACTION_SOLVED);
 	}
 }
 
@@ -1741,10 +1744,10 @@ GetTiles(TrianglesWidget w)
 	(void) fscanf(fp, "%d", &size);
 	if (size >= MINTILES) {
 		for (i = w->triangles.size; i < size; i++) {
-			setPuzzle(w, PUZZLE_INC);
+			setPuzzle(w, ACTION_INCREMENT);
 		}
 		for (i = w->triangles.size; i > size; i--) {
-			setPuzzle(w, PUZZLE_DEC);
+			setPuzzle(w, ACTION_DECREMENT);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: size ");
@@ -1765,7 +1768,7 @@ GetTiles(TrianglesWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
 	scanStartPosition(fp, w);
-	setPuzzle(w, PUZZLE_RESTORE);
+	setPuzzle(w, ACTION_RESTORE);
 	setStartPosition(w);
 	scanMoves(fp, w, moves);
 	(void) fclose(fp);
@@ -1839,7 +1842,7 @@ UndoTiles(TrianglesWidget w)
 		setMove(&redo, direction);
 		direction = (direction + (COORD / 2)) % COORD;
 		if (MoveTilesDir(w, direction, DOUBLE)) {
-			setPuzzle(w, PUZZLE_UNDO);
+			setPuzzle(w, ACTION_UNDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -1862,7 +1865,7 @@ RedoTiles(TrianglesWidget w)
 		getMove(&redo, &direction);
 		setMove(&undo, direction);
 		if (MoveTilesDir(w, direction, DOUBLE)) {
-			setPuzzle(w, PUZZLE_REDO);
+			setPuzzle(w, ACTION_REDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -1882,7 +1885,7 @@ ClearTiles(TrianglesWidget w)
 		return;
 	ResetTiles(w);
 	DrawAllTiles(w);
-	setPuzzle(w, PUZZLE_RESET);
+	setPuzzle(w, ACTION_RESET);
 }
 
 static void
@@ -1890,13 +1893,13 @@ SolveTiles(TrianglesWidget w)
 {
 	if (CheckSolved(w) || w->triangles.currentPosition >= 0)
 		return;
-	setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
+	setPuzzle(w, ACTION_SOLVE_MESSAGE);
 }
 
 static void
 IncrementTiles(TrianglesWidget w)
 {
-	setPuzzle(w, PUZZLE_INC);
+	setPuzzle(w, ACTION_INCREMENT);
 }
 
 static void
@@ -1904,7 +1907,7 @@ DecrementTiles(TrianglesWidget w)
 {
 	if (w->triangles.size <= MINTILES)
 		return;
-	setPuzzle(w, PUZZLE_DEC);
+	setPuzzle(w, ACTION_DECREMENT);
 }
 
 static void
@@ -1914,7 +1917,7 @@ SpeedTiles(TrianglesWidget w)
 	if (w->triangles.delay < 0)
 		w->triangles.delay = 0;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -1923,7 +1926,7 @@ SlowTiles(TrianglesWidget w)
 {
 	w->triangles.delay += 5;
 #ifdef HAVE_MOTIF
-	setPuzzle(w, PUZZLE_SPEED);
+	setPuzzle(w, ACTION_SPEED);
 #endif
 }
 
@@ -2194,45 +2197,45 @@ SetValuesPuzzle(Widget current, Widget request, Widget renew)
 		w->triangles.numSlices = ((w->triangles.delay < MAXSLICES) ?
 			w->triangles.delay + 1 : MAXSLICES);
 	}
-	if (w->triangles.menu != -1) {
+	if (w->triangles.menu != ACTION_IGNORE) {
 		int menu = w->triangles.menu;
 
-		w->triangles.menu = -1;
+		w->triangles.menu = ACTION_IGNORE;
 		switch (menu) {
-		case MENU_GET:
+		case ACTION_GET:
 			GetTiles(w);
 			break;
-		case MENU_WRITE:
+		case ACTION_WRITE:
 			WriteTiles(w);
 			break;
-		case MENU_UNDO:
+		case ACTION_UNDO:
 			UndoTiles(w);
 			break;
-		case MENU_REDO:
+		case ACTION_REDO:
 			RedoTiles(w);
 			break;
-		case MENU_CLEAR:
+		case ACTION_CLEAR:
 			ClearTiles(w);
 			break;
-		case MENU_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			RandomizeTiles(w);
 			break;
-		case MENU_SOLVE:
+		case ACTION_SOLVE:
 			SolveTiles(w);
 			break;
-		case MENU_INCREMENT:
+		case ACTION_INCREMENT:
 			IncrementTiles(w);
 			break;
-		case MENU_DECREMENT:
+		case ACTION_DECREMENT:
 			DecrementTiles(w);
 			break;
-		case MENU_SPEED:
+		case ACTION_SPEED:
 			SpeedTiles(w);
 			break;
-		case MENU_SLOW:
+		case ACTION_SLOW:
 			SlowTiles(w);
 			break;
-		case MENU_SOUND:
+		case ACTION_SOUND:
 			SoundTiles(w);
 			break;
 		default:
@@ -2422,8 +2425,8 @@ ResizePuzzle(TrianglesWidget w)
 		w->triangles.delta.x) / w->triangles.size, 0);
 	w->triangles.offset.y = MAX(((int) w->core.height -
 		2 * w->triangles.delta.y) / w->triangles.size, 0);
-	tempSize.x = (int) ((double) w->triangles.offset.y * 2.0 / sqrt_3);
-	tempSize.y = (int) ((double) w->triangles.offset.x * sqrt_3 / 2.0);
+	tempSize.x = (int) (w->triangles.offset.y * 2.0 / sqrt_3);
+	tempSize.y = (int) (w->triangles.offset.x * sqrt_3 / 2.0);
 	if (tempSize.y < w->triangles.offset.y)
 		w->triangles.offset.y = tempSize.y;
 	else /* tempSize.x <= w->triangles.offset.x */
@@ -2535,7 +2538,7 @@ HidePuzzle(TrianglesWidget w
 #endif
 )
 {
-	setPuzzle(w, PUZZLE_HIDE);
+	setPuzzle(w, ACTION_HIDE);
 }
 
 #ifndef WINVER
@@ -2571,13 +2574,13 @@ SelectPuzzle(TrianglesWidget w
 		if (rowType < 0) {
 			DrawTile(w, w->triangles.currentPosition,
 				w->triangles.currentPositionOrient,
-				rowType == PUZZLE_SPACE, False, TRUE, 0, 0);
+				rowType == SPACE, False, TRUE, 0, 0);
 			FLUSH(w);
 			Sleep(100);
 			DrawTile(w, w->triangles.currentPosition,
 				w->triangles.currentPositionOrient,
 				True, True, TRUE, 0, 0);
-			if (rowType != PUZZLE_SPACE)
+			if (rowType != SPACE)
 				DrawTile(w, w->triangles.currentPosition,
 					w->triangles.currentPositionOrient,
 					False, False, FALSE, 0, 0);
@@ -2622,7 +2625,7 @@ ClearPuzzleMaybe(TrianglesWidget w
 		ClearTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		setPuzzle(w, PUZZLE_CLEAR_QUERY);
+		setPuzzle(w, ACTION_CLEAR_QUERY);
 	}
 #endif
 }
