@@ -23,19 +23,9 @@
 #
 */
 
-/*-
-  Version 7: 03/11/15 X/Windows
-  Version 5: 95/10/01 Xt/Motif
-  Version 4: 94/06/07 Xt
-  Version 3: 93/04/01 Motif
-  Version 2: 92/01/06 XView
-  Version 1: 91/09/05 SunView
-*/
-
 #include "file.h"
 #ifdef WINVER
 #include "TrianglesP.h"
-#include "wtriangles.h"
 #define TITLE "wtriangles"
 
 static TrianglesRec widget;
@@ -63,8 +53,8 @@ static TrianglesRec widget;
 #include <Xm/CascadeB.h>
 #include <Xm/Scale.h>
 #ifdef MOUSEBITMAPS
-#include "icons/mouse-l.xbm"
-#include "icons/mouse-r.xbm"
+#include "pixmaps/mouse-l.xbm"
+#include "pixmaps/mouse-r.xbm"
 #endif
 #define PRINT_MESSAGE(b) PrintState(message, b)
 #else
@@ -74,9 +64,19 @@ static TrianglesRec widget;
 #include "Triangles.h"
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
-#include "icons/triangles.xpm"
+#include "pixmaps/triangles.t.xpm"
+#include "pixmaps/triangles.p.xpm"
+#include "pixmaps/triangles.s.xpm"
+#include "pixmaps/triangles.m.xpm"
+#include "pixmaps/triangles.l.xpm"
+#include "pixmaps/triangles.xpm"
+#define RESIZE_XPM(s) ((char **) ((s)<=32)?\
+(((s)<=22)?(((s)<=16)?triangles_t_xpm:triangles_p_xpm):\
+(((s)<=24)?triangles_s_xpm:triangles_m_xpm)):\
+(((s)<=48)?triangles_l_xpm:triangles_xpm))
 #endif
-#include "icons/triangles.xbm"
+#include "pixmaps/triangles.xbm"
+#define DEFINE_XBM (char *) triangles_bits, triangles_width, triangles_height
 #ifndef SCOREPATH
 #ifdef VMS
 #define SCOREPATH "SYS$LOGIN:"
@@ -88,7 +88,7 @@ static TrianglesRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Triangles Version 7.3\n"
+"Triangles Version 7.3.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -144,7 +144,7 @@ static const char featuresHelp[] = {
 "\n"
 "Press \"I\" or \"i\" keys to increase the number of tiles.\n"
 "\n"
-"Press \"O\" or \"o\" keys to decrease the number of tiles.\n"
+"Press \"D\" or \"d\" keys to decrease the number of tiles.\n"
 "\n"
 "Press \">\" or \".\" keys to speed up the movement of tiles.\n"
 "\n"
@@ -195,9 +195,9 @@ static const char solveHelp[] = {
 typedef struct {
 	int score;
 	char name[USERNAMELEN];
-} GameRecord;
+} PuzzleRecord;
 
-static GameRecord trianglesRecord[MAXTILES - MINTILES + 1];
+static PuzzleRecord puzzleRecord[MAXTILES - MINTILES + 1];
 static int movesDsp = 0;
 static char messageDsp[MESSAGELEN] = "Welcome";
 static char recordDsp[MESSAGELEN] = "NOT RECORDED";
@@ -225,8 +225,8 @@ static Boolean randomized = False;
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap icon = None;
-static Widget topLevel, triangles;
+static Pixmap pixmap = None;
+static Widget topLevel, puzzle;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -251,11 +251,11 @@ PrintRecords(void)
 	(void) printf("TRIANGLES  HALL OF FAME\n\n");
 	(void) printf(" TILES USER            MOVES\n");
 	for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-		if (trianglesRecord[i].score > 0)
+		if (puzzleRecord[i].score > 0)
 			(void) printf("%6d %-12s%9d\n",
 				i + 1,
-				trianglesRecord[i].name,
-				trianglesRecord[i].score);
+				puzzleRecord[i].name,
+				puzzleRecord[i].score);
 	}
 }
 #endif
@@ -266,8 +266,8 @@ InitRecords(void)
 	int i;
 
 	for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
-		trianglesRecord[i].score = NEVER;
-		(void) strncpy(trianglesRecord[i].name, NOACCESS, USERNAMELEN);
+		puzzleRecord[i].score = NEVER;
+		(void) strncpy(puzzleRecord[i].name, NOACCESS, USERNAMELEN);
 	}
 }
 
@@ -319,10 +319,10 @@ ReadRecords(void)
 	free(fname);
 	for (i = 0; i < MAXTILES - MINTILES + 1; i++) {
 		(void) fscanf(fp, "%d %s\n", &n, userName);
-		if (n <= trianglesRecord[i].score ||
-				trianglesRecord[i].score <= NEVER) {
-			trianglesRecord[i].score = n;
-			(void) strncpy(trianglesRecord[i].name, userName,
+		if (n <= puzzleRecord[i].score ||
+				puzzleRecord[i].score <= NEVER) {
+			puzzleRecord[i].score = n;
+			(void) strncpy(puzzleRecord[i].name, userName,
 				USERNAMELEN);
 		}
 	}
@@ -365,8 +365,8 @@ WriteRecords(void)
 #endif
 		for (i = 0; i < MAXTILES - MINTILES + 1; i++)
 			(void) fprintf(fp, "%d %s\n",
-				trianglesRecord[i].score,
-				trianglesRecord[i].name);
+				puzzleRecord[i].score,
+				puzzleRecord[i].name);
 #if HAVE_FCNTL_H
 		(void) close(lfd);
 		(void) unlink(lockFile);
@@ -382,11 +382,11 @@ PrintRecord(int size)
 
 	if (size > MAXTILES) {
 		(void) strncpy(recordDsp, "NOT RECORDED", MESSAGELEN);
-	} else if (trianglesRecord[i].score <= NEVER) {
+	} else if (puzzleRecord[i].score <= NEVER) {
 		(void) sprintf(recordDsp, "NEVER %s", NOACCESS);
 	} else {
 		(void) sprintf(recordDsp, "%d %s",
-			trianglesRecord[i].score, trianglesRecord[i].name);
+			puzzleRecord[i].score, puzzleRecord[i].name);
 	}
 #ifdef HAVE_MOTIF
 	PrintState(record, recordDsp);
@@ -410,7 +410,7 @@ PrintStatus(char *msg, int nMoves
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
-	XtVaSetValues(XtParent(triangles), XtNtitle, titleDsp, NULL);
+	XtVaSetValues(XtParent(puzzle), XtNtitle, titleDsp, NULL);
 #endif
 #endif
 }
@@ -420,11 +420,11 @@ HandleSolved(int counter, int size)
 {
 	int i = size - MINTILES;
 
-	if (size <= MAXTILES && (counter < trianglesRecord[i].score ||
-			trianglesRecord[i].score <= NEVER)) {
+	if (size <= MAXTILES && (counter < puzzleRecord[i].score ||
+			puzzleRecord[i].score <= NEVER)) {
 		ReadRecords();	/* Maybe its been updated by another */
-		trianglesRecord[i].score = counter;
-		(void) strncpy(trianglesRecord[i].name, userNameDsp,
+		puzzleRecord[i].score = counter;
+		(void) strncpy(puzzleRecord[i].name, userNameDsp,
 			USERNAMELEN);
 		WriteRecords();
 		PrintRecord(size);
@@ -548,7 +548,7 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-	case PUZZLE_HIDE:
+	case ACTION_HIDE:
 #ifdef WINVER
 		ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -558,15 +558,15 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 		break;
 #ifndef WINVER
-	case PUZZLE_CLEAR_QUERY:
+	case ACTION_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
 		XtManageChild(clearDialog);
 #else
-		XtVaSetValues(triangles, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 #endif
 		break;
 #endif
-	case PUZZLE_SOLVE_MESSAGE:
+	case ACTION_SOLVE_MESSAGE:
 #ifdef WINVER
 		(void) MessageBox(w->core.hWnd, solveHelp,
 			"Auto-Solve", MB_OK);
@@ -578,30 +578,30 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-	case PUZZLE_RESTORE:
-	case PUZZLE_RESET:
+	case ACTION_RESTORE:
+	case ACTION_RESET:
 		movesDsp = 0;
 		randomized = False;
 		break;
-	case PUZZLE_BLOCKED:
+	case ACTION_BLOCKED:
 		(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 		break;
-	case PUZZLE_SPACE:
+	case ACTION_SPACE:
 #if 0
 		/* Too annoying */
 		(void) strncpy(messageDsp, "Spaces can not slide",
 			MESSAGELEN);
 #endif
 		break;
-	case PUZZLE_IGNORE:
+	case ACTION_IGNORE:
 		(void) strncpy(messageDsp, "Clear to start",
 			MESSAGELEN);
 		break;
-	case PUZZLE_MOVED:
+	case ACTION_MOVED:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_SOLVED:
+	case ACTION_SOLVED:
 		if (cheat)
 			(void) sprintf(messageDsp,
 				"No cheating %s!!", userNameDsp);
@@ -613,27 +613,15 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 				MESSAGELEN);
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_RANDOMIZE:
+	case ACTION_COMPUTED:
+		SET_STARTED(w, False);
+		break;
+	case ACTION_RANDOMIZE:
 		movesDsp = 0;
 		randomized = True;
 		SET_STARTED(w, False);
 		break;
-	case PUZZLE_DEC:
-		movesDsp = 0;
-		size--;
-		PrintRecord(size);
-#ifdef WINVER
-		w->triangles.size = size;
-#else
-		XtVaSetValues(w, XtNsize, size, NULL);
-#ifdef HAVE_MOTIF
-		XmScaleSetValue(tile, size);
-		if (size >= MAXTILES)
-			XtVaSetValues(tile, XmNmaximum, size, NULL);
-#endif
-#endif
-		break;
-	case PUZZLE_INC:
+	case ACTION_INCREMENT:
 		movesDsp = 0;
 		size++;
 		PrintRecord(size);
@@ -648,23 +636,35 @@ CallbackPuzzle(Widget w, caddr_t clientData,
 #endif
 #endif
 		break;
-	case PUZZLE_COMPUTED:
-		SET_STARTED(w, False);
+	case ACTION_DECREMENT:
+		movesDsp = 0;
+		size--;
+		PrintRecord(size);
+#ifdef WINVER
+		w->triangles.size = size;
+#else
+		XtVaSetValues(w, XtNsize, size, NULL);
+#ifdef HAVE_MOTIF
+		XmScaleSetValue(tile, size);
+		if (size >= MAXTILES)
+			XtVaSetValues(tile, XmNmaximum, size, NULL);
+#endif
+#endif
 		break;
-	case PUZZLE_UNDO:
+	case ACTION_UNDO:
 		movesDsp--;
 		SET_STARTED(w, True);
 		break;
-	case PUZZLE_REDO:
+	case ACTION_REDO:
 		movesDsp++;
 		SET_STARTED(w, True);
 		break;
 #if defined(HAVE_MOTIF) && defined(ANIMATE)
-	case PUZZLE_SPEED:
+	case ACTION_SPEED:
 		{
 			int oldDelay, delay;
 
-			XtVaGetValues(triangles,
+			XtVaGetValues(puzzle,
 				XtNdelay, &delay, NULL);
 			oldDelay = delay;
 			if (delay > MAXSPEED - MINSPEED)
@@ -768,103 +768,103 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		break;
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
-		case IDM_GET:
+		case ACTION_GET:
 			GetPuzzle(&widget);
 			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_WRITE:
+		case ACTION_WRITE:
 			WritePuzzle(&widget);
 			break;
-		case IDM_EXIT:
+		case ACTION_EXIT:
 			DestroyPuzzle(&widget, brush);
 			break;
-		case IDM_HIDE:
+		case ACTION_HIDE:
 			HidePuzzle(&widget);
 			break;
-		case IDM_UNDO:
+		case ACTION_UNDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			UndoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_REDO:
+		case ACTION_REDO:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RedoPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_CLEAR:
+		case ACTION_CLEAR:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_RANDOMIZE:
+		case ACTION_RANDOMIZE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_SOLVE:
+		case ACTION_SOLVE:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			SolvePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_SPEED:
-			SpeedPuzzle(&widget);
-			break;
-		case IDM_SLOW:
-			SlowPuzzle(&widget);
-			break;
-		case IDM_SOUND:
-			SoundPuzzle(&widget);
-			break;
-		case IDM_TR:
-		case IDM_RIGHT:
-		case IDM_BR:
-		case IDM_BL:
-		case IDM_LEFT:
-		case IDM_TL:
+		case ACTION_TR:
+		case ACTION_RIGHT:
+		case ACTION_BR:
+		case ACTION_BL:
+		case ACTION_LEFT:
+		case ACTION_TL:
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
 			(void) MovePuzzle(&widget,
-				(int) LOWORD(wParam) - IDM_TR);
+				(int) LOWORD(wParam) - ACTION_TR);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
-		case IDM_DEC:
-			DecrementPuzzle(&widget);
-			SizePuzzle(&widget);
-			(void) InvalidateRect(hWnd, NULL, TRUE);
-			break;
-		case IDM_INC:
+		case ACTION_INCREMENT:
 			IncrementPuzzle(&widget);
 			SizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_ABOUT:
-			(void) DialogBox(widget.core.hInstance,
-				"About", hWnd, (DLGPROC) About);
+		case ACTION_DECREMENT:
+			DecrementPuzzle(&widget);
+			SizePuzzle(&widget);
+			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
-		case IDM_DESCRIPTION:
+		case ACTION_SPEED:
+			SpeedPuzzle(&widget);
+			break;
+		case ACTION_SLOW:
+			SlowPuzzle(&widget);
+			break;
+		case ACTION_SOUND:
+			SoundPuzzle(&widget);
+			break;
+		case ACTION_DESCRIPTION:
 			(void) MessageBox(hWnd, descriptionHelp,
 				"Description", MB_OK);
 			break;
-		case IDM_FEATURES:
+		case ACTION_FEATURES:
 			(void) MessageBox(hWnd, featuresHelp,
 				"Features", MB_OK);
 			break;
-		case IDM_REFERENCES:
+		case ACTION_REFERENCES:
 			(void) MessageBox(hWnd, referencesHelp,
 				"References", MB_OK);
 			break;
+		case ACTION_ABOUT:
+			(void) DialogBox(widget.core.hInstance,
+				"About", hWnd, (DLGPROC) About);
+			break;
 		}
 		break;
 	default:
@@ -975,7 +975,7 @@ CallbackPuzzleClear(Widget w, XtPointer clientData,
 		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(triangles, XtNmenu, MENU_CLEAR, NULL);
+		XtVaSetValues(puzzle, XtNmenu, ACTION_CLEAR, NULL);
 	}
 }
 
@@ -984,11 +984,11 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int size = cbs->value, old;
 
-	XtVaGetValues(triangles,
+	XtVaGetValues(puzzle,
 		XtNsize, &old, NULL);
 	if (old == size)
 		return;
-	XtVaSetValues(triangles,
+	XtVaSetValues(puzzle,
 		XtNsize, size, NULL);
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
@@ -1004,10 +1004,10 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
-	XtVaGetValues(triangles,
+	XtVaGetValues(puzzle,
 		XtNdelay, &oldDelay, NULL);
 	if (oldDelay != delay) {
-		XtVaSetValues(triangles, XtNdelay, delay, NULL);
+		XtVaSetValues(puzzle, XtNdelay, delay, NULL);
 	}
 }
 #endif
@@ -1015,19 +1015,19 @@ SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 static void
 fileCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_GET;
 
-	if (val == MENU_EXIT)
+	if (val == ACTION_EXIT)
 		exit(0);
-	XtVaSetValues(triangles, XtNmenu, val, NULL); /* GWQ */
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
-	int val = (int) value;
+	int val = (int) value + ACTION_UNDO;
 
-	XtVaSetValues(triangles, XtNmenu, val + MENU_EXIT + 1, NULL);
+	XtVaSetValues(puzzle, XtNmenu, val, NULL);
 }
 
 static Widget
@@ -1083,6 +1083,7 @@ createHelp(Widget w, char *text, char *title)
 	XmStringFree(messageString);
 	return messageBox;
 }
+
 static void
 helpCB(Widget w, XtPointer value, XtPointer clientData)
 {
@@ -1119,6 +1120,7 @@ helpCB(Widget w, XtPointer value, XtPointer clientData)
 int
 main(int argc, char **argv)
 {
+	int pixmapSize;
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
 	Widget menuBarPanel, mainPanel, controlPanel;
@@ -1137,34 +1139,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#ifdef HAVE_XPM
-	{
-		XpmAttributes xpmAttributes;
-		XpmColorSymbol transparentColor[1] = {{NULL,
-			(char *) "none", 0 }};
-		Pixel bg;
-
-		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
-		xpmAttributes.colorsymbols = transparentColor;
-		xpmAttributes.numsymbols = 1;
-		xpmAttributes.closeness = 40000;
-		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
-		transparentColor[0].pixel = bg;
-		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) triangles_xpm, &icon, NULL,
-			&xpmAttributes);
-	}
-	if (icon == (Pixmap) NULL)
-#endif
-		icon = XCreateBitmapFromData(XtDisplay(topLevel),
-			RootWindowOfScreen(XtScreen(topLevel)),
-			(char *) triangles_bits,
-			triangles_width, triangles_height);
 #ifdef HAVE_MOTIF
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
@@ -1181,7 +1156,8 @@ main(int argc, char **argv)
 	getString = XmStringCreateSimple((char *) "Get");
 	writeString = XmStringCreateSimple((char *) "Write");
 	quitString = XmStringCreateSimple((char *) "Quit");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu", 0, fileCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "file_menu",
+		0, fileCB,
 		XmVaPUSHBUTTON, getString, 'G', NULL, NULL,
 		XmVaPUSHBUTTON, writeString, 'W', NULL, NULL,
 		XmVaSEPARATOR,
@@ -1194,13 +1170,14 @@ main(int argc, char **argv)
 	redoString = XmStringCreateSimple((char *) "Redo");
 	clearString = XmStringCreateSimple((char *) "Clear");
 	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
-	solveString = XmStringCreateSimple((char *) "(Solve)");
+	solveString = XmStringCreateSimple((char *) "(Auto-Solve)");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
-	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
+	(void) XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu",
+		1, playCB,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
 		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
 		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
@@ -1328,7 +1305,6 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
-
 	tilesRowCol = XtVaCreateManagedWidget("tilesRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1360,24 +1336,53 @@ main(int argc, char **argv)
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	message = XtVaCreateManagedWidget("Play Triangles! (use mouse or keypad)",
 		xmLabelWidgetClass, messageRowCol, NULL);
-	triangles = XtCreateManagedWidget("triangles",
+	puzzle = XtCreateManagedWidget("triangles",
 		trianglesWidgetClass, mainPanel, NULL, 0);
 #else
-	XtVaSetValues(topLevel,
-		XtNiconPixmap, icon,
-		XtNinput, True, NULL);
-	triangles = XtCreateManagedWidget("triangles",
+	puzzle = XtCreateManagedWidget("triangles",
 		trianglesWidgetClass, topLevel, NULL, 0);
 #endif
-	XtAddCallback(triangles, XtNselectCallback,
+	XtVaGetValues(puzzle,
+		XtNpixmapSize, &pixmapSize, NULL);
+#ifdef HAVE_XPM
+	{
+		XpmAttributes xpmAttributes;
+		XpmColorSymbol transparentColor[1] = {{NULL,
+			(char *) "none", 0 }};
+		Pixel bg;
+
+		xpmAttributes.valuemask = XpmColorSymbols | XpmCloseness;
+		xpmAttributes.colorsymbols = transparentColor;
+		xpmAttributes.numsymbols = 1;
+		xpmAttributes.closeness = 40000;
+		XtVaGetValues(topLevel, XtNbackground, &bg, NULL);
+		transparentColor[0].pixel = bg;
+		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			RESIZE_XPM(pixmapSize), &pixmap, NULL,
+			&xpmAttributes);
+	}
+	if (pixmap == (Pixmap) NULL)
+#endif
+		pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
+			RootWindowOfScreen(XtScreen(topLevel)),
+			DEFINE_XBM);
+	XtVaSetValues(topLevel,
+#ifdef HAVE_MOTIF
+		XmNkeyboardFocusPolicy, XmPOINTER, /* not XmEXPLICIT */
+#else
+		XtNinput, True,
+#endif
+		XtNiconPixmap, pixmap, NULL);
+	XtAddCallback(puzzle, XtNselectCallback,
 		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
-	Initialize(triangles);
+	Initialize(puzzle);
 	XtRealizeWidget(topLevel);
-	XGrabButton(XtDisplay(triangles), (unsigned int) AnyButton, AnyModifier,
-		XtWindow(triangles), TRUE, (unsigned int) (ButtonPressMask |
+	XGrabButton(XtDisplay(puzzle), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(puzzle), TRUE, (unsigned int) (ButtonPressMask |
 		ButtonMotionMask | ButtonReleaseMask),
-		GrabModeAsync, GrabModeAsync, XtWindow(triangles),
-		XCreateFontCursor(XtDisplay(triangles), XC_crosshair));
+		GrabModeAsync, GrabModeAsync, XtWindow(puzzle),
+		XCreateFontCursor(XtDisplay(puzzle), XC_crosshair));
 	XtMainLoop();
 
 #ifdef VMS
